package client.UserInput;

import com.golden.gamedev.engine.BaseInput;
import com.golden.gamedev.engine.input.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/**
 * Checks keyboard and mouse inputs.
 *
 */
public class InputControls {

	private BaseInput bInput;
	private static InputControls singleton;
	
	private InputControls(BaseInput bInput){
		this.bInput = bInput;
	}
	/**
	 * Returns instance of singleton
	 * @return 
	 */
	public static InputControls getInstance(){
		if(singleton != null){
			return singleton;
		}
		return null;
	}
	/**
	 * Returns true if enter is pressed.
	 * @return
	 */
	public boolean getEnterPressed(){
		if(bInput.isKeyPressed(KeyEvent.VK_ENTER)){
			return true;
		}
		return false;
	}
	/**
	 * Returns true if ESC is pressed.
	 * @return
	 */
	public boolean getEscPressed(){
		if(bInput.isKeyPressed(KeyEvent.VK_ESCAPE)){
			return true;
		}
		return false;
	}
	/**
	 * Returens true if space is pressed.
	 * @return
	 */
	public boolean getSpacePressed(){
		if(bInput.isKeyPressed(KeyEvent.VK_SPACE)){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns type of input that is pressed.
	 * @return InputType
	 */
	public InputType getPressedButtons(){
		
		if(getLetterKeyPressed()){
			return InputType.LETTER;
		}
		else if(bInput.isKeyPressed(KeyEvent.VK_ENTER)){
			return InputType.ENTER;
		}
		else if(bInput.isKeyPressed(KeyEvent.VK_BACK_SPACE)){
			return InputType.BACKSPACE;
		}
		else if(bInput.isKeyPressed(KeyEvent.VK_TAB)){
			return InputType.TAB;
		}else if(bInput.isKeyPressed(KeyEvent.VK_SPACE)){
			return InputType.SPACE;
		}
		else{
			return InputType.NONE;
		}
		
	}
	
	
	/**
	 * Initializes the singleton
	 * @param bInput
	 */
	public static void TryInitialize(BaseInput bInput){
		if(singleton == null){
			singleton = new InputControls(bInput);
		}
	}
	/**
	 * Returns true if mousebutton 1 is pressed.
	 * @return
	 */
	public boolean getMouseButton1Pressed(){
		if(bInput.isMousePressed(MouseEvent.BUTTON1)){
			return true;
		}
		return false;
	}
	/**
	 * Returns true if mousebutton 2 is pressed.
	 * @return
	 */
	public boolean getMouseButton2Pressed(){
		if(bInput.isMousePressed(MouseEvent.BUTTON2)){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns if the mousebutton1 has been released this update
	 * 
	 * @return true if the mousebutton1 has been released
	 */
	public boolean getMouse1Released(){
		return bInput.isMouseReleased(1);
	}
	
	/**
	 * @return true when the right button is pressed, false otherwise
	 */
	public boolean getMouseButton3Pressed(){
		if(bInput.isMousePressed(MouseEvent.BUTTON3)){
			return true;
		}
		return false;
	}
	/**
	 * Returns X position of the mousepointer.
	 * @return
	 */
	public int getMouseX(){
		return bInput.getMouseX();
	}
	/**
	 * Returns Y position of the mousepointer.
	 * @return
	 */
	public int getMouseY(){
		return bInput.getMouseY();
	}
	/**
	 * Returns true if a letter key has been pressed.
	 * @return
	 */
	public boolean getLetterKeyPressed(){
		if(getPressedLetterKey() != ""){
			return true;
		}
		return false;
	}
	
	/**
	 * Returns a string with the letter of the key pressed.
	 * @return
	 */
	public String getPressedLetterKey(){
		
		int keyPressed = bInput.getKeyPressed();
			if(keyPressed != BaseInput.NO_KEY){
				if(bInput.isKeyDown(KeyEvent.VK_SHIFT)){
				switch (keyPressed) {
				case KeyEvent.VK_A: return "A";
				case KeyEvent.VK_B: return "B";	
				case KeyEvent.VK_C: return "C";	
				case KeyEvent.VK_D: return "D";	
				case KeyEvent.VK_E: return "E";	
				case KeyEvent.VK_F: return "F";	
				case KeyEvent.VK_G: return "G";	
				case KeyEvent.VK_H: return "H";	
				case KeyEvent.VK_I: return "I";	
				case KeyEvent.VK_J: return "J";	
				case KeyEvent.VK_K: return "K";	
				case KeyEvent.VK_L: return "L";	
				case KeyEvent.VK_M: return "M";	
				case KeyEvent.VK_N: return "N";	
				case KeyEvent.VK_O: return "O";	
				case KeyEvent.VK_P: return "P";	
				case KeyEvent.VK_Q: return "Q";	
				case KeyEvent.VK_R: return "R";
				case KeyEvent.VK_S: return "S";	
				case KeyEvent.VK_T: return "T";	
				case KeyEvent.VK_U: return "U";	
				case KeyEvent.VK_V: return "V";	
				case KeyEvent.VK_W: return "W";	
				case KeyEvent.VK_X: return "X";	
				case KeyEvent.VK_Y: return "Y";	
				case KeyEvent.VK_Z: return "Z";	
		
				default:
					return "";
				}
			}
			else{
				switch (keyPressed) {
				case KeyEvent.VK_A: return "a";
				case KeyEvent.VK_B: return "b";	
				case KeyEvent.VK_C: return "c";	
				case KeyEvent.VK_D: return "d";	
				case KeyEvent.VK_E: return "e";	
				case KeyEvent.VK_F: return "f";	
				case KeyEvent.VK_G: return "g";	
				case KeyEvent.VK_H: return "h";	
				case KeyEvent.VK_I: return "i";	
				case KeyEvent.VK_J: return "j";	
				case KeyEvent.VK_K: return "k";	
				case KeyEvent.VK_L: return "l";	
				case KeyEvent.VK_M: return "m";	
				case KeyEvent.VK_N: return "n";	
				case KeyEvent.VK_O: return "o";	
				case KeyEvent.VK_P: return "p";	
				case KeyEvent.VK_Q: return "q";	
				case KeyEvent.VK_R: return "r";
				case KeyEvent.VK_S: return "s";	
				case KeyEvent.VK_T: return "t";	
				case KeyEvent.VK_U: return "u";	
				case KeyEvent.VK_V: return "v";	
				case KeyEvent.VK_W: return "w";	
				case KeyEvent.VK_X: return "x";	
				case KeyEvent.VK_Y: return "y";	
				case KeyEvent.VK_Z: return "z";	
		
				default:
					return "";
				}
			}
		}
		return "";
	}
	
	
}
