package client.clientState;

import client.clientState.loginMenuState.LoginMenuState;
import client.clientState.worldState.WorldState;
import shared.network.messages.Message;
/**
 * 
 * Abstract class that is inherited by WorldState and LoginMenuState.
 */
public abstract class GameState {

	public static final int LOGINMENU = 1;
	public static final int WORLDMODE = 2;
	
	/**
	 * called on every update from the gameloop, when that GameState is the active one
	 * @param elapsedTime
	 * @return
	 */
	public abstract GameState update(long elapsedTime);
	
	/**
	 * called whenever a message is received, from lobby or world server
	 * @param msg
	 */
	public abstract void receivedMessage(Message msg);
	
	/**
	 * forces the GameState to draw everything, used when the state is switched to, etc.
	 */
	public abstract void forceDraw();
	
	private static LoginMenuState loginMenu;
	private static WorldState worldMode;
	/**
	 * Return WorldState
	 * @return
	 */
	public static WorldState getWorldMenuState(){
		if(worldMode == null){
			worldMode = new WorldState();
		}
		return worldMode;
	}
	/**
	 * Return LoginMenuState.
	 * @return
	 */
	public static LoginMenuState getLoginMenuState(){
		if(loginMenu == null){
			loginMenu = new LoginMenuState();
		}
		return loginMenu;
	}
	/**
	 * Returns the number of the state.
	 * @param state
	 * @return
	 */
	public static int getStateNumber(GameState state){
		if(state == getLoginMenuState()){
			return LOGINMENU;
		}
		else if(state == getWorldMenuState()){
			return WORLDMODE;
		}
		else{
			return -1;
		}
	}
	
}
