package client.clientState;

import shared.logger.LogSystem;
import shared.network.messages.Message;
/**
 * 
 * Controls which state the game is in.
 */
public class StateControl {

	private int activeState;
	private static StateControl singleton;
	
	public static StateControl getInstance(){
		if(singleton == null){
			singleton = new StateControl();
		}
		return singleton;
	}
	
	public StateControl(){
		
		GameState.getLoginMenuState();
		GameState.getWorldMenuState();
		
		activeState = GameState.LOGINMENU;
		getActiveState().forceDraw();
	}
	/**
	 * Is called by NetworkHandlerMain when a message is received.
	 * @param msg
	 */
	public void receivedMessage(Message msg){
		getActiveState().receivedMessage(msg);
	}
	/**
	 * Is called by ClientGameLoop in each update.
	 * @param arg0
	 */
	public void update(long arg0){
		GameState nextState = getActiveState().update(arg0);
		
		if(nextState != getActiveState()){
			activeState = GameState.getStateNumber(nextState);
			getActiveState().forceDraw();
		}
		
		
	}
	/**
	 * Returns the active GameState.
	 * @return
	 */
	public GameState getActiveState(){
		if(activeState == GameState.LOGINMENU){
			return GameState.getLoginMenuState();
		}
		else if(activeState == GameState.WORLDMODE){
			return GameState.getWorldMenuState();
		}
		else{
			return null;
		}
	}
	
	public static void writeLog(String msg){
		LogSystem.getInstance().writeClientLog(msg);
	}
	
	
	
}
