package client.clientState.loginMenuState;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import client.clientState.StateControl;
import client.graphics.GraphicObject;
import client.graphics.GraphicObjectFactory;
import client.graphics.components.WowButton;
import shared.models.Gender;
import shared.models.PlayableCharacter;
import shared.models.PlayableClasses;
import shared.models.RespawnPoint;
import shared.models.SimpleZone;
import shared.network.messages.LoginMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.RequestZoneMessage;
import shared.serverHelpers.ClientKingdomInfo;
import shared.serverHelpers.ShortCharacter;
import shared.serverHelpers.WorldServer;
/**
 * 
 * Shows the Character selection screen.
 * Player chooses a character to log in with.
 */
public class CharacterSelectMenu extends LoginMenu implements MouseListener {

	private ClientKingdomInfo chosenKingdom;
	private WowButton newCharacterButton;
	
	private ArrayList<GraphicObject> characterTextList;
	private ArrayList<WowButton> characterSelectButtons;
	
	private boolean gotCharacter;
	
	public CharacterSelectMenu(ClientKingdomInfo kingdom){
		
		chosenKingdom = kingdom;
		newCharacterButton = GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng("newchar", 200, 500, false);
		newCharacterButton.addMouseListener(this);
		
		characterTextList = new ArrayList<GraphicObject>();
		characterSelectButtons = new ArrayList<WowButton>();
		
		ArrayList<ShortCharacter> characters = chosenKingdom.getCharacters();
		
		for (int i = 0; i < characters.size(); i++) {
			
			characterTextList.add(client.graphics.GraphicObjectFactory.getInstance().buildStandardTextGraphicObject(
					characters.get(i).getName(), 200, 200 + (i * 50)));
			
			WowButton button = client.graphics.GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng(
					"play", 450, 200 + (i * 50) - 30, false);
			button.addMouseListener(this);
			characterSelectButtons.add(button);
			
		}
		
		gotCharacter = false;
		
	}
	
	
	@Override
	public void forceRedraw() {

		draw();
		
	}

	@Override
	public void receiveMessage(Message msg) {
		
		if(msg.getMessageType() == MessageType.OBJECT_MESSAGE){
			ObjectMessage objMsg = (ObjectMessage) msg;
			if(objMsg.getObjectType() == ObjectType.CHARACTER_OBJECT){
				System.out.println("Got character");
				PlayableCharacter cha = (PlayableCharacter) objMsg.getObject();
				client.clientState.GameState.getWorldMenuState().setCharacter(cha);
				gotCharacter = true;
			}
		}
		
	}

	@Override
	public LoginMenu update() {
	
		for (WowButton button : characterSelectButtons) {
			button.updateInput();
		}
		newCharacterButton.updateInput();
		
		if(gotCharacter){
			return new GoToWorldDummy();
		}
		
		
		return this;
	}
	
	private void draw(){
		
		for (GraphicObject graphicObject : characterTextList) {
			client.graphics.ViewFacade.getInstance().addMidFrontSprite(graphicObject);
		}
		for (WowButton button : characterSelectButtons) {
			client.graphics.ViewFacade.getInstance().addMidFrontSprite(button);
		}
		
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("LoginMenuBack1", 0, 0));
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("LeftTitleHigh", 0, 0));
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("Select Character", 200, 50));
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(newCharacterButton);
		
		
		
		client.graphics.ViewFacade.getInstance().updateAll();
	}

	@Override
	public void enterPressed() {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mouseClicked(MouseEvent arg0) {
		
		if(arg0.getSource() instanceof WowButton) {
			
			if(arg0.getSource() == newCharacterButton){
				//TODO: Temporary solution for creating new characters. 
			
				client.network.NetworkHandlerMain.getInstance().sendNewPlayableCharacterRequest("TestCharacter", PlayableClasses.FIGHTER, Gender.MALE, chosenKingdom.getServerInfo().getKingdomName());
			}
			else{
				for (int i = 0; i < characterSelectButtons.size(); i++) {
					
					if(arg0.getSource() == characterSelectButtons.get(i)){

						//client.network.NetworkHandlerMain.getInstance().connectToWorldServer(chosenKingdom.getServerInfo());
						client.network.NetworkHandlerMain.getInstance().connectCommand(chosenKingdom.getCharacter(i).getName(), chosenKingdom.getServerInfo());
						return;
						
					}
					
				}
			}
			
		}
		
	}


	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

}
