package client.clientState.loginMenuState;

import java.util.ArrayList;

import client.graphics.components.KeyBoardFocusableComponent;
import client.graphics.components.keyBoardFocusHandler;
import shared.network.messages.Message;


/**The abstract class for all menus that are part of the login process.
 * 
 * @author Henka
 *
 *	@see LoginMenuState
 */
public abstract class LoginMenu implements keyBoardFocusHandler{

	protected ArrayList<KeyBoardFocusableComponent> keyboardFocusComponents = new ArrayList<KeyBoardFocusableComponent>();
	protected KeyBoardFocusableComponent focusComponent;
	protected KeyBoardFocusableComponent nextFocusComponent;
	
	public abstract LoginMenu update();
	public abstract void receiveMessage(Message msg);
	public abstract void forceRedraw();
	
	/**
	 * Method for handling components requests for keyboardfocus.
	 */
	@Override
	public boolean requestKeyboardFocus( KeyBoardFocusableComponent requestingComponent) {
		
		if(keyboardFocusComponents.contains(requestingComponent)){

			nextFocusComponent = requestingComponent;
			
			return true;
		}
		return false;
	}


	@Override
	public void tabReturned() {
		if(focusComponent == null){
			focusComponent = keyboardFocusComponents.get(0);
		}
		else{
			int plass = keyboardFocusComponents.indexOf(focusComponent);
			int nextCompIndex = plass + 1;
			if(nextCompIndex >= keyboardFocusComponents.size()){
				nextCompIndex = 0;
			}
			
			nextFocusComponent = keyboardFocusComponents.get(nextCompIndex);
			
		}
		
		
	}
	
	public void addKeyboardFocusComponent(KeyBoardFocusableComponent component){
		keyboardFocusComponents.add(component);
	}
	public void removeKeyboardFocusComponent(KeyBoardFocusableComponent component){
		keyboardFocusComponents.remove(component);
	}
	
	public void updateKeyboardFocus(){
		if(nextFocusComponent != null && nextFocusComponent != focusComponent){
			changeKeyboardFocus(nextFocusComponent);
		}
	}
	
	private void changeKeyboardFocus(KeyBoardFocusableComponent component){
		if(focusComponent != null){
			focusComponent.releaseKeyboardFocus();
		}
		focusComponent = component;
		focusComponent.receiveKeyboardFocus();
	}
	
	
}
