package client.clientState.loginMenuState;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import shared.network.messages.*;
import shared.serverHelpers.ClientKingdomInfo;
import client.clientState.StateControl;
import client.graphics.*;
import client.graphics.components.KeyBoardFocusableComponent;
import client.graphics.components.WowButton;
import client.graphics.components.WowPasswordField;
import client.graphics.components.WowTextField;
import client.graphics.components.keyBoardFocusHandler;
/**
 * 
 * Main login window for inputting username and password.
 */
public class MainLoginMenu extends LoginMenu implements MouseListener {
	
	private boolean logedin = false;
	private WowButton loginButton;
	private WowTextField nameTextField;
	private WowPasswordField passWordTextField;
	
	private boolean shouldRedraw;
	
	private boolean showErrorMessage;
	private String errorMessage;
	
	private ArrayList<ClientKingdomInfo> serverList;
	private boolean gotServerList;
	
	public MainLoginMenu(){
		loginButton = GraphicObjectFactory.getInstance().buildWowButtonObjectPng("login", 640, 500, false, false);
		loginButton.addMouseListener(this);
		nameTextField = GraphicObjectFactory.getInstance().buildWowTextField("USER NAME", 200, 300, 200, this, false);
		nameTextField.addMouseListener(this);
		passWordTextField = GraphicObjectFactory.getInstance().buildWowPasswordField("PASSWORD", 200, 400, 200, this, false);
		shouldRedraw = false;
		
		//TODO: Just for testing!
		nameTextField.setText("Henka");
		passWordTextField.setText("lett");
		
	}
	
	
	@Override
	public void receiveMessage(Message msg) {

		if(msg.getMessageType() == MessageType.LOGIN_RESULT){
			if(((LoginResult)msg).isLoginSucessfull()){
				logedin = true;
			}
			else{
				showErrorMessage = true;
				shouldRedraw = true;
				errorMessage = "login failed, please retype username and password";
				nameTextField.resetToInitText();
				passWordTextField.resetToInitText();
			}
			
		}
		if(msg.getMessageType() == MessageType.OBJECT_MESSAGE){
			ObjectMessage objMsg = (ObjectMessage) msg;
			if(objMsg.getObjectType() == ObjectType.KINGDOM_INFO_LIST){
			
				gotServerList = true;
				serverList = (ArrayList<ClientKingdomInfo>)objMsg.getObject();
				
			}
		}

	}

	@Override
	public LoginMenu update() {
		
		loginButton.updateInput();
		nameTextField.updateInput();
		passWordTextField.updateInput();
		
		updateKeyboardFocus();
		
		if(gotServerList){
			return new ServerSelectLoginMenu(serverList);
		}
		
		updateKeyboardFocus();
		
		if(shouldRedraw){
			forceRedraw();
			shouldRedraw = false;
		}
		
		return this;
		
	}

	@Override
	public void forceRedraw() {

		draw();
	}
	
	private void draw(){
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("LoginMenuBack1", 0, 0));
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("LeftTitleHigh", 0, 0));
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("Login Main", 200, 50));
		if(showErrorMessage){
			client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildRedTextGraphicObject(errorMessage, 200, 460));
			
		}
		
		if(logedin){
			client.graphics.ViewFacade.getInstance().addMidFrontSprite(GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("LOGGED IN!", 200, 200));			
		}
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(loginButton);
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(nameTextField);
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(passWordTextField);
		client.graphics.ViewFacade.getInstance().updateAll();
		
	}


	@Override
	public void mouseClicked(MouseEvent e) {
		if(e.getSource() == loginButton){
			if( !(nameTextField.isInitText() || passWordTextField.isInitText()) ){
				client.network.NetworkHandlerMain.getInstance().sendLobbyLoginMessage(nameTextField.getText(), passWordTextField.getText());
			}
			else{
				showErrorMessage = true;
				shouldRedraw = true;
				errorMessage = "Please write both username and password";
			}
			
		}
	}


	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
	}


	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
	}


	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
	}


	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void enterPressed() {
		if( !(nameTextField.isInitText() || passWordTextField.isInitText()) ){
			client.network.NetworkHandlerMain.getInstance().sendLobbyLoginMessage(nameTextField.getText(), passWordTextField.getText());
		}
		else{
			showErrorMessage = true;
			shouldRedraw = true;
			errorMessage = "Please write both username and password";
		}
		
	}



	
}