package client.clientState.loginMenuState;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import client.clientState.StateControl;
import client.graphics.GraphicObject;
import client.graphics.GraphicObjectFactory;
import client.graphics.components.WowButton;
import shared.network.messages.LoginMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.serverHelpers.ClientKingdomInfo;
/**
 * 
 * Shows the Server Selection window.
 */
public class ServerSelectLoginMenu extends LoginMenu implements MouseListener {

	private ArrayList<GraphicObject> serverTextList;
	private ArrayList<WowButton> serverConnectButtons;
	
	private ArrayList<ClientKingdomInfo> serverList;
	
	private boolean serverChosen;
	private int chosenServerIndex;
	
	public ServerSelectLoginMenu(ArrayList<ClientKingdomInfo> kingdomList){
		
		this.serverList = kingdomList;
		serverTextList = new ArrayList<GraphicObject>();
		serverConnectButtons = new ArrayList<WowButton>();
		
		for (int i = 0; i < serverList.size(); i++) {
			
			serverTextList.add(client.graphics.GraphicObjectFactory.getInstance().buildStandardTextGraphicObject(
					serverList.get(i).getServerInfo().getKingdomName(), 200, 200 + (i * 50)));
			
			WowButton button = client.graphics.GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng(
					"connect", 450, 200 + (i * 50) - 30, false);
			button.addMouseListener(this);
			serverConnectButtons.add(button);
			
		}
		
	}
	
	
	@Override
	public void forceRedraw() {

		draw();
		
	}

	@Override
	public void receiveMessage(Message msg) {
		
		
	}

	@Override
	public LoginMenu update() {
		
//		if(client.UserInput.InputControls.getInstance().getEnterPressed()){
//			client.network.NetworkHandlerMain.getInstance().connectCommand("Tester");
//		}
		
		if(serverChosen){
			return new CharacterSelectMenu(serverList.get(chosenServerIndex));
		}
		
		for (WowButton button : serverConnectButtons) {
			button.updateInput();
		}
		
		return this;
	}
	
	private void draw(){
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("LoginMenuBack1", 0, 0));
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("LeftTitleHigh", 0, 0));
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("Select Server", 200, 50));
		
		for (GraphicObject graphicObject : serverTextList) {
			client.graphics.ViewFacade.getInstance().addMidBackSprite(graphicObject);
		}
		for (WowButton button : serverConnectButtons) {
			client.graphics.ViewFacade.getInstance().addMidBackSprite(button);
		}
		
		client.graphics.ViewFacade.getInstance().updateAll();
	}

	@Override
	public void enterPressed() {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mouseClicked(MouseEvent arg0) {

		if(arg0.getSource() instanceof WowButton){
			for (int i = 0; i < serverConnectButtons.size(); i++) {
				
				if(arg0.getSource() == serverConnectButtons.get(i)){
					chosenServerIndex = i;
					serverChosen = true;
					return;
				}
				
			}
		}
		
	}


	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

}
