package client.clientState.worldState;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import shared.models.Location;
import shared.network.messages.ChatMessage;

import client.clientState.Settings;
import client.graphics.SpriteGraphicObject;
import client.graphics.components.KeyBoardFocusableComponent;
import client.graphics.components.WowButton;
import client.graphics.components.WowTextField;
import client.graphics.components.keyBoardFocusHandler;
import client.graphics.components.TextGraphicObject;
import client.network.NetworkHandlerMain;
/**
 * View of the Chat.
 * @author thorgrun
 *
 */
public class ChatPanel implements keyBoardFocusHandler,MouseListener,ChatListener{

	private TextGraphicObject line1;
	private TextGraphicObject line2;
	private TextGraphicObject line3;
	private TextGraphicObject line4;
	private TextGraphicObject line5;
	private TextGraphicObject line6;
	private WowButton buttonUp;
	private WowButton buttonDown;
	private WowTextField chatField;
	private ChatSystem chatSystem;
	private SpriteGraphicObject background;
	private Location location;
	private int offSet;
	private int numberOfChatLines;
	public ChatPanel(){
		offSet = 0;
		background = client.graphics.GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("chatBackground", 0,Settings.playfieldHeight-50 );
		location = new Location(5,440);
		buttonUp = client.graphics.GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng("buttonUp", 175,415, false);
		buttonUp.addMouseListener(this);
		buttonDown = client.graphics.GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng("buttonDown", 175,535, false);
		buttonDown.addMouseListener(this);
		chatField = new WowTextField(location.getX(),location.getY()+120,200,"Chat",this,false);
		chatField.addMouseListener(this);
		Font chatFont = new Font("Arial",Font.PLAIN, 12);
		line1 = new TextGraphicObject("Line1",location.getX(),location.getY()+100,chatFont,Color.BLACK,true);
		line2 = new TextGraphicObject("Line2",location.getX(),location.getY()+80,chatFont,Color.BLACK,true);
		line3 = new TextGraphicObject("Line3",location.getX(),location.getY()+60,chatFont,Color.BLACK,true);
		line4 = new TextGraphicObject("Line4",location.getX(),location.getY()+40,chatFont,Color.BLACK,true);
		line5 = new TextGraphicObject("Line5",location.getX(),location.getY()+20,chatFont,Color.BLACK,true);
		line6 = new TextGraphicObject("Line6",location.getX(),location.getY(),chatFont,Color.BLACK,true);
		chatSystem = new ChatSystem(this,chatFont,200);
		updateChat();
	}
	/**
	 * Is called by the WorldState when a chat message is received from the world server.
	 * @param msg
	 */
	public void receiveChatMessage(ChatMessage msg){
		chatSystem.addMessage(msg.getToChannel(), msg.getFromName()+": "+msg.getMessage());
	}
	@Override
	public void addKeyboardFocusComponent(KeyBoardFocusableComponent component) {
		// TODO Auto-generated method stub
		
	}
	/**
	 * Checks input/button clicks on the up/down buttons, and the textfield.
	 */
	public void update(){
		buttonUp.updateInput();
		buttonDown.updateInput();
		chatField.updateInput();
	}
	
	/**
	 * Updates the viewed text in the Chat.
	 */
	public void updateChat() {
		ArrayList<String> chat = chatSystem.currentChat();
		numberOfChatLines= chat.size();
		line1.updateText(chat.get(chat.size()-1-offSet));
		line2.updateText(chat.get(chat.size()-2-offSet));
		line3.updateText(chat.get(chat.size()-3-offSet));
		line4.updateText(chat.get(chat.size()-4-offSet));
		line5.updateText(chat.get(chat.size()-5-offSet));
		line6.updateText(chat.get(chat.size()-6-offSet));
	}
	/**
	 * Fills ViewFacade buffer with ChatPanel elements.
	 */
	public void forceDraw(){
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(background);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(buttonUp);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(buttonDown);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(line1);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(line2);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(line3);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(line4);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(line5);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(line6);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(chatField);
		
	}
	/**
	 * When enter is pressed, sends a message to the WorldServer, and empties TextField.
	 */
	public void enterPressed() {
		NetworkHandlerMain.getInstance().sendChatMessage(chatField.getText(), "Main");
		chatField.setText("");
		
	}

	@Override
	public void removeKeyboardFocusComponent(KeyBoardFocusableComponent component) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean requestKeyboardFocus(KeyBoardFocusableComponent requestingComponent) {
		chatField.receiveKeyboardFocus();
		return true;
	}

	@Override
	public void tabReturned() {
		
	}
	/**
	 * When button up or down is pressed, it updates the Chat.
	 */
	public void mouseClicked(MouseEvent e) {
		if(e.getSource() == buttonUp){
			if(numberOfChatLines>offSet+6){
				offSet++;
				updateChat();
			}
		}else if(e.getSource()==buttonDown){
			if(offSet>0){
				offSet--;
				updateChat();
			}
		}
	}
	@Override
	public void mouseEntered(MouseEvent e) {
	}
	@Override
	public void mouseExited(MouseEvent e) {
		
	}
	@Override
	public void mousePressed(MouseEvent e) {
		
	}
	@Override
	public void mouseReleased(MouseEvent e) {
		
	}

}
