package client.clientState.worldState;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
/**
 * The ChatSystem.
 * @author thorgrun
 *
 */
public class ChatSystem {
	private ChatListener listener;
	private HashMap<String,ArrayList<String>> chatChannels;
	private String currentChatChannel;
	private Font chatFont;
	private int availableWidth;
	public ChatSystem(ChatListener listener,Font chatFont,int width){
		chatChannels = new HashMap<String, ArrayList<String>>();
		createNewChannel("Main");
		this.chatFont = chatFont;
		this.availableWidth = width;
		currentChatChannel = "Main";
		this.listener = listener;
	}
	/**
	 * Returns all the chat from the current chatchannel.
	 * @return
	 */
	public ArrayList<String> currentChat(){
		return chatChannels.get(currentChatChannel);
	}
	/**
	 * Creates a new chat channel.
	 * @param channel
	 */
	private void createNewChannel(String channel){
		ArrayList<String> chat = new ArrayList<String>();
		for(int i=0;i<6;i++){
			chat.add("");//Add 5 empty lines
		}
		chatChannels.put(channel,chat);
	}
	/**
	 * 
	 * @param str
	 * @return
	 */
	private int StringWidth(String str){
        FontMetrics metrics = new FontMetrics(chatFont) {};
        Rectangle2D bounds = metrics.getStringBounds(str, null);
		return (int) bounds.getWidth();
	}
	/**
	 * Wraps message.
	 * @param message
	 * @return
	 */
	private ArrayList<String> wrapMessage(String message){
		ArrayList<String> wrappedMessage = new ArrayList<String>();
		
		ArrayList<String> listOfWords = new ArrayList<String>();
		String tempString = "";
		for(int i =0 ; i < message.length(); i++){
			if(message.charAt(i) == ' '){
				listOfWords.add(tempString);
				tempString = "";
			}
			else {
				tempString += message.charAt(i);
			}
		}
		listOfWords.add(tempString);
		tempString = "";
		String finishedLine = "";
		boolean finished = false;
		while(! finished){
			if(listOfWords.size() > 0){
				tempString += listOfWords.get(0);
				tempString += " "; //adds a space
				listOfWords.remove(0);
			}
			else{
				finished = true;
				wrappedMessage.add(finishedLine);
				break;
			}
			if(availableWidth-20 > StringWidth(finishedLine + tempString)){
				finishedLine += tempString;
				tempString = "";
			}
			else{
				wrappedMessage.add(finishedLine);
				finishedLine = tempString;
				tempString ="";
			}
		}
        return wrappedMessage;
	}
	/**
	 * Adds a message to a channel.
	 * Wraps the message first to fit the ChatPanel.
	 * Creates new channel if it does not exist.
	 * @param channel
	 * @param message
	 */
	public void addMessage(String channel,String message){
		if(chatChannels.get(channel)==null){
			createNewChannel(channel);
		}
		for(String str:wrapMessage(message)){
			chatChannels.get(channel).add(str);
		}
		
		if(currentChatChannel.equals(channel)){
			listener.updateChat();
		}
	}
}
