package client.clientState.worldState;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import shared.models.AttributeTypes;

import client.clientState.Settings;
import client.graphics.SpriteGraphicObject;
import client.graphics.components.StatusBar;
import client.graphics.components.WowButton;
import client.graphics.*;
/**
 * 
 * View of the HUD. 
 */
public class HUDMenuPanel implements MouseListener{

	private SpriteGraphicObject bottomnPanelBackground;
	private SpriteGraphicObject sidePanelBackground;
	
	private StatusBar healthBar;
	private StatusBar manaBar;
	private StatusBar xpBar;
	
	private final int statusBarsX = 250;
	private final int healthBarY = 480;
	private final int manaBarY = 520;
	private final int xpBarY = 560;
	
	private WowButton mainMenuButton;
	private WowButton characterMenuButton;
	
	private WorldModel worldModel;
	
	private boolean shouldGoToCharscreen;
	
	public HUDMenuPanel(WorldModel worldModel) {
		
		int bottomnVertical = Settings.screenHeight - Settings.playfieldHeight;
		int bottomnHorizontal = Settings.screenWidth;
		int sideVertical = Settings.playfieldHeight;
		int sideHorizontal = Settings.screenWidth - Settings.playfieldWidth;
		
		this.worldModel = worldModel;
		
		if(bottomnVertical == 150 && bottomnHorizontal == 800 && sideHorizontal == 150 && sideVertical == 450){
			bottomnPanelBackground = client.graphics.GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("hudpanelbunn", 0, Settings.playfieldHeight);
			sidePanelBackground = client.graphics.GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("hudpanelside", Settings.playfieldWidth, 0);

		}
		
		characterMenuButton = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("characterMenuButton_up", 600, 500, false, false);
		mainMenuButton = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("MainMenuButton_up", 680, 500, false, false);
		characterMenuButton.addMouseListener(this);
		mainMenuButton.addMouseListener(this);
		
		healthBar = GraphicObjectFactory.getInstance().buildRedStatusBar(statusBarsX, healthBarY, 0, worldModel.getCharacter().getAttribute().getMaxHP(), AttributeTypes.HP);
		manaBar = GraphicObjectFactory.getInstance().buildBlueStatusBar(statusBarsX, manaBarY, 0, worldModel.getCharacter().getAttribute().getMaxMP(), AttributeTypes.MP);
		
		
		//TODO: right xp for next level as maxvalue/ right xp for minvalue?!
		xpBar = GraphicObjectFactory.getInstance().buildGoldStatusBar(statusBarsX, xpBarY, 0, 0, null);
		
		worldModel.getCharacter().getAttribute().addAttributeListener(healthBar);
		worldModel.getCharacter().getAttribute().addAttributeListener(manaBar);
		worldModel.getCharacter().getAttribute().addAttributeListener(xpBar);
		
		shouldGoToCharscreen = false;
		
	}
	/**
	 * Checks for input and updates status bars.
	 * @param elapsedtime
	 * @return
	 */
	public PopupState update(long elapsedtime) {
		
		characterMenuButton.updateInput();
		mainMenuButton.updateInput();
		
		healthBar.update(elapsedtime);
		manaBar.update(elapsedtime);
		xpBar.update(elapsedtime);
		
		if(shouldGoToCharscreen){
			shouldGoToCharscreen = false;
			return PopupState.CHARACTER_SCREEN_ACTIVE;
		}
		
		return PopupState.POPUP_INACTIVE;
		
	}
	/**
	 * Fills buffer with HUD elements.
	 */
	public void forceDraw() {
	
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(bottomnPanelBackground);
		client.graphics.ViewFacade.getInstance().addMidFrontSprite(sidePanelBackground);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(characterMenuButton);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(mainMenuButton);
		
		client.graphics.ViewFacade.getInstance().addForegroundSprite(healthBar);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(manaBar);
		client.graphics.ViewFacade.getInstance().addForegroundSprite(xpBar);
		
	}

	/**
	 * Checks if buttons has been clicked.
	 */
	public void mouseClicked(MouseEvent arg0) {
		
		if(arg0.getSource() == characterMenuButton){
			shouldGoToCharscreen = true;
		}
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	
//	
//	g.setColor(Color.GRAY);
//	g.fillRect(Settings.playfieldWidth, 0,Settings.screenWidth-Settings.playfieldWidth , Settings.screenHeight);
//	g.fillRect(0,Settings.playfieldHeight,Settings.playfieldWidth,Settings.screenHeight-Settings.playfieldHeight);	
	
	
}
