package client.clientState.worldState;

import java.util.ArrayList;
import java.util.Iterator;

import shared.CommonFunctions;
import shared.models.Attack;
import shared.models.AttributeTypes;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.SimpleCharacter;
import shared.models.SimpleZone;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.WorldObject;
import shared.models.SimpleCharacter.CharacterType;
import client.UserInput.InputControls;
import client.clientState.Settings;
import client.graphics.Background;
import client.graphics.GraphicObjectFactory;
import client.graphics.WorldObjectSprite;
import client.graphics.components.AttackPanel;
import client.graphics.components.QuestionPanel;
import client.graphics.components.SimpleStatusBar;
import client.graphics.components.TravelPanel;
import client.network.NetworkHandlerMain;
/**
 * 
 * View of the gameworld.
 */
public class MainWorldPanel {

	private Location screenFocus;
	private Background background;
	
	private WorldModel worldModel;

	
	private WorldState worldState;
	
	private TravelPanel travelPanel;
	private AttackPanel attackPanel;
	private PopupState popupState = PopupState.POPUP_INACTIVE;
	private boolean needToRedraw = false; // this will be used to update the screen when eg. 
									     // the attackpanel is destoyed
	private String previouslyClickedCharacterId;
	public MainWorldPanel(WorldModel worldModel, String startZoneId, WorldState worldState) {
		this.worldModel = worldModel;
		setCurrentZone(startZoneId);
		this.worldState = worldState;
	}
	/**
	 * Sets the current zone.
	 * Updates background.	
	 * @param zoneId
	 */
	public void setCurrentZone(String zoneId){
		SimpleZone zone = worldModel.getZone(zoneId);
		background = new Background();
		background.setBackgroundTileSet(zone.getBackgroundTileArray());
		screenFocus = worldModel.getCharacter().getCurrentLocation();
		background.setScreenFocus(screenFocus);
	}
	/**
	 * sets the screenfocus to character location, unless it is to close to edge of map
	 */
	private void updateScreenFocus(){
		Location charLoc = worldModel.getCharacter().getCurrentLocation();
		
		if(charLoc.getX()>Settings.playfieldWidth/2){
			
		}
	}
	
	private boolean collisionDetected(long elapsedTime,PlayableCharacter character){
		Location newPosition = CommonFunctions.updatePositions(elapsedTime, character.getCurrentLocation(),character.getTargetLocation());
		Iterator<StaticObject> itstaticObject = worldModel.currentZone().getStaticObjectList().values().iterator();
		while(itstaticObject.hasNext()){
			StaticObject staObj = itstaticObject.next();
			if(staObj.withinBounds(newPosition)){
				return true;
			}
		}
		Iterator<SimpleCharacter> itEnemyNPC = worldModel.currentZone().getEnemyNPCList().values().iterator();
		while(itEnemyNPC.hasNext()){
			SimpleCharacter npc = itEnemyNPC.next();
			if(npc.withinBounds(newPosition)){
				return true;
			}
		}
		Iterator<SimpleCharacter> itFriendNPC = worldModel.currentZone().getFriendNPCList().values().iterator();
		while(itFriendNPC.hasNext()){
			SimpleCharacter npc = itFriendNPC.next();
			if(npc.withinBounds(newPosition)){
				return true;
			}
		}
		//No collision, then move character.
		character.setCurrentLocation(newPosition);
		return false;
	}
	/**
	 * Update method of GameWorld, updates positions and checks userinputs.
	 * @param elapsedtime
	 * @return - true, if we wants to forceDraw false otherwise
	 */
	public boolean update(long elapsedtime) {
		InputControls input = client.UserInput.InputControls.getInstance();
		switch(popupState){
		case POPUP_INACTIVE:
			if(input.getMouseButton1Pressed()){
				int mouseX =input.getMouseX();
				int mouseY = input.getMouseY();
				if(background.withinBounds(mouseX, mouseY)&&(mouseX<Settings.playfieldWidth&&mouseY<Settings.playfieldHeight)){
					Location targetLocation= new Location(screenFocus.getX()+(mouseX-Settings.playfieldWidth/2),screenFocus.getY()+(mouseY-Settings.playfieldHeight/2));
				
					//TODO:fiks bug med currentlocation blir satt til 50-50 nr man sender over nettverk
					client.network.NetworkHandlerMain.getInstance().sendMovementRequest(worldModel.getCharacter().getName(), new Location(worldModel.getCharacter().getCurrentLocation().getX()+0,worldModel.getCharacter().getCurrentLocation().getY()+0), targetLocation);
				}
			}
			else if(input.getMouseButton3Pressed()){
				
				
				int mouseX =input.getMouseX();
				int mouseY = input.getMouseY();
				//if the attack panel not exist, create a new one 
				if(attackPanel == null&&travelPanel == null){
					SimpleCharacter clickedChar = isCharacterIsClicked(mouseX, mouseY);
					TravelTrigger clickedTrigger = isTriggerClicked(mouseX,mouseY);
					if(clickedChar!=null){
						switch(clickedChar.getCharacterType()){
						case ENEMY_NPC:
							attackPanel = new AttackPanel(mouseX, mouseY, 
									worldModel.getCharacter().getListOfPhysicalAttacks(),
									worldModel.getCharacter().getListOfMagicalAttacks(), 
									this, previouslyClickedCharacterId, CharacterType.ENEMY_NPC);
							popupState = PopupState.ATTACK_POPUP_ACTIVE;
							needToRedraw = true;
							break;
						case FRIENDLY_NPC:
							break;
						case PLAYER_CHARACTER:
							break;
						}
					}else if(clickedTrigger!=null){
						//Checks the distance to the character, need to be within 50 pixels of the trigger
						if(getDistanceToMouseClick(mouseX, mouseY)<50){
							
							travelPanel = new TravelPanel(mouseX,mouseY,clickedTrigger,this);
							popupState = PopupState.TRAVEL_POPUP_ACTIVE;
							needToRedraw = true;
						}
					}
				}
			}
			break;
		case TRAVEL_POPUP_ACTIVE:
				if(travelPanel!=null){
					travelPanel.update(elapsedtime);
					if(travelPanel.hasAnswered()){
						System.out.println("Answered");
						if(travelPanel.answeredYes()){
							System.out.println("answeredYES");
							NetworkHandlerMain.getInstance().sendTravelTrigger(
									worldModel.getCharacter().getName(),
									travelPanel.getTrigger().getDestinationZoneName(),
									travelPanel.getTrigger().getDestinationId());
							
							
						}
						
						
						destroyTravelPanel();
					}
				}
			
			break;
		case ATTACK_POPUP_ACTIVE:
			//we are in the attackPanel, we would like to cancel the 
			   //choice of the attackPanel 
				if(input.getMouseButton3Pressed()){
					popupState = PopupState.POPUP_INACTIVE;
					if(attackPanel != null){
						destroyAttackPanel();
						needToRedraw = true;
					}
				}
			break;
		}
		//we only need to update the attackPanel if it has been drawn on the screen
		 if(attackPanel != null){
			 attackPanel.update(elapsedtime);
		 }
		updatePositions(elapsedtime);
		if(collisionDetected(elapsedtime,getCharacter())){
			Location targetLocation = new Location(worldModel.getCharacter().getCurrentLocation().getX()+0,worldModel.getCharacter().getCurrentLocation().getY()+0);
		
			//TODO:fiks bug med currentlocation blir satt til 100-100 nr man sender over nettverk
			client.network.NetworkHandlerMain.getInstance().sendMovementRequest(
					worldModel.getCharacter().getName(), 
					new Location(worldModel.getCharacter().getCurrentLocation().getX()+0,worldModel.getCharacter().getCurrentLocation().getY()+0),
					targetLocation);
		}
		boolean returnvalue = false || needToRedraw;
		//if we need to force redraw, we do not need this next time the update is called
		needToRedraw = false;
		return returnvalue;
	}
	/**
	 * Returns distance between mouse click and the character object.
	 * Used by attacks skills to get distance between NPC and Player.
	 * TODO: Should be replaced by a check between the NPC location and Player location.
	 * @param mouseX
	 * @param mouseY
	 * @return
	 */
	public int getDistanceToMouseClick(int mouseX,int mouseY){
		return worldModel.getCharacter().getDistance(CommonFunctions.getXinPlayField(mouseX, screenFocus.getX()),CommonFunctions.getYinPlayField(mouseY, screenFocus.getY()));
	}
	private void updatePositions(long elapsedTime){
		Iterator<SimpleCharacter> itFriendNPC = worldModel.currentZone().getFriendNPCList().values().iterator();
		while(itFriendNPC.hasNext()){
			SimpleCharacter npc = itFriendNPC.next();
			npc.updatePosition(elapsedTime);
		}
		Iterator<SimpleCharacter> itEnemyNPC = worldModel.currentZone().getEnemyNPCList().values().iterator();
		while(itEnemyNPC.hasNext()){
			SimpleCharacter npc = itEnemyNPC.next();
			npc.updatePosition(elapsedTime);
		}
		Iterator<SimpleCharacter> itPC = worldModel.currentZone().getCharacterList().values().iterator();
		while(itPC.hasNext()){
			SimpleCharacter character = itPC.next();
			character.updatePosition(elapsedTime);
		}
	}
	private TravelTrigger isTriggerClicked(int mouseX,int mouseY){
		Iterator<String> itTrigger = worldModel.currentZone().getTravelTriggerList().keySet().iterator();
		while(itTrigger.hasNext()){
			String triggerId = itTrigger.next();
			TravelTrigger  traTri = worldModel.currentZone().getTravelTriggerList().get(triggerId);
			if(checkIfCharacterIsAtPosition(traTri, mouseX, mouseY)){
				if(traTri.getDestinationId()!=0){
					previouslyClickedCharacterId= triggerId;
					return traTri;
				}
				
			}
		}
		
		
		return null;
	}
	//this method checks if the mouse is on a character
	private SimpleCharacter isCharacterIsClicked(int mouseX,int mouseY){
		Iterator<String> itFriendNPC = worldModel.currentZone().getFriendNPCList().keySet().iterator();
		while(itFriendNPC.hasNext()){
			String fNPCid = itFriendNPC.next();
			SimpleCharacter fNPC = worldModel.currentZone().getFriendNPCList().get(fNPCid);
			if(checkIfCharacterIsAtPosition(fNPC, mouseX, mouseY)){
				previouslyClickedCharacterId= fNPCid;
				return fNPC;
			}
		}
		Iterator<String> itEnemyNPC = worldModel.currentZone().getEnemyNPCList().keySet().iterator();
		while(itEnemyNPC.hasNext()){
			String eNPCid = itEnemyNPC.next();
			SimpleCharacter eNPC = worldModel.currentZone().getEnemyNPCList().get(eNPCid);
			if(checkIfCharacterIsAtPosition(eNPC, mouseX, mouseY)){
				previouslyClickedCharacterId=eNPCid;
				return eNPC;
			}
		}
		Iterator<String> itPC = worldModel.currentZone().getCharacterList().keySet().iterator();
		while(itPC.hasNext()){
			String charId = itPC.next();
			SimpleCharacter character = worldModel.currentZone().getCharacterList().get(charId);
			if(checkIfCharacterIsAtPosition(character, mouseX, mouseY)){
				previouslyClickedCharacterId=charId;
				return character;
			}
		}
		previouslyClickedCharacterId= null;
		return null;
	}
	private boolean checkIfCharacterIsAtPosition(WorldObject object, int mouseX,int mouseY){
		int objectX = CommonFunctions.drawXinPlayField(object.getCurrentLocation().getX(), screenFocus.getX(),object.getImageWidth());
		int objectY = CommonFunctions.drawYinPlayField(object.getCurrentLocation().getY(), screenFocus.getY(),object.getImageHeight());
		
		if((mouseX<objectX+object.getImageWidth()&&mouseX>objectX)&&(mouseY<objectY+object.getImageHeight()&&mouseY>objectY)){
			return true;
		}
		return false;
	}
	
	/**
	 * This method will destroy the attackPanel.
	 * This should be used when there is no more need for an
	 * attack panel.
	 */
	public void destroyAttackPanel(){
		this.attackPanel = null;
		needToRedraw = true;
		popupState = PopupState.POPUP_INACTIVE;
	}
	
	private void drawStaticObjects(){
		Iterator<StaticObject> it = worldModel.currentZone().getStaticObjectList().values().iterator();
		while(it.hasNext()){
			StaticObject obj = it.next();
			client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildWorldObjectSprite(obj.getTextureId(), obj,screenFocus));
		}
	}
	
	private void drawFriendlyNPCS(){
		Iterator<SimpleCharacter> it = worldModel.currentZone().getFriendNPCList().values().iterator();
		while(it.hasNext()){
			SimpleCharacter npc = it.next();
			client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildWorldObjectSprite(npc.getImagePath(), npc,screenFocus));
		}
	}
	
	private void drawEnemyNPCS(){
		Iterator<SimpleCharacter> it = worldModel.currentZone().getEnemyNPCList().values().iterator();
		while(it.hasNext()){
			SimpleCharacter npc = it.next();
			client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildWorldObjectSprite(npc.getImagePath(),npc,screenFocus));
			SimpleStatusBar healthBar = new SimpleStatusBar(npc.getCurrentLocation(),screenFocus,0,npc.getMaxHP(),npc.getHP(),npc.getImageWidth(),npc.getImageHeight(),AttributeTypes.HP);
			npc.addAttributeListener(healthBar);
			client.graphics.ViewFacade.getInstance().addMidBackSprite(healthBar);
		}
	}
	
	private void drawPlayerCharacters(){
		Iterator<SimpleCharacter> it = worldModel.currentZone().getCharacterList().values().iterator();
		while(it.hasNext()){
			SimpleCharacter tempChar = it.next();
			client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildWorldObjectSprite(tempChar.getImagePath(), tempChar,screenFocus));
		}
	}
	private void drawTravelTriggers(){
		Iterator<TravelTrigger> it = worldModel.currentZone().getTravelTriggerList().values().iterator();
		while(it.hasNext()){
			client.graphics.ViewFacade.getInstance().addMidBackSprite(new WorldObjectSprite(null,it.next(),screenFocus));
		}
	}
	public PlayableCharacter getCharacter(){
		return worldModel.getCharacter();
	}
	/**
	 * Fills the buffer in ViewFacade with all the world objects.
	 */
	public void forceDraw() {
		//Draw background
		client.graphics.ViewFacade.getInstance().addBackgroundSprite(background);
		
		//Draw midback, static
		drawStaticObjects();
		drawFriendlyNPCS();
		drawEnemyNPCS();
		drawPlayerCharacters();
		drawTravelTriggers();
	
		//Draw dude
		client.graphics.ViewFacade.getInstance().addMidBackSprite(GraphicObjectFactory.getInstance().buildWorldObjectSprite("testDude", worldModel.getCharacter(),screenFocus));
		//draws the travelPanel
		if(travelPanel !=null){
			client.graphics.ViewFacade.getInstance().addForegroundSprite(travelPanel);
		}
		//Draws foreground
		if(attackPanel != null){
			client.graphics.ViewFacade.getInstance().addForegroundSprite(attackPanel);
		}
	}
	/**
	 * destroy travelpanel
	 */
	public void destroyTravelPanel(){
		this.travelPanel = null;
		needToRedraw = true;
		popupState = PopupState.POPUP_INACTIVE;
	}
	/**
	 * creates a qustionPanel
	 * @param attack - the attack which will be used
	 * @param targetId  - the targes's id
	 */
	public void createQuestionPanel(Attack attack, String targetId){
		worldState.createQuestionPanel(attack,targetId);
	}
}