package client.clientState.worldState;

import java.util.HashMap;

import client.graphics.Background;
import client.network.NetworkHandlerMain;

import shared.models.AttackType;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.SimpleZone;
/**
 * 
 * Model of the world used by WorldState.
 */
public class WorldModel {

	
	private PlayableCharacter character;
	private HashMap<String,SimpleZone> world;
	private String currentZoneId;
	private String getSessionId(){
		return NetworkHandlerMain.getInstance().getSessionId();
	}
	public WorldModel() {
		world = new HashMap<String, SimpleZone>();
	}
	
	public void setCharacter(PlayableCharacter character){
		this.character = character;
	}
	
	public SimpleZone currentZone(){
		return world.get(currentZoneId);
	}
	/**
	 * Updates the Movement of an object, by setting current position and target position.
	 * @param objectId
	 * @param current
	 * @param target
	 */
	public void updateMovement(String objectId,Location current,Location target){
		if(objectId.equals(character.getName())){
			character.setCurrentLocation(current);
			character.setTargetLocation(target);
		}else if(currentZone().getCharacterList().get(objectId)!=null){
			currentZone().getCharacterList().get(objectId).setCurrentLocation(current);
			currentZone().getCharacterList().get(objectId).setTargetLocation(target);
		}else if(currentZone().getEnemyNPCList().get(objectId)!=null){
			currentZone().getEnemyNPCList().get(objectId).setCurrentLocation(current);
			currentZone().getEnemyNPCList().get(objectId).setTargetLocation(target);
		}
	}
	/**
	 * Updates the health of an object. 
	 * TODO: Implement some feedback when someone misses.
	 * @param id
	 * @param health
	 * @param hit
	 */
	public void updateHealth(String id,int health,boolean hit){
		if(hit){
			if(currentZone().getEnemyNPCList().get(id)!=null){
				currentZone().getEnemyNPCList().get(id).setHP(health);
				System.out.println(id+" has this much hp left: "+currentZone().getEnemyNPCList().get(id).getHP());
			}else if(currentZone().getCharacterList().get(id)!=null){
				currentZone().getCharacterList().get(id).setHP(health);
			}else if(character.getName().equals(id)){
				character.getAttribute().setHP(health);
				System.out.println("MONSTER ATTACKED YOU!");
			}
		}else{
			System.out.println("you missed");
		}
		
		
	}
	/**
	 * Is called when an object dies. 
	 * @param id
	 * @param nameOfKiller
	 */
	public void isDead(String id,String nameOfKiller){
		//TODO: "kill NPC" ,change animation/picture, make him inactive/non attackable
		System.out.println(id+" is dead and was killed by "+nameOfKiller);
		if(id.equals(character.getName())){
			character.getAttribute().setHP(0);
			//TODO: player is dead, give option to respawn
			System.out.println("You are dead.");
		}else if(currentZone().getCharacterList().get(id)!=null){
			currentZone().getCharacterList().get(id).setHP(0);
		}else if(currentZone().getEnemyNPCList().get(id)!=null){
			currentZone().getEnemyNPCList().get(id).setHP(0);
		}
	}
	public void setCurrentZone(String zoneId){
		currentZoneId = zoneId;

	}
	
	public void addZone(SimpleZone zone){
		world.put(zone.getZoneId(),zone);
		setCurrentZone(zone.getZoneId());
	}
	
	public SimpleZone getZone(String zoneId){
		return world.get(zoneId);
	}

	public PlayableCharacter getCharacter() {
		return character;
	}

	public String getCurrentZoneId() {
		return currentZoneId;
	}
	
	
	
	
}
