package client.clientState.worldState.playerMenu;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import client.clientState.worldState.WorldModel;
import client.graphics.GraphicObjectFactory;
import client.graphics.SpriteGraphicObject;
import client.graphics.ViewFacade;
import client.graphics.components.AttackButton;
import client.graphics.components.TextGraphicObject;
import client.graphics.components.WowButtonState;
import shared.models.Attack;
import shared.models.PlayableCharacter;
import shared.network.messages.Message;
/**
 * 
 * Shows Attacksskills of the Character. 
 */
public class AttackTab extends PlayerMenuTab implements MouseListener {

	private SpriteGraphicObject backGround;
	private PlayableCharacter character;
	
	private ArrayList<AttackButton> magicalAttacks;
	private ArrayList<AttackButton> physicalAttacks;
	
	private TextGraphicObject detailHeader;
	
	private TextGraphicObject detailDamageText;
	private TextGraphicObject detailQuestionLevelText;
	private TextGraphicObject detailAttributeText;
	private TextGraphicObject detailReloadTimeText;
	private TextGraphicObject detailTypeText;
	private TextGraphicObject detailNumberOfQuestionText;
	private TextGraphicObject detailAnswerPercentText;
	
	private TextGraphicObject detailDamageValue;
	private TextGraphicObject detailQuestionLevelValue;
	private TextGraphicObject detailAttributeValue;
	private TextGraphicObject detailReloadTimeValue;
	private TextGraphicObject detailTypeValue;
	private TextGraphicObject detailNumberOfQuestionValue;
	private TextGraphicObject detailAnswerPercentValue;
	
	private AttackButton selectedButton;
	private boolean selectedAttackIsMagical;
	
	private final int attackButtonOffsetX = 20;
	
	private final int attackStartY = 40;
	private final int attackOffsetY = 40;
	private final int attackTextHeadOffsetY = 25;
	
	private final int detailHeaderX = 280;
	private final int detailHeaderY = 30;
	private final int detailsStartY = detailHeaderY + 20;
	private final int detailsOffsetY = 20;
	private final int detailsTextX = 260;
	private final int detailsValueX = 430;
	

	private final int detailDamageY = detailsStartY + (detailsOffsetY * 0); 
	private final int detailQuestionLevelY = detailsStartY + (detailsOffsetY * 1); 
	private final int detailAttributeY = detailsStartY + (detailsOffsetY * 2); 
	private final int detailReloadTimeY = detailsStartY + (detailsOffsetY * 3); 
	private final int detailTypeY = detailsStartY + (detailsOffsetY * 4); 
	private final int detailNumberOfQuestionY = detailsStartY + (detailsOffsetY * 5); 
	private final int detailAnswerPercentY = detailsStartY + (detailsOffsetY * 6); 
	
	public AttackTab(int xloc, int yloc, WorldModel worldModel) {
		super(xloc, yloc, worldModel);
		
		backGround = GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("MenuGraphics/PlayerMenu/attacktabback", xloc, yloc); 
		character = worldModel.getCharacter();
		magicalAttacks = new ArrayList<AttackButton>();
		physicalAttacks = new ArrayList<AttackButton>();
		
		selectedButton = null;
		selectedAttackIsMagical = false;
		
		generateDetailFields();
		
	}
	
	@Override
	public void forceRedraw() {

		generateAttackButtons();
		
		ViewFacade.getInstance().addMidFrontSprite(backGround);
		
		int counter = 0;
		if(magicalAttacks.size() > 0){
			ViewFacade.getInstance().addForegroundSprite(GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("  ..MAGICAL ATTACKS:", xLocation + attackButtonOffsetX, yLocation + attackStartY + attackTextHeadOffsetY));
			counter++;
			for (AttackButton buttn : magicalAttacks) {
				ViewFacade.getInstance().addForegroundSprite(buttn);
				counter++;
			}
		}
		
		if(physicalAttacks.size() > 0){
			ViewFacade.getInstance().addForegroundSprite(GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("  ..PHYSICAL ATTACKS:", xLocation + attackButtonOffsetX, yLocation + attackStartY + attackTextHeadOffsetY + (attackOffsetY * counter)));
			counter++;
			for (AttackButton buttn : physicalAttacks) {
				ViewFacade.getInstance().addForegroundSprite(buttn);
				counter++;
			}
		}
		
		ViewFacade.getInstance().addForegroundSprite(detailHeader);
		
		ViewFacade.getInstance().addForegroundSprite(detailDamageText);
		ViewFacade.getInstance().addForegroundSprite(detailQuestionLevelText);
		ViewFacade.getInstance().addForegroundSprite(detailAttributeText);
		ViewFacade.getInstance().addForegroundSprite(detailReloadTimeText);
		ViewFacade.getInstance().addForegroundSprite(detailTypeText);
		ViewFacade.getInstance().addForegroundSprite(detailNumberOfQuestionText);
		ViewFacade.getInstance().addForegroundSprite(detailAnswerPercentText);
		
		ViewFacade.getInstance().addForegroundSprite(detailDamageValue);
		ViewFacade.getInstance().addForegroundSprite(detailQuestionLevelValue);
		ViewFacade.getInstance().addForegroundSprite(detailAttributeValue);
		ViewFacade.getInstance().addForegroundSprite(detailReloadTimeValue);
		ViewFacade.getInstance().addForegroundSprite(detailTypeValue);
		ViewFacade.getInstance().addForegroundSprite(detailNumberOfQuestionValue);
		ViewFacade.getInstance().addForegroundSprite(detailAnswerPercentValue);
		
		
	}
	
	public void generateAttackButtons(){
		
		int offsetcounter = 0;
		ArrayList<Attack> magicalList = character.getListOfMagicalAttacks();
		if(magicalList.size() > 0){
			
			offsetcounter++;
			
			for (Attack attack : magicalList) {
				AttackButton buttn = GraphicObjectFactory.getInstance().buildAttackButtonObjectPng(xLocation + attackButtonOffsetX, yLocation + attackStartY + ( attackOffsetY * offsetcounter), attack);
				buttn.addMouseListener(this);
				magicalAttacks.add(buttn);
				offsetcounter++;
			}
		
		}
		
		ArrayList<Attack> PhysicalList = character.getListOfPhysicalAttacks();
		if(PhysicalList.size() > 0){
			
			offsetcounter++;
		
			for (Attack attack : PhysicalList) {
				AttackButton buttn = GraphicObjectFactory.getInstance().buildAttackButtonObjectPng(xLocation + attackButtonOffsetX, yLocation + attackStartY + ( attackOffsetY * offsetcounter), attack);
				buttn.addMouseListener(this);
				physicalAttacks.add(buttn);
				offsetcounter++;
			}
		
		}
	}
	
	/**
	 * Genereates the (empty) detailtextboxes
	 */
	public void generateDetailFields(){
		
		detailHeader = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailHeaderX, yLocation + detailHeaderY);
		
		detailDamageText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailDamageY);
		detailQuestionLevelText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailQuestionLevelY);
		detailAttributeText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailAttributeY);
		detailReloadTimeText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailReloadTimeY);
		detailTypeText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailTypeY);
		detailNumberOfQuestionText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailNumberOfQuestionY);
		detailAnswerPercentText = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsTextX, yLocation + detailAnswerPercentY);
		
		detailDamageValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailDamageY);
		detailQuestionLevelValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailQuestionLevelY);
		detailAttributeValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailAttributeY);
		detailReloadTimeValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailReloadTimeY);
		detailTypeValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailTypeY);
		detailNumberOfQuestionValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailNumberOfQuestionY);
		detailAnswerPercentValue = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailsValueX, yLocation + detailAnswerPercentY);
	}

	public void updateDetailFields(){
		if(selectedButton != null){
			
			detailHeader.updateText(selectedButton.getAttack().getName(), 0);
			
			detailDamageText.updateText("Damage:", 0);
			detailQuestionLevelText.updateText("Question level:", 0);
			detailAttributeText.updateText("Attribute used:", 0);
			detailReloadTimeText.updateText("Reload time:", 0);
			detailTypeText.updateText("Type:", 0);
			detailNumberOfQuestionText.updateText("Number of Questions:", 0);
			detailAnswerPercentText.updateText("Answer percent needed:", 0);
			
			detailDamageValue.updateText(selectedButton.getAttack().getMinDamage() + " - " + selectedButton.getAttack().getMaxDamage(), 0);
			detailQuestionLevelValue.updateText("" + selectedButton.getAttack().getQuestionLevel(), 0);
			detailAttributeValue.updateText(selectedButton.getAttack().getAttributeType().name(), 0);
			detailReloadTimeValue.updateText(selectedButton.getAttack().getReloadTime() + "", 0);
			detailTypeValue.updateText(selectedButton.getAttack().getAttackType().name(), 0);
			detailNumberOfQuestionValue.updateText(selectedButton.getAttack().getNumberOfQuestions() + "", 0);
			detailAnswerPercentValue.updateText(selectedButton.getAttack().getAnswerPercentage() + "", 0);

		}
	}
	
	@Override
	public void receiveMessage(Message msg) {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean update() {

		for (AttackButton buttn : magicalAttacks) {
			buttn.updateInput();
		}
		
		for (AttackButton buttn : physicalAttacks) {
			buttn.updateInput();
		}
		
		return false;
	}

	@Override
	public void enterPressed() {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if(magicalAttacks.contains(e.getSource())){
			if(selectedButton != null){
				selectedButton.toggleButtonState(WowButtonState.BUTTON_UP);
			}
			selectedButton = (AttackButton)e.getSource();
			selectedAttackIsMagical = true;
			updateDetailFields();
		}
		else if(physicalAttacks.contains(e.getSource())){
			if(selectedButton != null){
				selectedButton.toggleButtonState(WowButtonState.BUTTON_UP);
			}
			selectedButton = (AttackButton)e.getSource();
			selectedAttackIsMagical = false;
			updateDetailFields();
		}
		
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

}
