package client.clientState.worldState.playerMenu;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import client.UserInput.InputControls;
import client.clientState.worldState.WorldModel;
import client.graphics.GraphicObjectFactory;
import client.graphics.SpriteGraphicObject;
import client.graphics.ViewFacade;
import client.graphics.components.InventoryButton;
import client.graphics.components.TextGraphicObject;
import client.graphics.components.WowButton;
import client.graphics.components.WowButtonState;
import client.network.NetworkHandlerMain;
import shared.models.AttributeTypes;
import shared.models.EquipLocation;
import shared.models.Item;
import shared.models.PlayableCharacter;
import shared.network.messages.Message;
import sun.security.action.GetBooleanAction;


/**
 * Class for the handling of the inventory tab of the playermenu. The 
 * tab uses the character model and has basic drag and drop support for
 * equipping items. Also shows some details of the item selected.
 * 
 * @author Henka
 *
 */
public class InventoryTab extends PlayerMenuTab implements MouseListener{

	private SpriteGraphicObject backGround;
	private PlayableCharacter character;
	
	private ArrayList<InventoryButton> backPackButtons;
	
	private EquipLocation selectedLocation;
	private int selectedBackpackNumber;
	
	private SpriteGraphicObject detailPicture;
	private TextGraphicObject detailItemName;
	private TextGraphicObject detailItemAttributes;
	private TextGraphicObject detailItemEffect;
	
	private SpriteGraphicObject dragAndDropPicture;
	
	private final int detailPictureX = 211;
	private final int detailPictureY = 111;
	
	private final int detailTextOffsetY = 20;
	
	private final int detailItemNameX = 335;
	private final int detailItemNameY = 115;
	private final int detailItemAttributeX = detailPictureX + 64 + 10;
	private final int detailItemAttributeY = detailItemNameY + detailTextOffsetY;
	private final int detailItemAttributeWidth = 500 - detailItemAttributeX - 10;
	private final int detailItemEffectX = detailItemAttributeX;
	private final int detailItemEffectY = detailItemAttributeY + detailTextOffsetY * 2;
	
	
	private boolean isDraggingItem;
	private boolean shouldForceViewUpdate;
	
	private final int inventoryGridStartX = 17;
	private final int inventoryGridStartY = 203;
	private final int inventoryGridOffsetX = 31;
	private final int inventoryGridOffsetY = 31;
	
	private final int leftEquipLocationsX = 30;
	private final int centerEquipLocationsX = 83;
	private final int rightEquipLocationsX = 133;
	
	private final int headEquipLocationY = 30;
	private final int bodyAndRingsEquipLocationY = 85;
	private final int amuletEquipLocationY = 45;
	private final int armEquipLocationY = 123;
	private final int legEquipLocationY = 134;
	
	private boolean headEquiped;
	private boolean ringLeftEquiped;
	private boolean ringRightEquiped;
	private boolean armLeftEquiped;
	private boolean armRightEquiped;
	private boolean bodyEquiped;
	private boolean legsEquiped;
	private boolean amuletEquiped;
	
	private WowButton emptyLocButtonHead;
	private WowButton emptyLocButtonRingLeft;
	private WowButton emptyLocButtonRingRight;
	private WowButton emptyLocButtonArmLeft;
	private WowButton emptyLocButtonArmRight;
	private WowButton emptyLocButtonBody;
	private WowButton emptyLocButtonLegs;
	private WowButton emptyLocButtonAmulet;
	
	private InventoryButton locButtonHead;
	private InventoryButton locButtonRingLeft;
	private InventoryButton locButtonRingRight;
	private InventoryButton locButtonArmLeft;
	private InventoryButton locButtonArmRight;
	private InventoryButton locButtonBody;
	private InventoryButton locButtonLegs;
	private InventoryButton locButtonAmulet;
	
	
	/**
	 * Standard constructor. Initializes the needed graphicobjects.
	 * 
	 * @param xloc 			The xcoordinates of the upper-left corner of the panel
	 * @param yloc 			The ycoordinates of the upper-left corner of the panel
	 * @param worldModel 	the model currently in use by this panel
	 */
	public InventoryTab(int xloc, int yloc, WorldModel worldModel) {
		super(xloc, yloc, worldModel);
		
		backGround = GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("MenuGraphics/PlayerMenu/inventorytabback", xloc, yloc); 
		character = worldModel.getCharacter();
		
		detailPicture = GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("Buttons/items/detailpictureempty", xLocation + detailPictureX, yLocation + detailPictureY);
		detailItemName = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailItemNameX, yLocation + detailItemNameY);
		detailItemAttributes = GraphicObjectFactory.getInstance().buildWordWrappableTextObject("", xLocation + detailItemAttributeX, yLocation + detailItemAttributeY, detailItemAttributeWidth, true);
		detailItemEffect = GraphicObjectFactory.getInstance().buildStandardTextGraphicObject("", xLocation + detailItemEffectX, yLocation + detailItemEffectY);
		
		
		dragAndDropPicture = GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("Buttons/items/itemsmallblank", xLocation, yLocation);
		
		
		backPackButtons = new ArrayList<InventoryButton>();
		
		buildEmptyLocationButtons();
		
		headEquiped = false;
		ringLeftEquiped = false;
		ringRightEquiped = false;
		armLeftEquiped = false;
		armRightEquiped = false;
		bodyEquiped = false;
		legsEquiped = false;
		amuletEquiped = false;
		
		
		selectedLocation = null;
		selectedBackpackNumber = -1;
		
		isDraggingItem = false;
		shouldForceViewUpdate = false;
		
	}
	
	@Override
	public void forceRedraw() {

		buildItemButtons();
		
		ViewFacade.getInstance().addMidFrontSprite(backGround);
		
		for (WowButton buttn : backPackButtons) {
			ViewFacade.getInstance().addForegroundSprite(buttn);
		}
		
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.HEAD));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.LEFT_RING));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.RIGHT_RING));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.LEFT_ARM));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.RIGHT_ARM));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.BODY));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.LEGS));
		ViewFacade.getInstance().addForegroundSprite(getLocationButton(EquipLocation.AMULET));
		
		ViewFacade.getInstance().addForegroundSprite(detailItemName);
		ViewFacade.getInstance().addForegroundSprite(detailPicture);
		ViewFacade.getInstance().addForegroundSprite(detailItemAttributes);
		ViewFacade.getInstance().addForegroundSprite(detailItemEffect);
		
		
		ViewFacade.getInstance().addForegroundSprite(dragAndDropPicture);
		
	}
	
	/**
	 * This method traverses all the items of the character, and builds the neccesary buttons for this tab.
	 */
	private void buildItemButtons(){
		
		backPackButtons.clear();
		ArrayList<Item> inventory = character.getInventory();
		
		for (int i = 0; i < inventory.size(); i++) {

			String itemname = inventory.get(i).getName();
			String itemimage = inventory.get(i).getImagePath();
			
			if(itemimage != null){
				
				InventoryButton buttn = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(itemimage, xLocation + inventoryGridStartX + inventoryGridOffsetX * (i % 15), yLocation + inventoryGridStartY + ((int)(i / 15)) * inventoryGridOffsetY, false, inventory.get(i));
				buttn.addMouseListener(this);
				backPackButtons.add(buttn);
				
			}
		}
		
		Item item = character.getEquipedItemHead();
		if(item != null){
			locButtonHead = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + centerEquipLocationsX, yLocation + headEquipLocationY, false, item);
			locButtonHead.addMouseListener(this);
			headEquiped = true;
		}
		else{
			headEquiped = false;
		}
		
		item = character.getEquipedItemLeftRing();
		if(item != null){
			locButtonRingLeft = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + rightEquipLocationsX, yLocation + bodyAndRingsEquipLocationY, false, item);
			locButtonRingLeft.addMouseListener(this);
			ringLeftEquiped = true;
		}
		else{
			ringLeftEquiped = false;
		}
		
		item = character.getEquipedItemRightRing();
		if(item != null){
			locButtonRingRight = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + leftEquipLocationsX, yLocation + bodyAndRingsEquipLocationY, false, item);
			locButtonRingRight.addMouseListener(this);
			ringRightEquiped = true;
		}
		else{
			ringRightEquiped = false;
		}
		
		item = character.getEquipedItemLeftArm();
		if(item != null){
			locButtonArmLeft = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + rightEquipLocationsX, yLocation + armEquipLocationY, false, item);
			locButtonArmLeft.addMouseListener(this);
			armLeftEquiped = true;
		}
		else{
			armLeftEquiped = false;
		}
		
		item = character.getEquipedItemRightArm();
		if(item != null){
			locButtonArmRight = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + leftEquipLocationsX, yLocation + armEquipLocationY, false, item);
			locButtonArmRight.addMouseListener(this);
			armRightEquiped = true;
		}
		else{
			armRightEquiped = false;
		}
		
		item = character.getEquipedItemBody();
		if(item != null){
			locButtonBody = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + centerEquipLocationsX, yLocation + bodyAndRingsEquipLocationY, false, item);
			locButtonBody.addMouseListener(this);
			bodyEquiped = true;
		}
		else{
			bodyEquiped = false;
		}
		
		item = character.getEquipedItemLegs();
		if(item != null){
			locButtonLegs = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + centerEquipLocationsX, yLocation + legEquipLocationY, false, item);
			locButtonLegs.addMouseListener(this);
			legsEquiped = true;
		}
		else{
			legsEquiped = false;
		}
		
		item = character.getEquipedItemAmulet();
		if(item != null){
			locButtonAmulet = GraphicObjectFactory.getInstance().buildInventoryButtonObjectPng(item.getImagePath(), xLocation + rightEquipLocationsX, yLocation + amuletEquipLocationY, false, item);
			locButtonAmulet.addMouseListener(this);
			amuletEquiped = true;
		}
		else{
			amuletEquiped = false;
		}
		
		
	}
	
	
	/**
	 * This method creates the emptylocationbuttons, which are shown if nothing has
	 * been equiped for an equiplocation. Should just be called in constructor!
	 */
	private void buildEmptyLocationButtons(){
		emptyLocButtonHead = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_head", xLocation + centerEquipLocationsX, yLocation + headEquipLocationY, false, false);
		emptyLocButtonRingLeft  = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_ring", xLocation + rightEquipLocationsX, yLocation + bodyAndRingsEquipLocationY, false, false);
		emptyLocButtonRingRight = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_ring", xLocation + leftEquipLocationsX, yLocation + bodyAndRingsEquipLocationY, false, false); 
		emptyLocButtonArmLeft = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_arm", xLocation + rightEquipLocationsX, yLocation + armEquipLocationY, false, false);
		emptyLocButtonArmRight = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_arm", xLocation + leftEquipLocationsX, yLocation + armEquipLocationY, false, false);
		emptyLocButtonBody = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_body", xLocation + centerEquipLocationsX, yLocation + bodyAndRingsEquipLocationY, false, false);
		emptyLocButtonLegs = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_legs", xLocation + centerEquipLocationsX, yLocation + legEquipLocationY, false, false);
		emptyLocButtonAmulet = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage("items/EmptyLocations/emptylocation_amulet", xLocation + rightEquipLocationsX, yLocation + amuletEquipLocationY, false, false);
	}

	@Override
	public void receiveMessage(Message msg) {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean update() {

		for (WowButton buttn : backPackButtons) {
			buttn.updateInput();
		}
		
		getLocationButton(EquipLocation.HEAD).updateInput();
		getLocationButton(EquipLocation.LEFT_RING).updateInput();
		getLocationButton(EquipLocation.RIGHT_RING).updateInput();
		getLocationButton(EquipLocation.LEFT_ARM).updateInput();
		getLocationButton(EquipLocation.RIGHT_ARM).updateInput();
		getLocationButton(EquipLocation.BODY).updateInput();
		getLocationButton(EquipLocation.LEGS).updateInput();
		getLocationButton(EquipLocation.AMULET).updateInput();
		
		checkMouseRelease();
		
		if(isDraggingItem){
			dragAndDropPicture.setCoords(InputControls.getInstance().getMouseX() + 10, InputControls.getInstance().getMouseY() + 10);
		}
		
		if(shouldForceViewUpdate){
			shouldForceViewUpdate = false;
			return true;
		}
		
		return false;
	}

	@Override
	public void enterPressed() {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseClicked(MouseEvent e) {

		isDraggingItem = true;
		
		if(backPackButtons.contains(e.getSource())){
			changeSelected(EquipLocation.BACKPACK, backPackButtons.indexOf(e.getSource()));
		}
		else if(headEquiped && e.getSource() == locButtonHead){
			changeSelected(EquipLocation.HEAD, -1);
		}
		else if(ringLeftEquiped && e.getSource() == locButtonRingLeft){
			changeSelected(EquipLocation.LEFT_RING, -1);
		}
		else if(ringRightEquiped && e.getSource() == locButtonRingRight){
			changeSelected(EquipLocation.RIGHT_RING, -1);
		}
		else if(armLeftEquiped && e.getSource() == locButtonArmLeft){
			changeSelected(EquipLocation.LEFT_ARM, -1);
		}
		else if(armRightEquiped && e.getSource() == locButtonArmRight){
			changeSelected(EquipLocation.RIGHT_ARM, -1);
		}
		else if(bodyEquiped && e.getSource() == locButtonBody){
			changeSelected(EquipLocation.BODY, -1);
		}
		else if(legsEquiped && e.getSource() == locButtonLegs){
			changeSelected(EquipLocation.LEGS, -1);
		}
		else if(amuletEquiped && e.getSource() == locButtonAmulet){
			changeSelected(EquipLocation.AMULET, -1);
		}
		
	}
	
	/**
	 * Checks if the mouse has been released in a allowed location.
	 * 
	 * 
	 */
	private void checkMouseRelease(){
		if(isDraggingItem && InputControls.getInstance().getMouse1Released() && selectedLocation != null){
			
			isDraggingItem = false;
			dragAndDropPicture.updatePicture("Buttons/items/itemsmallblank");
			
			//get the mousecoordnates ( coordinates within the menu, for easier coding )
			int xRelease = InputControls.getInstance().getMouseX() - xLocation;
			int yRelease = InputControls.getInstance().getMouseY() - yLocation;
			
			//check the different equiplocations
			
			//first check if it's in the equiplocations area.
			if(xRelease < 182 && xRelease > 0 && yRelease < 190 && yRelease > 0){
				
				//check if x-coordinate is in one of the three boxrows:
				if(xRelease > leftEquipLocationsX && xRelease < leftEquipLocationsX + 32){
					//check rightring
					if(yRelease > bodyAndRingsEquipLocationY && yRelease < bodyAndRingsEquipLocationY + 32){
						equipSelectedItem(EquipLocation.RIGHT_RING);
					}
					//check rightarm
					else if(yRelease > armEquipLocationY && yRelease < armEquipLocationY + 32){
						equipSelectedItem(EquipLocation.RIGHT_ARM);
					}
					
					
				}
				else if(xRelease > centerEquipLocationsX && xRelease < centerEquipLocationsX + 32){
					//check head
					if(yRelease > headEquipLocationY && yRelease < headEquipLocationY + 32){
						equipSelectedItem(EquipLocation.HEAD);
					}
					//check body
					else if(yRelease > bodyAndRingsEquipLocationY && yRelease < bodyAndRingsEquipLocationY + 32){
						equipSelectedItem(EquipLocation.BODY);
					}
					//check legs
					else if(yRelease > legEquipLocationY && yRelease < legEquipLocationY + 32){
						equipSelectedItem(EquipLocation.LEGS);
					}
					
				}
				else if(xRelease > rightEquipLocationsX && xRelease < rightEquipLocationsX + 32){
					//check amulet
					if(yRelease > amuletEquipLocationY && yRelease < amuletEquipLocationY + 32){
						equipSelectedItem(EquipLocation.AMULET);
					}
					//check left ring
					else if(yRelease > bodyAndRingsEquipLocationY && yRelease < bodyAndRingsEquipLocationY + 32){
						equipSelectedItem(EquipLocation.LEFT_RING);
					}
					//check left arm
					else if(yRelease > armEquipLocationY && yRelease < armEquipLocationY + 32){
						equipSelectedItem(EquipLocation.LEFT_ARM);
					}
				}
				
				
			}
			//else, check if it's in the backpack ( doesn't matter where )
			else if(xRelease > inventoryGridStartX && xRelease < inventoryGridStartX + (inventoryGridOffsetX * 15) && yRelease > inventoryGridStartY && yRelease < inventoryGridStartY + (inventoryGridOffsetY * 4)){
				
				if(selectedLocation != EquipLocation.BACKPACK){
					Item item = getItemFromEquiplLocation(selectedLocation, -1);
					character.moveItemToInventory(selectedLocation);
					shouldForceViewUpdate = true;
					
					NetworkHandlerMain.getInstance().sendEquipMessage("unequip item " + item.getName() , EquipLocation.BACKPACK, selectedLocation, item.getItemId());
					
					
				}
				
			}
			
		}
		
	}
	
	/**
	 * Handles the equipping of the currently selected item. If the item cannot be equiped there, nothing
	 * happens.
	 * 
	 * @param targetLocation the location the item should be equiped at
	 */
	private void equipSelectedItem(EquipLocation targetLocation){
		
		Item item = getItemFromEquiplLocation(selectedLocation, selectedBackpackNumber);
		
		if(item != null){
			
			EquipLocation usedItemLocation = targetLocation;
			if(usedItemLocation == EquipLocation.LEFT_ARM || usedItemLocation == EquipLocation.RIGHT_ARM){
				usedItemLocation = EquipLocation.ARM;
			}
			else if(usedItemLocation == EquipLocation.LEFT_RING || usedItemLocation == EquipLocation.RIGHT_RING){
				usedItemLocation = EquipLocation.RING;
			}
			//check if the selected item can be equiped in that location!
			if(item.getEquipLocation() == usedItemLocation){
				
				character.equipItem(targetLocation, selectedLocation, item);
				
				shouldForceViewUpdate = true;
				
				NetworkHandlerMain.getInstance().sendEquipMessage("equip item " + item.getName() , targetLocation, selectedLocation, item.getItemId());
				
				
				selectedLocation = targetLocation;
				selectedBackpackNumber = -1;

				
			}
			
		}
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {}
	@Override
	public void mouseExited(MouseEvent e) {}
	@Override
	public void mousePressed(MouseEvent e) {}
	@Override
	public void mouseReleased(MouseEvent e) {}
	
	/**
	 * updates the selected button to the equiplocation, and backpacknumber ( if
	 * equiplocation is BACKPACK ). If the loc is not BACKPACK, this number is not
	 * used.
	 * 
	 * @param loc - the new selected location
	 * @param backPackNumber - the number in the backpack, if loc is BACKPACK
	 */
	private void changeSelected(EquipLocation loc, int backPackNumber){
		if(selectedLocation != null){
			if(selectedLocation == EquipLocation.BACKPACK){
				backPackButtons.get(selectedBackpackNumber).toggleButtonState(WowButtonState.BUTTON_UP);
			}
			else{
				getLocationButton(selectedLocation).toggleButtonState(WowButtonState.BUTTON_UP);
			}
		}
		
		
		selectedLocation = loc;
		selectedBackpackNumber = backPackNumber;
		
		//--------------------------------------------------------------------------
		//Update the detail view:
		Item item = getItemFromEquiplLocation(loc, backPackNumber);
		
		String attributestring = "";
		detailItemName.updateText(item.getName(), 0);
		detailPicture.updatePicture("Buttons/items/" + item.getImagePath() + "_large");
		
		ArrayList<AttributeTypes> atmods = item.getAttribute().getNotZeroAttributes();
		for (AttributeTypes attributeType : atmods) {
			
			attributestring += attributeType.name() + " : ";
			attributestring += item.getAttribute().getValueFromAttributeType(attributeType) + "  ";
			
		}
		detailItemAttributes.updateText(attributestring, 0);
		
		String effectText = "";
		if(item.getEffect() != null){
			effectText = item.getEffect().getName();
		}
		detailItemEffect.updateText(effectText, 0);
		
		//--------------------------------------------------------------------------
		
		dragAndDropPicture.updatePicture("Buttons/items/" + item.getImagePath() + "_down");
		
	}

	private Item getItemFromEquiplLocation(EquipLocation loc, int backPackNumber) {
		Item item;
		if(loc == EquipLocation.BACKPACK){
			 item = backPackButtons.get(backPackNumber).getItem();
		}
		else{
			WowButton buttn = getLocationButton(loc);
			if(buttn instanceof InventoryButton){
				item = ((InventoryButton)buttn).getItem();
			}
			else{
				return null;
			}
			
		}
		return item;
	}
	
	private WowButton getLocationButton(EquipLocation loc){
		if(loc == EquipLocation.HEAD){
			if(headEquiped) return locButtonHead;
			else return emptyLocButtonHead;
		}
		else if(loc == EquipLocation.LEFT_RING){
			if(ringLeftEquiped) return locButtonRingLeft;
			else return emptyLocButtonRingLeft;
		}
		else if(loc == EquipLocation.RIGHT_RING){
			if(ringRightEquiped) return locButtonRingRight;
			else return emptyLocButtonRingRight;
		}
		else if(loc == EquipLocation.LEFT_ARM){
			if(armLeftEquiped) return locButtonArmLeft;
			else return emptyLocButtonArmLeft;
		}
		else if(loc == EquipLocation.RIGHT_ARM){
			if(armRightEquiped) return locButtonArmRight;
			else return emptyLocButtonArmRight;
		}
		else if(loc == EquipLocation.BODY){
			if(bodyEquiped) return locButtonBody;
			else return emptyLocButtonBody;
		}
		else if(loc == EquipLocation.LEGS){
			if(legsEquiped) return locButtonLegs;
			else return emptyLocButtonLegs;
		}
		else if(loc == EquipLocation.AMULET){
			if(amuletEquiped) return locButtonAmulet;
			else return emptyLocButtonAmulet;
		}
		
		return null;
		
	}
	
	
	
	

}
