package client.clientState.worldState.playerMenu;

import java.awt.MenuBar;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import client.UserInput.InputControls;
import client.clientState.worldState.WorldModel;
import client.graphics.GraphicObjectFactory;
import client.graphics.SpriteGraphicObject;
import client.graphics.ViewFacade;
import client.graphics.components.WowButton;
import client.graphics.components.WowButtonState;
/**
 * Main Controller for the Player Menu. Handles the tabbing between the different
 * pages of the Player menu.
 * 
 * @author Henka
 *
 */
public class PlayerMenu implements MouseListener{

	private final int xLocation = 25;
	private final int yLocation = 25;
	
	private final int xSize = 500;
	private final int ySize = 400;
	
	private final int tabButtonYoffset = 30;
	
	private final int charTabX = 0;
	private final int skillsTabX = 82 + 4;
	private final int attackTabX = 82 + (79 * 1) + (4 * 2);
	private final int inventoryTabX = 82 + (79 * 2) + (4 * 3);
	private final int ledgerTabX = 82 + (79 * 3) + (4 * 4);
	private final int guildTabX = 82 + (79 * 4) + (4 * 5);
	
	private final int escbuttonX = 470;
	private final int escbuttonY = 3;
	
	private WowButton charTabButton;
	private WowButton skillsTabButton;
	private WowButton attackTabButton;
	private WowButton inventoryTabButton;
	private WowButton ledgerTabButton;
	private WowButton guildTabButton;
	
	private WowButton escButton;
	private SpriteGraphicObject backGroundSprite;
	
	private MainCharacterTab mainCharTab;
	private SkillsTab skillsTab;
	private AttackTab attackTab;
	private InventoryTab inventoryTab;
	private LedgerTab ledgerTab;
	private GuildTab guildTab;
	
	private MenuTabs activeTab;
	
	private boolean shouldRefreshDrawing;
	private boolean shouldClose;
	
	public PlayerMenu(WorldModel worldModel) {
		
		charTabButton = GraphicObjectFactory.getInstance().buildSimpleToggleWowButtonObjectPng("PlayerMenuTabs/charTab", xLocation + charTabX, yLocation + tabButtonYoffset, false);
		skillsTabButton = GraphicObjectFactory.getInstance().buildSimpleToggleWowButtonObjectPng("PlayerMenuTabs/skillsTab", xLocation + skillsTabX, yLocation + tabButtonYoffset, false);
		attackTabButton = GraphicObjectFactory.getInstance().buildSimpleToggleWowButtonObjectPng("PlayerMenuTabs/attackTab", xLocation + attackTabX, yLocation + tabButtonYoffset, false);
		inventoryTabButton = GraphicObjectFactory.getInstance().buildSimpleToggleWowButtonObjectPng("PlayerMenuTabs/inventoryTab", xLocation + inventoryTabX, yLocation + tabButtonYoffset, false);
		ledgerTabButton = GraphicObjectFactory.getInstance().buildSimpleToggleWowButtonObjectPng("PlayerMenuTabs/ledgerTab", xLocation + ledgerTabX, yLocation + tabButtonYoffset, false);
		guildTabButton = GraphicObjectFactory.getInstance().buildSimpleToggleWowButtonObjectPng("PlayerMenuTabs/guildTab", xLocation+ guildTabX, yLocation + tabButtonYoffset, false);
		
		escButton = GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng("playermenuescbutton", xLocation + escbuttonX, yLocation + escbuttonY, false);
		
		backGroundSprite = GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("MenuGraphics/PlayerMenu/playermenubaseback", xLocation, yLocation);
		
		charTabButton.addMouseListener(this);
		skillsTabButton.addMouseListener(this);
		attackTabButton.addMouseListener(this);
		inventoryTabButton.addMouseListener(this);
		ledgerTabButton.addMouseListener(this);
		guildTabButton.addMouseListener(this);
		
		escButton.addMouseListener(this);
		
		mainCharTab = new MainCharacterTab(xLocation, yLocation + 60, worldModel);
		skillsTab = new SkillsTab(xLocation, yLocation + 60, worldModel);
		attackTab = new AttackTab(xLocation, yLocation + 60, worldModel);
		inventoryTab = new InventoryTab(xLocation, yLocation + 60, worldModel);
		ledgerTab = new LedgerTab(xLocation, yLocation + 60, worldModel);
		guildTab = new GuildTab(xLocation, yLocation + 60, worldModel);
		
		activeTab = MenuTabs.PRIMARY_CHARACTERSCREEN;
		charTabButton.toggleButtonState(WowButtonState.BUTTON_DOWN);
		
		shouldRefreshDrawing = false;
		shouldClose = false;
	}
	
	public void forceDraw(){
		
		ViewFacade.getInstance().addMidFrontSprite(backGroundSprite);
		
		ViewFacade.getInstance().addForegroundSprite(charTabButton);
		ViewFacade.getInstance().addForegroundSprite(skillsTabButton);
		ViewFacade.getInstance().addForegroundSprite(attackTabButton);
		ViewFacade.getInstance().addForegroundSprite(inventoryTabButton);
		ViewFacade.getInstance().addForegroundSprite(ledgerTabButton);
		ViewFacade.getInstance().addForegroundSprite(guildTabButton);
		
		ViewFacade.getInstance().addForegroundSprite(escButton);
		
		getTabFromEnum(activeTab).forceRedraw();
		
	}
	
	/**
	 * The update method checks all input. Returns TRUE if the screen should
	 * be refreshed!
	 * 
	 * @return
	 */
	public PlayerMenuReturnType update(){
		
		charTabButton.updateInput();
		skillsTabButton.updateInput();
		attackTabButton.updateInput();
		inventoryTabButton.updateInput();
		ledgerTabButton.updateInput();
		guildTabButton.updateInput();
		
		escButton.updateInput();
		
		boolean tabRequestsRefresh = getTabFromEnum(activeTab).update();
		
		if(tabRequestsRefresh){
			tabRequestsRefresh = false;
			shouldRefreshDrawing = true;
		}
		
		if(shouldRefreshDrawing){
			shouldRefreshDrawing = false;
			return PlayerMenuReturnType.REDRAW;
		}
		
		if(InputControls.getInstance().getEscPressed() || shouldClose){
			shouldClose = false;
			return PlayerMenuReturnType.MENU_CLOSING;
		}
		
		return PlayerMenuReturnType.NO_CHANGE;
	}

	/**
	 * Method for handling mouseinput from the buttons.
	 * 
	 * If one of the tab buttons is pressed, change the current tab.
	 * 
	 * If the escape button is pressed, set the shouldClose flag, so the
	 * window will close on the next update.
	 */
	
	@Override
	public void mouseClicked(MouseEvent arg0) {

		if(arg0.getSource() == charTabButton || arg0.getSource() == skillsTabButton ||
					arg0.getSource() == attackTabButton || arg0.getSource() == inventoryTabButton ||
					arg0.getSource() == ledgerTabButton || arg0.getSource() == guildTabButton ){
			
				getButtonFromEnum(activeTab).toggleButtonState(WowButtonState.BUTTON_UP);
				activeTab = getEnumFromButton((WowButton)arg0.getSource());
			
				shouldRefreshDrawing = true;
			
		}
		else if(arg0.getSource() == escButton){
			shouldClose = true;
		}
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	/**
	 * Method for getting the tabbutton of a MenuTab enumeration
	 * 
	 * @param tab the MenuTab enum of the desired tab
	 * @return
	 */
	public WowButton getButtonFromEnum(MenuTabs tab){
		
		if(tab == MenuTabs.PRIMARY_CHARACTERSCREEN){
			return charTabButton;
		}
		else if(tab == MenuTabs.SKILLS_SCREEN){
			return skillsTabButton;
		}
		else if(tab == MenuTabs.ATTACK_MAGIC_SCREEN){
			return attackTabButton;
		}
		else if(tab == MenuTabs.INVENTORY_SCREEN){
			return inventoryTabButton;
		}
		else if(tab == MenuTabs.LEDGER_SCREEN){
			return ledgerTabButton;
		}
		else if(tab == MenuTabs.GUILD_SCREEN){
			return guildTabButton;
		}
		return null;

	}
	
	public MenuTabs getEnumFromButton(WowButton button){
		if(button == charTabButton){
			return MenuTabs.PRIMARY_CHARACTERSCREEN;
		}
		else if(button == skillsTabButton){
			return MenuTabs.SKILLS_SCREEN;
		}
		else if(button == attackTabButton){
			return MenuTabs.ATTACK_MAGIC_SCREEN;
		}
		else if(button == inventoryTabButton){
			return  MenuTabs.INVENTORY_SCREEN;
		}
		else if(button == ledgerTabButton){
			return MenuTabs.LEDGER_SCREEN;
		}
		else if(button == guildTabButton){
			return MenuTabs.GUILD_SCREEN;
		}
		return null;
	}
	
	public PlayerMenuTab getTabFromEnum(MenuTabs tab){
		if(tab == MenuTabs.PRIMARY_CHARACTERSCREEN){
			return mainCharTab;
		}
		else if(tab == MenuTabs.SKILLS_SCREEN){
			return skillsTab;
		}
		else if(tab == MenuTabs.ATTACK_MAGIC_SCREEN){
			return attackTab;
		}
		else if(tab == MenuTabs.INVENTORY_SCREEN){
			return inventoryTab;
		}
		else if(tab == MenuTabs.LEDGER_SCREEN){
			return ledgerTab;
		}
		else if(tab == MenuTabs.GUILD_SCREEN){
			return guildTab;
		}
		
		return null;
	}
	
}
