package client.clientState.worldState.playerMenu;

import java.util.ArrayList;

import client.clientState.worldState.WorldModel;
import client.graphics.components.KeyBoardFocusableComponent;
import client.graphics.components.keyBoardFocusHandler;
import shared.network.messages.Message;
/**
 * 
 * Abstract class. Inherited by the MenuTabs.
 */
public abstract class PlayerMenuTab implements keyBoardFocusHandler{

	protected ArrayList<KeyBoardFocusableComponent> keyboardFocusComponents = new ArrayList<KeyBoardFocusableComponent>();
	protected KeyBoardFocusableComponent focusComponent;
	protected KeyBoardFocusableComponent nextFocusComponent;
	
	public abstract boolean update();
	public abstract void receiveMessage(Message msg);
	public abstract void forceRedraw();
	
	protected int xLocation;
	protected int yLocation;
	protected WorldModel worldModel;
	
	public PlayerMenuTab(int xLocation, int yLocation, WorldModel worldModel){
		this.xLocation = xLocation;
		this.yLocation = yLocation;
		this.worldModel = worldModel;
	}
	
	@Override
	public boolean requestKeyboardFocus( KeyBoardFocusableComponent requestingComponent) {
		
		if(keyboardFocusComponents.contains(requestingComponent)){

			nextFocusComponent = requestingComponent;
			
			return true;
		}
		return false;
	}


	@Override
	public void tabReturned() {
		if(focusComponent == null){
			focusComponent = keyboardFocusComponents.get(0);
		}
		else{
			int plass = keyboardFocusComponents.indexOf(focusComponent);
			int nextCompIndex = plass + 1;
			if(nextCompIndex >= keyboardFocusComponents.size()){
				nextCompIndex = 0;
			}
			
			nextFocusComponent = keyboardFocusComponents.get(nextCompIndex);
			
		}
		
		
	}
	
	public void addKeyboardFocusComponent(KeyBoardFocusableComponent component){
		keyboardFocusComponents.add(component);
	}
	public void removeKeyboardFocusComponent(KeyBoardFocusableComponent component){
		keyboardFocusComponents.remove(component);
	}
	
	public void updateKeyboardFocus(){
		if(nextFocusComponent != null && nextFocusComponent != focusComponent){
			changeKeyboardFocus(nextFocusComponent);
		}
	}
	
	private void changeKeyboardFocus(KeyBoardFocusableComponent component){
		if(focusComponent != null){
			focusComponent.releaseKeyboardFocus();
		}
		focusComponent = component;
		focusComponent.receiveKeyboardFocus();
	}
	
	
}
