package client.graphics;

import java.awt.image.BufferedImage;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.awt.Color;
import java.awt.Font;
import java.awt.Transparency;

import shared.models.Attack;
import shared.models.AttributeTypes;
import shared.models.Item;
import shared.models.Location;
import shared.models.WorldObject;
import sun.security.x509.AVA;

import client.clientState.Settings;
import client.graphics.components.AttackButton;
import client.graphics.components.InventoryButton;
import client.graphics.components.StatusBar;
import client.graphics.components.WowButton;
import client.graphics.components.WowPasswordField;
import client.graphics.components.WowTextField;
import client.graphics.components.keyBoardFocusHandler;
import client.graphics.components.TextGraphicObject;

import com.golden.gamedev.object.Sprite;
/**
 * Factory for creating GraphicsObjects.
 * 
 */
public class GraphicObjectFactory {

	private static GraphicObjectFactory singleton;
	
	private GraphicObjectFactory(){
		
	}
	/**
	 * Returns the instance of the GraphicObjectFactory.
	 * @return
	 */
	public static GraphicObjectFactory getInstance(){
		if(singleton == null){
			singleton = new GraphicObjectFactory();
		}
		return singleton;
	}
	/**
	 * Returns a SpriteGraphicObject.
	 * @param textureName 
	 * @param xLocation
	 * @param yLocation
	 * @return
	 */
	public SpriteGraphicObject buildSpriteGraphicObjectPng(String textureName, int xLocation, int yLocation){
//			File file = new File(textureName + ".png");
//			fileURL = new URL("file:///d:/Diverse/NTNU/FordypningsProsjekt%20-%20World%20of%20Wisdom/Code/WowDev/HenrikTest"+".png");
//			fileURL = new URL("http://www.google.no/intl/no_no/images/logo.gif");
		 	URL fileURL = getFileURL("Graphics/" + textureName + ".png");
			BufferedImage image = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
			Sprite sprite = new Sprite( image, xLocation, yLocation);
			return new SpriteGraphicObject(sprite);
	}
	/**
	 * Returns a WorldObjectSprite
	 * @param textureName
	 * @param object
	 * @param screenFocus
	 * @return
	 */
	public WorldObjectSprite buildWorldObjectSprite(String textureName,WorldObject object,Location screenFocus){
	 	URL fileURL = getFileURL("Graphics/WorldObjects/" + textureName + ".png");
	 	BufferedImage image = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		return new WorldObjectSprite(image,object,screenFocus);
	
}
	/**
	 * Returns a standard TextGraphicObject
	 * @param text
	 * @param xLocation
	 * @param yLocation
	 * @return
	 */
	public TextGraphicObject buildStandardTextGraphicObject(String text, int xLocation, int yLocation){
		return new TextGraphicObject(text, xLocation, yLocation);
	}
	
	
	/**
	 * Returns a TextGraphicObject with red text.
	 * @param text
	 * @param xLocation
	 * @param yLocation
	 * @return
	 */
	public TextGraphicObject buildRedTextGraphicObject(String text, int xLocation, int yLocation){
		return new TextGraphicObject(text, xLocation, yLocation, Color.RED);
	}
	
	/**
	 * Returns a TextGraphicObject that wraps the text according to available width.
	 * @param text
	 * @param xLocation
	 * @param yLocation
	 * @param availableWidth
	 * @param doWordWrap
	 * @return
	 */
	public TextGraphicObject buildWordWrappableTextObject(String text, int xLocation, int yLocation,int availableWidth, boolean doWordWrap){
		return new TextGraphicObject(text, xLocation, yLocation, availableWidth,doWordWrap);
	}
	
	/**
	 * Returns a WowButton which uses only on image.
	 * @param buttonUpFileName
	 * @param xLocation
	 * @param yLocation
	 * @param toggelOn
	 * @param mouseOver
	 * @return
	 */
	public WowButton buildWowButtonObjectPngOneImage(String buttonUpFileName, int xLocation, int yLocation, 
						boolean toggelOn,  boolean mouseOver){
	 	URL fileURL = getFileURL("Graphics/Buttons/" + buttonUpFileName + ".png");
		BufferedImage image = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		WowButton wb = new WowButton(image, xLocation, yLocation,toggelOn, mouseOver);
		return wb;
	}
	/**
	 * Returns a InventoryButton
	 * @param buttonUpFileName
	 * @param xLocation
	 * @param yLocation
	 * @param mouseOver
	 * @param item
	 * @return
	 */
	public InventoryButton buildInventoryButtonObjectPng(String buttonUpFileName, int xLocation, int yLocation, 
				boolean mouseOver, Item item){
			URL fileURL = getFileURL("Graphics/Buttons/items/" + buttonUpFileName + "_down.png");
			BufferedImage down = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
			
			fileURL = getFileURL("Graphics/Buttons/items/" + buttonUpFileName + "_up.png");
			BufferedImage up = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
			
			
			BufferedImage image = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
			InventoryButton wb = new InventoryButton(up, down, xLocation, yLocation, true, mouseOver, item);
			return wb;
			
		}
	/**
	 * Returns an AttackButton
	 * @param xLocation
	 * @param yLocation
	 * @param attack
	 * @return
	 */
	public AttackButton buildAttackButtonObjectPng( int xLocation, int yLocation, Attack attack){
		
		URL fileURL = getFileURL("Graphics/Buttons/attacklistbg_down.png");
		BufferedImage down = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		fileURL = getFileURL("Graphics/Buttons/attacklistbg_up.png");
		BufferedImage up = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		fileURL = getFileURL("Graphics/Buttons/AttackThumbs/"+attack.getName().toLowerCase()+"_thumb_hoover.png");
		BufferedImage icon = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		return new AttackButton(icon, down, up, xLocation, yLocation, attack);
		
	}
	/**
	 * Returns a WowButton
	 * @param buttonName
	 * @param xLocation
	 * @param yLocation
	 * @param toggelOn
	 * @param mouseOver
	 * @return
	 */
	public WowButton buildWowButtonObjectPng(String buttonName, int xLocation, int yLocation, 
								boolean toggelOn,  boolean mouseOver){
	 	URL fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_clicked.png");
	 	BufferedImage clicked = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_down.png");
		BufferedImage down = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_up.png");
		BufferedImage up = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_hooverDown.png");
		BufferedImage hooverDown = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_hooverUp.png");
		BufferedImage hooverUp = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		
		WowButton wb = new WowButton(hooverUp,hooverDown,down,up,clicked, xLocation, yLocation,toggelOn, mouseOver);
		return wb;
	}
	
	/**
	 * For construct a simple, un-togglable buttons with just up and clicked pictures.
	 * @param buttonName - the name of the button
	 * @param xLocation - the x location where the button will be placed
	 * @param yLocation - the y location where the button will be placed
	 * @return a new button, with only two different states
	 */
	public WowButton buildSimpleWowButtonObjectPng(String buttonName, int xLocation, int yLocation, boolean mouseOver){
	 	URL fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_clicked.png");
	 	BufferedImage clicked = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_up.png");
		BufferedImage down = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		BufferedImage up = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		BufferedImage hooverDown = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		BufferedImage hooverUp = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		
		WowButton wb = new WowButton(hooverUp,hooverDown,down,up,clicked, xLocation, yLocation, false, mouseOver);
		return wb;
	}
	
	/**
	 * For construct a simple, togglable buttons with just up and down pictures.
	 * @param buttonName - the name of the button
	 * @param xLocation - the x location where the button will be placed
	 * @param yLocation - the y location where the button will be placed
	 * @return a new button, with only two different states
	 */
	public WowButton buildSimpleToggleWowButtonObjectPng(String buttonName, int xLocation, int yLocation, boolean mouseOver){
	 	URL fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_down.png");
	 	BufferedImage down = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_up.png");
		BufferedImage clicked = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		
		BufferedImage up = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		BufferedImage hooverDown = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		BufferedImage hooverUp = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		
		WowButton wb = new WowButton(hooverUp,hooverDown,down,up,clicked, xLocation, yLocation, true, mouseOver);
		return wb;
	}
	
	/**
	 * will create a new buttton, which will be used to choose the attack
	 * @param buttonName - the name on the ImageFile
	 * @param xLocation - the xLoction
	 * @param yLocation - the yLocation
	 * @return a new WoWButton
	 */
	public WowButton buildAttackButtonObject(String buttonName, int xLocation, int yLocation, boolean mouseOver){
		URL fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_clicked.png");
		BufferedImage clicked = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		BufferedImage down = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT );
		BufferedImage up = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		fileURL = getFileURL("Graphics/Buttons/" + buttonName + "_hoover.png");
		BufferedImage hooverDown = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		BufferedImage hooverUp = com.golden.gamedev.util.ImageUtil.getImage(fileURL, Transparency.TRANSLUCENT);
		WowButton wb = new WowButton(hooverUp,hooverDown,down,up,clicked, xLocation, yLocation, false, mouseOver);
		return wb;
	}
	
	/**
	 * Added a fileUrl checker method, that throws an exception if the file can't be found.
	 * @param fileName
	 * @return
	 */
	public URL getFileURL(String fileName){
		URL fileURL;
		fileURL = getClass().getResource("/Resources/"+fileName);
		if(fileURL==null){
			throw new NullPointerException("Can't find image at: /Resources/"+fileName);
		}
		return fileURL;
	}
	
	/**
	 * Returns a standard WowTextField
	 * @param initText
	 * @param xLocation
	 * @param yLocation
	 * @param width
	 * @param keyBoardFocusHandler
	 * @param mouseOver
	 * @return
	 */
	public WowTextField buildWowTextField(String initText, int xLocation, int yLocation, 
										int width, keyBoardFocusHandler keyBoardFocusHandler,
										boolean mouseOver){
		return new WowTextField(
				xLocation, yLocation, width, initText, keyBoardFocusHandler, mouseOver);
	}
	/**
	 * Returns a PasswordField, same as WowTextfield just with hidden text.
	 * @param initText
	 * @param xLocation
	 * @param yLocation
	 * @param width
	 * @param keyBoardFocusHandler
	 * @param mouseOver
	 * @return
	 */
	public WowPasswordField buildWowPasswordField(String initText, int xLocation, 
					int yLocation, int width, keyBoardFocusHandler keyBoardFocusHandler,
					boolean mouseOver){
		return new WowPasswordField(
				xLocation, yLocation, width, initText, keyBoardFocusHandler, mouseOver);
	}
	/**
	 * Returns a red StatusBar
	 */
	public StatusBar buildRedStatusBar(int xLocation, int yLocation, int minValue, int maxValue, AttributeTypes attributeType){

		return new StatusBar(buildBufferedImage("statusbarback"), buildBufferedImage("statusbarred"), maxValue, minValue, xLocation, yLocation, attributeType);
	}
	/**
	 * Returns a blue StatusBar
	 * @param xLocation
	 * @param yLocation
	 * @param minValue
	 * @param maxValue
	 * @param attributeType
	 * @return
	 */
	public StatusBar buildBlueStatusBar(int xLocation, int yLocation, int minValue, int maxValue, AttributeTypes attributeType){

		return new StatusBar(buildBufferedImage("statusbarback"), buildBufferedImage("statusbarblue"), maxValue, minValue, xLocation, yLocation, attributeType);
	}
	
	public StatusBar buildGoldStatusBar(int xLocation, int yLocation, int minValue, int maxValue, AttributeTypes attributeType){

		return new StatusBar(buildBufferedImage("statusbarback"), buildBufferedImage("statusbargold"), maxValue, minValue, xLocation, yLocation, attributeType);
	}

	/**
	 * 
	 * @param textureName - the filename to which the picture should be returned from
	 * @return a picture based on the filename
	 */
	public BufferedImage buildBufferedImage(String textureName){
	 	URL fileURL = getFileURL("Graphics/" + textureName + ".png");
	 	BufferedImage image = com.golden.gamedev.util.ImageUtil.
					getImage(fileURL, Transparency.TRANSLUCENT );
		return image;
	}
}