package client.graphics;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Collections;

import client.clientState.Settings;

import com.golden.gamedev.object.Sprite;
/**
 * Handles the object that is drawn to the screen.
 */
public class ViewFacade {

	private static ViewFacade singleton;
	
	private ArrayList<GraphicObject> foregroundList;
	private ArrayList<GraphicObject> bufferForegroundList;
	
	private ArrayList<GraphicObject> midFrontList;
	private ArrayList<GraphicObject> bufferMidFrontList;
	
	private ArrayList<GraphicObject> midBackList;
	private ArrayList<GraphicObject> bufferMidBackList;
	
	private ArrayList<GraphicObject> backgroundList;
	private ArrayList<GraphicObject> bufferBackgroundList;
	
	private ViewFacade(){
		foregroundList = new ArrayList<GraphicObject>();
		bufferForegroundList = new ArrayList<GraphicObject>();
		midFrontList = new ArrayList<GraphicObject>();
		bufferMidFrontList = new ArrayList<GraphicObject>();
		midBackList = new ArrayList<GraphicObject>();
		bufferMidBackList = new ArrayList<GraphicObject>();
		backgroundList = new ArrayList<GraphicObject>();
		bufferBackgroundList = new ArrayList<GraphicObject>();
	}
	/**
	 * Returns the instance of the ViewFacade.
	 * @return
	 */
	public static ViewFacade getInstance(){
		if(singleton == null){
			singleton = new ViewFacade();
		}
		return singleton;
	}
	/**
	 * Updates all the layers.
	 * Clears the Layers, and move the buffer to layer.
	 */
	public void updateAll(){
		updateForeground();
		updateMidFront();
		updateMidBack();
		updateBackground();
	}
	/**
	 * Clears the ForegroundList, and moves the buffer to layer.
	 */
	public void updateForeground(){
		foregroundList.clear();
		for (GraphicObject s : bufferForegroundList) {
			foregroundList.add(s);
		}
		bufferForegroundList.clear();
	}
	/**
	 * Clears the midFrontList, and moves the buffer to layer.
	 */
	public void updateMidFront(){
		midFrontList.clear();
		for (GraphicObject s : bufferMidFrontList) {
			midFrontList.add(s);
		}
		bufferMidFrontList.clear();
	}
	/**
	 * Clears the midBackList, and moves the buffer to layer.
	 */
	public void updateMidBack(){
		midBackList.clear();
		for (GraphicObject s : bufferMidBackList) {
			midBackList.add(s);
		}
		bufferMidBackList.clear();
	}
	/**
	 * Clears the BackgroundList, and moves the buffer to layer.
	 */
	public void updateBackground(){
		backgroundList.clear();
		for (GraphicObject s : bufferBackgroundList) {
			backgroundList.add(s);
		}
		bufferBackgroundList.clear();
	}
	/**
	 * Add a GraphicObject to buffer.
	 * @param s
	 */
	public void addForegroundSprite(GraphicObject s){
		bufferForegroundList.add(s);
	}
	/**
	 * Add a GraphicObject to buffer.
	 * @param s
	 */
	public void addMidFrontSprite(GraphicObject s){
		bufferMidFrontList.add(s);
	}
	/**
	 * Add a GraphicObject to buffer.
	 * @param s
	 */
	public void addMidBackSprite(GraphicObject s){
		bufferMidBackList.add(s);
	}
	/**
	 * Add a GraphicObject to buffer.
	 * @param s
	 */
	public void addBackgroundSprite(GraphicObject e){
		bufferBackgroundList.add(e);
	}
	/**
	 * Renders all the objects in the layerlists.
	 * @param g
	 */
	public synchronized void render(Graphics2D g){
		g.setBackground(Color.BLACK);
		g.clearRect(0,0, Settings.screenWidth, Settings.screenHeight);
		if(backgroundList.size() != 0){
			for (GraphicObject s : backgroundList) {
				s.draw(g);
			}
		}
		
		if(midBackList.size() != 0){
			Collections.sort(midBackList);
			for (GraphicObject s : midBackList) {
				s.draw(g);
			}
		}
		
		if(midFrontList.size() != 0){
			for (GraphicObject s : midFrontList) {
				s.draw(g);
			}
		}
		
		if(foregroundList.size() != 0){
			for (GraphicObject s : foregroundList) {
				s.draw(g);
			}
		}
	}
	
}
