package client.graphics;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import shared.CommonFunctions;
import shared.models.Location;
import shared.models.WorldObject;

import client.clientState.Settings;

import com.golden.gamedev.object.Sprite;
/**
 * GraphicObject for WorldObjects.
 * @author thorgrun
 */
public class WorldObjectSprite implements GraphicObject{
	private BufferedImage image;
	private WorldObject object;
	private Location screenFocus;
	public WorldObjectSprite(BufferedImage image,WorldObject object,Location screenFocus){
		this.image = image;
		this.screenFocus = screenFocus;
		this.object = object;
		if(image!=null){
			this.object.setImageHeight(this.image.getHeight());
			this.object.setImageWidth(this.image.getWidth());
		}
		
	}
	/**
	 * Draws the WorldObject relative to the position on the map and the screen focus.
	 */
	public void draw(Graphics2D g) {
		int drawX= CommonFunctions.drawXinPlayField(object.getCurrentLocation().getX(), screenFocus.getX(),this.object.getImageWidth());
		int drawY = CommonFunctions.drawYinPlayField(object.getCurrentLocation().getY(), screenFocus.getY(),this.object.getImageHeight());
		if((drawX<Settings.playfieldWidth&&drawX>-40)&&(drawY<Settings.playfieldHeight&&drawY>-40)){
			if(image!=null){
				g.drawImage(image,null,drawX,drawY);
			}else{
				//Traveltrigger drawn.
				g.setColor(Color.BLUE);
				g.fillRect(drawX, drawY, object.getImageWidth(),object.getImageHeight());
			}
			
		}
		
		
		
	}
	@Override
	public int getYLoc() {
		// TODO Auto-generated method stub
		return object.getCurrentLocation().getY();
	}
	@Override
	public int compareTo(GraphicObject obj) {
		if(obj.getYLoc()>this.getYLoc()){
			return -1;
		}else if(obj.getYLoc()<this.getYLoc()){
			return 1;
		}
		return 0;
	}

}
