package client.graphics.components;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import com.golden.gamedev.object.Timer;

import client.graphics.GraphicObject;
import client.graphics.GraphicObjectFactory;

/**
 * This class will contain a questionPanel
 * @author Esben Fllesdal
 *
 */
public class QuestionPanel implements GraphicObject, MouseListener{

	private final int offsetX_QuestionText_Header = 100;
	private final int offsetY_QuestionText_Header = 100;
	
	private int width = 400; 
	private int height = 300;
	private ArrayList<String> answers = new ArrayList<String>();
	private String questionText ;
	private int xPos = 50;
	private int yPos = 10;
	private BufferedImage background;
	private ArrayList<WowButton> buttons = new ArrayList<WowButton>();
	private int timeLeft;
	private Timer timerTick;
	private boolean questionAnswered = false; // when the player answers, this becomes true
	private int selectedAnswer = -1;//this will contain the answer the player selected
	private TextGraphicObject questionGraphicsObject;
	private ArrayList<TextGraphicObject> graphicsObjectList = new ArrayList<TextGraphicObject>(); 
	private int questionId;
	private String questionHeader;
	private TextGraphicObject questionHeaderGraphicsObject;
	
	public QuestionPanel(String questionText, ArrayList<String> answers, int timeLimit, 
						int questionId, String questionHeader){
		this.questionText = questionText;
		//creates questionTextGraphicsObject, which will draw the text on the screen
		questionHeaderGraphicsObject = GraphicObjectFactory.getInstance().buildWordWrappableTextObject(questionHeader, getX()+offsetX_QuestionText_Header, getY()+offsetY_QuestionText_Header,getWidth(), true);
		questionGraphicsObject = GraphicObjectFactory.getInstance().buildWordWrappableTextObject(questionText, getX()+offsetX_QuestionText_Header, getY()+offsetY_QuestionText_Header+questionHeaderGraphicsObject.getFont().getSize(),getWidth(), true); 
		this.questionHeader = questionHeader;
		this.answers = answers;
		this.questionId = questionId;
		//creates a background
		background = GraphicObjectFactory.getInstance().buildBufferedImage("QuestionPanel/background");
		//creates one buttons for each answer
		for(int i= 0; i<answers.size();i++){
			int x = 160;
			int y = 260;
			buttons.add(GraphicObjectFactory.getInstance().buildSimpleWowButtonObjectPng("answer", x, y+(60*i), false));
			graphicsObjectList.add(GraphicObjectFactory.getInstance().buildWordWrappableTextObject(answers.get(i),x+10,y+30+(60*i), buttons.get(i).getWidth(), false));
		}
		//adds mouse listener to all the buttons
		for(WowButton button: buttons){
			button.addMouseListener(this);
			button.updateInput();
		}
		//lager en timer
		timeLeft = timeLimit;
		timerTick = new Timer(1000);
	}
	
	public int getQuestionId() {
		return questionId;
	}

	/**
	 * Draws the graphics to the screen
	 * @Override 
	 */
	public void draw(Graphics2D g) {
	if(!questionAnswered){
			g.setColor(Color.RED);
			g.drawImage(background, null, getX(), getY());
			questionHeaderGraphicsObject.draw(g);
			questionGraphicsObject.draw(g);
			for(int i =0; i< buttons.size();i++){
				WowButton button =buttons.get(i); 
				button.draw(g);
				graphicsObjectList.get(i).draw(g);
			}
			g.drawString("Time left : " +timeLeft+"", 100, 200);
		}
	}
	
	/**
	 * updates the questionPanel
	 */
	public void Update(long elapsedTimer){
		for(WowButton button: buttons){
			button.updateInput();
		}
		//updates the timer
		if(timeLeft > 0 && !questionAnswered){
			if(timerTick.action(elapsedTimer)){
				timeLeft= timeLeft - 1;
			}
		}
	}
	
	public int getX() {
		return xPos;
	}

	public void setX(int x) {
		this.xPos = x;
	}

	public int getY() {
		return yPos;
	}

	public void setY(int y) {
		this.yPos = y;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public ArrayList<String> getAnswers() {
		return answers;
	}

	public void setAnswers(ArrayList<String> answers) {
		this.answers = answers;
	}

	public String getQuestionText() {
		return questionText;
	}

	public void setQuestionText(String questionText) {
		this.questionText = questionText;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		for(int i=0; i<buttons.size();i++){
			if(e.getSource().equals(buttons.get(i))){
				questionAnswered = true;
				selectedAnswer = i;
				break;
			}
		}
	}

	public int getTimeLeft() {
		return timeLeft;
	}

	public boolean isQuestionAnswered() {
		return questionAnswered;
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
	}
	
	public int getSelectedAnswer() {
		return selectedAnswer;
	}
	
	public String getSelectedAnswerText(){
		return answers.get(getSelectedAnswer());
	}

	@Override
	public int getYLoc() {
		return yPos;
	}

	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}
}