package client.graphics.components;

import java.awt.Color;
import java.awt.Graphics2D;

import shared.CommonFunctions;
import shared.models.AttributeListener;
import shared.models.AttributeTypes;
import shared.models.Location;
import client.graphics.GraphicObject;
/**
 * 
 * SimpleStatus bar that is used for displaying health over the NPCs.
 */
public class SimpleStatusBar  implements GraphicObject, AttributeListener {

	
	private int minValue;
	private int maxValue;
	private int npcWidth;
	private int npcHeight;
	private int currentValue;
	private Location npcLocation;
	private AttributeTypes attributeType;
	private AttributeTypes maxAttributeType;
	private Location screenFocus;
	public SimpleStatusBar(Location npcLocation,Location screenFocus,int minValue,int maxValue,int currentValue,int npcWidth,int npcHeight,AttributeTypes attributeType){
		this.minValue = minValue;
		this.npcHeight = npcHeight;
		this.npcWidth = npcWidth;
		this.maxValue = maxValue;
		this.screenFocus = screenFocus;
		this.currentValue = currentValue;
		this.npcLocation = npcLocation;
		this.attributeType = attributeType;
		if(attributeType == AttributeTypes.HP){
			maxAttributeType = AttributeTypes.MAX_HP;
		}
		else if(attributeType == AttributeTypes.MP){
			maxAttributeType = AttributeTypes.MAX_MP;
		}
		else{
			maxAttributeType = null;
		}
	}
	/**
	 * Draws the statusbar
	 */
	public void draw(Graphics2D g) {
		double lifeLeftPercentage=(double)currentValue/(double)maxValue;
		int drawX = CommonFunctions.drawXinPlayField(npcLocation.getX(), screenFocus.getX())-npcWidth/2;
		int drawY = CommonFunctions.drawYinPlayField(npcLocation.getY(), screenFocus.getY())-npcHeight-10;
		g.setColor(Color.BLACK);
		g.fillRect(drawX, drawY, 32, 7);
		g.setColor(Color.WHITE);
		g.fillRect(drawX+1, drawY+1, 30, 5);
		g.setColor(Color.RED);
		g.fillRect(drawX+1, drawY+1, (int)(lifeLeftPercentage*30), 5);
	}

	@Override
	public int getYLoc() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public int compareTo(GraphicObject arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	/**
	 * Is called when the attribute is changed.
	 */
	public void AttributeChanged(AttributeTypes attribute, int oldValue,
			int newValue) {
		if(attribute == this.attributeType){
			currentValue = newValue;
		}
		else if(attribute == this.maxAttributeType){
			maxValue = newValue;
		}
		
	}

}
