package client.graphics.components;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import com.sun.org.apache.bcel.internal.classfile.Attribute;

import shared.models.AttributeListener;
import shared.models.AttributeTypes;

import client.graphics.GraphicObject;
/**
 * Statusbar component.
 */
public class StatusBar implements GraphicObject, AttributeListener {

	private int minValue;
	private int maxValue;
	private int targetValue;
	private int currentValue;
	
	private AttributeTypes attributeType;
	private AttributeTypes maxAttributeType;
	
	private int xLocation;
	private int yLocation;
	
	private int stringLocationX;
	private int stringLocationY;
	
	private BufferedImage background;
	private BufferedImage theBar;
	
	private int totalChange;
	
	
	public StatusBar(BufferedImage background, BufferedImage barImage, int maxValue, int minValue, int xLocation, int yLocation, AttributeTypes attributeType) {
		this.minValue = minValue;
		this.maxValue = maxValue;
		this.background = background;
		this.theBar = barImage;
		this.xLocation = xLocation;
		this.yLocation = yLocation;
		
		stringLocationX = xLocation + 100;
		stringLocationY = yLocation + 22;
		
		this.currentValue = maxValue;
		this.targetValue = maxValue;
		this.totalChange = 0;
		
		this.attributeType = attributeType;
		
		if(attributeType == AttributeTypes.HP){
			maxAttributeType = AttributeTypes.MAX_HP;
		}
		else if(attributeType == AttributeTypes.MP){
			maxAttributeType = AttributeTypes.MAX_MP;
		}
		else{
			maxAttributeType = null;
		}
		
		
		
	}
	
	/**
	 * Draws the Status bar
	 */
	public void draw(Graphics2D g) {
		
		g.drawImage(background, xLocation, yLocation, null);
		
		//TODO: calculate and draw correct width!
		double pixelWidth = 271;
		double distance = maxValue - minValue;
		double step = (pixelWidth / distance);
		
		int width = (int) ((currentValue - minValue) * step);
		
		g.drawImage(theBar, xLocation + 4, yLocation + 4, width, theBar.getHeight(), null);
		g.setColor(Color.BLACK);
		g.setFont(new Font("Arial", Font.BOLD, 22));
		//g.drawString("TEST", 300,  500);
		
		g.drawString(currentValue + " / " + maxValue, stringLocationX, stringLocationY);
	}
	/**
	 * Updates the value of the status bar, to give a smooth change. 
	 * @param elapsedtime
	 */
	public void update(long elapsedtime){
		
		
		if(targetValue != currentValue){
				float elapsedFloat = (float) elapsedtime;
				float changeThisTick = ((elapsedFloat / 2000) * totalChange);
				
				if(targetValue < currentValue){
					if(changeThisTick == 0){
						currentValue -= 1;
					}
					else{
						currentValue -= changeThisTick;
					}
					
					if(currentValue < targetValue){
						currentValue = targetValue;
					}
					
				}
				else{
					if(changeThisTick == 0){
						currentValue += 1;
					}
					else{
						currentValue += changeThisTick;
					}
					
					if(currentValue > targetValue){
						currentValue = targetValue;
					}
				}
			
				
			
		}
		
	}

	/**
	 * Is called if an attribute is changed.
	 */
	public void AttributeChanged(AttributeTypes attribute, int oldValue,
			int newValue) {

		if(attribute == this.attributeType){
			targetValue = newValue;
		}
		else if(attribute == this.maxAttributeType){
			maxValue = newValue;
		}
		
	}

	@Override
	public int getYLoc() {
		return yLocation;
	}

	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}
	
}
