package client.graphics.components;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import shared.models.TravelTrigger;

import client.clientState.worldState.MainWorldPanel;
import client.graphics.GraphicObject;
import client.graphics.GraphicObjectFactory;
import client.graphics.SpriteGraphicObject;


public class TravelPanel implements GraphicObject,MouseListener{

	private WowButton yesButton;
	private WowButton noButton;
	private boolean hasAnswered;
	private boolean answeredYes;
	private TextGraphicObject travelQuestion;
	private TextGraphicObject destination;

	private SpriteGraphicObject background;
	private int xLoc = 0;
	private int yLoc = 0;
	private TravelTrigger clickedTrigger;
	private MainWorldPanel myWorldPanel;
	public TravelPanel(int xLoc, int yLoc,TravelTrigger trigger, MainWorldPanel myWorldPanel){
		this.xLoc = xLoc;
		this.yLoc = yLoc;
		yesButton = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage(
				"yes_up", xLoc+100, yLoc+150, false, false);
		
		yesButton.addMouseListener(this);
		
		noButton = GraphicObjectFactory.getInstance().buildWowButtonObjectPngOneImage(
				"no_up", xLoc+200, yLoc+150, false, false);
		
		noButton.addMouseListener(this);
		this.myWorldPanel = myWorldPanel;
		this.clickedTrigger = trigger;
		hasAnswered = false;
		background = GraphicObjectFactory.getInstance().buildSpriteGraphicObjectPng("travelPanelBackground", xLoc, yLoc);
		travelQuestion = new TextGraphicObject("Do you want to travel to:",xLoc+20,yLoc+30,Color.black);
		destination = new TextGraphicObject(trigger.getDestinationZoneName()+"?",xLoc+20,yLoc+60,Color.black);
	}
	
	public void update(long elapsedTime){
		yesButton.updateInput();
		noButton.updateInput();
	}
	public boolean hasAnswered(){
		return hasAnswered;
	}
	public boolean answeredYes(){
		return answeredYes;
	}
	@Override
	public void draw(Graphics2D g) {
		background.draw(g);
		yesButton.draw(g);
		noButton.draw(g);
		travelQuestion.draw(g);
		destination.draw(g);
	}
	public TravelTrigger getTrigger(){
		return clickedTrigger;
	}
	@Override
	public int getYLoc() {
		// TODO Auto-generated method stub
		return yLoc;
	}

	@Override
	public int compareTo(GraphicObject arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if(e.getSource()==yesButton){
			answeredYes = true;
			hasAnswered = true;
		}else if(e.getSource()==noButton){
			hasAnswered = true;
			answeredYes = false;
		}
		
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

}
