package client.graphics.components;

import java.awt.Graphics2D;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import client.graphics.GraphicObject;

/**
 * Creates a button object.
 * @author Esben Fllesdal
 * @see GraphicObject
 * @see SelectableComponent
 */
public class WowButton extends SelectableComponent implements GraphicObject{
	//all pictures have to be at the same size
	private BufferedImage hooverButtonUp;
	private BufferedImage hooverButtonDown;
	private BufferedImage buttonDown;
	private BufferedImage buttonUp;
	private BufferedImage buttonClick;
	private WowButtonState buttonState = WowButtonState.BUTTON_UP;
	private boolean toggleOn = false;
	private boolean hoover = false;
	private boolean clicked = false;
	
	/**
	 * 
	 * @param hooverButtonUp - picture when mouse hoovering over a button which is up
	 * @param hooverButtonDown - picture when mouse hoovering over a button which is down
	 * @param buttonDown - picture when a button is down
	 * @param buttonUp - - picture when a button is up
	 * @param buttonClick -picture when the button is clicked
	 * @param xLocation - the location where the button starts (x)
	 * @param yLocation - the location where the button starts (y)
	 * @param mouseOver - if there should be possibility to do mouseHoovering
	 */
	public WowButton(BufferedImage hooverButtonUp, BufferedImage hooverButtonDown, 
			BufferedImage buttonDown, BufferedImage buttonUp, BufferedImage buttonClick,
			int xLocation, int yLocation, boolean toggleOn, boolean mouseOver){
		super(xLocation, yLocation, buttonUp.getWidth(), buttonUp.getHeight(), mouseOver);
		this.hooverButtonUp = hooverButtonUp;
		this.hooverButtonDown = hooverButtonDown;
		this.buttonDown = buttonDown;
		this.buttonUp = buttonUp;
		this.buttonClick = buttonClick;
		this.toggleOn = toggleOn;
	}
	
	/**
	 * 
	 * @param hooverButtonUp - picture when mouse hoovering over a button which is up
	 * @param hooverButtonDown - picture when mouse hoovering over a button which is down
	 * @param buttonDown - picture when a button is down
	 * @param buttonUp - - picture when a button is up
	 * @param buttonClick -picture when the button is clicked
	 * @param xLocation - the location where the button starts (x)
	 * @param yLocation - the location where the button starts (y)
	 * @param mouseOver - if there should be possibility to do mouseHoovering
	 */
	public WowButton(BufferedImage hooverButton, 
			BufferedImage buttonDown, BufferedImage buttonUp, BufferedImage buttonClick,
			int xLocation, int yLocation, boolean toggleOn, boolean mouseOver){
		super(xLocation, yLocation, buttonUp.getWidth(), buttonUp.getHeight(), mouseOver);
		this.hooverButtonUp = hooverButton;
		this.hooverButtonDown = hooverButton;
		this.buttonDown = buttonDown;
		this.buttonUp = buttonUp;
		this.buttonClick = buttonClick;
		this.toggleOn = toggleOn;
	}

	/**
	 * 	this constructor have the same image on all buttonState.
	 *  this should not be used, for other purposes than testing
	 *  @param buttonImage - an image
	 *  @param xLocation - the x-location for the button
	 *  @param yLocation - the y-location for the button
	 *  @param mouseOver - if there should be possibility to do mouseHoovering
	 */
	public WowButton(BufferedImage buttonImage, int xLocation, int yLocation, boolean toggleOn,
			 boolean mouseOver){
		super(xLocation, yLocation, buttonImage.getWidth(), buttonImage.getHeight(), mouseOver);
		this.buttonUp = buttonImage;
		this.hooverButtonDown = buttonImage;
		this.hooverButtonUp = buttonImage;
		this.buttonDown = buttonImage;
		this.buttonClick = buttonImage;
		this.toggleOn = toggleOn;
	}
	/**
	 * Draws the Button.
	 */
	public void draw(Graphics2D g) {
		if(clicked){
			g.drawImage(buttonClick, null, getXLocation(), getYLocation());
		}
		else if(buttonState == WowButtonState.BUTTON_UP){
			if(hoover){
				g.drawImage(hooverButtonUp, null, getXLocation(), getYLocation());
			}
			else{
				g.drawImage(buttonUp, null, getXLocation(), getYLocation());
			}
		}
		else if(buttonState == WowButtonState.BUTTON_DOWN){
			if(hoover){
				g.drawImage(hooverButtonDown, null, getXLocation(), getYLocation());
			}
			else{
				g.drawImage(buttonDown, null, getXLocation(), getYLocation());
			}
		}
	}


	/**
	 * Checks if the button has been clicked, or if the cursor hoover over it.
	 */
	public void updateInput() {
		boolean pressed = checkIfPressed(); 
		if(pressed){
			if(!clicked){
				clicked = true;
				fireMouseClickedEvent();
			}
			else{
				return;
			}
		}
		else{
			if(clicked){
				clicked = false;
				if(toggleOn){
					if(buttonState == WowButtonState.BUTTON_UP){
						buttonState = WowButtonState.BUTTON_DOWN;
					}
					else{
						buttonState = WowButtonState.BUTTON_UP;
					}
				}
			}
		}
		if(isMouseOver()){
			//updates if the component should listen on hoover effects
			boolean entered = checkIfEntered();
			if(entered){
				hoover = true;
				fireMouseEnteredEvent();
			}
			else{
				if(hoover){
					//only fire exit if it just has exited
					hoover = false;
					fireMouseExitedEvent();
				}
			}
		}
	}
	/**
	 * Toggles the buttonstate.
	 * @param state
	 */
	public void toggleButtonState(WowButtonState state){
		
		if(toggleOn){
			buttonState = state;
		}
	}

	@Override
	public int getYLoc() {
		// TODO Auto-generated method stub
		return getYLocation();
	}

	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}

	public BufferedImage getButtonDown() {
		return buttonDown;
	}

	public BufferedImage getButtonUp() {
		return buttonUp;
	}

	public WowButtonState getButtonState() {
		return buttonState;
	}

	public boolean isClicked() {
		return clicked;
	}
	
}