package client.graphics.components;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;

import client.UserInput.*;
import client.clientState.Settings;
import client.graphics.GraphicObject;
/**
 * 
 * A TextField component.
 */
public class WowTextField extends KeyBoardFocusableComponent {


	private String text;
	private String initText;
	private boolean isInitText;
	/**
	 * Creates the textfield.
	 * @param xLocation
	 * @param yLocation
	 * @param width
	 * @param initalText
	 * @param keyBoardFocusHandler
	 * @param mouseOver
	 */
	public WowTextField(int xLocation, int yLocation, int width, String initalText,
				keyBoardFocusHandler keyBoardFocusHandler, boolean mouseOver){
		super(xLocation, yLocation, width, Settings.standardFont.getSize()+4, keyBoardFocusHandler, mouseOver);
		this.text = initalText;
		this.initText = initalText;
		isInitText = true;
		hasKeyboardFocus = false;
	}
	
	
	/**
	 * Checks if the Textfield has focus, and writes when a letter is typed.
	 */
	public void updateInput(){
		
		if(super.checkIfPressed()){
			keyFocusHandler.requestKeyboardFocus(this);
		}
		
		
		if(hasKeyboardFocus){
			InputType input = InputControls.getInstance().getPressedButtons();
			if(input == InputType.BACKSPACE){
				if(isInitText){
					isInitText = false;
					text = "";
				}
				if(text.length() > 0){
					text = text.substring(0, text.length() - 1);
				}
			}
			else if(input == InputType.TAB){
				keyFocusHandler.tabReturned();
			}
			else if(input == InputType.ENTER){
				keyFocusHandler.enterPressed();
			}
			else if(input == InputType.SPACE){
				String s=" ";
				if(s != ""){
					if(isInitText){
						isInitText = false;
						text = "";
					}
					text = text + s;
					
				}
			}
			else if(input == InputType.LETTER){
				String s = "";
				if( client.UserInput.InputControls.getInstance().getLetterKeyPressed()){
					s = client.UserInput.InputControls.getInstance().getPressedLetterKey();
				}
				else if(client.UserInput.InputControls.getInstance().getSpacePressed()){
					s = " ";
				}
				if(s != ""){
					if(isInitText){
						isInitText = false;
						text = "";
					}
					text = text + s;
					
				}
				
			}
			
		}
		
		
	}
	private int StringWidth(String str){
        FontMetrics metrics = new FontMetrics(Settings.standardFont) {};
        Rectangle2D bounds = metrics.getStringBounds(str, null);
		return (int) bounds.getWidth();
	}
	protected String drawSubString(String str){
		String drawText = "";
		int letterIndex = str.length()-1;
		if(letterIndex<0){
			return str;
		}
		while(StringWidth(drawText)<getWidth()-30){
			if(letterIndex == 0){
				drawText= str.substring(letterIndex, letterIndex+1)+drawText;
				break;
			}else{
				drawText= str.substring(letterIndex, letterIndex+1)+drawText;
			}
			letterIndex--;
		}
		
		return drawText;
	}
	
	
	/**
	 * Draws the TextField.
	 */
	public void draw(Graphics2D g) {
		// TODO Auto-generated method stub
		g.setBackground(Color.GRAY);
		g.drawRect(getXLocation(), getYLocation(), super.getWidth(), super.getHeight());
		g.setColor(Color.BLACK);
		
		
		g.drawString(drawSubString(text), getXLocation()+ 5, getYLocation() + getHeight()  - 2);
	}

	/**
	 * Returns the text in the TextField
	 * @return
	 */
	public String getText() {
		return text;
	}
	
	


	public void setText(String text) {
		this.text = text;
		isInitText = false;
	}


	public boolean isInitText() {
		return isInitText;
	}
	
	public void resetToInitText(){
		text = initText;
		isInitText = true;
	}


	@Override
	public int getYLoc() {
		
		return getYLocation();
	}


	@Override
	public int compareTo(GraphicObject o) {
		// TODO Auto-generated method stub
		return 0;
	}
	
	
	
	
}
