package database.databaseConnection;

import java.sql.*;

import shared.fileHandler.Settings;

import lobbyServer.clientSessionHandler.LobbyControl;

/**
 * A collection of useful databaseMethods
 * @author Esben Fllesdal
 *
 */
public class DBMethods{
	private static DBMethods singleton;
	private String postgreIpAdress;
	private String databaseName;
	private String userName;
	private String password;
	private Settings settings;
	
	private DBMethods(){
		settings = new Settings("database");
		postgreIpAdress = settings.getPostgreSQLIp();
		databaseName = settings.getDatabaseName();
	}
	public void setUserName(String userName){
		this.userName = userName;
	}
	public void setPassword(String password){
		this.password = password;
	}
	public static DBMethods getInstance(){
		
		if(singleton == null){
			singleton = new DBMethods();
		}
		
		return singleton;
	}
	/**
	 * Establishes a connection to the database
	 * @author Esben Fllesdal
	 * @return A connection to the database
	 */
	public Connection ConnectToDB() 
	{
		 try{
			    Class.forName("org.postgresql.Driver");
	   } catch (ClassNotFoundException cnfe){
			    System.out.println("Could not find the JDBC driver!");
			    cnfe.printStackTrace();
			    System.exit(1);
			 }
		 Connection conn = null;
		 try {
			 conn = DriverManager.getConnection
			 ("jdbc:postgresql://"+postgreIpAdress+"/"+databaseName,
                     userName, password);
		 } catch (SQLException sqle) {
			System.out.println("Could not connect");
			sqle.printStackTrace();
			System.exit(1);
			}
		return conn;
	}
	
	/**
	 * Disconnect a database connection
	 * @param c - a connection
	 * @author Esben Fllesdal
	 */
	public void DisconnectFromDB(Connection c){
		try {
			c.close();
		} catch (SQLException sqle) {
			System.out.println("Could not disconnect");
			sqle.printStackTrace();
			System.exit(1);
		}
	}
	
	/**
	 * Precompiles the query 
	 * @param sql a sql query
	 * @param c a database connection
	 * @return a prepared SQL statement
	 */
	public static PreparedStatement PrepareStatement(String sql, Connection c){
		try{
		return c.prepareStatement(sql);
		}
		catch(SQLException sqle){
			System.out.println("Could not prepareStatement");
			sqle.printStackTrace();
			System.exit(1);
		}
		return null;
	}
	
	public static PreparedStatement SelectQuery(String select, String from, String where,
			Connection dbConnection) {
		String sqlText = "Select "+select+" From \""+from+"\" where "+where;
		try{
		return dbConnection.prepareStatement(sqlText);
		}
		catch(SQLException sqle){
			System.out.println("Could not prepareStatement");
			sqle.printStackTrace();
			System.exit(1);
		}
		return null;
	}
	
	public static PreparedStatement SelectQuery(String select, String from,
			Connection dbConnection) {
		String sqlText = "Select "+select+" From \""+from+"\"";
		try{
		return dbConnection.prepareStatement(sqlText);
		}
		catch(SQLException sqle){
			System.out.println("Could not prepareStatement");
			sqle.printStackTrace();
			System.exit(1);
		}
		return null;
	}

	public static PreparedStatement InsertQuery(String tableName, String values, 
						Connection dbConnection){
		try{
			String sqlText ="INSERT INTO \""+tableName+"\" values("+values+")";
			return dbConnection.prepareStatement(sqlText);
			}
			catch(SQLException sqle){
				System.out.println("Could not prepareStatement");
				sqle.printStackTrace();
				System.exit(1);
			}
			return null;
	}
	
	public static PreparedStatement InsertQuery(String tableName, String columNames, 
			String values ,Connection dbConnection){
			try{
			String sqlText ="INSERT INTO \""+tableName+"\" (\""+columNames+"\") values("+values+")";
			return dbConnection.prepareStatement(sqlText);
			}
			catch(SQLException sqle){
				System.out.println("Could not prepareStatement");
				sqle.printStackTrace();
				System.exit(1);
			}
			return null;
	}
	
	public static PreparedStatement InsertQuery(String tableName, String columNames, 
			String values, String returning ,Connection dbConnection){
			try{
			String sqlText ="INSERT INTO \""+tableName+"\" (\""+columNames+"\") values("+values+") " +
					"RETURNING \""+ returning+"\"";
			return dbConnection.prepareStatement(sqlText);
			}
			catch(SQLException sqle){
				System.out.println("Could not prepareStatement");
				sqle.printStackTrace();
				System.exit(1);
			}
			return null;
	}
	
	public static PreparedStatement InsertQueryWithInt(String tableName, String columNames, 
			String values ,Connection dbConnection){
			try{
			String sqlText ="INSERT INTO \""+tableName+"\" (\""+columNames+"\") values("+values+")";
			return dbConnection.prepareStatement(sqlText);
			}
			catch(SQLException sqle){
				System.out.println("Could not prepareStatement");
				sqle.printStackTrace();
				System.exit(1);
			}
			return null;
	}

	
	public static PreparedStatement UpdateQuery(String tableName, String set, String where,
						Connection dbConnection){
		try{
			String sqlText = "UPDATE \""+tableName+"\" SET "+set+" WHERE "+where;
			return dbConnection.prepareStatement(sqlText);
			}
			catch(SQLException sqle){
				System.out.println("Could not prepareStatement");
				sqle.printStackTrace();
				System.exit(1);
			}
			return null;
	}
	
	/**
	 * 
	 * @param tableName
	 * @param where
	 * @param dbConnection
	 * @return
	 */
	public static PreparedStatement DeleteQuery(String tableName, String where,
			Connection dbConnection){
		try{
		String sqlText = "DELETE FROM \""+tableName+"\" WHERE "+where;
		return dbConnection.prepareStatement(sqlText);
		}
		catch(SQLException sqle){
			System.out.println("Could not prepareStatement");
			sqle.printStackTrace();
			System.exit(1);
		}
		return null;
	}
}