package database.databaseServer;

import java.io.IOException;
import java.net.ServerSocket;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import database.databaseConnection.DBMethods;

import shared.fileHandler.Settings;
import sun.security.acl.WorldGroupImpl;
/**
 * 
 * @author Esben Fllesdal
 * Starts the Database server
 * Listens for connections from lobbyserver and worldservers
 */
public class DBServerMain{
	private Settings networkSettings;
	private DBMethods methods;
	public DBServerMain(){
		JFrame frame = new JFrame("Database Server");
		frame.setVisible(true);
		frame.setLocation(300, 300);
		frame.setSize(200, 50);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JLabel lbl = new JLabel("Server is running..");
		frame.add(lbl);
		
		networkSettings = new Settings("database");
		methods = DBMethods.getInstance();
		
		String username = (String)JOptionPane.showInputDialog(
                frame,
                "Please enter the username for the PostgreSql Database",
                "Database Username",
                JOptionPane.PLAIN_MESSAGE,
                null,
                null,
                "");
		if(username==null||username.equals("")){
			System.exit(0);
		}
		methods.setUserName(username);
		String password = (String)JOptionPane.showInputDialog(
                frame,
                "Please enter the password for the PostgreSql Database",
                "Database Password",
                JOptionPane.PLAIN_MESSAGE,
                null,
                null,
                "");
		if(password==null||password.equals("")){
			System.exit(0);
		}
		methods.setPassword(password);
		
		//Attempt connection with current userName and password
		
		 try{
			    Class.forName("org.postgresql.Driver");
		 } catch (ClassNotFoundException cnfe){
			 	JOptionPane.showMessageDialog(frame, "Could not find the JDBC driver.");
			    cnfe.printStackTrace();
			    System.exit(1);
		 }
		 Connection conn = null;
		 try {
			 conn = DriverManager.getConnection
			 ("jdbc:postgresql://"+networkSettings.getPostgreSQLIp()+"/"+networkSettings.getDatabaseName(),
                  username, password);
			 conn.close();
		 } catch (SQLException sqle) {
			 JOptionPane.showMessageDialog(frame, "Could not connect to the database.");
			sqle.printStackTrace();
			System.exit(1);
		}
		
		
		
		
		new LobbyServerListener(networkSettings.getLobbyListenPort()).start();
		new WorldServerListener(networkSettings.getWorldServerListenPort()).start();
		new EditorListener(networkSettings.getEditorListenPort()).start();
	}
	
	public static void main(String[] args) {
		
		
		new DBServerMain();
	}
	
	

	
}