package database.databaseServer;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import com.sun.org.apache.xml.internal.security.utils.HelperNodeList;

import database.databaseConnection.DBMethods;
import database.helperMethods.DatabaseGetters;
import database.helperMethods.DatabaseSetters;
import database.helperMethods.KingdomInfo;

import shared.models.Answer;
import shared.models.Attack;
import shared.models.AttackType;
import shared.models.Attributes;
import shared.models.Character;
import shared.models.Effect;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Gender;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.PlayableClasses;
import shared.models.Question;
import shared.models.RespawnPoint;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
import shared.models.helpers.EnumHelpers;
import shared.network.messages.DatabaseObjectMessage;
import shared.network.messages.KingdomMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.SessionIdMessage;

/**
 * This class reperesents the listeneing thread for the worldListening server
 * @author Esben Fllesdal
 *
 */
public class EditorConnectionThread extends Thread {

	private Socket socket = null;
	private boolean running = true;
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private String myId;
	private String worldId;
	private String sessionId;
	public EditorConnectionThread(Socket socket){
		super("EditorListenerConnectionThread");
		this.socket  = socket;
	}
	private void sendSucessfullDatabaseOperationMsg(String message, MessageType type){
		Message messageToEditor = new Message(myId, worldId, message,
				type);
		try {
			out.writeObject(messageToEditor);
			out.flush();
		}
		catch (IOException ioe) {
			System.out.println("Connection error in EditorListenerConnectionThread " +ioe.getMessage());
		}
	}
	private void sendSuccessfullKingdomSaved(String kingdomName){
		SessionIdMessage messageToEditor = new SessionIdMessage(myId, worldId,"Kingdom successfully stored" ,
				MessageType.SUCCESSFUL_DATABASE_OPERATION_KINGDOM,"",kingdomName);
		try {
			out.writeObject(messageToEditor);
			out.flush();
		}
		catch (IOException ioe) {
			System.out.println("Connection error in EditorListenerConnectionThread " +ioe.getMessage());
		}
	}
	private void sendIllegalOperationMsg(String message){
		Message messageToEditor = new Message(myId, worldId, 
				message, MessageType.ILLEGAL_DATABASE_OPERATION);
		try {
		out.writeObject(messageToEditor);
		out.flush();
		}
		catch (IOException ioe) {
			System.out.println("Connection error in EditorListenerConnectionThread " +ioe.getMessage());
		}
	}
	private void send(Message msg) throws IOException{
		out.writeObject(msg);
		out.flush();
	}
	public void run(){
			try {			
				in = new ObjectInputStream(socket.getInputStream());
				out = new ObjectOutputStream(socket.getOutputStream());
				
				while(running){
					try{
						Message messageFromEditor = (Message)in.readObject();
						Message messageToEditor;
						myId = messageFromEditor.getTargetId();
						worldId = messageFromEditor.getSourceId();
						
						SessionIdMessage sessionIdMessage;
						String characterName;
						int kingdomId;					
					switch (messageFromEditor.getMessageType()) {
								
					case GET_PLAYABLE_CHARACTER:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						characterName = sessionIdMessage.getCharacterName();
						PlayableCharacter pc = DatabaseGetters.GetPlayableCharcter(characterName);	
						if(pc != null){	
						messageToEditor = new ObjectMessage(myId, worldId, 
									"Character object returning",
									MessageType.OBJECT_MESSAGE ,
									pc,
									ObjectType.CHARACTER_OBJECT,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Character succesfully sent..");
						}
						else{
							messageToEditor = new Message(myId, worldId, "Could not get character : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Could not execute get character");
						}
						break;
					case GET_ALL_FRIENDLY_NPCS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						HashMap<String,FriendlyNPC> fnpcs = DatabaseGetters.GetAllFriendlyNPCAsAHashMap();
						if(fnpcs !=null){
							messageToEditor = new ObjectMessage(myId,worldId,
									"FriendlyNPCs object returning",
									MessageType.OBJECT_MESSAGE,
									fnpcs,
									ObjectType.FRIENDLY_NPC_HASHMAP,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("All friendlyNPCs succesfully sent..");
						}else {
							messageToEditor = new Message(myId, worldId, 
									"Could not get all friendlyNPCs : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Could not execute get all friendlyNPCs");
						}
						break;
					case GET_ALL_KINGDOMNAMES:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<String> kings = DatabaseGetters.GetAllKingdomNames();
						if(kings!=null){
							messageToEditor = new ObjectMessage(myId,worldId,
									"KingdomNames returning",
									MessageType.OBJECT_MESSAGE,
									kings,
									ObjectType.KINGDOM_NAME_LIST,
									sessionId);
							send(messageToEditor);
							System.out.println("KingdomNames sent");
							
						}
						else{
							sendIllegalOperationMsg("Could not get kingdom name list");
						}
						break;
					case GET_ALL_PLAYERNAMES:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<String> playerNames = DatabaseGetters.getAllPlayerNames();
						if(playerNames!=null){
							messageToEditor = new ObjectMessage(myId,worldId,
									"PlayerNames returning",
									MessageType.OBJECT_MESSAGE,
									playerNames,
									ObjectType.PLAYER_NAME_LIST,
									sessionId);
							send(messageToEditor);
							System.out.println("PlayerNames sent");
							
						}
						else{
							sendIllegalOperationMsg("Could not get Player name list");
						}
						
						break;
						
					case GET_ALL_ZONENAMES:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						String[] names = DatabaseGetters.GetAllZoneTemplateNames();
						if(names!=null){
							messageToEditor = new ObjectMessage(myId,worldId,
									"Zonenames returning",
									MessageType.OBJECT_MESSAGE,
									names,
									ObjectType.ZONE_NAME_LIST,
									sessionId);
							send(messageToEditor);
							System.out.println("ZoneNames sent");
						}else{
							sendIllegalOperationMsg("Could not get zone name list");
						}
						
						break;
					case GET_ALL_ENEMY_NPCS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						HashMap<String, EnemyNPC> enpcs = DatabaseGetters.GetAllEnemyNPCAsAHashMap();
						if(enpcs != null){
							messageToEditor = new ObjectMessage(myId, worldId, 
									"EnemyNPCs object returning",
									MessageType.OBJECT_MESSAGE ,
									enpcs,
									ObjectType.ENEMY_NPC_HASHMAP,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("All enemyNPCs succesfully sent..");
						}
						else {
							messageToEditor = new Message(myId, worldId, 
									"Could not get all enemyNPCs : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Could not execute get all enemyNPCs");
						}
						break;
					
					case KINGDOM_OBJECT:
						KingdomMessage kingMsg = (KingdomMessage)messageFromEditor;
						
						if(kingMsg.getKingdomId()==0){
							kingdomId = DatabaseSetters.insertKingdom(kingMsg.getKingdomName(), kingMsg.getSubjectName());
							if(kingdomId!=0){
								DatabaseSetters.insertPlayerAccess(kingdomId,kingMsg.getPlayerAccessList());
								
								for(String str: kingMsg.getZoneList().keySet()){
									int zoneId =DatabaseSetters.insertZone(kingdomId,(Zone)kingMsg.getZoneList().get(str));
									if(zoneId!=0){
										if(str.equals(kingMsg.getStartZoneName())){
											if(!DatabaseSetters.addStartZoneToKingdom(zoneId, kingdomId)){
												sendIllegalOperationMsg("Could not add startZone to kingdom");
												break;
											}
										}
									}else{
										sendIllegalOperationMsg("Could not insertZone");
										break;
									}
								}
								Iterator<Zone> it = kingMsg.getZoneList().values().iterator();
								while(it.hasNext()){
									Zone z = it.next();
									for(TravelTrigger trigger : z.getTravelTriggerList()){
										if(trigger.getDestinationId()!=0){
											if(!DatabaseSetters.addTravelTriggerConnections(trigger.getTravelTriggerId(), trigger.getDestinationId())){
												sendIllegalOperationMsg("Could not create TravelTrigger connection"+trigger.getTravelTriggerId()+" to "+trigger.getDestinationId());
												break;
											}
										}
										
									}
								}
								sendSuccessfullKingdomSaved(kingMsg.getKingdomName());
								
							}else{
								sendIllegalOperationMsg("Could not insert Kingdom");
							}
							
						}else{//Save changes to kingdom
							if(DatabaseSetters.updateKingdomInformation(kingMsg.getKingdomName(),kingMsg.getSubjectName(), kingMsg.getKingdomId(),kingMsg.getPlayerAccessList())){
								if(DatabaseSetters.updateZonesInKingdom(kingMsg.getKingdomId(),kingMsg.getStartZoneName(),kingMsg.getZoneList())){
									
									
									for(Zone z : kingMsg.getZoneList().values()){
										for(TravelTrigger trigger : z.getTravelTriggerList()){
											System.out.println(trigger.getTravelTriggerId()+"-"+trigger.getDestinationId());
											Zone destZone = kingMsg.getZoneList().get(trigger.getDestinationZoneName());
											if(destZone!=null){
												trigger.setDestinationId(destZone.getTravelTrigger(trigger.getDestinationId()+"").getTravelTriggerId());
											}else{
												trigger.setDestinationId(0);
											}
											if(trigger.getDestinationId()!=0){
												
												if(!DatabaseSetters.addTravelTriggerConnections(trigger.getTravelTriggerId(), trigger.getDestinationId())){
													sendIllegalOperationMsg("Could not create TravelTrigger connection"+trigger.getTravelTriggerId()+" to "+trigger.getDestinationId());
													break;
												}
											}
											
										}
									}
									sendSuccessfullKingdomSaved(kingMsg.getKingdomName());									
								}else{
									sendIllegalOperationMsg("Could not remove old zones from kingdom");
								}
							
							
							}else{
								sendIllegalOperationMsg("Could not update kingdom information.");
							}
						}
						
						
						break;
					
					case DATABASE_INSERT:
						DatabaseObjectMessage dbInsMsg = (DatabaseObjectMessage)messageFromEditor;
						switch(dbInsMsg.getObjectType()){
							case FRIENDLY_NPC:
								if(DatabaseSetters.insertFriendlyNPC((FriendlyNPC)dbInsMsg.getObject())!=0){
									sendSucessfullDatabaseOperationMsg("FriendlyNPC created",MessageType.SUCCESSFUL_DATABASE_OPERATION_FRIENDLYNPC);
								}else{
									sendIllegalOperationMsg("Could not insert FriendlyNPC");
								}
							break;
							
							case ENEMY_NPC:
								
								if(DatabaseSetters.insertEnemyNPC((EnemyNPC)dbInsMsg.getObject())!=0){
									sendSucessfullDatabaseOperationMsg("EnemyNPC created", MessageType.SUCCESSFUL_DATABASE_OPERATION_ENEMYNPC);
								}
								else{
									sendIllegalOperationMsg("Could not insert EnemyNPC");
								}
							break;
							case STATIC_OBJECT:
								if(DatabaseSetters.insertStaticObject((StaticObject)dbInsMsg.getObject())!=null){
									sendSucessfullDatabaseOperationMsg("StaticObject created", MessageType.SUCCESSFUL_DATABASE_OPERATION_STATICOBJECT);
								}
								else{
									sendIllegalOperationMsg("Could not insert StaticObject");
								}
								break;
							case ZONE_TEMPLATE:
								if(DatabaseSetters.insertZoneTemplate((Zone)dbInsMsg.getObject())!=0){
									sendSucessfullDatabaseOperationMsg("Zone template created", MessageType.SUCCESSFUL_DATABASE_OPERATION_ZONETEMPLATE);
								}else{
									sendIllegalOperationMsg("Could not insert zone template");
								}
								
							break;
							case QUESTION:
								Question question= (Question)dbInsMsg.getObject();
								boolean topicExists=false;
								for(String str: DatabaseGetters.GetTopics()){
									if(question.getQuestionTopic().equals(str)){
										topicExists = true;
									}
								}
								if(!topicExists){
									if(!DatabaseSetters.InsertTopic(question.getQuestionTopic())){
										sendIllegalOperationMsg("Could not insert Topic");
										break;
									}
								}
								int questionId = DatabaseSetters.InsertQuestion(question.getQuestionText(), question.getQuestionLevel(), question.getQuestionTopic(), question.getQuestionHeading());
								if(questionId==0){
									sendIllegalOperationMsg("Could not insert question");
									break;
								}
								for(Answer ans : question.getAnswers()){
									DatabaseSetters.InsertAnswer(ans.getAnswerText(), ans.isCorrectAnswer(), questionId);
								}
								sendSucessfullDatabaseOperationMsg("Question added", MessageType.SUCCESSFUL_DATABASE_OPERATION_QUESTION);
								
								
							break;
								
						}
						break;
					case DATABASE_ALTER:
						DatabaseObjectMessage dbAlterMsg = (DatabaseObjectMessage)messageFromEditor;
						switch(dbAlterMsg.getObjectType()){
						case FRIENDLY_NPC:
							if(DatabaseSetters.alterFriendlyNPC((FriendlyNPC)dbAlterMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("FriendlyNPC altered successfully", MessageType.SUCCESSFUL_DATABASE_OPERATION_FRIENDLYNPC);
							}else{
								sendIllegalOperationMsg("Could not change FriendlyNPC");
							}
							break;
						case ENEMY_NPC:
							if(DatabaseSetters.alterEnemyNPC((EnemyNPC)dbAlterMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("EnemyNPC altered successfully", MessageType.SUCCESSFUL_DATABASE_OPERATION_ENEMYNPC);
							}else{
								sendIllegalOperationMsg("Could not alter EnemyNPC");
							}
							break;
						case STATIC_OBJECT:
							if(DatabaseSetters.alterStaticObject((StaticObject)dbAlterMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("Static object altered successfully", MessageType.SUCCESSFUL_DATABASE_OPERATION_STATICOBJECT);
							}else{
								sendIllegalOperationMsg("Could not alter StaticObject");
							}
							break;
						case QUESTION:
							Question question= (Question)dbAlterMsg.getObject();
							boolean topicExists=false;
							for(String str: DatabaseGetters.GetTopics()){
								if(question.getQuestionTopic().equals(str)){
									topicExists = true;
								}
							}
							if(!topicExists){
								if(!DatabaseSetters.InsertTopic(question.getQuestionTopic())){
									sendIllegalOperationMsg("Could not insert Topic");
									break;
								}
							}
							if(DatabaseSetters.updateQuestion((Question)dbAlterMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("Question updated successfully", MessageType.SUCCESSFUL_DATABASE_OPERATION_QUESTION);
							}else{
								sendIllegalOperationMsg("Could not update Question");
							}
							
							
							break;
							
						}
						break;
						
					case DATABASE_DELETE:
						DatabaseObjectMessage dbDelMsg = (DatabaseObjectMessage)messageFromEditor;
						switch(dbDelMsg.getObjectType()){
						case FRIENDLY_NPC:
							if(DatabaseSetters.deleteFriendlyNPC((Integer)dbDelMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("FriendlyNPC was deleted", MessageType.SUCCESSFUL_DATABASE_OPERATION_FRIENDLYNPC);
							}else{
								sendIllegalOperationMsg("Could not delete FriendlyNPC");
							}
							break;
						case ENEMY_NPC:
							if(DatabaseSetters.deleteEnemyNPC((Integer)dbDelMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("EnemyNPC was deleted", MessageType.SUCCESSFUL_DATABASE_OPERATION_ENEMYNPC);
							}else{
								sendIllegalOperationMsg("Could not delete EnemyNPC");
							}
							break;
						case STATIC_OBJECT:
							if(DatabaseSetters.deleteStaticObject((Integer)dbDelMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("StaticObject was deleted", MessageType.SUCCESSFUL_DATABASE_OPERATION_STATICOBJECT);
							}else{
								sendIllegalOperationMsg("Could not delete StaticObject");
							}
							break;
						case QUESTION:
							if(DatabaseSetters.deleteQuestion((Integer)dbDelMsg.getObject())){
								sendSucessfullDatabaseOperationMsg("Question was deleted", MessageType.SUCCESSFUL_DATABASE_OPERATION_QUESTION);
							}else{
								sendIllegalOperationMsg("Could not delete Question");
							}
							break;
						}
						
						
						break;
					case OBJECT_MESSAGE:
						ObjectMessage objMsg = (ObjectMessage)messageFromEditor;
						switch(objMsg.getObjectType()){
							
						}
						
						break;
					case GET_ENEMY_NPC:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						characterName = sessionIdMessage.getCharacterName();
						EnemyNPC enpc = DatabaseGetters.GetEnemyNPC(characterName);
						if(enpc != null){
							messageToEditor = new ObjectMessage(myId, worldId, 
									"EnemyNPCs object returning",
									MessageType.OBJECT_MESSAGE ,
									enpc,
									ObjectType.ENEMY_NPC,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("EnemyNPC succesfully sent..");
						}
						else{
							messageToEditor = new Message(myId, worldId, 
									"Could not get enemyNPC : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Could not execute get enemyNPC");
						}
						break;
					case GET_ZONE_TEMPLATE:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						String zoneName = sessionIdMessage.getCharacterName();
						Zone template = DatabaseGetters.getZoneFromTemplate(zoneName);
						if(template!=null){
							ObjectMessage obMsg = new ObjectMessage(myId,worldId,
									"Returning template",MessageType.OBJECT_MESSAGE,
									template,ObjectType.ZONE_TEMPLATE,sessionId);
							send(obMsg);
							System.out.println("Template sent successfully");
						}else{
							sendIllegalOperationMsg("Could not get template");
						}
						
						
						break;
					case GET_KINGDOM:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						String kingdomName = sessionIdMessage.getCharacterName();
						KingdomInfo kingdom = DatabaseGetters.getKingdom(kingdomName);
						HashMap<String,Zone> zones = DatabaseGetters.getZones(kingdomName);
						ArrayList<String> playerAccess = DatabaseGetters.getPlayerNames(kingdom.getKingdomId());
						if(kingdom.getSubjectName()!=null&&kingdom.getKingdomId()!=0&&zones!=null&&kingdom.getStartZoneName()!=null){
							KingdomMessage returnKingMsg = new KingdomMessage(kingdom.getKingdomName(),
									kingdom.getSubjectName(),
									kingdom.getKingdomId(),
									zones,kingdom.getStartZoneName(),playerAccess,
									"Returning kingdom",myId,worldId);
							send(returnKingMsg);
							System.out.println("Kingdom succesfully sent..");
						}else{
							sendIllegalOperationMsg("Could not get kingdom");
						}
						
						break;
					case GET_ALL_STATIC_OBJECTS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						HashMap<String,StaticObject> sObjList = database.helperMethods.DatabaseGetters.GetAllStaticObjectAsAHashMap();
						if(sObjList!=null){
							messageToEditor = new ObjectMessage(myId, worldId,
									"StaticObjectList returning",
									MessageType.OBJECT_MESSAGE,
									sObjList,
									ObjectType.STATIC_OBJECT_LIST,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Staticobject list succesfully sent..");
						}
						else{
							sendIllegalOperationMsg("Could not get staticobject list");
						}
						break;
						
					case GET_ALL_TOPICS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<String> topicList = database.helperMethods.DatabaseGetters.GetTopics();
						if(topicList != null){
							messageToEditor = new ObjectMessage(myId, worldId, 
									"TopicList returning",
									MessageType.OBJECT_MESSAGE ,
									topicList,
									ObjectType.TOPIC_LIST,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Topic list succesfully sent..");
						}
						else{
							messageToEditor = new Message(myId, worldId, 
									"Could not get topicList : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.err.println("Could not get all Topics");
						}
						break;
					case GET_ALL_QUESTIONS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<Question> questionList = database.helperMethods.DatabaseGetters.getAllQuestionsFromDB();
						if(questionList != null){
							messageToEditor = new ObjectMessage(myId, worldId, 
									"QuestionList returning",
									MessageType.OBJECT_MESSAGE ,
									questionList,
									ObjectType.QUESTION_LIST,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Question list succesfully sent..");
						}
						else{
							messageToEditor = new Message(myId, worldId, 
									"Could not get questionList : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.err.println("Could not get all questions");
						}
						break;
					case GET_ALL_PHYSICAL_ATTACKS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<Attack> attackPList = database.helperMethods.DatabaseGetters.GetAllPhysicalAttacks();
						if(attackPList != null){
							messageToEditor = new ObjectMessage(myId,worldId,
									"Physical Attack list returning.",
									MessageType.OBJECT_MESSAGE,
									attackPList,
									ObjectType.PHYSICAL_ATTACK_LIST,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Physical attack list successfully sent.");
						}
						else{
							messageToEditor = new Message(myId,worldId,
									"Could not get attackList : ",
									MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.err.println("Could not get all attacks");
						}
						
						
						break;
					case GET_ALL_MAGICAL_ATTACKS:
						sessionIdMessage = (SessionIdMessage)messageFromEditor;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<Attack> attackMList = database.helperMethods.DatabaseGetters.GetAllMagicAttacks();
						if(attackMList != null){
							messageToEditor = new ObjectMessage(myId,worldId,
									"Magical Attack list returning.",
									MessageType.OBJECT_MESSAGE,
									attackMList,
									ObjectType.MAGICAL_ATTACK_LIST,
									sessionId);
							out.writeObject(messageToEditor);
							out.flush();
							System.out.println("Magical attack list successfully sent.");
						}
						else{
							messageToEditor = new Message(myId,worldId,
									"Could not get attackList : ",
									MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToEditor);
							out.flush();
							System.err.println("Could not get all attacks");
						}
						
						
						break;
					case CLOSE_CONNECTION:
							messageToEditor = new Message(myId, worldId, 
								"Connection is closed!", MessageType.CONNECTION_CLOSED);
							out.writeObject(messageToEditor);
							out.flush();
							//closing the socket and this thread
							in.close();
							out.close();
							socket.close();
							running = false;
							System.out.println("Connection closed");
						break;
						}
					}
					catch(ClassNotFoundException e){
						System.out.println("Class not found in DatabaseServerListener" + e.getMessage());
					}
				}
			}
			catch (IOException ioe) {
				System.out.println("Connection error in WorldServerListenerThread " +ioe.getMessage());
			}
	}
}