package database.databaseServer;

import java.awt.print.Printable;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;

import com.sun.org.apache.xml.internal.security.utils.HelperNodeList;

import database.databaseConnection.*;
import database.helperMethods.DatabaseGetters;
import database.helperMethods.DatabaseSetters;
import shared.network.messages.*;
import shared.models.*;
import shared.network.messages.Message;
import shared.serverHelpers.BanInfo;
import shared.serverHelpers.ShortCharacter;
/**
 * 
 * @author Esben Fllesdal
 * This thread is created for each connection from the lobbyserver
 */
public class LobbyServerConnectionThread extends Thread {

	private Socket socket = null;
	private boolean running = true;
		
	public LobbyServerConnectionThread(Socket socket){
		super("LobbyServerConnectionThread");
		this.socket  = socket;
	}
	
	/**
	 * When start is called on the thread, this method will be running
	 */
	public void run(){
		try {			
			ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
			ObjectOutputStream out = new ObjectOutputStream(socket.getOutputStream());
			
			while(running){
			try {
				Message messageFromLobby = (Message)in.readObject();
				//the message contains the id to the lobbyserver and databaseserver
				//we save this as a variable here.
				String myId = messageFromLobby.getTargetId();
				String lobbyId = messageFromLobby.getSourceId();
				Message messageToLobby;
				Connection databaseConnection;
				String username; 
				String password;
				String sessionId;
				String email;
				String oldUserName;
				String oldPassword;
				int playerId;
				String playerName;
				
				switch (messageFromLobby.getMessageType()) {
				case GET_PLAYER:
					//casts the message to a getPlayerMessage
					PlayerInformationMessage getPlayerMessage = (PlayerInformationMessage)messageFromLobby;
					username = getPlayerMessage.getUserName();
					password = getPlayerMessage.getPassword();
					sessionId = getPlayerMessage.getSessionId();
					Player player = DatabaseGetters.getPlayer(username, password);
					if(player != null){
						messageToLobby = new ObjectMessage(myId, lobbyId, "Player object returning",
								MessageType.OBJECT_MESSAGE ,player, ObjectType.PLAYER,
								sessionId);
						System.out.println("Player succesfully sent..");
						out.writeObject(messageToLobby);
						out.flush();
					}
					else{
						messageToLobby = new ObjectMessage(myId, lobbyId, "User identification ERROR",
								MessageType.OBJECT_MESSAGE, null, ObjectType.IDENTIFICATION_ERROR,
								sessionId);
						out.writeObject(messageToLobby);
						out.flush();
						System.out.println("Player could not be sent..");
					}
					break;
				
				case CREATE_PLAYER:
					//casts the message to a getPlayerMessage
					PlayerInformationMessage createPlayerMessage = 
											(PlayerInformationMessage)messageFromLobby;
					username = createPlayerMessage.getUserName();
					password = createPlayerMessage.getPassword();
					sessionId = createPlayerMessage.getSessionId();
					email = createPlayerMessage.getEmail();
					if(DatabaseSetters.insertPlayer(username, password, email)){
						messageToLobby = new Message(myId, lobbyId, "Player created successfully",
								MessageType.SUCCESSFUL_DATBASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Could not create player. " +
								"Username in use!", MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					break;
				
				case ALTER_PLAYER:
					//casts the message to a getPlayerMessage
					PlayerInformationMessage alterPlayerMessage = 
											(PlayerInformationMessage)messageFromLobby;
					username = alterPlayerMessage.getUserName();
					password = alterPlayerMessage.getPassword();
					sessionId = alterPlayerMessage.getSessionId();
					email = alterPlayerMessage.getEmail();
					oldUserName = alterPlayerMessage.getOldUserName();
					oldPassword = alterPlayerMessage.getOldPassword();
					//connects to the database
					if(DatabaseSetters.alterPlayer(oldUserName, oldPassword,
							username, email, password)){
						messageToLobby = new Message(myId, lobbyId, "Player altered successfully",
								MessageType.SUCCESSFUL_DATBASE_OPERATION);
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Can not alter player. " +
								"Wrong username or password or username in use",
								MessageType.ILLEGAL_DATABASE_OPERATION);
					}
					out.writeObject(messageToLobby);
					out.flush();
					break;
					
				case GET_BAN_LIST:
					//connects to the database
					HashMap<String, BanInfo> banhash = DatabaseGetters.getBanList();
					if(banhash != null){
						messageToLobby = new ObjectMessage(myId, lobbyId, "List over banned player",
								MessageType.OBJECT_MESSAGE, banhash,
								ObjectType.BANNED_PLAYER_LIST_HASH,null);
						out.writeObject(messageToLobby);
						out.flush();
						System.out.println("Ban list sent...");
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Could not get Ban linst",
								MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					break;
				
					
				case BAN_PLAYER:
					ObjectMessage banPlayer  = (ObjectMessage)messageFromLobby;
					BanInfo banPlayerInfo = (BanInfo)banPlayer.getObject();
					playerId = banPlayerInfo.getPlayerId();
					String reason = banPlayerInfo.getReason();
					Timestamp time = banPlayerInfo.getTimeStamp();
					if(DatabaseSetters.banPlayer(playerId, reason, time)){
						messageToLobby = new Message(myId, lobbyId, "Player banned!", 
								MessageType.SUCCESSFUL_DATBASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
						System.out.println(playerId + "has been banned");
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Could not ban player",
								MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					break;
					
				case UNBAN_PLAYER:
					ObjectMessage unBanPlayer  = (ObjectMessage)messageFromLobby;
					BanInfo unBanPlayerInfo = (BanInfo)unBanPlayer.getObject();
					playerId = unBanPlayerInfo.getPlayerId();
					if(DatabaseSetters.unbanPlayer(playerId)){
						messageToLobby = new Message(myId, lobbyId, "Player unbanned!",
								MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
						System.out.println(playerId+"has been unbanned");
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Could not unban player",
								MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					break;
					
				case REQUEST_SHORT_CHARACTERS:
					RequestShortCharacterListMessage request = 
										(RequestShortCharacterListMessage)messageFromLobby;
					sessionId = request.getSessionID();
					playerName = request.getUserName();
					myId = request.getTargetId();
					lobbyId = request.getSourceId();
					ArrayList<ShortCharacter> shortCharList = DatabaseGetters.getShortCharacter(playerName); 
					if(shortCharList != null){
						messageToLobby = new ObjectMessage(myId, lobbyId, "List over shortCharacters",
								MessageType.OBJECT_MESSAGE, shortCharList,
								ObjectType.SHORT_CHARACTER_LIST,sessionId);
						out.writeObject(messageToLobby);
						out.flush();
						System.out.println("short character list has been sent");
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Could not get short" +
								" character list",
						MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					break;
					
				case REQUEST_KINGDOM_ACCESS:
					RequestKingdomAccessMessage kingdomAccess = 
											(RequestKingdomAccessMessage)messageFromLobby;
					sessionId = kingdomAccess.getSessionID();
					playerName = kingdomAccess.getUserName();
					myId = kingdomAccess.getTargetId();
					lobbyId = kingdomAccess.getSourceId();
					HashMap<String, Boolean > kingdomHash = DatabaseGetters.getKingdomAccessList(playerName);
					if(kingdomHash != null){
						messageToLobby = new ObjectMessage(myId, lobbyId, 
								"List over accessableKingdoms",
								MessageType.OBJECT_MESSAGE, kingdomHash,
								ObjectType.KINGDOM_ACCESS_LIST,sessionId);
						out.writeObject(messageToLobby);
						out.flush();
						System.out.println("Kingdom access list has been sent");
						}
					else{
					messageToLobby = new Message(myId, lobbyId, "Could not get kingdomAccessList",
							MessageType.ILLEGAL_DATABASE_OPERATION);
					out.writeObject(messageToLobby);
					out.flush();
					}
					break;
					
				case NEW_PLAYABLE_CHARACTER:
					NewPlayableCharacterMessage playableCharacterMessage = 
								(NewPlayableCharacterMessage)messageFromLobby;
					sessionId = playableCharacterMessage.getSessionId();
					Integer playableCharacterId = DatabaseSetters.insertPlayableCharacter(
							playableCharacterMessage.getPlayableCharacter(), 
							playableCharacterMessage.getPlayerName(), 
							playableCharacterMessage.getKingdomName());
					if(playableCharacterId != null){
						messageToLobby = new  ObjectMessage(myId, lobbyId, 
								"New playyableCharacter created",
								MessageType.OBJECT_MESSAGE, playableCharacterId,
								ObjectType.INTEGER,sessionId);
						out.writeObject(messageToLobby);
						out.flush();
					}
					else{
						messageToLobby = new Message(myId, lobbyId, "Could not create new playableCharacters",
								MessageType.ILLEGAL_DATABASE_OPERATION);
						out.writeObject(messageToLobby);
						out.flush();
					}
					break;
					
				case CLOSE_CONNECTION:
					messageToLobby = new Message(myId, lobbyId, 
							"Connection is closed!", MessageType.CONNECTION_CLOSED);
					out.writeObject(messageToLobby);
					out.flush();
					//closing the socket and this thread
					in.close();
					out.close();
					socket.close();
					running = false;
					break;
				
				default:
					messageToLobby = new Message(myId, lobbyId, 
							"Unkown message type", MessageType.UNKNOWN_MESSAGE);
					out.writeObject(messageToLobby);
					out.flush();
					break;
				}
				
			} catch (ClassNotFoundException e) {
				System.out.println("Class not found in LobbyServerListener");
			}
			
			}
			
		} catch (IOException e) {
			System.out.println("IOException in LobbyServerListener: someone disconnected?");
		}	
	}
}