package database.databaseServer;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

import com.sun.org.apache.xml.internal.security.utils.HelperNodeList;

import database.databaseConnection.DBMethods;
import database.helperMethods.DatabaseGetters;
import database.helperMethods.KingdomInfo;

import shared.models.Attack;
import shared.models.AttackType;
import shared.models.Attributes;
import shared.models.Character;
import shared.models.Effect;
import shared.models.EnemyNPC;
import shared.models.Gender;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.PlayableClasses;
import shared.models.Question;
import shared.models.RespawnPoint;
import shared.models.Zone;
import shared.models.helpers.EnumHelpers;
import shared.network.messages.KingdomMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.SessionIdMessage;

/**
 * This class reperesents the listeneing thread for the worldListening server
 * @author Esben Fllesdal
 *
 */
public class WorldServerConnectionThread extends Thread {

	private Socket socket = null;
	private boolean running = true;
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private String myId;
	private String worldId;
	private String sessionId;
	public WorldServerConnectionThread(Socket socket){
		super("WorldServerConnectionThread");
		this.socket  = socket;
	}
	private void sendSucessfullDatabaseOperationMsg(String message, MessageType type){
		Message messageToEditor = new Message(myId, worldId, message,
				type);
		try {
			out.writeObject(messageToEditor);
			out.flush();
		}
		catch (IOException ioe) {
			System.out.println("Connection error in EditorListenerConnectionThread " +ioe.getMessage());
		}
	}
	private void sendIllegalOperationMsg(String message){
		Message messageToEditor = new Message(myId, worldId, 
				message, MessageType.ILLEGAL_DATABASE_OPERATION);
		try {
		out.writeObject(messageToEditor);
		out.flush();
		}
		catch (IOException ioe) {
			System.out.println("Connection error in EditorListenerConnectionThread " +ioe.getMessage());
		}
	}
	private void send(Message msg) throws IOException{
		out.writeObject(msg);
		out.flush();
	}
	public void run(){
			try {			
				in = new ObjectInputStream(socket.getInputStream());
				out = new ObjectOutputStream(socket.getOutputStream());
				
				while(running){
					try{
						Message messageFromWorldServer = (Message)in.readObject();
						Message messageToWorldServer;
						myId = messageFromWorldServer.getTargetId();
						worldId = messageFromWorldServer.getSourceId();
						SessionIdMessage sessionIdMessage;
						String characterName;
												
					switch (messageFromWorldServer.getMessageType()) {
								
					case GET_PLAYABLE_CHARACTER:
						sessionIdMessage = (SessionIdMessage)messageFromWorldServer;
						sessionId = sessionIdMessage.getSessionID();
						characterName = sessionIdMessage.getCharacterName();
						PlayableCharacter pc = DatabaseGetters.GetPlayableCharcter(characterName);	
						if(pc != null){	
						messageToWorldServer = new ObjectMessage(myId, worldId, 
									"Character object returning",
									MessageType.OBJECT_MESSAGE ,
									pc,
									ObjectType.CHARACTER_OBJECT,
									sessionId);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("Character succesfully sent..");
						}
						else{
							messageToWorldServer = new Message(myId, worldId, "Could not get character : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("Could not execute get character");
						}
						break;
					
					case GET_ALL_ENEMY_NPCS:
						sessionIdMessage = (SessionIdMessage)messageFromWorldServer;
						sessionId = sessionIdMessage.getSessionID();
						HashMap<String, EnemyNPC> enpcs = DatabaseGetters.GetAllEnemyNPCAsAHashMap();
						if(enpcs != null){
							messageToWorldServer = new ObjectMessage(myId, worldId, 
									"EnemyNPCs object returning",
									MessageType.OBJECT_MESSAGE ,
									enpcs,
									ObjectType.ENEMY_NPC_HASHMAP,
									sessionId);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("All enemyNPCs succesfully sent..");
						}
						else {
							messageToWorldServer = new Message(myId, worldId, 
									"Could not get all enemyNPCs : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("Could not execute get all enemyNPCs");
						}
						break;
					case GET_ALL_KINGDOMNAMES:
						sessionIdMessage = (SessionIdMessage)messageFromWorldServer;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<String> kings = DatabaseGetters.GetAllKingdomNames();
						if(kings!=null){
							messageToWorldServer = new ObjectMessage(myId,worldId,
									"KingdomNames returning",
									MessageType.OBJECT_MESSAGE,
									kings,
									ObjectType.KINGDOM_NAME_LIST,
									sessionId);
							send(messageToWorldServer);
							System.out.println("KingdomNames sent");
							
						}
						else{
							sendIllegalOperationMsg("Could not get kingdom name list");
						}
						break;
					case GET_KINGDOM:
						sessionIdMessage = (SessionIdMessage)messageFromWorldServer;
						sessionId = sessionIdMessage.getSessionID();
						String kingdomName = sessionIdMessage.getCharacterName();
						KingdomInfo kingdom = DatabaseGetters.getKingdom(kingdomName);
						HashMap<String,Zone> zones = DatabaseGetters.getZones(kingdomName);
						if(kingdom.getSubjectName()!=null&&kingdom.getKingdomId()!=0&&zones!=null&&kingdom.getStartZoneName()!=null){
							KingdomMessage returnKingMsg = new KingdomMessage(kingdom.getKingdomName(),
									kingdom.getSubjectName(),
									kingdom.getKingdomId(),
									zones,kingdom.getStartZoneName(),new ArrayList<String>(),
									"Returning kingdom",myId,worldId);
							send(returnKingMsg);
							System.out.println("Kingdom succesfully sent..");
						}else{
							sendIllegalOperationMsg("Could not get kingdom");
						}
						
						break;
					case GET_ENEMY_NPC:
						sessionIdMessage = (SessionIdMessage)messageFromWorldServer;
						sessionId = sessionIdMessage.getSessionID();
						characterName = sessionIdMessage.getCharacterName();
						EnemyNPC enpc = DatabaseGetters.GetEnemyNPC(characterName);
						if(enpc != null){
							messageToWorldServer = new ObjectMessage(myId, worldId, 
									"EnemyNPCs object returning",
									MessageType.OBJECT_MESSAGE ,
									enpc,
									ObjectType.ENEMY_NPC,
									sessionId);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("EnemyNPC succesfully sent..");
						}
						else{
							messageToWorldServer = new Message(myId, worldId, 
									"Could not get enemyNPC : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("Could not execute get enemyNPC");
						}
						break;
					case GET_ALL_QUESTIONS:
						sessionIdMessage = (SessionIdMessage)messageFromWorldServer;
						sessionId = sessionIdMessage.getSessionID();
						ArrayList<Question> questionList = database.helperMethods.DatabaseGetters.getAllQuestionsFromDB();
						if(questionList != null){
							messageToWorldServer = new ObjectMessage(myId, worldId, 
									"QuestionList returning",
									MessageType.OBJECT_MESSAGE ,
									questionList,
									ObjectType.QUESTION_LIST,
									sessionId);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.out.println("Question list succesfully sent..");
						}
						else{
							messageToWorldServer = new Message(myId, worldId, 
									"Could not get questionList : ",
									 MessageType.ILLEGAL_DATABASE_OPERATION);
							out.writeObject(messageToWorldServer);
							out.flush();
							System.err.println("Could not get all questions");
						}
						break;
						
					case CLOSE_CONNECTION:
							messageToWorldServer = new Message(myId, worldId, 
								"Connection is closed!", MessageType.CONNECTION_CLOSED);
							out.writeObject(messageToWorldServer);
							out.flush();
							//closing the socket and this thread
							in.close();
							out.close();
							socket.close();
							running = false;
							System.out.println("Connection closed");
						break;
						}
					}
					catch(ClassNotFoundException e){
						System.out.println("Class not found in DatabaseServerListener" + e.getMessage());
					}
				}
			}
			catch (IOException ioe) {
				System.out.println("Connection error in WorldServerListenerThread " +ioe.getMessage());
			}
	}
}