package database.helperMethods;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;

import database.databaseConnection.DBMethods;
import shared.fileHandler.ReadBackgroundFromFile;
import shared.models.Answer;
import shared.models.Armour;
import shared.models.Attack;
import shared.models.AttackType;
import shared.models.Attributes;
import shared.models.Effect;
import shared.models.EnemyNPC;
import shared.models.EquipLocation;
import shared.models.FriendlyNPC;
import shared.models.Gender;
import shared.models.Guild;
import shared.models.Item;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.PlayableClasses;
import shared.models.Player;
import shared.models.QuestJournal;
import shared.models.Question;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.UseableItem;
import shared.models.Weapon;
import shared.models.Zone;
import shared.models.helpers.EnumHelpers;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.serverHelpers.BanInfo;
import shared.serverHelpers.ShortCharacter;
import sun.font.LayoutPathImpl.EndType;

/**
 * This class contains the logic to extract information from 
 * the database 
 * @author Esben Fllesdal
 */
public class DatabaseGetters {
	
	/**
	 * @param characterName -the playablecharacter name
	 * @param location - the equiplocation
	 * @return - the item on the location
	 */
	public static Item getEquippedItemOnLocation(String characterName, EquipLocation location){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetArmourInInventory",
				"\"CharacterName\" = ?  AND \"Inventory_EquipLocation\" = ?", dbConnection);
		try {
			prpState.setString(1, characterName);
			prpState.setString(2, location.toString());
			ResultSet rsArmour = prpState.executeQuery();
			Armour armour = null;
			if(rsArmour.next()){
				//adds an armour to the inventoryList
				//-----------------------------------------------------------
				//checks if one of the statsId contains null or a real number
				Integer armourOffensiveStatsId = rsArmour.getInt("OffensiveStatsId");
				if(armourOffensiveStatsId == null) armourOffensiveStatsId = 0;
				Integer armourDefensiveStatsId = rsArmour.getInt("DefensiveStatsId");
				if(armourDefensiveStatsId == null) armourDefensiveStatsId = 0;
				Integer armourPrimaryStatsId = rsArmour.getInt("PrimaryStatsId");
				if(armourPrimaryStatsId == null) armourPrimaryStatsId = 0;
				Integer armourReqPrimaryStatsId = rsArmour.getInt("RequiredStatsId");
				if(armourReqPrimaryStatsId == null)armourPrimaryStatsId = 0;
				//------------------------------------------------------------
					//only the armour in the backPack should be added to the
					//inventorylist
					armour = new Armour(
						rsArmour.getInt("ItemId"),
						rsArmour.getString("ItemName"),
						GetAttributeFromDB(armourPrimaryStatsId, armourOffensiveStatsId,
								armourDefensiveStatsId),
						rsArmour.getInt("Range"),
						rsArmour.getInt("Price"),
						GetAttributeFromDB(armourPrimaryStatsId, 0, 0),
						GetEffectFromDB(rsArmour.getInt("EffectId")),
						rsArmour.getString("ImagePath"),
						shared.models.helpers.EnumHelpers.
							GetEquipLocation(rsArmour.getString("LegalEquipLocation"))
						);
			
			System.out.println(armour.getName() + " added to " + location);
			return armour;
			}
			
			prpState  = DBMethods.SelectQuery("*", "GetWeaponsInInventory",
					"\"CharacterName\" = ?  AND \"EquipLocation\" = ?", dbConnection);
			prpState.setString(1, characterName);
			prpState.setString(2, location.toString());
			ResultSet rsWeapon = prpState.executeQuery();
			Weapon weapon = null;
			if(rsWeapon.next()){
				//adds a weapon to the inventoryList
				//-----------------------------------------------------------
				//checks if one of the statsId contains null or a real number
				Integer weaponOffensiveStatsId = rsWeapon.getInt("OffensiveStatsId");
				if(weaponOffensiveStatsId == null) weaponOffensiveStatsId = 0;
				Integer weaponDefensiveStatsId = rsWeapon.getInt("DefensiveStatsId");
				if(weaponDefensiveStatsId == null) weaponDefensiveStatsId = 0;
				Integer weaponPrimaryStatsId = rsWeapon.getInt("PrimaryStatsId");
				if(weaponPrimaryStatsId == null) weaponPrimaryStatsId = 0;
				Integer weaponReqPrimaryStatsId = rsWeapon.getInt("RequiredStatsId");
				if(weaponReqPrimaryStatsId == null)weaponReqPrimaryStatsId = 0;
				//------------------------------------------------------------
					//only the weapons in the backPack should be added to the
					//inventorylist
					weapon = new Weapon(
							rsWeapon.getInt("ItemId"),
							rsWeapon.getString("ItemName"),
							GetAttributeFromDB(weaponReqPrimaryStatsId, weaponOffensiveStatsId,
									weaponDefensiveStatsId),
							rsWeapon.getInt("Range"),
							rsWeapon.getInt("Price"),
							GetAttributeFromDB(weaponPrimaryStatsId, 0, 0),
							GetEffectFromDB(rsWeapon.getInt("EffectId")),
							shared.models.helpers.EnumHelpers.
									GetWeaponType(rsWeapon.getString("WeaponType")),
							rsWeapon.getBoolean("TwoHand"),
							rsWeapon.getInt("Damage"),
							rsWeapon.getString("ImagePath"),
							shared.models.helpers.EnumHelpers.
								GetEquipLocation(rsWeapon.getString("LegalEquipLocation"))
							);
					System.out.println(weapon.getName() + " added to" + location);
					return weapon;
			}
			
			prpState  = DBMethods.SelectQuery("*", "GetUseableItemInInventory",
					"\"CharacterName\" = ?   AND \"EquipLocation\" = ?", dbConnection);
			prpState.setString(1, characterName);
			prpState.setString(2, location.toString());
			ResultSet rsUseableItem = prpState.executeQuery();
			UseableItem useableItem = null;
			if(rsUseableItem.next()){
				//adds an UseableItem to the inventoryList
				//-----------------------------------------------------------
				//checks if one of the statsId contains null or a real number
				Integer useableItemOffensiveStatsId = rsUseableItem.getInt("OffensiveStatsId");
				if(useableItemOffensiveStatsId == null) useableItemOffensiveStatsId = 0;
				Integer useableItemDefensiveStatsId = rsUseableItem.getInt("DefensiveStatsId");
				if(useableItemDefensiveStatsId == null) useableItemDefensiveStatsId = 0;
				Integer useableItemPrimaryStatsId = rsUseableItem.getInt("PrimaryStatsId");
				if(useableItemPrimaryStatsId == null) useableItemPrimaryStatsId = 0;
				Integer useableItemReqPrimaryStatsId = rsUseableItem.getInt("RequiredStatsId");
				if(useableItemReqPrimaryStatsId == null) useableItemReqPrimaryStatsId = 0;
				//------------------------------------------------------------
						//only the items in the backPack should be added to the
						//inventorylist
					useableItem = new UseableItem(
							rsUseableItem.getInt("ItemId"),
							rsUseableItem.getString("ItemName"),
							GetAttributeFromDB(useableItemPrimaryStatsId, useableItemOffensiveStatsId,
									useableItemDefensiveStatsId),
					    	rsUseableItem.getInt("Range"),
							rsUseableItem.getInt("Price"),
							GetAttributeFromDB(useableItemReqPrimaryStatsId, 0, 0),
							GetEffectFromDB(rsUseableItem.getInt("EffectId")),
							shared.models.helpers.EnumHelpers.
									GetUseableItemType(rsUseableItem.getString("UseableItemType")),
							rsUseableItem.getString("ImagePath"),
							shared.models.helpers.EnumHelpers.
								GetEquipLocation(rsUseableItem.getString("LegalEquipLocation"))
							);
					System.out.println(useableItem.getName() + " added to " + location);
					return useableItem;
				}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
		}
			catch (SQLException sqle) {
				DBMethods.getInstance().DisconnectFromDB(dbConnection);
				System.err.println("Could not get equipped item" + sqle.getMessage());
				sqle.printStackTrace();
			}
			return null;
	}
	
	/**
	 * Gets the inventory to a playable character
	 * @param characterName - the characterName to which extract the inventoryList
	 * @return an arraylist of Items, which belongs to a playableCharacter
	 * @see Item
	 * @see Weapon
	 * @see UseableItem
	 * @see Armour
	 * @see DBMethods
	 */
	public static ArrayList<Item> GetInventory(String characterName){
		ArrayList<Item> inventoryList = new ArrayList<Item>();
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetArmourInInventory",
				"\"CharacterName\" = ? ", dbConnection);
		try {
			prpState.setString(1, characterName);
			ResultSet rsArmour = prpState.executeQuery();
			Armour armour = null;
			while(rsArmour.next()){
				//adds an armour to the inventoryList
				//-----------------------------------------------------------
				//checks if one of the statsId contains null or a real number
				Integer armourOffensiveStatsId = rsArmour.getInt("OffensiveStatsId");
				if(armourOffensiveStatsId == null) armourOffensiveStatsId = 0;
				Integer armourDefensiveStatsId = rsArmour.getInt("DefensiveStatsId");
				if(armourDefensiveStatsId == null) armourDefensiveStatsId = 0;
				Integer armourPrimaryStatsId = rsArmour.getInt("PrimaryStatsId");
				if(armourPrimaryStatsId == null) armourPrimaryStatsId = 0;
				Integer armourReqPrimaryStatsId = rsArmour.getInt("RequiredStatsId");
				if(armourReqPrimaryStatsId == null)armourPrimaryStatsId = 0;
				//------------------------------------------------------------
				if(rsArmour.getString("Inventory_EquipLocation").equals("BACKPACK")){
					//only the armour in the backPack should be added to the
					//inventorylist
					armour = new Armour(
						rsArmour.getInt("ItemId"),
						rsArmour.getString("ItemName"),
						GetAttributeFromDB(armourPrimaryStatsId, armourOffensiveStatsId,
								armourDefensiveStatsId),
						rsArmour.getInt("Range"),
						rsArmour.getInt("Price"),
						GetAttributeFromDB(armourPrimaryStatsId, 0, 0),
						GetEffectFromDB(rsArmour.getInt("EffectId")),
						rsArmour.getString("ImagePath"),
						shared.models.helpers.EnumHelpers.
							GetEquipLocation(rsArmour.getString("LegalEquipLocation"))
						);
				inventoryList.add(armour);
				System.out.println(armour.getName() + " added to inventory");
				}
			}
			
			prpState  = DBMethods.SelectQuery("*", "GetWeaponsInInventory",
					"\"CharacterName\" = ? ", dbConnection);
			prpState.setString(1, characterName);
			ResultSet rsWeapon = prpState.executeQuery();
			Weapon weapon = null;
			while(rsWeapon.next()){
				//adds a weapon to the inventoryList
				//-----------------------------------------------------------
				//checks if one of the statsId contains null or a real number
				Integer weaponOffensiveStatsId = rsWeapon.getInt("OffensiveStatsId");
				if(weaponOffensiveStatsId == null) weaponOffensiveStatsId = 0;
				Integer weaponDefensiveStatsId = rsWeapon.getInt("DefensiveStatsId");
				if(weaponDefensiveStatsId == null) weaponDefensiveStatsId = 0;
				Integer weaponPrimaryStatsId = rsWeapon.getInt("PrimaryStatsId");
				if(weaponPrimaryStatsId == null) weaponPrimaryStatsId = 0;
				Integer weaponReqPrimaryStatsId = rsWeapon.getInt("RequiredStatsId");
				if(weaponReqPrimaryStatsId == null)weaponReqPrimaryStatsId = 0;
				//------------------------------------------------------------
				if(rsWeapon.getString("EquipLocation").equals("BACKPACK")){
					//only the weapons in the backPack should be added to the
					//inventorylist
					weapon = new Weapon(
							rsWeapon.getInt("ItemId"),
							rsWeapon.getString("ItemName"),
							GetAttributeFromDB(weaponReqPrimaryStatsId, weaponOffensiveStatsId,
									weaponDefensiveStatsId),
							rsWeapon.getInt("Range"),
							rsWeapon.getInt("Price"),
							GetAttributeFromDB(weaponPrimaryStatsId, 0, 0),
							GetEffectFromDB(rsWeapon.getInt("EffectId")),
							shared.models.helpers.EnumHelpers.
									GetWeaponType(rsWeapon.getString("WeaponType")),
							rsWeapon.getBoolean("TwoHand"),
							rsWeapon.getInt("Damage"),
							rsWeapon.getString("ImagePath"),
							shared.models.helpers.EnumHelpers.
								GetEquipLocation(rsWeapon.getString("LegalEquipLocation"))
							);
//					weapon.setEquipLocation(EnumHelpers.GetEquipLocation(
//								rsWeapon.getString("EquipLocation")));
					inventoryList.add(weapon);
					System.out.println(weapon.getName() + " added to inventory");
				}
			}
			
			prpState  = DBMethods.SelectQuery("*", "GetUseableItemInInventory",
					"\"CharacterName\" = ? ", dbConnection);
			prpState.setString(1, characterName);
			ResultSet rsUseableItem = prpState.executeQuery();
			UseableItem useableItem = null;
			while(rsUseableItem.next()){
				//adds an UseableItem to the inventoryList
				//-----------------------------------------------------------
				//checks if one of the statsId contains null or a real number
				Integer useableItemOffensiveStatsId = rsUseableItem.getInt("OffensiveStatsId");
				if(useableItemOffensiveStatsId == null) useableItemOffensiveStatsId = 0;
				Integer useableItemDefensiveStatsId = rsUseableItem.getInt("DefensiveStatsId");
				if(useableItemDefensiveStatsId == null) useableItemDefensiveStatsId = 0;
				Integer useableItemPrimaryStatsId = rsUseableItem.getInt("PrimaryStatsId");
				if(useableItemPrimaryStatsId == null) useableItemPrimaryStatsId = 0;
				Integer useableItemReqPrimaryStatsId = rsUseableItem.getInt("RequiredStatsId");
				if(useableItemReqPrimaryStatsId == null) useableItemReqPrimaryStatsId = 0;
				//------------------------------------------------------------
				if(rsUseableItem.getString("EquipLocation").equals("BACKPACK")){
						//only the items in the backPack should be added to the
						//inventorylist
					useableItem = new UseableItem(
							rsUseableItem.getInt("ItemId"),
							rsUseableItem.getString("ItemName"),
							GetAttributeFromDB(useableItemPrimaryStatsId, useableItemOffensiveStatsId,
									useableItemDefensiveStatsId),
					    	rsUseableItem.getInt("Range"),
							rsUseableItem.getInt("Price"),
							GetAttributeFromDB(useableItemReqPrimaryStatsId, 0, 0),
							GetEffectFromDB(rsUseableItem.getInt("EffectId")),
							shared.models.helpers.EnumHelpers.
									GetUseableItemType(rsUseableItem.getString("UseableItemType")),
							rsUseableItem.getString("ImagePath"),
							shared.models.helpers.EnumHelpers.
								GetEquipLocation(rsUseableItem.getString("LegalEquipLocation"))
							);
//					useableItem.setEquipLocation(EnumHelpers.GetEquipLocation(
//							rsUseableItem.getString("EquipLocation")));
					inventoryList.add(useableItem);
					System.out.println(useableItem.getName() + " added to inventory");
				}
				}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
		} catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get inventoryList" + sqle.getMessage());
			sqle.printStackTrace();
		}
		return inventoryList;
	}
	
	/**
	 * Gets an attribute object, based on the input ids.
	 * @param primaryStatsId
	 * @param offensiveStatsId
	 * @param defensiveStatsId
	 * @return an attribute object
	 * @see Attributes
	 */
	public static Attributes GetAttributeFromDB(int primaryStatsId, int offensiveStatsId, 
								int defensiveStatsId){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "PrimaryStats",
				"\"PrimaryStatsId\" = ? ", dbConnection);	
		Attributes att = new Attributes();
		try {
			if(primaryStatsId != 0){
				prpState.setInt(1, primaryStatsId);
				ResultSet rsPrimay = prpState.executeQuery();
				rsPrimay.next();
				att.setHP(rsPrimay.getInt("HP"));
				att.setMP(rsPrimay.getInt("MP"));
				att.setSTR(rsPrimay.getInt("STR"));
				att.setINT(rsPrimay.getInt("INT"));
				att.setDEX(rsPrimay.getInt("DEX"));
				att.setCON(rsPrimay.getInt("CON"));
				att.setMaxHP(rsPrimay.getInt("MaxHP"));
				att.setMaxMP(rsPrimay.getInt("MaxMp"));
			}
			else{//if there is no attributes, set it to zero
				att.setHP(0);
				att.setMP(0);
				att.setSTR(0);
				att.setINT(0);
				att.setDEX(0);
				att.setCON(0);
				att.setMaxHP(0);
				att.setMaxMP(0);
			}
			
			if(defensiveStatsId != 0){
				prpState = DBMethods.SelectQuery("*", "DefensiveStats",
						"\"DefensiveStatsId\" = ? ", dbConnection);
				prpState.setInt(1, defensiveStatsId);
				ResultSet rsDefense = prpState.executeQuery();
				rsDefense.next();
				att.setPiercingDef(rsDefense.getInt("PiercingDef"));
				att.setBluntDef(rsDefense.getInt("BluntDef"));
				att.setSlashingDef(rsDefense.getInt("SlashingDef"));
				att.setFireDef(rsDefense.getInt("FireDef"));
				att.setWaterDef(rsDefense.getInt("WaterDef"));
				att.setAirDef(rsDefense.getInt("AirDef"));
				att.setEarthDef(rsDefense.getInt("EarthDef"));
			}
			else{//if there is no attributes, set it to zero
				att.setPiercingDef(0);
				att.setBluntDef(0);
				att.setSlashingDef(0);
				att.setFireDef(0);
				att.setWaterDef(0);
				att.setAirDef(0);
				att.setEarthDef(0);
			}
			if(offensiveStatsId != 0){
				prpState = DBMethods.SelectQuery("*", "OffensiveStats",
						"\"OffensiveStatsId\" = ? ", dbConnection);
				prpState.setInt(1, offensiveStatsId);
				ResultSet rsOffense = prpState.executeQuery();
				rsOffense.next();
				att.setPiercingOff(rsOffense.getInt("PiercingOff"));
				att.setBluntOff(rsOffense.getInt("BluntOff"));
				att.setSlashingOff(rsOffense.getInt("SlashingOff"));
				att.setFireOff(rsOffense.getInt("FireOff"));
				att.setWaterOff(rsOffense.getInt("WaterOff"));
				att.setAirOff(rsOffense.getInt("AirOff"));
				att.setEarthOff(rsOffense.getInt("EarthOff"));
			}
			else{//if there is no attributes, set it to zero
				att.setPiercingDef(0);
				att.setBluntOff(0);
				att.setSlashingOff(0);
				att.setFireOff(0);
				att.setWaterOff(0);
				att.setAirOff(0);
				att.setEarthOff(0);
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
		} 
		catch (SQLException sqle) {
		DBMethods.getInstance().DisconnectFromDB(dbConnection);
		System.err.println("Could not get attribute" + sqle.getMessage());
		}
		return att;
	}
	
	/**
	 * gets and returns an effectObject from the database
	 * @param effectId
	 * @return an effectObject
	 */
	public static Effect GetEffectFromDB(Integer effectId){
		Effect effect = null;
		//if there is no object references in the database
		if(effect == null) return effect;
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "Effect",
				"\"EffectId\" = ? ", dbConnection);
		try{
			prpState.setInt(1, effectId);
			ResultSet rsEffect = prpState.executeQuery();
			rsEffect.next();
			//checks if one of the statsId contains null or a real number
			Integer effectOffensiveStatsId = rsEffect.getInt("OffensiveStatsId");
			if(effectOffensiveStatsId == null) effectOffensiveStatsId = 0;
			Integer effectDefensiveStatsId = rsEffect.getInt("DefensiveStatsId");
			if(effectDefensiveStatsId == null) effectDefensiveStatsId = 0;
			Integer effectPrimaryStatsId = rsEffect.getInt("PrimaryStatsId");
			if(effectPrimaryStatsId == null) effectPrimaryStatsId = 0;
			//------------------------------------------------------------
			effect = new Effect(
			rsEffect.getString("Name"),
			rsEffect.getInt("Duration"),
			GetAttributeFromDB(effectPrimaryStatsId, effectOffensiveStatsId, effectDefensiveStatsId),
			rsEffect.getInt("DamagePrSec")
			);
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get effect" + sqle.getMessage());
		}
		return effect;
	}
	
	/**
	 * Gets a question object from the database
	 * @param questionId -the id to a question
	 * @return a question object 
	 */
	public static Question GetQuestionFromDB(int questionId){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetQuestion", 
				"\"QuestionId\" = ? ",dbConnection);
		try{
			prpState.setInt(1, questionId);
			ResultSet rsQuestions = prpState.executeQuery();
			ArrayList<Answer> listOfAnswers = new ArrayList<Answer>();
			Answer answer = null;
			int id = 0;
			String questionText = "";
			String questionHeader = "";
			String questionTopic = "";
			int questionLevel = 1;
			int timeLimit = 0;
			while(rsQuestions.next()){
				answer = new Answer(rsQuestions.getInt("AnswerId"),
									rsQuestions.getBoolean("CorrectAnswer"),
									rsQuestions.getString("AnswerText"));
				listOfAnswers.add(answer);
				//we need the question information. This information is available
				//for every answer. But we only need it once. Thats why this if.
				if(id == 0){
					id= rsQuestions.getInt("QuestionId");
					questionText = rsQuestions.getString("QuestionText");
					questionHeader = rsQuestions.getString("QuestionHeader");
					questionTopic = rsQuestions.getString("TopicTheme");
					questionLevel = rsQuestions.getInt("DifficultyLevel");
					timeLimit = rsQuestions.getInt("TimeLimit");
				}
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			Question question = new Question(
						id,
						listOfAnswers,
						questionText,
						questionHeader,
						questionTopic,
						questionLevel,
						timeLimit
						);
			return question;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get the Questions from the DB" + sqle.getMessage());
		}
		return null;
	}
	
	/**
	 * This will return all the questionIds in the database
	 * @return - all the questionIds in the db
	 */
	public static ArrayList<Integer> GetAllQuestionIDs(){
		ArrayList<Integer> idList = new ArrayList<Integer>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"QuestionId\"", "Question", 
														dbConnection);
		try{
			ResultSet rsQuestionId = prpState.executeQuery();
			while(rsQuestionId.next()){
				idList.add(rsQuestionId.getInt("QuestionId"));
				
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return idList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get the Question IDs from the DB" + sqle.getMessage());
		}
		return null;
	}
	
	/**
	 * Will get all the questions in the database
	 * @return - an arraylist with all the questions
	 * @see Question 
	 */ 
	public static ArrayList<Question> getAllQuestionsFromDB(){
		ArrayList<Question> questionList  = new ArrayList<Question>();
		ArrayList<Integer> idList = GetAllQuestionIDs();
		for(Integer i : idList){
			questionList.add(GetQuestionFromDB(i));
		}
		return questionList;
 	}
	
	/**
	 * 
	 * @return all the topics in the database
	 */
	public static ArrayList<String> GetTopics(){
		ArrayList<String> topicList = new ArrayList<String>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"TopicTheme\"", "TopicTheme", 
														dbConnection);
		try{
			ResultSet rsTopic = prpState.executeQuery();
			while(rsTopic.next()){
				topicList.add(rsTopic.getString("TopicTheme"));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return topicList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get the topic theme from the DB" + sqle.getMessage());
		}
		return null;
	}
	/**
	 * return all zone templates from database
	 */
	public static String[] GetAllZoneTemplateNames(){
		ArrayList<String> names = new ArrayList<String>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"Name\"", "Zones", "\"KingdomId\" IS NULL", 
														dbConnection);
		
		
		try{
			ResultSet rsName = prpState.executeQuery();
			while(rsName.next()){
				names.add(rsName.getString("Name"));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			
			String[] templateNames = new String[names.size()];
			for(int i = 0 ; i<names.size();i++){
				templateNames[i] = names.get(i);
			}
			return templateNames;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get the topic theme from the DB" + sqle.getMessage());
		}
		return null;
		
	}
	/**
	 * returns a enemyNPC based on the characterName
	 * @param characterName
	 * @return an enemyNPCobject
	 * @see EnemyNPC
	 */
	public static EnemyNPC GetEnemyNPC(int characterId){
		EnemyNPC npc = null;
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetEnemyNPC", 
				"\"EnemyNPCId\" =?",dbConnection);
		try{
			prpState.setInt(1, characterId);
			ResultSet rsNPC = prpState.executeQuery();
			if(rsNPC.next()){
				Attributes att = GetAttributeFromDB(rsNPC.getInt("PrimaryStatsId"), 
						rsNPC.getInt("OffensiveStatsId"), 
						rsNPC.getInt("DefensiveStatsId"));
				npc = new EnemyNPC(rsNPC.getString("CharacterName"),
						shared.models.helpers.EnumHelpers.GetGender(rsNPC.getString("Sex")),
						rsNPC.getInt("LVL"),
						att,
						rsNPC.getInt("Agressive"),
						rsNPC.getInt("SleepTime"),
						rsNPC.getInt("XP"),
						GetPhysicalAttacks(rsNPC.getString("CharacterName")),
						GetMagicAttacks(rsNPC.getString("CharacterName")),
						rsNPC.getString("ImagePath")
						);
				npc.setEnemyNPCId(rsNPC.getInt("EnemyNPCId"));
				return npc;
			}
			return null;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get enemyNPC from the DB:" + sqle.getMessage());
		}
		return null;
	}
	/**
	 * returns a enemyNPC based on the characterName
	 * @param characterName
	 * @return an enemyNPCobject
	 * @see EnemyNPC
	 */
	public static EnemyNPC GetEnemyNPC(String characterName){
		EnemyNPC npc = null;
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetEnemyNPC", 
				"\"CharacterName\" =?",dbConnection);
		try{
			prpState.setString(1, characterName);
			ResultSet rsNPC = prpState.executeQuery();
			if(rsNPC.next()){
				Attributes att = GetAttributeFromDB(rsNPC.getInt("PrimaryStatsId"), 
						rsNPC.getInt("OffensiveStatsId"), 
						rsNPC.getInt("DefensiveStatsId"));
				npc = new EnemyNPC(rsNPC.getString("CharacterName"),
						shared.models.helpers.EnumHelpers.GetGender(rsNPC.getString("Sex")),
						rsNPC.getInt("LVL"),
						att,
						rsNPC.getInt("Agressive"),
						rsNPC.getInt("SleepTime"),
						rsNPC.getInt("XP"),
						GetPhysicalAttacks(characterName),
						GetMagicAttacks(characterName),
						rsNPC.getString("ImagePath")
						);
				npc.setEnemyNPCId(rsNPC.getInt("EnemyNPCId"));
				return npc;
			}
			return null;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get enemyNPC from the DB:" + sqle.getMessage());
		}
		return null;
	}
	/**
	 * returns a friendlyNPC based on the characterName
	 * @param characterName
	 * @return an enemyNPCobject
	 * @see friendlyNPC
	 */
	public static FriendlyNPC GetFriendlyNPC(int characterId){
		FriendlyNPC npc = null;
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetFriendlyNPC", 
				"\"FriendlyNPCId\" =?",dbConnection);
		try{
			prpState.setInt(1, characterId);
			ResultSet rsNPC = prpState.executeQuery();
			if(rsNPC.next()){
				Attributes att = GetAttributeFromDB(rsNPC.getInt("PrimaryStatsId"), 
						rsNPC.getInt("OffensiveStatsId"), 
						rsNPC.getInt("DefensiveStatsId"));
				//TODO: Should retrieve questlists/merchants list aswell from db.
				npc = new FriendlyNPC(rsNPC.getString("CharacterName"),
						shared.models.helpers.EnumHelpers.GetGender(rsNPC.getString("Sex")),
						rsNPC.getInt("LVL"),
						att,
						null,null,null,null,GetMagicAttacks(rsNPC.getString("CharacterName"))
						,GetPhysicalAttacks(rsNPC.getString("CharacterName"))
						,rsNPC.getString("ImagePath")
						);
				npc.setFriendlyNPCId(rsNPC.getInt("FriendlyNPCId"));
				return npc;
			}
			return null;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get FriendlyNPC from the DB:" + sqle.getMessage());
		}
		return null;
	}
	/**
	 * returns a friendlyNPC based on the characterName
	 * @param characterName
	 * @return an enemyNPCobject
	 * @see friendlyNPC
	 */
	public static FriendlyNPC GetFriendlyNPC(String characterName){
		FriendlyNPC npc = null;
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetFriendlyNPC", 
				"\"CharacterName\" =?",dbConnection);
		try{
			prpState.setString(1, characterName);
			ResultSet rsNPC = prpState.executeQuery();
			if(rsNPC.next()){
				Attributes att = GetAttributeFromDB(rsNPC.getInt("PrimaryStatsId"), 
						rsNPC.getInt("OffensiveStatsId"), 
						rsNPC.getInt("DefensiveStatsId"));
				//TODO: Should retrieve questlists/merchants list aswell from db.
				npc = new FriendlyNPC(rsNPC.getString("CharacterName"),
						shared.models.helpers.EnumHelpers.GetGender(rsNPC.getString("Sex")),
						rsNPC.getInt("LVL"),
						att,
						null,null,null,null,GetMagicAttacks(characterName)
						,GetPhysicalAttacks(characterName)
						,rsNPC.getString("ImagePath")
						);
				npc.setFriendlyNPCId(rsNPC.getInt("FriendlyNPCId"));
				return npc;
			}
			return null;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get FriendlyNPC from the DB:" + sqle.getMessage());
		}
		return null;
	}
	
	public static ArrayList<String> GetAllKingdomNames(){
		ArrayList<String> kingdomNames = new ArrayList<String>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"Name\"", "Kingdom", dbConnection);
		try{
			ResultSet rs = prpState.executeQuery();
			while(rs.next()){
				kingdomNames.add(rs.getString("Name"));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return kingdomNames;
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get all Kingdomnames from the DB"+sqle.getMessage());
		}
		return null;
	}
	public static StaticObject GetStaticObject(String name){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "StaticObject",
				"\"StaticObjectName\" =?", dbConnection);
		try{
			prpState.setString(1, name);
			ResultSet rs = prpState.executeQuery();
			while(rs.next()){
				StaticObject obj = new StaticObject(rs.getString("StaticObjectName"),rs.getString("ImagePath"),null);
				obj.setStaticObjectId(rs.getInt("StaticObjectId"));
				return obj;
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get static object from the DB"+sqle.getMessage());
		}
		return null;
	}
	public static ArrayList<StaticObject> GetAllStaticObjects(){
		ArrayList<StaticObject> listOfAllStaticObjects = new ArrayList<StaticObject>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "StaticObject", dbConnection);
		try{
			ResultSet rsSObj = prpState.executeQuery();
			while(rsSObj.next()){
				StaticObject obj = new StaticObject(rsSObj.getString("StaticObjectName"),rsSObj.getString("ImagePath"),null);
				obj.setStaticObjectId(rsSObj.getInt("StaticObjectId"));
				listOfAllStaticObjects.add(obj);
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return listOfAllStaticObjects;
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get all static objects from the DB"+sqle.getMessage());
		}
		return null;
	}
	/**
	 * gets all the enemyNPCs from the database
	 * @return an arraylist with all the enemyNPCs
	 */
	public static ArrayList<EnemyNPC> GetAllEnemyNPCs(){
		ArrayList<EnemyNPC> listOfAllEnemyNPCs = new ArrayList<EnemyNPC>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"CharacterName\"", "GetEnemyNPC", 
				dbConnection);
		try{
			ResultSet rsNPCs = prpState.executeQuery();
			while(rsNPCs.next()){
				listOfAllEnemyNPCs.add(GetEnemyNPC(rsNPCs.getString("CharacterName")));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return listOfAllEnemyNPCs;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get all enemyNPCs from the DB" + sqle.getMessage());
		}
		return null;
	}
	/**
	 * gets all the friendlyNPCs from the database
	 * @return an arraylist with all the friendlyNPCs
	 */
	public static ArrayList<FriendlyNPC> GetAllFriendlyNPCs(){
		ArrayList<FriendlyNPC> listOfAllFriendlyNPCs = new ArrayList<FriendlyNPC>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"CharacterName\"", "GetFriendlyNPC", 
				dbConnection);
		try{
			ResultSet rsNPCs = prpState.executeQuery();
			while(rsNPCs.next()){
				listOfAllFriendlyNPCs.add(GetFriendlyNPC(rsNPCs.getString("CharacterName")));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return listOfAllFriendlyNPCs;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get all FriendlyNPCs from the DB" + sqle.getMessage());
		}
		return null;
	}
	/**
	 * Return list of all magical attacks in Database.
	 * @return
	 */
	public static ArrayList<Attack> GetAllMagicAttacks(){
		ArrayList<Attack> listOfMagicalAttacks = new ArrayList<Attack>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetAllMagicalAttack",
				dbConnection);
		try{
			ResultSet rsMagic = prpState.executeQuery();
			Attack magicAttack = null;
			Effect effectObject = null;
			while(rsMagic.next()){
				int effectId = 0; 
				//the effect id may be null, so we have to try to convert it								
				try{effectId = rsMagic.getInt("EffectId");}
				catch(Exception e){}
				if(effectId != 0){
					effectObject = GetEffectFromDB(rsMagic.getInt("EffectId"));
				}
				magicAttack = new Attack(
						rsMagic.getInt("MaxDamage"),
						rsMagic.getInt("MinDamage"),
						rsMagic.getInt("QuestionLevel"),
						shared.models.helpers.EnumHelpers.
							GetAttributeType(rsMagic.getString("Attribute")),
						rsMagic.getInt("ReloadTime"),
						rsMagic.getInt("NumberOfQuestions"),
						rsMagic.getString("Name"),
						shared.models.helpers.EnumHelpers.
							GetPlayableClass(rsMagic.getString("ClassName")),
							rsMagic.getInt("LevelActivation"),
						shared.models.helpers.EnumHelpers.
							GetDamageType(rsMagic.getString("Attack_DamageType")),
						null, //there is no weaponType on a magic attack
						effectObject, //effect
						AttackType.MAGICAL,
						rsMagic.getInt("Range"),
						rsMagic.getInt("AnswerPercentage"),
						rsMagic.getBoolean("CanAttackFriendly"),
						rsMagic.getInt("Mana"));
				listOfMagicalAttacks.add(magicAttack);
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return listOfMagicalAttacks;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get magicAttacks from the DB" + sqle.getMessage());
		}
		return null;
		
	}
	/**
	 * Returns a list of all the magically attacks
	 * @param characterName - the characterName which the magically attackt belongs
	 * @return - list of magical attacks 
	 */
	public static ArrayList<Attack> GetMagicAttacks(String characterName){
		ArrayList<Attack> listOfMagicalAttacks = new ArrayList<Attack>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetMagicalAttack",
				"\"CharacterName\" = ? ", dbConnection);
		try{
			prpState.setString(1, characterName);
			ResultSet rsMagic = prpState.executeQuery();
			Attack magicAttack = null;
			Effect effectObject = null;
			while(rsMagic.next()){
				int effectId = 0; 
				//the effect id may be null, so we have to try to convert it								
				try{effectId = rsMagic.getInt("EffectId");}
				catch(Exception e){}
				if(effectId != 0){
					effectObject = GetEffectFromDB(rsMagic.getInt("EffectId"));
				}
				magicAttack = new Attack(
						rsMagic.getInt("MaxDamage"),
						rsMagic.getInt("MinDamage"),
						rsMagic.getInt("QuestionLevel"),
						shared.models.helpers.EnumHelpers.
							GetAttributeType(rsMagic.getString("Attribute")),
						rsMagic.getInt("ReloadTime"),
						rsMagic.getInt("NumberOfQuestions"),
						rsMagic.getString("Name"),
						shared.models.helpers.EnumHelpers.
							GetPlayableClass(rsMagic.getString("ClassName")),
							rsMagic.getInt("LevelActivation"),
						shared.models.helpers.EnumHelpers.
							GetDamageType(rsMagic.getString("Attack_DamageType")),
						null, //there is no weaponType on a magic attack
						effectObject, //effect
						AttackType.MAGICAL,
						rsMagic.getInt("Range"),
						rsMagic.getInt("AnswerPercentage"),
						rsMagic.getBoolean("CanAttackFriendly"),
						rsMagic.getInt("Mana"));
				listOfMagicalAttacks.add(magicAttack);
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return listOfMagicalAttacks;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get magicAttacks from the DB" + sqle.getMessage());
		}
		return null;
	}
	
	public static ArrayList<Attack> GetAllPhysicalAttacks(){
		ArrayList<Attack> listOfPhysicalAttacks = new ArrayList<Attack>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetAllPhysicalAttack",
				dbConnection);
		try{
			ResultSet rsAttack = prpState.executeQuery();
			Attack attack;
			while(rsAttack.next()){
				attack = new Attack(
						rsAttack.getInt("MaxDamage"),
						rsAttack.getInt("MinDamage"),
						rsAttack.getInt("QuestionLevel"),
						shared.models.helpers.EnumHelpers.
							GetAttributeType(rsAttack.getString("Attribute")),
						rsAttack.getInt("ReloadTime"),
						rsAttack.getInt("NumberOfQuestions"),
						rsAttack.getString("Name"),
						shared.models.helpers.EnumHelpers.
							GetPlayableClass(rsAttack.getString("ClassName")),
						rsAttack.getInt("LevelActivation"),
						shared.models.helpers.EnumHelpers.
							GetDamageType(rsAttack.getString("Weapon_DamageType")),
						shared.models.helpers.EnumHelpers.
							GetWeaponType(rsAttack.getString("WeaponType")),
						null, //there is no effect on a physical attack
						AttackType.PHYSICAL,
						rsAttack.getInt("Range"),
						rsAttack.getInt("AnswerPercentage"),
						rsAttack.getBoolean("CanAttackFriendly"),
						0);//physical attacks does not need mana
				listOfPhysicalAttacks.add(attack);
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			
			return listOfPhysicalAttacks;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get physicalAttacks from the DB" + sqle.getMessage());
		}
		return null;
	}
	
	
	/**
	 * Returns an arraylist populated with the physical attacks to an character
	 * @param characterName - the characterName
	 * @return - an arraylist populated with the physical attacks to an character
	 */
	public static ArrayList<Attack> GetPhysicalAttacks(String characterName){
		ArrayList<Attack> listOfPhysicalAttacks = new ArrayList<Attack>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetPhysicalAttack",
				"\"CharacterName\" = ? ", dbConnection);
		try{
			prpState.setString(1, characterName);
			ResultSet rsAttack = prpState.executeQuery();
			Attack attack;
			while(rsAttack.next()){
				attack = new Attack(
						rsAttack.getInt("MaxDamage"),
						rsAttack.getInt("MinDamage"),
						rsAttack.getInt("QuestionLevel"),
						shared.models.helpers.EnumHelpers.
							GetAttributeType(rsAttack.getString("Attribute")),
						rsAttack.getInt("ReloadTime"),
						rsAttack.getInt("NumberOfQuestions"),
						rsAttack.getString("Name"),
						shared.models.helpers.EnumHelpers.
							GetPlayableClass(rsAttack.getString("ClassName")),
						rsAttack.getInt("LevelActivation"),
						shared.models.helpers.EnumHelpers.
							GetDamageType(rsAttack.getString("Weapon_DamageType")),
						shared.models.helpers.EnumHelpers.
							GetWeaponType(rsAttack.getString("WeaponType")),
						null, //there is no effect on a physical attack
						AttackType.PHYSICAL,
						rsAttack.getInt("Range"),
						rsAttack.getInt("AnswerPercentage"),
						rsAttack.getBoolean("CanAttackFriendly"),
						0);//physical attacks does not need mana
				listOfPhysicalAttacks.add(attack);
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			
			return listOfPhysicalAttacks;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get physicalAttacks from the DB" + sqle.getMessage());
		}
		return null;
	}
	
	/**
	 * Gets a playablecharacter from the database
	 * @param playableCharacter - the name of the playable character
	 * @return - a playableCharacter object
	 */
	public static PlayableCharacter GetPlayableCharcter(String playableCharacter){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetCharacter",
				"\"CharacterName\" = ? ", dbConnection);
		try{
			prpState.setString(1, playableCharacter);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			int primaryStatsId = rs.getInt("PrimaryStatsId");
			int defensiveStatsId = rs.getInt("DefensiveStatsId");
			int offensiveStatsId = rs.getInt("OffensiveStatsId");
			String characterFromDbName = rs.getString("CharacterName"); // the character 
			// returned from the database
			System.out.println(primaryStatsId+ ":, "+ defensiveStatsId+ ":, "+offensiveStatsId+ ":, "+characterFromDbName );
			PlayableCharacter pc = new PlayableCharacter(
									characterFromDbName,
									shared.models.helpers.EnumHelpers.GetGender
									(rs.getString("Sex")),
									rs.getInt("LVL"),	
									DatabaseGetters.GetAttributeFromDB(primaryStatsId, 
											offensiveStatsId,defensiveStatsId),// the characters attribute
									database.helperMethods.DatabaseGetters.
										GetInventory(characterFromDbName), //inventory
									rs.getInt("PlayerId"),// plyerId
									shared.models.helpers.EnumHelpers.GetPlayableClass
									(rs.getString("ClassName")), 
									rs.getString("RespawnZoneName"), //respawnPoint
									rs.getInt("Money"),
									DatabaseGetters.GetPhysicalAttacks(playableCharacter),//list of physicalAttacks
									DatabaseGetters.GetMagicAttacks(characterFromDbName),//list of magicalAttacks
									rs.getString("ImagePath"), 
									getEquippedItemOnLocation(playableCharacter, EquipLocation.HEAD),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.LEFT_RING),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.RIGHT_RING),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.LEFT_ARM),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.RIGHT_ARM),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.BODY),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.LEGS),
									getEquippedItemOnLocation(playableCharacter, EquipLocation.AMULET),
									rs.getInt("XP")
									);
			System.out.println(characterFromDbName +" has " +rs.getString("ImagePath"));
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return pc;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get playableCharacter from the DB" + sqle.getMessage());
		}
		return null;
	}
	
	/**
	 * gets the characterId based on the characterName to a character
	 * @param characterName - name to the character
	 * @return - the id belonging to a character
	 */
	public static int GetCharacterId(String characterName){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"CharacterStatsId\"", "CharacterStats",
				"\"CharacterName\" = ? ", dbConnection);
		try{
			prpState.setString(1, characterName);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("CharacterStatsId");
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get characterId from the DB " + sqle.getMessage());
			return 0;
		}
	}
	
	/**
	 * gets the id to an item
	 * @param itemName - the item name to which we would like to find the id to  
	 * @return -the id to an item
	 */
	public static int GetItemId(String itemName){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"ItemId\"", "Items",
				"\"ItemName\" = ? ", dbConnection);
		try{
			prpState.setString(1, itemName);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("ItemId");
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get itemId from the DB" + sqle.getMessage());
			return 0;
		}
	}
	
	/**
	 * @return an hashmap with all possible enemyNPCs in the database
	 */
	public static HashMap<String, EnemyNPC> GetAllEnemyNPCAsAHashMap(){
		ArrayList<EnemyNPC> enemy = GetAllEnemyNPCs();
		HashMap<String, EnemyNPC> enemyHash = new HashMap<String, EnemyNPC>();
		for(EnemyNPC npc: enemy){
			enemyHash.put(npc.getName(), npc);
		}
		return enemyHash;
	}
	/**
	 * @return an hashmap with all possible friendlyNPCs in the database
	 */
	public static HashMap<String, FriendlyNPC> GetAllFriendlyNPCAsAHashMap(){
		ArrayList<FriendlyNPC> friendly = GetAllFriendlyNPCs();
		HashMap<String, FriendlyNPC> friendlyHash = new HashMap<String, FriendlyNPC>();
		for(FriendlyNPC npc: friendly){
			friendlyHash.put(npc.getName(), npc);
		}
		return friendlyHash;
	}
	public static HashMap<String, StaticObject> GetAllStaticObjectAsAHashMap(){
		ArrayList<StaticObject> sObjList = GetAllStaticObjects();
		HashMap<String,StaticObject> sObjHash = new HashMap<String, StaticObject>();
		for(StaticObject obj : sObjList){
			sObjHash.put(obj.getName(), obj);
			
		}
		return sObjHash;
	}
	/**
	 * @param playerName - the playername to which we want to get the playerId
	 * @return - the playerId to a given player
	 */
	public static Integer getPlayerId(String playerName){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"playerid\"", "Player",
				"\"playername\" = ?", dbConnection);
		try{
			prpState.setString(1, playerName);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("playerid");
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not get playerId from the DB" + sqle.getMessage());
			return 0;
		}
	}
	
	/**
	 * Returns a player object, based on the username an password
	 * @param username - the username to the player
	 * @param password - the password to the player
	 * @return a player object, if the username and password is correct. null otherwise 
	 */
	public static Player getPlayer(String username, String password){
		//connects to the database
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			Statement sql = databaseConnection.createStatement();
			PreparedStatement prpState = DBMethods.SelectQuery
							("playername, email, playerid", 
							 "Player", "password= ? and playername= ?",
							 databaseConnection);
			//prevents sql-injection
			prpState.setString(1, password);
			prpState.setString(2, username);
			ResultSet rs = prpState.executeQuery();
			//disconnects from the database as soon as possible,
			// to let others access the database
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			rs.next();
			if(rs != null){
				//creates the player object to return
				Player player = new Player(rs.getString("playername"), 
								rs.getString("email"));
				player.setPlayerId(rs.getInt("playerid"));
				return player;
			}
			else{
				return null;
			}
		} catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.err.println("Most likely there is wrong username and/ or password");
			return null;
		}
	}
	
	/**
	 * gets the playableClassId from the database
	 * @param - playableClass - the playableClass 
	 * @return - the playableClass Id 
 	 */
	public static Integer getClasId(PlayableClasses playableClass){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"ClassId\"", "Classes",
				"\"ClassName\" = ?", dbConnection);
		try {
			prpState.setString(1, playableClass.toString());
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("ClassId");
			}
			catch (SQLException sqle) {
				DBMethods.getInstance().DisconnectFromDB(dbConnection);
				System.err.println("Could not get playerId from the DB" + sqle.getMessage());
				return null;
			}
		}
	
	/**
	 * Based on the kingdomName, the kingdomId is returned from the database
	 * @param kingdomName - the kingdomName
	 * @return the kingdomId
	 */
	public static Integer getKingdomId(String kingdomName){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"KingdomId\"", "Kingdom",
				"\"Name\" = ?", dbConnection);
		try {
			prpState.setString(1,kingdomName);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("KingdomId");
			}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get kingdomId from the DB" + sqle.getMessage());
			return null;
		}
	}
	
	public static KingdomInfo getKingdom(String kingdomName){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetKingdom",
				"\"Name\" = ?", dbConnection);
		try {
			prpState.setString(1,kingdomName);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return new KingdomInfo(rs.getInt("KingdomId"),rs.getString("Name"),rs.getString("StartZoneName"),rs.getString("Subject"));
			}else{
				return null;
			}
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get Kingdom from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static String getKingdomSubject(String kingdomName){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"Subject\"", "Kingdom",
				"\"Name\" = ?", dbConnection);
		try {
			prpState.setString(1,kingdomName);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getString("Subject");
			}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get subject from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<String> getPlayerNames(int kingdomId){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"playername\"", "GetHasAccesToKingdom", 
				"\"KingdomId\" = ?",dbConnection);
		ArrayList<String> playerAccessList = new ArrayList<String>();
		try {
			prpState.setInt(1, kingdomId);
			ResultSet rs = prpState.executeQuery();
			while(rs.next()){
				playerAccessList.add(rs.getString("playername"));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return playerAccessList;
			}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get player access names from DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<String> getAllPlayerNames(){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"playername\"", "Player", 
				dbConnection);
		ArrayList<String> playerNames = new ArrayList<String>();
		try{
			ResultSet rs = prpState.executeQuery();
			while(rs.next()){
				playerNames.add(rs.getString("playername"));
			}
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return playerNames;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get player names from DB" + sqle.getMessage());
			return null;
		}
	}
	public static HashMap<String, Zone> getZones(String kingdomName){
		HashMap<String, Zone> zones = new HashMap<String, Zone>();
		int kingdomId = getKingdomId(kingdomName);
		ArrayList<Integer> zoneIds = getZoneIds(kingdomId);
		
		for(int i : zoneIds){
			Zone tmpZone = getZone(i);
			if(tmpZone==null){
				System.err.println("Could not load zone");
				return null;
			}
			zones.put(tmpZone.getZoneName(), tmpZone);
		}
		
		return zones;
	}
	public static Zone getZoneFromTemplate(String zoneName){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"ZoneId\"", "Zones", 
				"\"Name\" =? AND \"KingdomId\" IS NULL",dbConnection);
		try {
			prpState.setString(1,zoneName);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return getZone(rs.getInt("ZoneId"));
			}
			
		}catch (SQLException sqle) {
				DBMethods.getInstance().DisconnectFromDB(dbConnection);				
				System.err.println("Could not get zoneIds from the DB" + sqle.getMessage());
				return null;
		
		}
		return null;
	}
	public static Zone getZone(int zoneId){
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "Zones", 
				"\"ZoneId\" =?",dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				
				HashMap<String, EnemyNPC> eNPCList = getEnemyNPCLocation(zoneId);
				HashMap<String, FriendlyNPC> fNPCList = getFriendlyNPCLocation(zoneId);
				HashMap<String, StaticObject> sObjList = getStaticObjectLocation(zoneId);
				HashMap<String, TravelTrigger> triggerList = getTravelTrigger(zoneId);
				
				Zone zoneFromDB = new Zone(zoneId+"",rs.getString("Name"),ReadBackgroundFromFile.get(rs.getString("BackgroundPath"), rs.getInt("HTiles"),rs.getInt("VTiles")),new Location(rs.getInt("RespawnX"),rs.getInt("RespawnY")),eNPCList,fNPCList,sObjList,triggerList);
				
				return zoneFromDB;
			}
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get zoneIds from the DB" + sqle.getMessage());
			return null;
		}catch(IOException fileE){
			System.err.println("Could not get background file" + fileE.getMessage());
			return null;
		}
		return null;
	}
	
	private static HashMap<String, FriendlyNPC> getFriendlyNPCLocation(int zoneId){
		HashMap<String, FriendlyNPC> npcLoc = new HashMap<String, FriendlyNPC>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "FriendlyNPCLocation", 
				"\"ZoneId\" =?",dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				FriendlyNPC npc = GetFriendlyNPC(rs.getInt("FriendlyNPCId"));
				npc.setFriendlyNPCId(rs.getInt("LocationId"));
				npc.setCurrentLocation(new Location(rs.getInt("LocationX"),rs.getInt("LocationY")));
				npc.setTargetLocation(new Location(rs.getInt("LocationX"),rs.getInt("LocationY")));
				npcLoc.put(npc.getFriendlyNPCId()+"", npc);
			}
			
			
			return npcLoc;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get FriendlyNPCLocations from the DB" + sqle.getMessage());
			return null;
		}
	}
	private static HashMap<String, EnemyNPC> getEnemyNPCLocation(int zoneId){
		HashMap<String,EnemyNPC> npcLoc = new HashMap<String, EnemyNPC>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "EnemyNPCLocation", 
				"\"ZoneId\" =?",dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				EnemyNPC npc = GetEnemyNPC(rs.getInt("EnemyNPCId"));
				npc.setEnemyNPCId(rs.getInt("LocationId"));
				npc.setCurrentLocation(new Location(rs.getInt("LocationX"),rs.getInt("LocationY")));
				npc.setTargetLocation(new Location(rs.getInt("LocationX"),rs.getInt("LocationY")));
				npc.setTopicTheme(rs.getString("TopicTheme"));
				npcLoc.put(npc.getEnemyNPCId()+"", npc);
			}
			
			
			return npcLoc;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get EnemyLocations from the DB" + sqle.getMessage());
			return null;
		}
	}
	private static HashMap<String, TravelTrigger> getTravelTrigger(int zoneId){
		HashMap<String, TravelTrigger> triggerList = new HashMap<String, TravelTrigger>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetTravelTriggers", 
				"\"ZoneId\" =?",dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			//Add triggers with connections
			while(rs.next()){
				TravelTrigger trigger = new TravelTrigger(rs.getString("Name"),rs.getInt("Width"),rs.getInt("Height"),new Location(rs.getInt("LocationX"),rs.getInt("LocationY")));
				trigger.setTravelTriggerId(rs.getInt("TriggerId"));
				trigger.setDestinationId(rs.getInt("ToTriggerId"));
				trigger.setDestinationZoneName(rs.getString("DestinationZoneName"));
				triggerList.put(rs.getInt("TriggerId")+"",trigger);
			}
			//Add triggers without connections
			PreparedStatement prpState2 = DBMethods.SelectQuery("*", "TravelTrigger", 
					"\"ZoneId\" =? AND \"ToTriggerId\" IS NULL",dbConnection);
			prpState2.setInt(1,zoneId);
			ResultSet rs2 = prpState2.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs2.next()){
				TravelTrigger trigger = new TravelTrigger(rs2.getString("Name"),rs2.getInt("Width"),rs2.getInt("Height"),new Location(rs2.getInt("LocationX"),rs2.getInt("LocationY")));
				trigger.setTravelTriggerId(rs2.getInt("TriggerId"));
				triggerList.put(rs2.getInt("TriggerId")+"",trigger);
			}
			return triggerList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get StaticObjectLocations from the DB" + sqle.getMessage());
			return null;
		}
	}
	private static HashMap<String, StaticObject> getStaticObjectLocation(int zoneId){
		HashMap<String,StaticObject> sObjLoc = new HashMap<String, StaticObject>();
		Connection dbConnection = database.databaseConnection.DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("*", "GetStaticObjectLocations", 
				"\"ZoneId\" =?",dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				StaticObject sObj = new StaticObject(rs.getString("StaticObjectName"),rs.getString("ImagePath"),new Location(rs.getInt("LocationX"),rs.getInt("LocationY")));
				sObj.setStaticObjectId(rs.getInt("LocationId"));
				sObjLoc.put(rs.getInt("LocationId")+"",sObj);
			}
			return sObjLoc;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get StaticObjectLocations from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<Integer> getEnemyNPCIds(int zoneId){
		ArrayList<Integer> idList = new ArrayList<Integer>();
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"LocationId\"", "EnemyNPCLocation",
				"\"ZoneId\" = ?", dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				idList.add(rs.getInt("LocationId"));
			}
			return idList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get enemyNPCids from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<Integer> getFriendlyNPCids(int zoneId){
		ArrayList<Integer> idList = new ArrayList<Integer>();
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"LocationId\"", "FriendlyNPCLocation",
				"\"ZoneId\" = ?", dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				idList.add(rs.getInt("LocationId"));
			}
			return idList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get friendlyNPCIds from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<Integer> getStaticObjectIds(int zoneId){
		ArrayList<Integer> idList = new ArrayList<Integer>();
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"LocationId\"", "StaticObjectLocation",
				"\"ZoneId\" = ?", dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				idList.add(rs.getInt("LocationId"));
			}
			return idList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get staticObjectIDs from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<Integer> getTravelTriggerIds(int zoneId){
		ArrayList<Integer> idList = new ArrayList<Integer>();
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"TriggerId\"", "TravelTrigger",
				"\"ZoneId\" = ?", dbConnection);
		try {
			prpState.setInt(1,zoneId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
				idList.add(rs.getInt("TriggerId"));
			}
			return idList;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get TravelTriggerIds from the DB" + sqle.getMessage());
			return null;
		}
	}
	public static ArrayList<Integer> getZoneIds(int kingdomId){
		ArrayList<Integer> zoneIds = new ArrayList<Integer>();
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"ZoneId\"", "Zones",
				"\"KingdomId\" = ?", dbConnection);
		try {
			prpState.setInt(1,kingdomId);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			while(rs.next()){
			  zoneIds.add(rs.getInt("ZoneId"));
			}
			return zoneIds;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get zoneIds from the DB" + sqle.getMessage());
			return null;
		}
	}
	
	/**
	 * Gets the zoneId form the database based on the kingdomId
	 * TODO: Reimplement to return more than one zone. The zones have not been
	 * Properly implemented, thats why this just returns one zone.
	 * @param kingdomId - the kingdomId 
	 * @return a zoneId 
	 */
	public static Integer getZone(Integer kingdomId){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"ZoneId\"", "Zones",
				"\"KingdomId\" = ?", dbConnection);
		try {
			prpState.setInt(1,kingdomId);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("ZoneId");
			}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get zoneId from the DB" + sqle.getMessage());
			return null;
		}
	}
	
	/**
	 * gets the respawnPoint from the dataBase
	 * @param zoneId - the zoneId to which the respawnPoint belongs 
	 * @return a respawnPoint
	 */
	public static Integer getRespawnPoint(Integer kingdomId){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		PreparedStatement prpState = DBMethods.SelectQuery("\"StartZoneId\"", "Kingdom",
				"\"KingdomId\" = ?", dbConnection);
		try {
			prpState.setInt(1,kingdomId);
			ResultSet rs = prpState.executeQuery();
			rs.next();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			return rs.getInt("StartZoneId");
			}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);				
			System.err.println("Could not get StartZoneId from the DB" + sqle.getMessage());
			return null;
		}
	}
	
	/**
	 * gets the guildId, based on the guildName
	 * TODO: implement guilds and redesign guilds in the database
	 */
	public static Integer getGuildId(String guildName){
		return null;
	}
	
	/**
	 * get a list which determins if a player has access to a given kingdom
	 * @param playerName 
	 * @return a hashmap with all kingdoms. True if there is access rigth, false otherwise
	 */
	public static HashMap<String, Boolean> getKingdomAccessList(String playerName){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.SelectQuery("*",
					"GetHasAccesToKingdom",
					"\"playername\"= ?", databaseConnection);
			prpState.setString(1, playerName);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			HashMap<String, Boolean> kingdomHash = new HashMap<String, Boolean>();
			while(rs.next()){
				kingdomHash.put(rs.getString("Name"), true);
			}
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return kingdomHash;
			
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not execute getKingdomAccessList : " + 
					sqle.getMessage());
			return null;
		}
	}
	
	/**
	 * gets all the short characters to a player from the database
	 * @param playerName - the playename
	 * @return - a list with all the shortCharacter to a player 
	 */
	public static ArrayList<ShortCharacter> getShortCharacter(String playerName){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.SelectQuery("*", "GetShortCharacter",
					"\"playername\"= ?", databaseConnection);
			prpState.setString(1, playerName);
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			ArrayList<ShortCharacter> shortCharList = new ArrayList<ShortCharacter>();
			while(rs.next()){
				ShortCharacter sc = new ShortCharacter(rs.getString("CharacterName"),
						rs.getInt("LVL"), rs.getString("Name"),
						shared.models.helpers.EnumHelpers.GetPlayableClass(rs.getString("ClassName")),
						shared.models.helpers.EnumHelpers.GetGender(rs.getString("Sex")));
				shortCharList.add(sc);
			}
			return shortCharList;
			
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not execute getShortCharacterList : " + 
					sqle.getMessage());
			return null;
		}
	}
	
	public static HashMap<String, BanInfo> getBanList(){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		int playerId;
		HashMap<String, BanInfo> banhash = new HashMap<String, BanInfo>();
		try {
			PreparedStatement prpState = DBMethods.SelectQuery("*", "BanList", 
										databaseConnection);
			ResultSet rs = prpState.executeQuery();
			while(rs.next()){
				playerId = rs.getInt("PlayerId");
				//gets the playerName
				prpState = DBMethods.SelectQuery("playername", "Player",
								"playerid = ?", databaseConnection);
				prpState.setInt(1, playerId);
				ResultSet rs2 = prpState.executeQuery();
				rs2.next();
				BanInfo bI = new BanInfo(playerId, rs.getString("Reason"),
						(Timestamp)rs.getObject("TimeStamp"), rs.getInt("BanId"),
						rs2.getString("playername"));
				banhash.put(rs2.getString("playername"), bI);
			}
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return banhash;
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not execute getBanlist");
			System.out.println(sqle.getMessage());
			sqle.printStackTrace();
			return null;
		}
	}
}