package database.helperMethods;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;

import shared.fileHandler.ReadBackgroundFromFile;
import shared.models.Answer;
import shared.models.AttributeTypes;
import shared.models.Attributes;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Item;
import shared.models.PlayableCharacter;
import shared.models.Player;
import shared.models.Question;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
import sun.awt.PlatformFont;
import sun.print.resources.serviceui;
import shared.models.Character;
import shared.network.messages.Message;
import shared.network.messages.MessageType;

import database.databaseConnection.DBMethods;

/**
 * This class contains the logic to insert information into 
 * the database 
 * @author Esben Fllesdal
 */
public class DatabaseSetters {

	/**
	 * Inserts a new topic into the database
	 * @param topic - a new topic
	 * @return true if the insert was successful, false otherwise
	 */
	public static boolean InsertTopic(String topic){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		PreparedStatement prpState = DBMethods.InsertQuery("TopicTheme", "TopicTheme", 
				"?", dbConnection);
		try{
			prpState.setString(1,topic);
			prpState.executeUpdate();
			return true;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Topic "+ topic+ " : " + sqle.getMessage());
			return false;
		}
	}
	
	/**
	 * Inserts a new question into the database
	 * @param questionText - the question text
	 * @param difficultylevel - the dificulty level of the question
	 * @param topicTheme - the topic theme 
	 * @param questionHeading - the question heading 
	 * @param returning - the column to return
	 * @return the quesitionId to the last inserted question
	 */
	public static Integer InsertQuestion(String questionText, int difficultylevel,
			String topicTheme, String questionHeading){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("Question", 
					"QuestionText\",\"DifficultyLevel\",\"TopicTheme\",\"QuestionHeader",
					"?,?,?,?", "Question\".\"QuestionId", dbConnection);
			prpState.setString(1,questionText);
			prpState.setInt(2,difficultylevel);
			prpState.setString(3,topicTheme);
			prpState.setString(4,questionHeading);
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				return rs.getInt("QuestionId");
			}
			return 0;
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Question : " + sqle.getMessage());
			return 0;
		}
	}
	
	/**
	 * Inserts a new answer into the database
	 * @param answerText - the answer text
	 * @param correct - if this answer i correct
	 * @param questionId - the questionId to which this answer belongs
	 */
	public static void InsertAnswer(String answerText, boolean correct, int questionId){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("Answer",
					"AnswerText\", \"CorrectAnswer\", \"QuestionId",
					"?,?,?", dbConnection);
			prpState.setString(1, answerText);
			prpState.setBoolean(2, correct);
			prpState.setInt(3, questionId);
			prpState.executeUpdate();
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Answers : " + sqle.getMessage());
		}
	}
	private static boolean updateAnswer(Answer ans){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("Answer",
					"\"AnswerText\"= ? , \"CorrectAnswer\"= ?",
					"\"AnswerId\" = ?", databaseConnection);
			prpState.setString(1, ans.getAnswerText());
			prpState.setBoolean(2, ans.isCorrectAnswer());
			prpState.setInt(3, ans.getAnswerId());
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
			
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not update question");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	public static boolean deleteQuestion(int questionId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("Question",
					"\"QuestionId\"= ?", databaseConnection);
			prpState.setInt(1, questionId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete Question:"+questionId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	
	public static boolean deleteStaticObject(int objId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("StaticObject",
					"\"StaticObjectId\"= ?", databaseConnection);
			prpState.setInt(1, objId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete StaticObject:"+objId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	public static boolean updateQuestion(Question question){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("Question",
					"\"QuestionText\"= ? , \"DifficultyLevel\"= ? , \"TopicTheme\"= ?  " +
					", \"QuestionHeader\"= ?, \"TimeLimit\"= ?",
					"\"QuestionId\" = ?", databaseConnection);
			prpState.setString(1, question.getQuestionText());
			prpState.setInt(2, question.getQuestionLevel());
			prpState.setString(3, question.getQuestionTopic());
			prpState.setString(4, question.getQuestionHeading());
			prpState.setInt(5, question.getTimeLimit());
			prpState.setInt(6, question.getQuestionID());
			prpState.executeUpdate();
			
			
			for(Answer ans : question.getAnswers()){
				if(!updateAnswer(ans)){
					DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
					return false;
				}
			}
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not update question");
			System.err.println(sqle.getMessage());
			return false;
		}
		
	}
	/**
	 * inserts the inventory to the database
	 * @param characterName
	 * @param inventory
	 */
	public static void InsertInventory(String characterName, ArrayList<Item> inventory){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		int characterId = DatabaseGetters.GetCharacterId(characterName);
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("InventorySquare",
					"CharacterStatsId\", \"EquipLocation\", \"ItemId",
					"?,?,?", dbConnection);
			for(Item itemInInventory : inventory){
				prpState.setInt(1, characterId);
				prpState.setString(2, itemInInventory.getEquipLocation()+"");
				prpState.setInt(3,itemInInventory.getItemId());
				prpState.executeUpdate();
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert ItemList : " + sqle.getMessage());
		}
	}
	
	/**
	 * deletes all the items to a character
	 * @param characterName
	 */
	public static void RemoveFromInventory(String characterName){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("InventorySquare", 
					"\"CharacterStatsId\" = ?", dbConnection);
			prpState.setInt(1, DatabaseGetters.GetCharacterId(characterName));
			prpState.executeUpdate();
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not delete from inventory : " + sqle.getMessage());
		}
	}
	
	/**
	 * inserts a character into the database
	 * @param primaryStatsId - the corresponding primaryStatsId to the character
	 * @param defensiveStatsId - the corresponding defensiveStatsId to the character
	 * @param offensiveStatsId - the corresponding offensiveStatsId to the character
	 * @param charaterName - the characterName
	 * @param sex - the character's gender
	 * @param level - the charactersLevel
	 * @param imagePath - the imagePath to the characters picture
	 * @return - the databaseId to the inserted row
	 * @see Character
	 */
	public static Integer insertCharacter(int primaryStatsId, int defensiveStatsId, 
			int offensiveStatsId, String charaterName, String sex, int level, String imagePath){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("CharacterStats", 
					"CharacterName\",\"Sex\",\"LVL\",\"OffensiveStatsId\",\"DefensiveStatsId\",\"PrimaryStatsId\",\"ImagePath", 
					"?,?,?,?,?,?,?", "CharacterStats\".\"CharacterStatsId", dbConnection);
			prpState.setString(1,charaterName);
			prpState.setString(2,sex+"");
			prpState.setInt(3,level);
			prpState.setInt(4,offensiveStatsId);
			prpState.setInt(5,defensiveStatsId);
			prpState.setInt(6,primaryStatsId);
			prpState.setString(7,imagePath);
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				return rs.getInt("CharacterStatsId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert primaryStats : " + sqle.getMessage());
			return null;
		}
		DBMethods.getInstance().DisconnectFromDB(dbConnection);
		return null;
	}
	
	/**
	 * Inserts a playable character object into the database
	 * @param playableCharacter - the playable character object, which will be inserted 
	 * @param playerName - the playerName to the player to which the playableCharacter belongs
	 * @param kingdomName - the kingdomName to where the playableCharacter should belong
	 * @return - playablecharacterId to the inserted row
	 */
	public static Integer insertPlayableCharacter(PlayableCharacter playableCharacter, 
							String playerName, String kingdomName){
		//to create a playableCharacter in the database, we need all the foreignkey constraints 
		Integer primaryStatsId = insertPrimaryStats(playableCharacter.getAttribute());
		Integer defensiveStatsId = insertDefensiveStats(playableCharacter.getAttribute());
		Integer offensiveStatsId = insertOffensiveStats(playableCharacter.getAttribute());
		if(primaryStatsId == null || defensiveStatsId  == null || offensiveStatsId  == null ){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		Integer characterStatsId = insertCharacter(primaryStatsId, defensiveStatsId, 
				offensiveStatsId, playableCharacter.getName(), playableCharacter.getSex().toString(),
				playableCharacter.getLVL(), playableCharacter.getImagePath());
		Integer playerId = DatabaseGetters.getPlayerId(playerName);
		Integer classId = DatabaseGetters.getClasId(playableCharacter.getPlayerClass());
		Integer kingdomId = DatabaseGetters.getKingdomId(kingdomName);
		if(characterStatsId ==null || playerId == null || classId == null || kingdomId==null){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		Integer respawnPointId = DatabaseGetters.getRespawnPoint(kingdomId);
		if(respawnPointId == null){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		//TODO: implement guilds and add it to the playableCharacter
		
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("PlayableCharacter", 
					"CharacterStatsId\",\"PlayerId\",\"XP\",\"RespawnZoneId\",\"KingdomId\",\"ClassId\",\"Money", 
					"?,?,?,?,?,?,?", "PlayableCharacter\".\"PlayableCharacterId", dbConnection);
			prpState.setInt(1,characterStatsId);
			prpState.setInt(2,playerId);
			prpState.setInt(3,playableCharacter.getXP());
			prpState.setInt(4,respawnPointId);
			prpState.setInt(5,kingdomId);
			prpState.setInt(6,classId);
			prpState.setInt(7,playableCharacter.getMoney());
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("PlayableCharacterId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert playableCharacter : " + sqle.getMessage());
			return null;
		}
		return null;
	}
	public static Integer insertEnemyNPC(EnemyNPC npc){
		//to create a playableCharacter in the database, we need all the foreignkey constraints 
		Integer primaryStatsId = insertPrimaryStats(npc.getAttribute());
		Integer defensiveStatsId = insertDefensiveStats(npc.getAttribute());
		Integer offensiveStatsId = insertOffensiveStats(npc.getAttribute());
		if(primaryStatsId == null || defensiveStatsId  == null || offensiveStatsId  == null ){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		Integer characterStatsId = insertCharacter(primaryStatsId, defensiveStatsId, 
				offensiveStatsId, npc.getName(), npc.getSex().toString(),
				npc.getLVL(), npc.getImagePath());
		
		if(characterStatsId ==null){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("EnemyNPC", 
					"CharacterStatsId\",\"Agressive\",\"SleepTime\",\"XP", 
					"?,?,?,?", "EnemyNPC\".\"EnemyNPCId", dbConnection);
			prpState.setInt(1,characterStatsId);
			prpState.setInt(2,npc.getLevelOfAgressivenes());
			prpState.setInt(3,npc.getSleepTime());
			prpState.setInt(4,npc.getXP());
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("EnemyNPCId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert enemyNPC : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static String insertStaticObject(StaticObject obj){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("StaticObject", 
					"StaticObjectName\",\"ImagePath", 
					"?,?", "StaticObject\".\"StaticObjectName", dbConnection);
			prpState.setString(1, obj.getName());
			prpState.setString(2, obj.getTextureId());
		
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getString("StaticObjectName");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert StaticObject : " + sqle.getMessage());
			return null;
		}
		return null;
	}
	public static Integer insertKingdom(String kingdomName, String subjectName){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("Kingdom", 
					"Name\",\"Subject", 
					"?,?", "Kingdom\".\"KingdomId", dbConnection);
			prpState.setString(1, kingdomName);
			prpState.setString(2, subjectName);
		
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("KingdomId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Kingdom : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	private static boolean insertAccessToKingdom(int kingdomId, int playerId){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("HasAccessToKingdom", 
					"KingdomId\",\"PlayerId", 
					"?,?","HasAccessToKingdom\".\"KingdomId", dbConnection);
			prpState.setInt(1, kingdomId);
			prpState.setInt(2, playerId);
			ResultSet rs =prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("KingdomId")==kingdomId;
			}
			return true;
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert AccessToKingdom : " + sqle.getMessage());
			return false;
		}
	}
	
	public static boolean insertPlayerAccess(int kingdomId, ArrayList<String> playerAccessList){
			for(String str : playerAccessList){
				if(!insertAccessToKingdom(kingdomId,DatabaseGetters.getPlayerId(str))){
					return false;
				}
			}
			return true;
	
	}
	private static Integer updateZone(int kingdomId, Zone zone){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			int zoneId =Integer.parseInt(zone.getZoneId());
			PreparedStatement prpState = DBMethods.UpdateQuery("Zones",
					"\"KingdomId\"= ? , \"Name\"= ? , \"HTiles\"= ? , \"VTiles\"= ? , \"BackgroundPath\"= ? , \"RespawnX\"= ? , \"RespawnY\"= ?", 
					"\"ZoneId\" = ?", dbConnection);
			prpState.setInt(1, kingdomId);
			prpState.setString(2, zone.getZoneName());
			prpState.setInt(3, zone.getHorizontalSize());
			prpState.setInt(4, zone.getVerticalSize());
			prpState.setString(5, kingdomId+"/"+zone.getZoneName());
			prpState.setInt(6, zone.getRespawnPoint().getX());
			prpState.setInt(7, zone.getRespawnPoint().getY());
			prpState.setInt(8, zoneId);
			
			prpState.executeUpdate();
			ReadBackgroundFromFile.saveBackgroundToFile(kingdomId,zone.getZoneName(), zone.getBackgroundArray());
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			//Delete objects from db that is not in Zone
			ArrayList<Integer> enemyInDB = DatabaseGetters.getEnemyNPCIds(zoneId);
			ArrayList<Integer> friendlyInDB = DatabaseGetters.getFriendlyNPCids(zoneId);
			ArrayList<Integer> staticObjectInDB = DatabaseGetters.getStaticObjectIds(zoneId);
			ArrayList<Integer> travelTriggersInDB = DatabaseGetters.getTravelTriggerIds(zoneId);
			ArrayList<Integer> toBeDeletedIDs = new ArrayList<Integer>();
			//DELETE ENEMYNPC LOCATIONS
			for(Integer id : enemyInDB){
				if(zone.getEnemyNPC(id+"")==null){
					toBeDeletedIDs.add(id);
				}
			}
			for(Integer id : toBeDeletedIDs){
				deleteEnemyNPCLocation(id);
			}
			toBeDeletedIDs = new ArrayList<Integer>();
			//DELETE FRIENDLYNPC LOCATIONS
			for(Integer id : friendlyInDB){
				if(zone.getFriendlyNPC(id+"")==null){
					toBeDeletedIDs.add(id);
				}
			}
			for(Integer id : toBeDeletedIDs){
				deleteFriendlyNPCLocation(id);
			}
			toBeDeletedIDs = new ArrayList<Integer>();
			//DELETE StaticObject LOCATIONS
			for(Integer id : staticObjectInDB){
				if(zone.getStaticObject(id+"")==null){
					toBeDeletedIDs.add(id);
				}
			}
			for(Integer id : toBeDeletedIDs){
				deleteStaticObjectLocation(id);
			}
			toBeDeletedIDs = new ArrayList<Integer>();
			//DELETE TravelTriggers
			for(Integer id : travelTriggersInDB){
				if(zone.getTravelTrigger(id+"")==null){
					toBeDeletedIDs.add(id);
				}
			}
			for(Integer id : toBeDeletedIDs){
				deleteTravelTrigger(id);
			}
			toBeDeletedIDs = new ArrayList<Integer>();
			
			
			
			for(EnemyNPC npc:zone.getEnemyList()){
				if(npc.getEnemyNPCId()<0){//Its a new npcLocation
					insertEnemyNPCInZone(zoneId,npc);
				}else{//Update old npcLocation
					updateEnemyNPCLocation(npc);
				}
				
			}
			for(FriendlyNPC npc:zone.getFriendlyList()){
				if(npc.getFriendlyNPCId()<0){
					insertFriendlyNPCInZone(zoneId, npc);
				}else{
					updateFriendlyNPCLocation(npc);
				}
				
			}
			for(StaticObject obj:zone.getStaticObjectList()){
				if(obj.getStaticObjectId()<0){
					insertStaticObjectInZone(zoneId, obj);
				}else{
					updateStaticObjectLocation(obj);
				}
				
			}
			for(TravelTrigger trigger: zone.getTravelTriggerList()){
				if(trigger.getTravelTriggerId()<0){
					int triggerId =insertTravelTriggerInZone(zoneId, trigger);
					trigger.setTravelTriggerId(triggerId);
				}else{
					updateTravelTrigger(trigger);
				}
				
			}
			
			
			return Integer.parseInt(zone.getZoneId());
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Zone : " + sqle.getMessage());
			return 0;
		}
	}
	private static boolean updateEnemyNPCLocation(EnemyNPC npc){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("EnemyNPCLocation",
					"\"LocationX\"= ? , \"LocationY\"= ? , \"TopicTheme\"= ?",
					"\"LocationId\" = ?", databaseConnection);
			prpState.setInt(1, npc.getCurrentLocation().getX());
			prpState.setInt(2, npc.getCurrentLocation().getY());
			prpState.setString(3, npc.getTopicTheme());
			prpState.setInt(4, npc.getEnemyNPCId());
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute update Enemy Location");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean updateFriendlyNPCLocation(FriendlyNPC npc){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("FriendlyNPCLocation",
					"\"LocationX\"= ? , \"LocationY\"= ?",
					"\"LocationId\" = ?", databaseConnection);
			prpState.setInt(1, npc.getCurrentLocation().getX());
			prpState.setInt(2, npc.getCurrentLocation().getY());
			prpState.setInt(3, npc.getFriendlyNPCId());
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute update Friendly Location");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean updateStaticObjectLocation(StaticObject obj){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("StaticObjectLocation",
					"\"LocationX\"= ? , \"LocationY\"= ?",
					"\"LocationId\" = ?", databaseConnection);
			prpState.setInt(1, obj.getCurrentLocation().getX());
			prpState.setInt(2, obj.getCurrentLocation().getY());
			prpState.setInt(3, obj.getStaticObjectId());
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute update StaticObject Location");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean updateTravelTrigger(TravelTrigger trigger){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("TravelTrigger",
					"\"Name\"= ? , \"LocationX\"= ? , \"LocationY\"= ? , \"Width\"= ? , \"Height\"= ?",
					"\"TriggerId\" = ?", databaseConnection);
			prpState.setString(1, trigger.getName());
			prpState.setInt(2, trigger.getCurrentLocation().getX());
			prpState.setInt(3, trigger.getCurrentLocation().getY());
			prpState.setInt(4, trigger.getImageWidth());
			prpState.setInt(5, trigger.getImageHeight());
			prpState.setInt(6, trigger.getTravelTriggerId());
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute update TravelTrigger");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	public static Integer insertZoneTemplate(Zone zone){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("Zones", 
					"Name\",\"HTiles\",\"VTiles\",\"BackgroundPath\",\"RespawnX\",\"RespawnY", 
					"?,?,?,?,?,?", "Zones\".\"ZoneId", dbConnection);
			
			prpState.setString(1, zone.getZoneName());
			prpState.setInt(2, zone.getHorizontalSize());
			prpState.setInt(3, zone.getVerticalSize());
			prpState.setString(4, 0+"/"+zone.getZoneName());
			prpState.setInt(5, zone.getRespawnPoint().getX());
			prpState.setInt(6, zone.getRespawnPoint().getY());
			ResultSet rs = prpState.executeQuery();
			ReadBackgroundFromFile.saveBackgroundToFile(0,zone.getZoneName(), zone.getBackgroundArray());
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				int zoneId = rs.getInt("ZoneId");
			
				for(EnemyNPC npc:zone.getEnemyList()){
					insertEnemyNPCInZone(zoneId,npc);
				}
				for(FriendlyNPC npc:zone.getFriendlyList()){
					insertFriendlyNPCInZone(zoneId, npc);
				}
				for(StaticObject obj:zone.getStaticObjectList()){
					insertStaticObjectInZone(zoneId, obj);
				}
				return zoneId;
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Zone template: " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static Integer insertZone(int kingdomId, Zone zone){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("Zones", 
					"KingdomId\",\"Name\",\"HTiles\",\"VTiles\",\"BackgroundPath\",\"RespawnX\",\"RespawnY", 
					"?,?,?,?,?,?,?", "Zones\".\"ZoneId", dbConnection);
			prpState.setInt(1, kingdomId);
			prpState.setString(2, zone.getZoneName());
			prpState.setInt(3, zone.getHorizontalSize());
			prpState.setInt(4, zone.getVerticalSize());
			prpState.setString(5, kingdomId+"/"+zone.getZoneName());
			prpState.setInt(6, zone.getRespawnPoint().getX());
			prpState.setInt(7, zone.getRespawnPoint().getY());
			ResultSet rs = prpState.executeQuery();
			ReadBackgroundFromFile.saveBackgroundToFile(kingdomId,zone.getZoneName(), zone.getBackgroundArray());
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				int zoneId = rs.getInt("ZoneId");
			
				for(EnemyNPC npc:zone.getEnemyList()){
					insertEnemyNPCInZone(zoneId,npc);
				}
				for(FriendlyNPC npc:zone.getFriendlyList()){
					insertFriendlyNPCInZone(zoneId, npc);
				}
				for(StaticObject obj:zone.getStaticObjectList()){
					insertStaticObjectInZone(zoneId, obj);
				}
				for(TravelTrigger trigger: zone.getTravelTriggerList()){
					int triggerId =insertTravelTriggerInZone(zoneId, trigger);
					trigger.setTravelTriggerId(triggerId);
				}
				return zoneId;
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert Zone : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static Integer insertEnemyNPCInZone(int zoneId,EnemyNPC npc){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("EnemyNPCLocation", 
					"EnemyNPCId\",\"ZoneId\",\"LocationX\",\"LocationY\",\"TopicTheme", 
					"?,?,?,?,?", "EnemyNPCLocation\".\"LocationId", dbConnection);
			prpState.setInt(1,DatabaseGetters.GetEnemyNPC(npc.getName()).getEnemyNPCId());
			prpState.setInt(2,zoneId);
			prpState.setInt(3,npc.getCurrentLocation().getX());
			prpState.setInt(4,npc.getCurrentLocation().getY());
			prpState.setString(5,npc.getTopicTheme());
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("LocationId");
			}
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert EnemyNPCLocation : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	
	public static Integer insertFriendlyNPCInZone(int zoneId,FriendlyNPC npc){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("FriendlyNPCLocation", 
					"FriendlyNPCId\",\"ZoneId\",\"LocationX\",\"LocationY", 
					"?,?,?,?", "FriendlyNPCLocation\".\"LocationId", dbConnection);
			prpState.setInt(1,DatabaseGetters.GetFriendlyNPC(npc.getName()).getFriendlyNPCId());
			prpState.setInt(2,zoneId);
			prpState.setInt(3,npc.getCurrentLocation().getX());
			prpState.setInt(4,npc.getCurrentLocation().getY());
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("LocationId");
			}
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert FriendlyNPCLocation : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static Integer insertStaticObjectInZone(int zoneId,StaticObject obj){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("StaticObjectLocation", 
					"StaticObjectId\",\"ZoneId\",\"LocationX\",\"LocationY", 
					"?,?,?,?", "StaticObjectLocation\".\"LocationId", dbConnection);
			prpState.setInt(1,DatabaseGetters.GetStaticObject(obj.getName()).getStaticObjectId());
			prpState.setInt(2,zoneId);
			prpState.setInt(3,obj.getCurrentLocation().getX());
			prpState.setInt(4,obj.getCurrentLocation().getY());
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("LocationId");
			}
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert StaticObjectLocation : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static Integer insertTravelTriggerInZone(int zoneId,TravelTrigger trigger){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("TravelTrigger",
					"ZoneId\",\"LocationX\",\"LocationY\",\"Width\",\"Height\",\"Name",
					"?,?,?,?,?,?","TravelTrigger\".\"TriggerId", dbConnection);
			prpState.setInt(1,zoneId);
			prpState.setInt(2, trigger.getCurrentLocation().getX());
			prpState.setInt(3, trigger.getCurrentLocation().getY());
			prpState.setInt(4, trigger.getImageWidth());
			prpState.setInt(5, trigger.getImageHeight());
			prpState.setString(6, trigger.getName());
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("TriggerId");
			}
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert TravelTrigger : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static Integer insertFriendlyNPC(FriendlyNPC npc){
		//to create a playableCharacter in the database, we need all the foreignkey constraints 
		Integer primaryStatsId = insertPrimaryStats(npc.getAttribute());
		Integer defensiveStatsId = insertDefensiveStats(npc.getAttribute());
		Integer offensiveStatsId = insertOffensiveStats(npc.getAttribute());
		if(primaryStatsId == null || defensiveStatsId  == null || offensiveStatsId  == null ){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		Integer characterStatsId = insertCharacter(primaryStatsId, defensiveStatsId, 
				offensiveStatsId, npc.getName(), npc.getSex().toString(),
				npc.getLVL(), npc.getImagePath());
		
		if(characterStatsId ==null){
			//chekcs if any of the above databaseoperations returns null
			return null;
		}
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("FriendlyNPC", 
					"CharacterStatsId", 
					"?", "FriendlyNPC\".\"FriendlyNPCId", dbConnection);
			prpState.setInt(1,characterStatsId);
		
			ResultSet rs = prpState.executeQuery();
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			if(rs.next()){
				return rs.getInt("FriendlyNPCId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert friendlyNPC : " + sqle.getMessage());
			return 0;
		}
		return 0;
	}
	public static boolean addTravelTriggerConnections(int triggerId, int toTriggerId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("TravelTrigger",
					"\"ToTriggerId\"= ?",
					"\"TriggerId\" = ?", databaseConnection);
			prpState.setInt(1, triggerId);
			prpState.setInt(2, toTriggerId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute ALTER_ENEMYNPC");
			System.err.println(sqle.getMessage());
			return false;
		}
	
	}
	private static boolean deleteAccessList(int kingdomId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("HasAccessToKingdom",
					"\"KingdomId\"= ?", databaseConnection);
			prpState.setInt(1, kingdomId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete access list to kingdom with id:"+kingdomId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	public static boolean updatePlayerAccessList(int kingdomId,ArrayList<String> playerAccessList){
		//Delete old access information
		if(!deleteAccessList(kingdomId)){
			return false;
		}
		//insert new 
		for(String name: playerAccessList){
			if(!insertAccessToKingdom(kingdomId, DatabaseGetters.getPlayerId(name))){
				return false;
			}
		}
		return true;
	
	}
	public static boolean updateKingdomInformation(String kingdomName, String kingdomSubject, int kingdomId, ArrayList<String> playerAccessList){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("Kingdom",
					"\"Name\"= ? , \"Subject\"= ? , \"StartZoneId\"= null",
					"\"KingdomId\" = ?", databaseConnection);
			prpState.setString(1, kingdomName);
			prpState.setString(2, kingdomSubject);
			prpState.setInt(3, kingdomId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return updatePlayerAccessList(kingdomId,playerAccessList);
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not update kingdominformation to kingdom");
			System.err.println(sqle.getMessage());
			return false;
		}
		
		
		
	}
	public static boolean alterStaticObject(StaticObject obj){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("StaticObject",
					"\"StaticObjectName\"= ? , \"ImagePath\"= ?",
					"\"StaticObjectId\" = ?", databaseConnection);
			prpState.setString(1, obj.getName());
			prpState.setString(2, obj.getTextureId());
			prpState.setInt(3, obj.getStaticObjectId());
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute update StaticObject");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean deleteZone(int zoneId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("Zones",
					"\"ZoneId\"= ?", databaseConnection);
			prpState.setInt(1, zoneId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete zone:"+zoneId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean deleteEnemyNPCLocation(int npcId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("EnemyNPCLocation",
					"\"LocationId\"= ?", databaseConnection);
			prpState.setInt(1, npcId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete enemyNPCLocation:"+npcId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean deleteFriendlyNPCLocation(int npcId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("FriendlyNPCLocation",
					"\"LocationId\"= ?", databaseConnection);
			prpState.setInt(1, npcId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete friendlyNPCLocation:"+npcId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean deleteStaticObjectLocation(int objId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("StaticObjectLocation",
					"\"LocationId\"= ?", databaseConnection);
			prpState.setInt(1, objId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete StaticObjectLocation:"+objId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	private static boolean deleteTravelTrigger(int triggerId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("TravelTrigger",
					"\"TriggerId\"= ?", databaseConnection);
			prpState.setInt(1, triggerId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete TravelTrigger:"+triggerId);
			System.err.println(sqle.getMessage());
			return false;
		}
	}

	public static boolean updateZonesInKingdom(int kingdomId,String startZoneName, HashMap<String, Zone> zoneList){
		//Delete zones from db that is not in kingdom anymore..
		ArrayList<Integer> inDBList = DatabaseGetters.getZoneIds(kingdomId);
		ArrayList<Integer> zonesToBeDeletedFromDB = new ArrayList<Integer>();
		for(Integer iDB : inDBList){
			boolean zoneExists = false;
			for(Zone z : zoneList.values()){
				if(z.getZoneId().equals(iDB+"")){
					zoneExists = true;
				}
			}
			if(!zoneExists){
				zonesToBeDeletedFromDB.add(iDB);
			}
			
			
		}
		for(Integer deleteZoneId : zonesToBeDeletedFromDB){
			deleteZone(deleteZoneId);
		}
		
		for(String str: zoneList.keySet()){
			if(Integer.parseInt(zoneList.get(str).getZoneId())<=0){//new Zone
				int zoneId =DatabaseSetters.insertZone(kingdomId,(Zone)zoneList.get(str));
				if(zoneId!=0){
					if(str.equals(startZoneName)){
						if(!DatabaseSetters.addStartZoneToKingdom(zoneId, kingdomId)){
							return false;
						}
					}
				}else{
					return false;
				}
			}else{//Allready in DB
				int zoneId = updateZone(kingdomId, zoneList.get(str));
				if(zoneId!=0){
					if(str.equals(startZoneName)){
						if(!DatabaseSetters.addStartZoneToKingdom(zoneId, kingdomId)){
							return false;
						}
					}
				}else{
					return false;
				}
				
				
			}
		
		}
		return true;
	}
	public static boolean addStartZoneToKingdom(int zoneId, int kingdomId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.UpdateQuery("Kingdom",
					"\"StartZoneId\"= ?",
					"\"KingdomId\" = ?", databaseConnection);
			prpState.setInt(1, zoneId);
			prpState.setInt(2, kingdomId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			return true;
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute add startZone to kingdom");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	
	public static boolean alterFriendlyNPC(FriendlyNPC npc){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			PreparedStatement prpState = DBMethods.SelectQuery("*",
					"FriendlyNPC", "\"FriendlyNPCId\" = ?", databaseConnection);
			prpState.setInt(1, npc.getFriendlyNPCId());
			//executes the query
			
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				prpState = DBMethods.SelectQuery("*",
						"CharacterStats", "\"CharacterStatsId\" = ?", databaseConnection);
				prpState.setInt(1, rs.getInt("CharacterStatsId"));
				
				ResultSet rsC = prpState.executeQuery();
				if(rsC.next()){
					prpState = prepareCharacterStatsUpdate(npc, rs.getInt("CharacterStatsId"), databaseConnection);
					prpState.executeUpdate();
					prpState = preparePrimaryStatsUpdate(npc, rsC.getInt("PrimaryStatsId"), databaseConnection);
					prpState.executeUpdate();
					prpState = prepareDefensiveStatsUpdate(npc, rsC.getInt("DefensiveStatsId"), databaseConnection);
					prpState.executeUpdate();
					prpState = prepareOffensiveStatsUpdate(npc, rsC.getInt("OffensiveStatsId"), databaseConnection);
					prpState.executeUpdate();
				}
				DBMethods.getInstance().DisconnectFromDB(databaseConnection);
				return true;
			}
			else{
				return false;
			}
		}catch (SQLException sqle) {
				DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
				System.out.println("Could not execute ALTER_ENEMYNPC");
				System.err.println(sqle.getMessage());
				return false;
			}
		
	}
	
	public static boolean alterEnemyNPC(EnemyNPC npc){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			Statement sql = databaseConnection.createStatement();
			PreparedStatement prpState = DBMethods.SelectQuery("*",
					"EnemyNPC", "\"EnemyNPCId\" = ?", databaseConnection);
			prpState.setInt(1, npc.getEnemyNPCId());
			//executes the query
			
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				prpState = DBMethods.UpdateQuery("EnemyNPC", 
						"\"Agressive\"= ? , \"SleepTime\"= ? , \"XP\"= ? ", 
						"\"EnemyNPCId\"="+npc.getEnemyNPCId(), databaseConnection);
				prpState.setInt(1, npc.getLevelOfAgressivenes());
				prpState.setInt(2,npc.getSleepTime());
				prpState.setInt(3, npc.getXP());
				prpState.executeUpdate();
				prpState = DBMethods.SelectQuery("*",
						"CharacterStats", "\"CharacterStatsId\" = ?", databaseConnection);
				prpState.setInt(1, rs.getInt("CharacterStatsId"));
				
				ResultSet rsC = prpState.executeQuery();
				if(rsC.next()){
					prpState = prepareCharacterStatsUpdate(npc, rs.getInt("CharacterStatsId"), databaseConnection);
					prpState.executeUpdate();
					prpState = preparePrimaryStatsUpdate(npc, rsC.getInt("PrimaryStatsId"), databaseConnection);
					prpState.executeUpdate();
					prpState = prepareDefensiveStatsUpdate(npc, rsC.getInt("DefensiveStatsId"), databaseConnection);
					prpState.executeUpdate();
					prpState = prepareOffensiveStatsUpdate(npc, rsC.getInt("OffensiveStatsId"), databaseConnection);
					prpState.executeUpdate();
					
					
				}
				DBMethods.getInstance().DisconnectFromDB(databaseConnection);
				return true;
			}
			else{
				return false;
			}
		}catch (SQLException sqle) {
				DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
				System.out.println("Could not execute ALTER_ENEMYNPC");
				System.err.println(sqle.getMessage());
				return false;
			}
		
		
		
		
		
	}
	public static boolean deleteEnemyNPC(int npcId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			Statement sql = databaseConnection.createStatement();
			
			PreparedStatement prpState = DBMethods.SelectQuery("*", "EnemyNPC",
					"\"EnemyNPCId\"= ?", databaseConnection);
			prpState.setInt(1,npcId);
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				prpState = DBMethods.SelectQuery("*", "CharacterStats",
						"\"CharacterStatsId\"= ?", databaseConnection);
				prpState.setInt(1,rs.getInt("CharacterStatsId"));
				ResultSet rsC = prpState.executeQuery();
				if(rsC.next()){
					prpState = DBMethods.DeleteQuery("PrimaryStats",
							"\"PrimaryStatsId\"= ?", databaseConnection);
					prpState.setInt(1, rsC.getInt("PrimaryStatsId"));
					prpState.executeUpdate();
					
					prpState = DBMethods.DeleteQuery("DefensiveStats",
							"\"DefensiveStatsId\"= ?", databaseConnection);
					prpState.setInt(1, rsC.getInt("DefensiveStatsId"));
					prpState.executeUpdate();
					
					prpState = DBMethods.DeleteQuery("OffensiveStats",
							"\"OffensiveStatsId\"= ?", databaseConnection);
					prpState.setInt(1, rsC.getInt("OffensiveStatsId"));
					prpState.executeUpdate();
					
				}
				
			}
				
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
	
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute DELETE_ENEMYNPC");
			System.err.println(sqle.getMessage());
			return false;
		}
		
		
		
	}
	/**
	 * Delete all zones from kingdom, On delete-cascade is on, so all the objects will be deleted aswell.
	 * @param kingdomId
	 * @return
	 */
	public static boolean deleteZonesFromKingdom(int kingdomId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("Zones",
					"\"KingdomId\"= ?", databaseConnection);
			prpState.setInt(1, kingdomId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not delete zones");
			System.err.println(sqle.getMessage());
			return false;
		}
	}
	public static boolean deleteFriendlyNPC(int npcId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			Statement sql = databaseConnection.createStatement();
			
			PreparedStatement prpState = DBMethods.SelectQuery("*", "FriendlyNPC",
					"\"FriendlyNPCId\"= ?", databaseConnection);
			prpState.setInt(1,npcId);
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				prpState = DBMethods.SelectQuery("*", "CharacterStats",
						"\"CharacterStatsId\"= ?", databaseConnection);
				prpState.setInt(1,rs.getInt("CharacterStatsId"));
				ResultSet rsC = prpState.executeQuery();
				if(rsC.next()){
					prpState = DBMethods.DeleteQuery("PrimaryStats",
							"\"PrimaryStatsId\"= ?", databaseConnection);
					prpState.setInt(1, rsC.getInt("PrimaryStatsId"));
					prpState.executeUpdate();
					
					prpState = DBMethods.DeleteQuery("DefensiveStats",
							"\"DefensiveStatsId\"= ?", databaseConnection);
					prpState.setInt(1, rsC.getInt("DefensiveStatsId"));
					prpState.executeUpdate();
					
					prpState = DBMethods.DeleteQuery("OffensiveStats",
							"\"OffensiveStatsId\"= ?", databaseConnection);
					prpState.setInt(1, rsC.getInt("OffensiveStatsId"));
					prpState.executeUpdate();
				}
							
			}
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
	
		}catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
			System.out.println("Could not execute DELETE_FRIENDLYNPC");
			System.err.println(sqle.getMessage());
			return false;
		}
		
		
		
	}
	private static PreparedStatement prepareCharacterStatsUpdate(Character npc, int CharacterStatsId, Connection databaseConnection) throws SQLException {
		PreparedStatement prpState = DBMethods.UpdateQuery("CharacterStats",
				"\"CharacterName\"= ? , \"Sex\"= ? , \"LVL\"= ? , \"ImagePath\"= ?",
				"\"CharacterStatsId\" = ?", databaseConnection);
		prpState.setString(1, npc.getName());
		prpState.setString(2,npc.getSex().toString());
		prpState.setInt(3, npc.getLVL());
		prpState.setString(4, npc.getImagePath());
		prpState.setInt(5, CharacterStatsId);

		return prpState;
	}
	private static PreparedStatement preparePrimaryStatsUpdate(Character npc, int PrimaryStatsId, Connection databaseConnection) throws SQLException{
		PreparedStatement prpState = DBMethods.UpdateQuery("PrimaryStats",
				"\"HP\"= ? , \"MP\"= ? , \"STR\"= ? , \"INT\"= ? , \"DEX\"= ? , \"CON\"= ? , \"MaxHP\"= ? , \"MaxMp\"= ?"
				, "\"PrimaryStatsId\" = ?", databaseConnection);
		Attributes attr = npc.getAttribute();
		prpState.setInt(1, attr.getHP());
		prpState.setInt(2, attr.getMP());
		prpState.setInt(3, attr.getSTR());
		prpState.setInt(4, attr.getINT());
		prpState.setInt(5, attr.getDEX());
		prpState.setInt(6, attr.getCON());
		prpState.setInt(7, attr.getMaxHP());
		prpState.setInt(8, attr.getMaxMP());
		prpState.setInt(9, PrimaryStatsId);
		
		return prpState;
	}
	private static PreparedStatement prepareDefensiveStatsUpdate(Character npc, int DefensiveStatsId, Connection databaseConnection) throws SQLException{
		PreparedStatement prpState = DBMethods.UpdateQuery("DefensiveStats",
				"\"PiercingDef\"= ? , \"BluntDef\"= ? , \"SlashingDef\"= ? , \"FireDef\"= ? , \"WaterDef\"= ? , \"EarthDef\"= ? , \"AirDef\"= ?"
				, "\"DefensiveStatsId\" = ?", databaseConnection);
		Attributes attr = npc.getAttribute();
		prpState.setInt(1, attr.getPiercingDef());
		prpState.setInt(2, attr.getBluntDef());
		prpState.setInt(3, attr.getSlashingDef());
		prpState.setInt(4, attr.getFireDef());
		prpState.setInt(5, attr.getWaterDef());
		prpState.setInt(6, attr.getEarthDef());
		prpState.setInt(7, attr.getAirDef());
		prpState.setInt(8, DefensiveStatsId);
		
		return prpState;
	}
	
	private static PreparedStatement prepareOffensiveStatsUpdate(Character npc, int OffensiveStatsId, Connection databaseConnection) throws SQLException{
		PreparedStatement prpState = DBMethods.UpdateQuery("OffensiveStats",
				"\"PiercingOff\"= ? , \"BluntOff\"= ? , \"SlashingOff\"= ? , \"FireOff\"= ? , \"WaterOff\"= ? , \"EarthOff\"= ? , \"AirOff\"= ?"
				, "\"OffensiveStatsId\" = ?", databaseConnection);
		Attributes attr = npc.getAttribute();
		prpState.setInt(1, attr.getPiercingOff());
		prpState.setInt(2, attr.getBluntOff());
		prpState.setInt(3, attr.getSlashingOff());
		prpState.setInt(4, attr.getFireOff());
		prpState.setInt(5, attr.getWaterOff());
		prpState.setInt(6, attr.getEarthOff());
		prpState.setInt(7, attr.getAirOff());
		prpState.setInt(8, OffensiveStatsId);
		
		return prpState;
	}
	/**
	 * Inserts all the primaryAttributes into the database
	 * @param attributes - the attributes which will be inserted
	 * @return - the databaseId to the inserted row
	 */
	public static Integer insertPrimaryStats(Attributes attributes ){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("PrimaryStats", 
					"HP\",\"MP\",\"STR\",\"INT\",\"DEX\",\"CON\",\"MaxHP\",\"MaxMp", 
					"?,?,?,?,?,?,?,?", "PrimaryStats\".\"PrimaryStatsId", dbConnection);
			prpState.setInt(1,attributes.getHP());
			prpState.setInt(2,attributes.getMP());
			prpState.setInt(3,attributes.getSTR());
			prpState.setInt(4,attributes.getINT());
			prpState.setInt(5,attributes.getDEX());
			prpState.setInt(6,attributes.getCON());
			prpState.setInt(7,attributes.getMaxHP());
			prpState.setInt(8,attributes.getMaxMP());
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				return rs.getInt("PrimaryStatsId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert primaryStats : " + sqle.getMessage());
			return null;
		}
		DBMethods.getInstance().DisconnectFromDB(dbConnection);
		return null;
	}
	
	/**
	 * Inserts all the defensiveAttributes into the database
	 * @param attributes - the attributes which will be inserted
	 * @return - the databaseId to the inserted row
	 */
	public static Integer insertDefensiveStats(Attributes attributes ){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("DefensiveStats", 
					"PiercingDef\",\"BluntDef\",\"SlashingDef\",\"FireDef\",\"WaterDef\",\"EarthDef\",\"AirDef", 
					"?,?,?,?,?,?,?", "DefensiveStats\".\"DefensiveStatsId", dbConnection);
			prpState.setInt(1,attributes.getPiercingDef());
			prpState.setInt(2,attributes.getBluntDef());
			prpState.setInt(3,attributes.getSlashingDef());
			prpState.setInt(4,attributes.getFireDef());
			prpState.setInt(5,attributes.getWaterDef());
			prpState.setInt(6,attributes.getEarthDef());
			prpState.setInt(7,attributes.getAirDef());
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				return rs.getInt("DefensiveStatsId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert defensivestats : " + sqle.getMessage());
			return null;
		}
		DBMethods.getInstance().DisconnectFromDB(dbConnection);
		return null;
	}
	
	/**
	 * Inserts all the offensiveStats into the database
	 * @param attributes - the attributes which will be inserted
	 * @return - the databaseId to the inserted row
	 */
	public static Integer insertOffensiveStats(Attributes attributes ){
		Connection dbConnection = DBMethods.getInstance().ConnectToDB();	
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("OffensiveStats", 
					"PiercingOff\",\"BluntOff\",\"SlashingOff\",\"FireOff\",\"WaterOff\",\"EarthOff\",\"AirOff", 
					"?,?,?,?,?,?,?", "OffensiveStats\".\"OffensiveStatsId", dbConnection);
			prpState.setInt(1,attributes.getPiercingOff());
			prpState.setInt(2,attributes.getBluntOff());
			prpState.setInt(3,attributes.getSlashingOff());
			prpState.setInt(4,attributes.getFireOff());
			prpState.setInt(5,attributes.getWaterOff());
			prpState.setInt(6,attributes.getEarthOff());
			prpState.setInt(7,attributes.getAirOff());
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				return rs.getInt("OffensiveStatsId");
			}
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(dbConnection);
			System.err.println("Could not insert ofensiveStats : " + sqle.getMessage());
			return null;
		}
		DBMethods.getInstance().DisconnectFromDB(dbConnection);
		return null;
	}
	
	/**
	 * creates a new player in the database
	 * @param username - the username to the player, this must be unique
	 * @param password - the players password
	 * @param email - the email to the player
	 * @return true if the insertion was successful, false otherwise
	 */
	public static boolean insertPlayer(String username, String password, String email){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();					
		try{
			Statement sql = databaseConnection.createStatement();
			PreparedStatement prpState = DBMethods.InsertQuery("Player",
								" ? , ? , ? ", databaseConnection);
			prpState.setString(1, username);
			prpState.setString(2, email);
			prpState.setString(3, password);
			//executes the query
			prpState.executeUpdate();
			//disconnects from the database as soon as possible,
			// to let others access the database
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
			
		} catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not execute insert statement");
			return false;
		}
	}
	
	/**
	 * Unbans a player
	 * @param playerId - the playerId to the selected player
	 * @return true if the unban was successfuk, false otherwise
	 */
	public static boolean unbanPlayer(int playerId){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.DeleteQuery("BanList",
					"\"PlayerId\"= ? ", databaseConnection);
			prpState.setInt(1, playerId);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}
		catch(SQLException sqle){
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			System.out.println("Could not execute unBanPlayer");
			return false;
		}
	}
	
	/**
	 * bans the given player
	 * @param playerId - the banned playerId
	 * @param reason - the reason for banning
	 * @param time - the time when the player was banned
	 * @return - true if successful, false otherwise.
	 */
	public static boolean banPlayer(int playerId, String reason, Timestamp time){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try{
			PreparedStatement prpState = DBMethods.InsertQuery("BanList", 
					"Reason\", \"PlayerId\", \"TimeStamp", 
					" ? , ? , ? ", databaseConnection);
			prpState.setString(1, reason);
			prpState.setInt(2, playerId);
			prpState.setTimestamp(3, time);
			prpState.executeUpdate();
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return true;
		}
		catch (SQLException sqle) {
			DBMethods.getInstance().DisconnectFromDB(databaseConnection);
			return false;
		}
	}
	
	/**
	 * Alters a player in the database
	 * @param oldUserName  -the oldUsername to the player
	 * @param oldPassword - the oldPassword
	 * @param username - the new userName
	 * @param email - the new email
	 * @param password - the new password
	 * @return true if success, false otherwise
	 */
	public static boolean alterPlayer(String oldUserName, String oldPassword, String username, 
			String email, String password){
		Connection databaseConnection = DBMethods.getInstance().ConnectToDB();
		try {
			Statement sql = databaseConnection.createStatement();
			PreparedStatement prpState = DBMethods.SelectQuery("playerid",
					"Player", "playername= ? AND password = ?", databaseConnection);
			prpState.setString(1, oldUserName);
			prpState.setString(2, oldPassword);
			//executes the query
			ResultSet rs = prpState.executeQuery();
			if(rs.next()){
				prpState = DBMethods.UpdateQuery("Player", 
						"playername= ? , email= ? , password= ? ", 
						"playerid="+rs.getString("playerid"), databaseConnection);
				prpState.setString(1, username);
				prpState.setString(2, email);
				prpState.setString(3, password);
				prpState.executeUpdate();
				DBMethods.getInstance().DisconnectFromDB(databaseConnection);
				return true;
			}
			else{
				return false;
			}
		}catch (SQLException sqle) {
				DBMethods.getInstance().DisconnectFromDB(databaseConnection);	
				System.out.println("Could not execute ALTER_PLAYER");
				System.err.println(sqle.getMessage());
				return false;
			}
	}
}