package editor;

import java.util.ArrayList;
import java.util.HashMap;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Location;
import shared.models.Question;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
import editor.graphics.EditorGUI;
import editor.models.WorldModel;
import editor.models.WorldObjectLists;
/**
 * Main class of Editor. Initializes all objects, graphics and storageHandler.
 * @author thorgrun
 *
 */
public class EditorMain implements ModelUpdater, ModelControls{
	//MODELS
	private WorldObjectLists objectLists;
	private WorldModel kingdom;
	private ArrayList<String> kingdomNames;
	
	//NETWORK
	//GRAPHICS
	private EditorGUI gui;
	private static ModelControls modelControls;
	/**
	 * Constructor of EditorMain.
	 * Initializes graphics, models and storagehandler.
	 * Adds graphics as listeners to models.
	 */
	public EditorMain(){
		modelControls = this;
		kingdomNames = new ArrayList<String>();
		objectLists = new WorldObjectLists();
		gui = new EditorGUI();
		kingdom = new WorldModel(gui.getMapEditor());
		kingdom.setAccessListener(gui);
		//Add listeners to objectLists
		objectLists.addEnemyListener(gui.getNPCEdit());
		objectLists.addFriendlyListener(gui.getNPCEdit());
		objectLists.addStaticObjListener(gui.getNPCEdit());
		objectLists.addEnemyListener(gui.getMapEditor());
		objectLists.addFriendlyListener(gui.getMapEditor());
		objectLists.addStaticObjListener(gui.getMapEditor());
		objectLists.addTopicListener(gui.getMapEditor());
		objectLists.addZoneNameListener(gui.getMapEditor());
		objectLists.addTopicListener(gui.getQuestionEditor());
		objectLists.addQuestionListener(gui.getQuestionEditor());
		StorageHandler.initialize(this);
		
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new EditorMain();
	}
	/**
	 * Returns class as interface, Singleton.
	 * @return
	 */
	public static ModelControls getControls(){
		return modelControls;
	}
	
	@Override
	public void updateAllEnemyNPCList(HashMap<String, EnemyNPC> npcList) {
		objectLists.setEnemyList(npcList);
		
	}
	@Override
	public void updateAllFriendlyNPCList(HashMap<String, FriendlyNPC> npcList) {
		objectLists.setFriendlyList(npcList);		
	}
	@Override
	public void updateAllStaticObjectList(HashMap<String, StaticObject> objList) {
		objectLists.setStaticObjectList(objList);
		
	}

	@Override
	public void updateTopicList(ArrayList<String> topicList) {
		objectLists.setTopicList(topicList);
	}
	@Override
	public void updateQuestionList(ArrayList<Question> questionList) {
		objectLists.setQuestionList(questionList);
		
	}
	@Override
	public boolean createNewZone(Zone zone) {
		return kingdom.createNewZone(zone);
		
	}
	@Override
	public boolean checkIfUniqueZoneName(String zoneName) {
		if(kingdom.getZone(zoneName)!=null){
			return false;
		}else{
			return true;
		}
	}

	@Override
	public boolean addTravelTriggerToZone(String zoneName,TravelTrigger trigger) {
		
		return kingdom.addTravelTriggerToZone(zoneName, trigger);
	}
	@Override
	public void removeTravelTriggerFromZone(String zoneName, int triggerId) {
		kingdom.removeTravelTriggerFromZone(zoneName, triggerId);
		
	}
	@Override
	public void getSelectedZone(String zoneName) {
		kingdom.selectZone(zoneName);
	}

	@Override
	public void addEnemyToZone(String zoneName, String npcName, String topic,
			int x, int y) {
		EnemyNPC npc = objectLists.getEnemyNPC(npcName).getCopy();
		npc.setTopicTheme(topic);
		npc.setCurrentLocation(new Location(x,y));
		kingdom.addEnemyNPCToZone(zoneName, npc);
		
	}
	@Override
	public void addFriendlyToZone(String zoneName, String npcName, int x, int y) {
		FriendlyNPC npc = objectLists.getFriendlyNPC(npcName).getCopy();
		npc.setCurrentLocation(new Location(x,y));
		kingdom.addFriendlyNPCToZone(zoneName, npc);
		
	}
	@Override
	public void addStaticObjectToZone(String zoneName, String objName, int x,
			int y) {
		StaticObject obj = objectLists.getStaticObject(objName).getCopy();
		obj.setCurrentLocation(new Location(x,y));
		kingdom.addStaticObjectToZone(zoneName, obj);
	}
	@Override
	public EnemyNPC getEnemyNPC(String npcName) {
		return objectLists.getEnemyNPC(npcName);
		
	}
	@Override
	public FriendlyNPC getFriendlyNPC(String npcName) {
		return objectLists.getFriendlyNPC(npcName);
		
	}
	@Override
	public StaticObject getStaticObject(String obj) {
		return objectLists.getStaticObject(obj);
		
	}
	@Override
	public void updateEnemyNPC(String zoneName, int npcId, String themeTopic) {
		kingdom.updateEnemyNPC(zoneName,npcId,themeTopic);
		
	}
	@Override
	public void updateTravelTrigger(int triggerId, String name, int width,
			int height, int destinationTriggerId) {
		kingdom.updateTravelTrigger(triggerId,name,width,height,destinationTriggerId);
		
	}
	@Override
	public void removeEnemyNPCFromZone(String zoneName, int npcId) {
		kingdom.removeEnemyNPCFromZone(zoneName, npcId);
		
	}
	@Override
	public void removeFriendlyNPCFromZone(String zoneName, int npcId) {
		kingdom.removeFriendlyNPCFromZone(zoneName, npcId);
		
	}
	@Override
	public void removeStaticObjectFromZone(String zoneName, int objId) {
		kingdom.removeStaticObjectFromZone(zoneName, objId);
		
	}
	@Override
	public void removeZoneFromWorld(String zoneName) {
		kingdom.removeZoneFromWorld(zoneName);
		
	}
	@Override
	public void saveWorld(String zoneName) {
		kingdom.saveKingdomInDatabase(zoneName);
		
	}
	@Override
	public boolean saveWorldToHD() {
		return kingdom.saveKingdomToHD();
		
	}
	@Override
	public void saveZoneTemplate(String zoneTemplateName) {
		kingdom.saveZoneAsTemplate(zoneTemplateName);
		
	}
	@Override
	public void updateKingdomNameList(ArrayList<String> kingdomNameList) {
		kingdomNames = kingdomNameList;
		
	}
	@Override
	public void updatePlayerNameList(ArrayList<String> playerNames) {
		this.kingdom.updatePlayerNameList(playerNames);
		
	}
	@Override
	public String[] getKingdomNames() {
		String[] names = new String[kingdomNames.size()];
		for(int i = 0;i<kingdomNames.size();i++){
			names[i] = kingdomNames.get(i);
		}
		return names;
	}
	@Override
	public void updateWorldObject(String kingdomName, String kingdomSubject,
			int kingdomId, HashMap<String, Zone> zoneList, ArrayList<String> playerAccessList) {
		kingdom.setWorld(kingdomName,kingdomSubject,kingdomId,zoneList,playerAccessList);
		
	}
	@Override
	public String[] getZoneNames() {
		return kingdom.getZoneNames();
	}
	@Override
	public boolean createNewKingdom(String kingdomName, String subjectName) {
		if(!kingdomNames.contains(kingdomName)){
			kingdom.setWorld(kingdomName, subjectName, 0, null, new ArrayList<String>());
			return true;
		}
		return false;
	}
	@Override
	public void updateZoneNameList(String[] zoneNameList) {
		this.objectLists.setZoneNameList(zoneNameList);
	}
	
	@Override
	public void addZoneFromTemplate(Zone zoneTemplate) {
		String zoneName = gui.getZoneNameForTemplate();
		if(zoneName.equals("")){
			
		}else{
			zoneTemplate.setZoneName(zoneName);
			kingdom.addZoneFromTemplate(zoneTemplate);
		}
		
	}
	@Override
	public void addPlayerAccess(String playerName) {
		kingdom.addPlayerAccess(playerName);
		
	}
	@Override
	public void removePlayerAccess(String playerName) {
		kingdom.removePlayerAccess(playerName);
		
	}
	@Override
	public String[] getZoneTemplateNames() {
		
		return objectLists.getZoneNames();
	}
	
	

	

	


}
