package editor;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;
/**
 * Interface for action on model objects.
 * @author thorgrun
 *
 */
public interface ModelControls {
	/**
	 * Creates a new zone in Kingdom.
	 * @param zone
	 * @return
	 */
	public boolean createNewZone(Zone zone);
	/**
	 * Checks if zone name unique.
	 * @param zoneName
	 * @return
	 */
	public boolean checkIfUniqueZoneName(String zoneName);
	/**
	 * Creates new empty kingdom.
	 * @param kingdomName
	 * @param subjectName
	 * @return
	 */
	public boolean createNewKingdom(String kingdomName,String subjectName);
	/**
	 * Adds a EnemyNPC to zone.
	 * @param zoneName
	 * @param npcName
	 * @param topic
	 * @param x
	 * @param y
	 */
	public void addEnemyToZone(String zoneName, String npcName, String topic, int x, int y);
	/**
	 * Adds a FriendlyNPC to zone.
	 * @param zoneName
	 * @param npcName
	 * @param x
	 * @param y
	 */
	public void addFriendlyToZone(String zoneName, String npcName, int x, int y);
	/**
	 * Adds a staticobject to zone.
	 * @param zoneName
	 * @param objName
	 * @param x
	 * @param y
	 */
	public void addStaticObjectToZone(String zoneName, String objName,int x, int y);
	/**
	 * Adds a traveltrigger to zone.
	 * @param zoneName
	 * @param trigger
	 * @return
	 */
	public boolean addTravelTriggerToZone(String zoneName, TravelTrigger trigger);
	/**
	 * Sets a zone as selected zone. Displays it in GUI.
	 * @param zoneName
	 */
	public void getSelectedZone(String zoneName);
	/**
	 * Returns EnemyNPC.
	 * @param npcName
	 * @return
	 */
	public EnemyNPC getEnemyNPC(String npcName);
	/**
	 * Returns FriendlyNPC.
	 * @param npcName
	 * @return
	 */
	public FriendlyNPC getFriendlyNPC(String npcName);
	/**
	 * Returns StaticObject.
	 * @param sObj
	 * @return
	 */
	public StaticObject getStaticObject(String sObj);
	/**
	 * Updates Travel Trigger in Kingdom.
	 * @param triggerId
	 * @param name
	 * @param width
	 * @param height
	 * @param destinationTriggerId
	 */
	public void updateTravelTrigger(int triggerId,String name,int width,int height, int destinationTriggerId);
	/**
	 * Updates EnemyNPC in kingdom.
	 * @param zoneName
	 * @param npcId
	 * @param themeTopic
	 */
	public void updateEnemyNPC(String zoneName,int npcId, String themeTopic);
	/**
	 * Removes EnemyNPC from zone.
	 * @param zoneName
	 * @param npcId
	 */
	public void removeEnemyNPCFromZone(String zoneName,int npcId);
	/**
	 * Removes FriendlyNPC from zone.
	 * @param zoneName
	 * @param npcId
	 */
	public void removeFriendlyNPCFromZone(String zoneName,int npcId);
	/**
	 * Removes StaticObject from Zone.
	 * @param zoneName
	 * @param objId
	 */
	public void removeStaticObjectFromZone(String zoneName, int objId);
	/**
	 * Removes Travel Trigger from zone.
	 * @param zoneName
	 * @param triggerId
	 */
	public void removeTravelTriggerFromZone(String zoneName, int triggerId);
	/**
	 * Removes zone from Kingdom.
	 * @param zoneName
	 */
	public void removeZoneFromWorld(String zoneName);
	/**
	 * Saves current world to database.
	 * @param zoneName
	 */
	public void saveWorld(String zoneName);
	/**
	 * Saves current world to harddrive.
	 * @return
	 */
	public boolean saveWorldToHD();
	/**
	 * Save selected zone as template to database with new name.
	 * @param zoneTemplateName
	 * @param zoneName
	 */
	public void saveZoneTemplate(String zoneTemplateName);
	/**
	 * Return names of all kingdoms.
	 * @return
	 */
	public String[] getKingdomNames();
	/**
	 * Return names of all zones in Kingdom.
	 * @return
	 */
	public String[] getZoneNames();
	/**
	 * Return names of all zone templates.
	 * @return
	 */
	public String[] getZoneTemplateNames();
	/**
	 * Add player to accessList to kingdom.
	 * @param playerName
	 */
	public void addPlayerAccess(String playerName);
	/**
	 * Remove player from accesslist to kingdom.
	 * @param playerName
	 */
	public void removePlayerAccess(String playerName);
}
