package editor;

import java.util.ArrayList;
import java.util.HashMap;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Question;
import shared.models.StaticObject;
import shared.models.Zone;
/**
 * Interface for updating the Models.
 * @author thorgrun
 *
 */
public interface ModelUpdater {
	/**
	 * Updates the EnemyNPCList.
	 * @param npcList
	 */
	public void updateAllEnemyNPCList(HashMap<String,EnemyNPC> npcList);
	/**
	 * Updates the FriendlyNPCList
	 * @param npcList
	 */
	public void updateAllFriendlyNPCList(HashMap<String,FriendlyNPC> npcList);
	/**
	 * Updates the StaticObjectList.
	 * @param objectList
	 */
	public void updateAllStaticObjectList(HashMap<String,StaticObject> objectList);
	/**
	 * Updates the worldObject with a new kingdom.
	 * @param kingdomName Name of kingdom
	 * @param kingdomSubject Name of course subject
	 * @param kingdomId DatabaseId of kingdom
	 * @param zoneList List of Zone objects
	 * @param playerAccessList List of players with access to the kingdom.
	 */
	public void updateWorldObject(String kingdomName,String kingdomSubject, int kingdomId,HashMap<String,Zone> zoneList, ArrayList<String> playerAccessList);
	/**
	 * Updates the TopicList.
	 * @param topicList
	 */
	public void updateTopicList(ArrayList<String> topicList);
	/**
	 * Updates the QuestionList
	 * @param questionList
	 */
	public void updateQuestionList(ArrayList<Question> questionList);
	/**
	 * Updates the Kingdom names list.
	 * @param kingdomNameList
	 */
	public void updateKingdomNameList(ArrayList<String> kingdomNameList);
	/**
	 * Updates list of all players names.
	 * @param playerNames
	 */
	public void updatePlayerNameList(ArrayList<String> playerNames);
	/**
	 * Updates list of all zone templates names.
 	 * @param zoneNameList
	 */
	public void updateZoneNameList(String[] zoneNameList);
	
	/**
	 * Adds a zone from template.
	 * @param zoneTemplate
	 */
	public void addZoneFromTemplate(Zone zoneTemplate);
}
