package editor.graphics;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import editor.EditorMain;
import editor.StorageHandler;
import editor.graphics.editorPanels.MapEditor;
import editor.graphics.editorPanels.WorldObjectEditor;
import editor.graphics.editorPanels.QuestionEditor;
import editor.models.changeListeners.AccessChangedListener;

/**
 * Main Graphical window in the Editor.
 * Controls the different part of the editor as JTabbedPanes.
 * @author thorgrun
 *
 */
public class EditorGUI extends JFrame implements GuiControls,ActionListener, AccessChangedListener, ChangeListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static GuiControls controls;
	private static String NOKINGDOM = "No kingdoms found.";
	private static String NOZONES = "No zones found.";
	private static String NOTEMPLATES = "No templates found in DB";
	private JMenuItem newWorld;
	private JMenuItem loadWorld;
	private JMenuItem saveWorld;
	private JMenuItem reconnect;
	private JMenuItem kingdomAccess;
	private JMenuItem saveWorldToHarddrive;
	private JMenuItem loadWorldFromHarddrive;
	private JMenuItem loadZoneTemplate;
	private JMenuItem saveZoneTemplate;
	private JTabbedPane pane;
	private LoadingDataWindow loadingWindow;
	private UpdateAccessListToKingdom updateAccess;
	//Editors
	//private ItemEditor itemEdit;
	private MapEditor mapEdit;
	private WorldObjectEditor npcEdit;
	private QuestionEditor questionEdit;
	
	//Menu
	JMenu mapMenu;
	JMenu zoneMenu;
	/**
	 * Returns instance of EditorGUI as interface.
	 * @return
	 */
	public static GuiControls guiControls(){
		return controls;
	}
	/**
	 * Constructor
	 */
	public EditorGUI(){
		super("World of Wisdom - Editor");
		JMenuBar menu = new JMenuBar();
		setSize(1024,768);
		JMenu filemenu = new JMenu("File");
		mapMenu = new JMenu("Kingdom");
		zoneMenu = new JMenu("Zone Template");
		reconnect = new JMenuItem("Reconnect to DB");
		reconnect.addActionListener(this);
		newWorld = new JMenuItem("Create new kingdom");
		newWorld.addActionListener(this);
		saveWorld = new JMenuItem("Save world");
		saveWorld.addActionListener(this);
		loadWorld = new JMenuItem("Load world");
		loadWorld.addActionListener(this);
		saveWorldToHarddrive = new JMenuItem("Save world to harddrive");
		saveWorldToHarddrive.addActionListener(this);
		loadWorldFromHarddrive = new JMenuItem("Load world from harddrive");
		loadWorldFromHarddrive.addActionListener(this);
		kingdomAccess = new JMenuItem("Give access to kingdom");
		kingdomAccess.addActionListener(this);
		loadZoneTemplate = new JMenuItem("Load zone");
		loadZoneTemplate.addActionListener(this);
		saveZoneTemplate = new JMenuItem("Save zone");
		saveZoneTemplate.addActionListener(this);
		filemenu.add(reconnect);
		mapMenu.add(newWorld);
		mapMenu.add(loadWorld);
		mapMenu.add(saveWorld);
		mapMenu.add(kingdomAccess);
		mapMenu.add(loadWorldFromHarddrive);
		mapMenu.add(saveWorldToHarddrive);
		zoneMenu.add(loadZoneTemplate);
		zoneMenu.add(saveZoneTemplate);
		menu.add(filemenu);
		menu.add(mapMenu);
		menu.add(zoneMenu);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		//itemEdit = new ItemEditor();
		mapEdit = new MapEditor();
		npcEdit = new WorldObjectEditor();
		questionEdit = new QuestionEditor();

		pane = new JTabbedPane();
		pane.addChangeListener(this);
		//Add panels
		pane.addTab("Map Editor",mapEdit);
		pane.addTab("World Object Editor",npcEdit);
		//pane.addTab("Item Editor",itemEdit);
		pane.addTab("Question Editor",questionEdit);
		
		//Set tabbed pane attributes
		pane.setTabLayoutPolicy(javax.swing.JTabbedPane.SCROLL_TAB_LAYOUT);
		pane.setPreferredSize(new Dimension(1000,700));
		this.add(pane);
		
		
		
		this.setJMenuBar(menu);
		setVisible(true);
		updateAccess = new UpdateAccessListToKingdom(getLocationX(),getLocationY());
		controls = this;
		
	}
	/**
	 * Returns WorldObject editor
	 * @return
	 */
	public WorldObjectEditor getNPCEdit(){
		return npcEdit;
	}
	/**
	 * Returns MapEditor.
	 * @return
	 */
	public MapEditor getMapEditor(){
		return mapEdit;
	}
	/**
	 * Return question editor.
	 * @return
	 */
	public QuestionEditor getQuestionEditor(){
		return questionEdit;
	}
	
	@Override
	public void disableGui() {
		this.setEnabled(false);
		
	}
	@Override
	public void enableGui() {
		this.setEnabled(true);
		
	}
	/**
	 * Returns extension of File.
	 * Method gotten from Java Tutorial:
	 * http://java.sun.com/docs/books/tutorial/uiswing/components/filechooser.html#filters
	 * @param f
	 * @return
	 */
	private String getExtension(File f){
		 String ext = null;
	        String s = f.getName();
	        int i = s.lastIndexOf('.');

	        if (i > 0 &&  i < s.length() - 1) {
	            ext = s.substring(i+1).toLowerCase();
	        }
	        return ext;

	}
	/**
	 * Returns name of image, after moving it.
	 */
	public String browseForImage(){
		JFileChooser chooser = new JFileChooser();
		
	    chooser.setFileFilter(new FileFilter(){
	    	public boolean accept(File f){
	    		
	    		String ext = getExtension(f);
	    		if(ext!=null){
	    			if(ext.equals("png")){
		    			return true;
		    		}else{
		    			return false;
		    		}
	    		}
	    		if(f.isDirectory()){
	    			return true;
	    		}
	    		return false;
	    		
	    		
	    	}
	    	public String getDescription(){
	    		return "Images(.png)";
	    	}
	    });
	    chooser.setCurrentDirectory(new File("Resources/Graphics"));
	    int returnVal = chooser.showOpenDialog(this);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	    	String filePath =chooser.getSelectedFile().getAbsolutePath();
			return ImageFactory.getInstance().copyImageToWorldObjectFolder(filePath);
	    }else{
	    	 return null;
	    }

	   
		/*
		FileDialog fd = new FileDialog(this, "Please choose an image:",
			    FileDialog.LOAD);
		fd.setFilenameFilter(new FilenameFilter(){
		    public boolean accept(File dir, String name){
		      return (name.endsWith(".png"));
		    }
		 });
		fd.setDirectory("");
		fd.setModal(true);
		fd.setVisible(true);
		if(fd.getFile()!=null){
			String filePath =fd.getDirectory()+ fd.getFile();
			return ImageFactory.getInstance().copyImageToWorldObjectFolder(filePath);
		}else{
			return null;
		}
		*/
	}
	/**
	 * Shows the Access List GUI.
	 */
	public void showUpdateAccess(){
		disableGui();
		updateAccess.setVisible(true);
		
	}
	/**
	 * Prompts user for new name for the zone they want to save as template.
	 * @return
	 */
	public String getZoneNameForTemplate(){
		String choice="";
		boolean uniqueName = false;
		while(!uniqueName){
			choice = JOptionPane.showInputDialog ("Enter a name for the new zone"); 
				
			uniqueName = EditorMain.getControls().checkIfUniqueZoneName(choice);
			if(!uniqueName){
				JOptionPane.showMessageDialog(null, "Zone with name \""+choice+"\" allready exists in the kingdom. Please enter a unique name.");
			}			
		}
		return choice;
	}
	
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==newWorld){
			int result = JOptionPane.showConfirmDialog(this, "Any unsaved changes will be overwritten, are you sure you want to continue?", "Create new world confirmation", JOptionPane.YES_NO_OPTION);
			if(result == JOptionPane.YES_OPTION){
				disableGui();
				new CreateKingdomWindow(getLocationX(),getLocationY());
			}
		}else if(e.getSource()==saveWorld){
			String[] zoneNames = EditorMain.getControls().getZoneNames();
			String selectedValue;
			if(zoneNames.length>0){
				selectedValue = zoneNames[0];
			}else{
				selectedValue = NOZONES;
			}
			String choice = (String)JOptionPane.showInputDialog(
                    this,
                    "Select the zone you want as starting zone in the world.",
                    "Select zone",
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    zoneNames,
                    selectedValue);
			if(choice!=null&&!choice.equals(NOZONES)){
				EditorMain.getControls().saveWorld(choice);
				return;
			}
		
		}else if(e.getSource()==loadWorld){
			int result = JOptionPane.showConfirmDialog(this, "Any unsaved changes will be overwritten, are you sure you want to continue?", "Load world from DB confirmation", JOptionPane.YES_NO_OPTION);
			if(result == JOptionPane.YES_OPTION){
				String[] kingdomNames = EditorMain.getControls().getKingdomNames();
				String selectedValue;
				if(kingdomNames.length>0){
					selectedValue = kingdomNames[0];
				}else{
					selectedValue = NOKINGDOM;
				}
				String choice = (String)JOptionPane.showInputDialog(
	                    this,
	                    "Select the kingdom you want to load from database.",
	                    "Select kingdom",
	                    JOptionPane.PLAIN_MESSAGE,
	                    null,
	                    kingdomNames,
	                    selectedValue);
				if(choice!=null&&!choice.equals(NOKINGDOM)){
					StorageHandler.getInstance().requestKingdom(choice);
					return;
				}
			}
		}else if(e.getSource()==loadWorldFromHarddrive){
			FileDialog fd = new FileDialog(this, "Please choose a file:",
				    FileDialog.LOAD);
			fd.setDirectory("Resources/KingdomBackups/");
			fd.setModal(true);
			fd.setVisible(true);
			String file = fd.getFile();
			if(file!=null){
				StorageHandler.getInstance().loadWorldFromHarddrive(fd.getDirectory()+file);
			}
		
		}else if(e.getSource()==saveWorldToHarddrive){
			if(EditorMain.getControls().saveWorldToHD()){
				JOptionPane.showMessageDialog(null, "World was successfully saved to the HardDrive.");
			}
		}else if(e.getSource() == reconnect){
			StorageHandler.getInstance().reconnectToDB();
		}else if(e.getSource() == kingdomAccess){
			showUpdateAccess();
		}else if(e.getSource()==loadZoneTemplate){
				String[] zoneNames = EditorMain.getControls().getZoneTemplateNames();
				String selectedValue;
				if(zoneNames.length>0){
					selectedValue = zoneNames[0];					
				}else{
					zoneNames = new String[1];
					zoneNames[0] = NOTEMPLATES;
					selectedValue = NOTEMPLATES;
				}
				String choice = (String)JOptionPane.showInputDialog(
	                    this,
	                    "Select the zone template you want to load from database.",
	                    "Select zone template",
	                    JOptionPane.PLAIN_MESSAGE,
	                    null,
	                    zoneNames,
	                    selectedValue);
				if(choice!=null&&!choice.equals(NOTEMPLATES)){
					StorageHandler.getInstance().loadZoneTemplate(choice);
					return;
				}
		}else if(e.getSource()==saveZoneTemplate){
			String[] zoneNames = EditorMain.getControls().getZoneTemplateNames();
				String choice="";
				boolean uniqueName = false;
				choice = JOptionPane.showInputDialog ("Enter a name for the new zone"); 
				if(choice !=null){
					uniqueName = true;
					for(String str : zoneNames){
						if(str.equals(choice)){
							uniqueName = false;
						}
					}
					if(!uniqueName){
						JOptionPane.showMessageDialog(null, "Template with name \""+choice+"\" allready exists in the Database. Please enter a unique name.");
					}else if(choice.equals("")){
						JOptionPane.showMessageDialog(null, "You need to enter a name for the template.");
					}else{
						EditorMain.getControls().saveZoneTemplate(choice);
					}
				}
				
			
				
				
				
			}
			
		
	}

	@Override
	public void isLoading(String message) {
		disableGui();
		loadingWindow = new LoadingDataWindow(getLocationX(),getLocationY());
		loadingWindow.addMessage(message);
		
	}

	@Override
	public void doneLoading() {
		if(loadingWindow!=null){
			loadingWindow.allDone();
		}
	}
	@Override
	public void finishedLoading(String message) {
		if(loadingWindow!=null){
			loadingWindow.finishedLoading(message);
		}
		
	}
	@Override
	public void failedDBOperation(String message) {
		if(loadingWindow!=null){
			loadingWindow.failedDBOperation(message);
		}
		
	}
	@Override
	public void addLoadMessage(String message) {
		if(loadingWindow!=null){
			loadingWindow.addMessage(message);
		}
		
	}
	@Override
	public int getLocationX() {
		return this.getLocationOnScreen().x;
	}
	@Override
	public int getLocationY() {
		// TODO Auto-generated method stub
		return this.getLocationOnScreen().y;
	}
	@Override
	public void updatePlayerAccessList(ArrayList<String> playerAccessList) {
		updateAccess.updateAccessList(playerAccessList);
		
	}
	@Override
	public void updatePlayerNames(ArrayList<String> playerNames) {
		updateAccess.updatePlayerNamesList(playerNames);
		
	}
	@Override
	public void addPlayer(String playerName) {
		updateAccess.addPlayer(playerName);
		
	}
	@Override
	public void removePlayer(String playerName) {
		updateAccess.removePlayer(playerName);
	}
	@Override
	public void stateChanged(ChangeEvent e) {
		if(e.getSource()==pane){
			Component selectedComponent = pane.getSelectedComponent();
			if(selectedComponent == npcEdit){
				mapMenu.setEnabled(false);
				zoneMenu.setVisible(false);
			}else if(selectedComponent == mapEdit){
				mapMenu.setEnabled(true);
				zoneMenu.setVisible(true);
			}else if(selectedComponent == questionEdit){
				mapMenu.setEnabled(false);
				zoneMenu.setVisible(false);
			}
		}
		
	}

	
	
}
