package editor.graphics;

import java.awt.Color;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
/**
 * Used for giving user feedback on what is being loaded, and its progress.
 * @author thorgrun
 *
 */
public class LoadingDataWindow extends JFrame implements ActionListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JButton okButton;
	private HashMap<String,JLabel> loadingLabels;
	/**
	 * Constructor.
	 * Initializes the window at coordinates x,y.
	 * @param x
	 * @param y
	 */
	public LoadingDataWindow(int x, int y){
		super("Database connection");
		loadingLabels = new HashMap<String, JLabel>();
		Container content = getContentPane();
		okButton = new JButton("OK");
		
		okButton.addActionListener(this);
		setLocation(x+400,y+300);//Set location relative to EditorGUI
		this.setLayout(new BoxLayout(content,BoxLayout.Y_AXIS));
		setSize(300,25);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setVisible(true);
		setResizable(false);
	}
	/**
	 * Adds a new message to the GUI.
	 * @param message
	 */
	public void addMessage(String message){
		loadingLabels.put(message, new JLabel(message));
		this.getContentPane().add(loadingLabels.get(message));
		this.setSize(300,20*(loadingLabels.size()+2));
	}
	/**
	 * Marks a message as finished.
	 * @param message
	 */
	public void finishedLoading(String message){
		loadingLabels.get(message).setText(message+" *Finished*");
	}
	/**
	 * Called when a DB operation fails, gives a message in red.
	 * And shows OK button.
	 * @param message
	 */
	public void failedDBOperation(String message){
		JLabel failedMsg = new JLabel(message);
		failedMsg.setForeground(Color.RED);
		loadingLabels.put(message, failedMsg);
		this.getContentPane().add(loadingLabels.get(message));
		this.setSize(300,20*(loadingLabels.size()+2));
		allDone();
	}
	/**
	 * Called when all are finished loading, shows an OK button that will close the window.
	 */
	public void allDone(){
		this.getContentPane().add(okButton);
		this.setSize(300,20*(loadingLabels.size()+4));
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==okButton){
			EditorGUI.guiControls().enableGui();
			this.dispose();
			
		}
		
	}
}
