package editor.graphics;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.SpringLayout;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import editor.EditorMain;
/**
 * Graphics that displays list of players with and without access to Kingdom.
 * Used to alter accesslists.
 * @author thorgrun
 *
 */
public class UpdateAccessListToKingdom extends JFrame implements ActionListener,WindowListener,ListSelectionListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JList allPlayers;
	private JList playersWithAccess;
	private DefaultListModel allPlayersModel;
	private DefaultListModel playersWithAccessModel;
	private ArrayList<String> allPlayersList;
	private JButton okButton;
	private String allPlayerSelected;
	private String playerWithAccessSelected;
	private SpringLayout layout;
	private JButton addButton;
	private JButton removeButton;
	public UpdateAccessListToKingdom(int x, int y){
		super("Give players access to current kingdom.");
		layout = new SpringLayout();
		allPlayerSelected="";
		playerWithAccessSelected="";
		this.setLayout(layout);
		setSize(400,400);
		setVisible(false);
		setResizable(false);
		setLocation(x+400,y+300);
		addButton = new JButton(">>");
		addButton.addActionListener(this);
		removeButton = new JButton("<<");
		removeButton.addActionListener(this);
		okButton = new JButton("Ok");
		okButton.addActionListener(this);
		allPlayers = new JList();
		playersWithAccess = new JList();
		
		allPlayersModel = initializeJList(5, allPlayers);
		playersWithAccessModel = initializeJList(230, playersWithAccess);
		place(180,4,addButton);
		place(180,6,removeButton);
		place(140,10,okButton);
		place(5,1,new JLabel("All players"));
		place(230,1,new JLabel("Players with access."));
	
		addWindowListener(this);
	}
	private DefaultListModel initializeJList(int x,JList list){
		DefaultListModel listModel = new DefaultListModel();
		list.setModel(listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		list.setLayoutOrientation(JList.VERTICAL);
		list.addListSelectionListener(this);
		JScrollPane scroll = new JScrollPane(list);
		scroll.setPreferredSize(new Dimension(150,250));
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		place(x,2,scroll);
		return listModel;
	}
	private void place(int x,int rowY,Component comp){
		layout.putConstraint(SpringLayout.WEST,comp,
							x,
							SpringLayout.WEST,this);
		layout.putConstraint(SpringLayout.NORTH, comp,
							(rowY*30-25),
							SpringLayout.NORTH, this);
		this.add(comp);
	}
	/**
	 * Updates the list of all players.
	 * @param playerNames
	 */
	public void updatePlayerNamesList(ArrayList<String> playerNames){
		allPlayersList = playerNames;
		allPlayersModel.clear();
		for(String str : playerNames){
			allPlayersModel.addElement(str);
		}
	}
	/**
	 * Updates the list of players with access to kingdom.
	 * @param playerAccessList
	 */
	public void updateAccessList(ArrayList<String> playerAccessList){
		updatePlayerNamesList(allPlayersList);
		playersWithAccessModel.clear();
		for(String str : playerAccessList){
			playersWithAccessModel.addElement(str);
			allPlayersModel.removeElement(str);
		}
	}
	/**
	 * Adds a player to accessList
	 * @param playerName
	 */
	public void addPlayer(String playerName) {
		playersWithAccessModel.addElement(playerName);
		allPlayersModel.removeElement(playerName);
		
	}
	/**
	 * Removes a player from accessList.
	 * @param playerName
	 */
	public void removePlayer(String playerName) {
		playersWithAccessModel.removeElement(playerName);
		allPlayersModel.addElement(playerName);
		
	}
	@Override
	public void windowActivated(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void windowClosed(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void windowClosing(WindowEvent arg0) {
		EditorGUI.guiControls().enableGui();
		this.setVisible(false);
	
	}
	@Override
	public void windowDeactivated(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void windowDeiconified(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void windowIconified(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void windowOpened(WindowEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void valueChanged(ListSelectionEvent e) {
		if(e.getSource()==allPlayers){
			if(allPlayers.getSelectedValue()!=null){
				allPlayerSelected = allPlayers.getSelectedValue().toString();
			}else{
				allPlayerSelected = "";
			}
			
			
		}else if(e.getSource()==playersWithAccess){
			if(playersWithAccess.getSelectedValue()!=null){
				playerWithAccessSelected = playersWithAccess.getSelectedValue().toString();
				
			}else{
				playerWithAccessSelected = "";
			}
		}
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==addButton){
			if(allPlayerSelected!=null||!allPlayerSelected.equals("")){
				EditorMain.getControls().addPlayerAccess(allPlayerSelected);
			}
			
			
		}else if(e.getSource() == removeButton){
			if(playerWithAccessSelected!=null||!playerWithAccessSelected.equals("")){
				EditorMain.getControls().removePlayerAccess(playerWithAccessSelected);
			}
			
		}else if(e.getSource() == okButton){
			EditorGUI.guiControls().enableGui();
			this.setVisible(false);
		}
		
	}
}
