package editor.graphics.editorPanels;

import java.awt.Component;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.JPanel;
import javax.swing.SpringLayout;


import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Location;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.Zone;

import editor.EditorMain;
import editor.graphics.editorPanels.mapEditorPanels.MapGraphics;
import editor.graphics.editorPanels.mapEditorPanels.MapObjects;
import editor.graphics.editorPanels.mapEditorPanels.MapPlaceables;
import editor.graphics.editorPanels.mapEditorPanels.ObjectSelectedListener;
import editor.graphics.editorPanels.mapEditorPanels.PlaceableListener;
import editor.models.changeListeners.EnemyListChangedListener;
import editor.models.changeListeners.FriendlyListChangedListener;
import editor.models.changeListeners.StaticObjectListChangedListener;
import editor.models.changeListeners.TopicListChangedListener;
import editor.models.changeListeners.WorldModelChangedListener;
import editor.models.changeListeners.ZoneNameListChangedListener;
/**
 * JPanel for the MapEditor.
 * @author thorgrun
 *
 */
public class MapEditor extends JPanel implements EnemyListChangedListener,
												FriendlyListChangedListener,
												StaticObjectListChangedListener,
												TopicListChangedListener,
												ZoneNameListChangedListener,
												WorldModelChangedListener,
												ObjectSelectedListener, 
												PlaceableListener{
		

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final static int playfieldHeight=700;
	private final static int playfieldWidth=600;
	private MapGraphics worldGraphic;
	private MapObjects mapObjects;
	private MapPlaceables mapPlaceables;
	private SpringLayout layout;
	/**
	 * Initializes the three parts of the MapEditor.
	 * WorldGraphics, ObjectTreePanel and Placeables.
	 */
	public MapEditor(){
		layout = new SpringLayout();
		this.setLayout(layout);
		worldGraphic = new MapGraphics(this,playfieldWidth,playfieldHeight);
		mapObjects = new MapObjects(this);
		mapPlaceables = new MapPlaceables(this);
		place(5,1,mapObjects);
		place(210,1,worldGraphic);
		place(215+playfieldWidth,1,mapPlaceables);
		mapObjects.setVisible(false);
		worldGraphic.setVisible(false);
		mapPlaceables.setVisible(false);
	}
	/**
	 * Helper method for placing components in the JPanel.
	 * @param x
	 * @param rowY
	 * @param comp
	 */
	private void place(int x,int rowY,Component comp){
		layout.putConstraint(SpringLayout.WEST,comp,
							x,
							SpringLayout.WEST,this);
		layout.putConstraint(SpringLayout.NORTH, comp,
							(rowY*30-25),
							SpringLayout.NORTH, this);
		this.add(comp);
	}
	@Override
	public void updatedEnemyNPCList(HashMap<String, EnemyNPC> npcList) {
		mapPlaceables.updateEnemyNPCList(npcList);
		
	}
	@Override
	public void updatedFriendlyNPCList(HashMap<String, FriendlyNPC> npcList) {
		mapPlaceables.updateFriendlyNPCList(npcList);
		
	}
	@Override
	public void updatedStaticObjectList(HashMap<String, StaticObject> objList) {
		mapPlaceables.updateStaticObjectList(objList);
		
	}



	@Override
	public void newEnemyNPC(String zoneName, EnemyNPC npc) {
		mapObjects.addEnemyToZone(zoneName, npc);
		worldGraphic.selectEnemyNPC(npc.getEnemyNPCId()+"");
		worldGraphic.setScreenFocus(npc.getCurrentLocation());
		worldGraphic.repaint();
	}


	@Override
	public void newFriendlyNPC(String zoneName, FriendlyNPC npc) {
		mapObjects.addFriendlyToZone(zoneName, npc);
		worldGraphic.selectFriendlyNPC(npc.getFriendlyNPCId()+"");
		worldGraphic.setScreenFocus(npc.getCurrentLocation());
		worldGraphic.repaint();
	}


	@Override
	public void newStaticObject(String zoneName, StaticObject obj) {
		mapObjects.addStaticObjectToZone(zoneName, obj);
		worldGraphic.selectStaticObject(obj.getStaticObjectId()+"");
		worldGraphic.setScreenFocus(obj.getCurrentLocation());
		worldGraphic.repaint();
	}
	@Override
	public void newTravelTrigger(String zoneName, TravelTrigger trigger) {
		mapObjects.addTravelTriggerToZone(zoneName,trigger);
		worldGraphic.selectTravelTrigger(trigger.getTravelTriggerId()+"");
		worldGraphic.setScreenFocus(trigger.getCurrentLocation());
		worldGraphic.repaint();
	}

	@Override
	public void newWorld(String kingdomName) {
		mapObjects.clearWorld(kingdomName);
		worldGraphic.setVisible(false);
		mapPlaceables.setVisible(false);
	}


	@Override
	public void newZone(Zone newZone) {
		mapObjects.addZoneToTree(newZone);
	}




	@Override
	public void removeZone(String zoneName) {
		mapObjects.removeZone(zoneName);
		rootSelected();
	
	}

	@Override
	public void removeEnemyNPC(String zoneName, int npcId) {
		mapObjects.removeEnemyNPC(zoneName, npcId);
		worldGraphic.removeSelectionOfObject();
		
	}

	@Override
	public void removeFriendlyNPC(String zoneName, int npcId) {
		mapObjects.removeFriendlyNPC(zoneName, npcId);
		worldGraphic.removeSelectionOfObject();
		
	}

	@Override
	public void removeStaticObject(String zoneName, int objId) {
		
		mapObjects.removeStaticObject(zoneName, objId);
		worldGraphic.removeSelectionOfObject();
	}

	@Override
	public void removeTravelTrigger(String zoneName, int triggerId) {
		mapObjects.removeTravelTrigger(zoneName, triggerId);
		worldGraphic.removeSelectionOfObject();
		
	}




	@Override
	public void rootSelected() {
		worldGraphic.setVisible(false);
		mapPlaceables.setVisible(false);
		mapObjects.collapseZone();
	}


	@Override
	public void staticObjectSelected(String zoneName,String objectId) {
		worldGraphic.selectStaticObject(objectId);
		mapObjects.selectedStaticObject(zoneName, objectId);
	}
	@Override
	public void travelTriggerSelected(String zoneName,String triggerId) {
		worldGraphic.selectTravelTrigger(triggerId);
		mapObjects.selectedTravelTrigger(zoneName, triggerId);
		
	}
	@Override
	public void enemyNPCSelected(String zoneName,String npcId) {
		worldGraphic.selectEnemyNPC(npcId);
		mapObjects.selectedEnemyNPC(zoneName, npcId);
	}
	@Override
	public void friendlyNPCSelected(String zoneName,String npcId) {
		worldGraphic.selectFriendlyNPC(npcId);
		mapObjects.selectedFriendlyNPC(zoneName, npcId);
	}
	@Override
	public void zoneSelected(String zoneName) {
		EditorMain.getControls().getSelectedZone(zoneName);
		worldGraphic.setVisible(true);
		mapPlaceables.setVisible(true);
	}

	@Override
	public void currentZone(Zone selectedZone) {
		worldGraphic.setDrawZone(selectedZone);
		Location centerOfZone = new Location(selectedZone.getHorizontalSize()/2*16,selectedZone.getVerticalSize()/2*16);
		worldGraphic.setScreenFocus(centerOfZone);
		mapPlaceables.setCurrentZoneName(selectedZone.getZoneName());
	}


	@Override
	public void updateTopicList(ArrayList<String> topicList) {
		mapPlaceables.updateTopicList(topicList);
		mapObjects.updateTopicList(topicList);
	}


	@Override
	public void disablePainting() {
		worldGraphic.disableTilePainting();
	}


	@Override
	public void setEnablePainting(String tileName) {
		worldGraphic.enablePainting(tileName);
		
	}

	@Override
	public void updateZoneNameList(String[] zoneNameList) {
		mapObjects.updateZoneNameList(zoneNameList);
		
	}

	@Override
	public void addEnemyToZone(String zoneName, String npcName, String topic) {
		
		
		EditorMain.getControls().addEnemyToZone(zoneName, npcName, topic,
				worldGraphic.getScreenFocus().getX(),worldGraphic.getScreenFocus().getY() );
		
	}

	@Override
	public void addFriendlyToZone(String zoneName, String npcName) {
		EditorMain.getControls().addFriendlyToZone(zoneName, npcName,
				worldGraphic.getScreenFocus().getX(),worldGraphic.getScreenFocus().getY() );
		
	}

	@Override
	public void addStaticObjectToZone(String zoneName, String objName) {
		EditorMain.getControls().addStaticObjectToZone(zoneName, objName,
				worldGraphic.getScreenFocus().getX(),worldGraphic.getScreenFocus().getY() );
		
	}











	



	
	
	
}
