package editor.graphics.editorPanels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SpinnerNumberModel;
import javax.swing.SpringLayout;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import editor.EditorMain;
import editor.StorageHandler;
import editor.models.changeListeners.QuestionListChangedListener;
import editor.models.changeListeners.TopicListChangedListener;

import shared.models.Answer;
import shared.models.Question;

public class QuestionEditor extends JPanel implements TreeSelectionListener, ActionListener,QuestionListChangedListener,TopicListChangedListener, KeyListener  {

	private final String saveString = "Save changes";
	private final String addString = "Add new question";
	private final int numAns = 4;
	
	private ArrayList<Question> listOfQuestions;
	private ArrayList<String> topicList;
	
	private JTree questionTree;
	private JTextField questionHeadingTextField = new JTextField(20);
	private JTextArea questionText = new JTextArea(2, 20);
	private JTextField answer1TextField = new JTextField(20);
	private JTextField answer2TextField = new JTextField(20);
	private JTextField answer3TextField = new JTextField(20);
	private JTextField answer4TextField = new JTextField(20);
	private JButton cancelButton = new JButton("Cancel");
	private JButton okButton = new JButton(addString);
	private JTextField newTopicField = new JTextField(20);
	private JRadioButton correctAnswer1 = new JRadioButton("Correct answer", true);
	private JRadioButton correctAnswer2 = new JRadioButton("Correct answer", false);
	private JRadioButton correctAnswer3 = new JRadioButton("Correct answer", false);
	private JRadioButton correctAnswer4 = new JRadioButton("Correct answer", false);
	private ButtonGroup correctAnswerGroup = new ButtonGroup();
	private JScrollPane treeView;
	private JScrollPane textView;
	private JSpinner questionLevelSpinner = new JSpinner();
	
	private JLabel headingLabel = new JLabel("Question heading :");
	private JLabel questionLabel = new JLabel("Question text :");
	private JLabel topicLable = new JLabel("Question topic :");
	private JLabel questionLevelLabel = new JLabel("Question level");
	private SpringLayout layout;
	private JPanel eastPanel;
	private JPanel treePanel;
	private DefaultTreeModel treeModel;
	/**
	 *  creates a new panel
	*/
	public QuestionEditor(){
		//gets all the question from the database
		treeModel = new DefaultTreeModel(new DefaultMutableTreeNode("WoW - Questions"));
		questionTree = new JTree(treeModel);
		questionTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		questionTree.setSelectionRow(0);
		questionTree.addTreeSelectionListener(this);
		questionTree.addKeyListener(this);
		questionTree.expandRow(0);
		//GUI --------------------------------------------------------
		treeView = new JScrollPane(questionTree);
		treeView.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		treeView.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		treeView.setPreferredSize(new Dimension(300,600));
		SpinnerNumberModel spm = new SpinnerNumberModel(1,1,5,1);
		questionLevelSpinner.setModel(spm);
		
		textView = new JScrollPane(questionText);
		textView.setPreferredSize(new Dimension(300,70));
		setLayout(new BorderLayout());
		layout = new SpringLayout();
		treePanel = new JPanel();
		treePanel.add(treeView);
		eastPanel = new JPanel();
		eastPanel.setLayout(layout);
		place(5,1,headingLabel);
		place(140,1,questionHeadingTextField);
		place(5,2,topicLable);
		place(140,2,newTopicField);
		place(5,3,questionLabel);
		place(140,3,textView);
		place(5,6,questionLevelLabel);
		place(140,6,questionLevelSpinner);
		place(5,8,new JLabel("Answer alternatives"));
		place(5,9,answer1TextField);
		place(250,9,correctAnswer1);
		place(5,10,answer2TextField);
		place(250,10,correctAnswer2);
		place(5,11,answer3TextField);
		place(250,11,correctAnswer3);
		place(5,12,answer4TextField);
		place(250,12,correctAnswer4);
		place(50,14,cancelButton);
		place(140,14,okButton);
		eastPanel.add(headingLabel);
		eastPanel.add(questionHeadingTextField);
		eastPanel.add(topicLable);
		eastPanel.add(newTopicField);
		eastPanel.add(questionLabel);
		eastPanel.add(textView);
		eastPanel.add(questionLevelLabel);
		eastPanel.add(questionLevelSpinner);
		eastPanel.add(answer1TextField);
		eastPanel.add(correctAnswer1);
		eastPanel.add(answer2TextField);
		eastPanel.add(correctAnswer2);
		eastPanel.add(answer3TextField);
		eastPanel.add(correctAnswer3);
		eastPanel.add(answer4TextField);
		eastPanel.add(correctAnswer4);		
		cancelButton.addActionListener(this);
		eastPanel.add(cancelButton);
		okButton.addActionListener(this);
		eastPanel.add(okButton);
		correctAnswerGroup.add(correctAnswer1);
		correctAnswerGroup.add(correctAnswer2);
		correctAnswerGroup.add(correctAnswer3);
		correctAnswerGroup.add(correctAnswer4);
		
		add(treePanel, BorderLayout.WEST);
		add(eastPanel, BorderLayout.CENTER);
		
	}
	@Override
	public void updateQuestionList(ArrayList<Question> questionList) {
		this.listOfQuestions = questionList;
		if(topicList!=null){
			createNodes();
		}
		
	}
	
	@Override
	public void updateTopicList(ArrayList<String> topicList) {
		this.topicList = topicList;
		if(listOfQuestions!=null){
			createNodes();
		}
		
	}
	
	private void place(int x,int rowY,Component comp){
		layout.putConstraint(SpringLayout.WEST,comp,
							x,
							SpringLayout.WEST,this);
		layout.putConstraint(SpringLayout.NORTH, comp,
							(rowY*30-25),
							SpringLayout.NORTH, this);
	}
	/**
	 * Creates the nodes in the jtree
	 * @param topNode - the only node without parrents
	 */
	public void createNodes(){
		
	    DefaultMutableTreeNode categoryNode = null;
	    DefaultMutableTreeNode questionNode = null;
	    DefaultMutableTreeNode root = (DefaultMutableTreeNode)treeModel.getRoot();
	    questionTree.setSelectionRow(0);
	    //Clear list
	    while(root.getChildCount()>0){
			treeModel.removeNodeFromParent((DefaultMutableTreeNode)root.getChildAt(0));
		}
	    for(String topic : topicList){
	    	categoryNode = new DefaultMutableTreeNode(topic);
	    	treeModel.insertNodeInto(categoryNode, root, 0);
	    	
			for(Question q : listOfQuestions){
				if(topic.equals(q.getQuestionTopic())){
					questionNode = new DefaultMutableTreeNode(q);
					treeModel.insertNodeInto(questionNode, categoryNode, 0);
				}
			}
	    }
	}

	/**
	 * treeselectionlistener
	 */
	public void valueChanged(TreeSelectionEvent e) {
		 DefaultMutableTreeNode node = (DefaultMutableTreeNode)
         					questionTree.getLastSelectedPathComponent();
		 if (node == null)
			 //Nothing is selected.	
			 return;
		 Object nodeInfo = node.getUserObject();
		 if (node.isLeaf()&&nodeInfo instanceof Question) {
			 	Question questionNode = (Question)nodeInfo;
			 	setAllFields(questionNode);
			 	okButton.setText(saveString);
			 }
		 else if(node.isRoot()){
			 //can add a new topic here
			 resetAllFields();
			 okButton.setText(addString);
		 }
		 else {
			 resetAllFields();
			 newTopicField.setText((String)nodeInfo);
			 okButton.setText(addString);
		 }
	}
	/**
	 * resets all the fields
	 */
	private void resetAllFields(){
		 answer1TextField.setText("");
		 answer2TextField.setText("");
		 answer3TextField.setText("");
		 answer4TextField.setText("");
		 questionHeadingTextField.setText("");
		 questionText.setText("");
		 questionLevelSpinner.setValue(1);
	}

	/**
	 * sets all the fields
	 * @param questionNode - a question object
	 */
	private void setAllFields(Question questionNode){
		questionHeadingTextField.setText(questionNode.getQuestionHeading());
	 	questionText.setText(questionNode.getQuestionText());
	 	newTopicField.setText(questionNode.getQuestionTopic());
	 	questionLevelSpinner.setValue(questionNode.getQuestionLevel());
	 	if(questionNode.getAnswers().size()>0){
	 		answer1TextField.setText(questionNode.getAnswers().get(0).getAnswerText());
	 		//checks if this question is correct
	 		if(questionNode.getAnswers().get(0).isCorrectAnswer()){
	 			correctAnswer1.setSelected(true);
	 		}
	 	}
	 	else{
	 		answer1TextField.setText("");
	 	}
	 	if(questionNode.getAnswers().size()>1){
	 		answer2TextField.setText(questionNode.getAnswers().get(1).getAnswerText());
	 		//checks if this question is correct
	 		if(questionNode.getAnswers().get(1).isCorrectAnswer()){
	 			correctAnswer2.setSelected(true);
	 		}
	 	}
	 	else{
	 		answer2TextField.setText("");
	 	}
	 	if(questionNode.getAnswers().size()>2){
	 		answer3TextField.setText(questionNode.getAnswers().get(2).getAnswerText());
	 		//checks if this question is correct
	 		if(questionNode.getAnswers().get(2).isCorrectAnswer()){
	 			correctAnswer3.setSelected(true);
	 		}
	 	}
	 	else{
	 		answer3TextField.setText("");
	 	}
	 	if(questionNode.getAnswers().size()>3){
	 		answer4TextField.setText(questionNode.getAnswers().get(3).getAnswerText());
	 		//checks if this question is correct
	 		if(questionNode.getAnswers().get(3).isCorrectAnswer()){
	 			correctAnswer4.setSelected(true);
	 		}
	 	}
	 	else{
	 		answer4TextField.setText("");
	 	}
	 	// sets the correct radiobutton
	 	
	}
	
	/**
	 * Action performed method, used for buttons
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()== okButton){
	
			
			
			if(e.getActionCommand().equals(addString)){
				ArrayList<Answer> answers = new ArrayList<Answer>();
				
				for(int i =1;i<=numAns; i++){
						answers.add(new Answer(0,getSelectedRadio()==i,getAnswerText(i)));
				}
				Question question = new Question(0,answers,questionText.getText(),questionHeadingTextField.getText(),newTopicField.getText(),Integer.parseInt(questionLevelSpinner.getValue()+""),60);
				
				if(question.getQuestionText().equals("")){
					JOptionPane.showMessageDialog(this, "Please enter a name for the question");
					return;
				}
				StorageHandler.getInstance().newQuestion(question);
			}
			else if(e.getActionCommand().equals(saveString)){
				Question questionNode = (Question)((DefaultMutableTreeNode)
				questionTree.getLastSelectedPathComponent()).getUserObject();
			
				ArrayList<Answer> answers = new ArrayList<Answer>();
				
				for(int i =1;i<=numAns; i++){
						answers.add(new Answer(questionNode.getAnswers().get(i-1).getAnswerId(),getSelectedRadio()==i,getAnswerText(i)));
				}
				Question question = new Question(questionNode.getQuestionID(),answers,questionText.getText(),questionHeadingTextField.getText(),newTopicField.getText(),Integer.parseInt(questionLevelSpinner.getValue()+""),60);
				
				if(question.getQuestionText().equals("")){
					JOptionPane.showMessageDialog(this, "Please enter a name for the question");
					return;
				}
				StorageHandler.getInstance().updateQuestion(question);
			}
		}
		else if (e.getSource() == cancelButton){
			try{
			    DefaultMutableTreeNode node = (DefaultMutableTreeNode)
					questionTree.getLastSelectedPathComponent();
				Object nodeInfo = node.getUserObject();
				Question questionNode = (Question)nodeInfo;
				setAllFields(questionNode);
			}
			catch(Exception ex){
				resetAllFields();
			}
		}
	}
	
	/**
	 * 
	 * @return an int which corresponds to the correct answer
	 */
	private int getSelectedRadio(){
		if(correctAnswer1.isSelected()){
			return 1;
		}
		else if(correctAnswer2.isSelected()){
			return 2;
		}
		else if(correctAnswer3.isSelected()){
			return 3;
		}
		else if(correctAnswer4.isSelected()){
			return 4;
		}	
		return 0;
	}
	
	/**
	 * Checks if a topic exists
	 * @param topic - a topic
	 * @return true if a topic already exist, false otherwise
	 */
	private boolean doesTopicExsist(String topic){
		for(String t : topicList){
			if(topic.equals(t)){
				return true;
			}
		}
		return false;
	}
	/**
	 * returns the answer text
	 * @param i - the number of the answer
	 * @return the corresponing answerText
	 */
	private String getAnswerText(int i){
		switch (i) {
		case 1:
			return answer1TextField.getText(); 
		case 2:
			return answer2TextField.getText();
		case 3:
			return answer3TextField.getText();
		case 4:
			return answer4TextField.getText();
		default:
			return "";
		}
	}
	@Override
	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode()==KeyEvent.VK_DELETE){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)
				questionTree.getLastSelectedPathComponent();
			if (node == null){
				//Nothing is selected.	
				return;
			}
			
			Object nodeInfo = node.getUserObject();
			if (node.isLeaf()) {
		
				if(nodeInfo instanceof Question){
					Question questionNode = (Question)nodeInfo;
					int yesno = JOptionPane.showConfirmDialog(
						    this,
						    "Are you sure you want to delete question \""+nodeInfo.toString()+"\"?",
						    "Delete question confirmation",
						    JOptionPane.YES_NO_OPTION);
					if(yesno == JOptionPane.YES_OPTION){
						StorageHandler.getInstance().deleteQuestion(questionNode.getQuestionID());
					}
				}
				
			}
			
		}
		
	}
	@Override
	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

}