package editor.graphics.editorPanels;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.HashMap;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SpinnerNumberModel;
import javax.swing.SpringLayout;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import editor.StorageHandler;
import editor.graphics.EditorGUI;
import editor.graphics.ImageFactory;
import editor.models.changeListeners.EnemyListChangedListener;
import editor.models.changeListeners.FriendlyListChangedListener;
import editor.models.changeListeners.StaticObjectListChangedListener;

import shared.models.Attributes;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Gender;
import shared.models.StaticObject;

public class WorldObjectEditor extends JPanel implements ActionListener,
												TreeSelectionListener,
												EnemyListChangedListener,
												FriendlyListChangedListener,
												StaticObjectListChangedListener,
												ChangeListener,
												KeyListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//Statics
	private static String saveChanges = "Save changes";
	private static String createNewNPC = "Save new object to database";
	private static String ENEMY = "Enemy NPC";
	private static String FRIENDLY = "Friendly NPC";
	private static String STATICOBJECT = "Static objects";
	//Model
	private HashMap<String,EnemyNPC> eNpcList;
	private HashMap<String,FriendlyNPC> fNpcList;
	private HashMap<String,StaticObject> sObjList;
	private int npcId;
	//**GUI Components**
	private SpringLayout layout;
	private JTree npcTree;
	private DefaultTreeModel npcTreeModel;
	private JButton saveNPCButton;
	private JButton cancelButton;
	private JButton deleteButton;
	private JPanel mainFieldsPanel;
	private JPanel priAttPanel;
	private JPanel statsPanel;
	private JPanel imageSelectionPanel;
	private JPanel enemyPanel;
	//Main fields
	private JTextField fieldName;
	private JRadioButton radFemale;
	private JRadioButton radMale;
	private JTextField fieldmaxHP;
	private JTextField fieldmaxMP;
	private JTextField fieldSleepTime;
	private JTextField fieldAgressive;
	private JTextField fieldXP;
	private JSpinner fieldLVL;
	
	//ImagePanel
	private JTextField ImagePath;
	private JLabel imageIcon;
	private JButton browseButton;
	//Primary Attributes
	private JTextField fieldSTR;
	private JTextField fieldINT;
	private JTextField fieldDEX;
	private JTextField fieldCON;
	
	
	//private JList allPhysicalAtt;
	//private JList allMagicalAtt;
	//Defense and Offensive attributes
	private JTextField fieldPiercingDef;
	private JTextField fieldBluntDef;
	private JTextField fieldSlashingDef;
	private JTextField fieldFireDef;
	private JTextField fieldWaterDef;
	private JTextField fieldEarthDef;
	private JTextField fieldAirDef;
	private JTextField fieldPiercingOff;
	private JTextField fieldBluntOff;
	private JTextField fieldSlashingOff;
	private JTextField fieldFireOff;
	private JTextField fieldWaterOff;
	private JTextField fieldEarthOff;
	private JTextField fieldAirOff;
	
	
	public WorldObjectEditor(){
		layout = new SpringLayout();
		this.setLayout(layout);
		this.npcId = 0;
		//Initialize JTree
		
		DefaultMutableTreeNode topNode = new DefaultMutableTreeNode("World Objects");
		
		npcTreeModel = new DefaultTreeModel(topNode);
		npcTreeModel.insertNodeInto(new DefaultMutableTreeNode(ENEMY),(MutableTreeNode)npcTreeModel.getRoot(), 0);
		npcTreeModel.insertNodeInto(new DefaultMutableTreeNode(FRIENDLY),(MutableTreeNode)npcTreeModel.getRoot(), 1);
		npcTreeModel.insertNodeInto(new DefaultMutableTreeNode(STATICOBJECT),(MutableTreeNode)npcTreeModel.getRoot(),2);
		npcTree = new JTree(npcTreeModel);
		npcTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		npcTree.setSelectionRow(0);
		npcTree.addTreeSelectionListener(this);
		JScrollPane treeScroll = new JScrollPane(npcTree);
		treeScroll.setPreferredSize(new Dimension(300,650));
		treeScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		treeScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		place(5,1,treeScroll);
		
		//Initialize components
		initializeComponents();
		place(700,22,saveNPCButton);
		place(500,22,cancelButton);
		place(600,22,deleteButton);
		
		mainFieldsPanel = new JPanel();
		mainFieldsPanel.setLayout(new SpringLayout());
		mainFieldsPanel.setPreferredSize(new Dimension(250,120));
		
		fieldSleepTime.setToolTipText("This decides how many seconds between each attack.");
		//Add components to mainFieldsPanel
	
		//Row 1
		place(5,1,new JLabel("Name"),mainFieldsPanel);
		place(40,1,fieldName,mainFieldsPanel);
		//Row 2
		place(5,2,new JLabel("Gender"),mainFieldsPanel);
		place(50,2,radFemale,mainFieldsPanel);
		place(115,2,radMale,mainFieldsPanel);
		//Row 3
		place(5,3,new JLabel("MaxHP"),mainFieldsPanel);
		place(70,3,fieldmaxHP,mainFieldsPanel);
		place(125,3,new JLabel("MaxMP"),mainFieldsPanel);
		place(190,3,fieldmaxMP,mainFieldsPanel);
	
		place(5,4,new JLabel("Level"),mainFieldsPanel);
		place(70,4,fieldLVL,mainFieldsPanel);
	
		enemyPanel = new JPanel();
		enemyPanel.setLayout(new SpringLayout());
		enemyPanel.setPreferredSize(new Dimension(250,60));

		//Row 4
		place(5,1,new JLabel("Sleeptime"),enemyPanel);
		place(70,1,fieldSleepTime,enemyPanel);
		place(125,1,new JLabel("Agressive"),enemyPanel);
		place(190,1,fieldAgressive,enemyPanel);
		//Row 5
		place(5,2,new JLabel("XP reward"),enemyPanel);
		place(70,2,fieldXP,enemyPanel);
		
		
		//Primary Attributes
		priAttPanel = new JPanel();
		priAttPanel.setLayout(new SpringLayout());
		priAttPanel.setPreferredSize(new Dimension(250,150));
	
		place(5,1,new JLabel("Primary Attributes"),priAttPanel);
		place(5,2,new JLabel("Strenght"),priAttPanel);
		place(85,2,fieldSTR,priAttPanel);
		place(5,3,new JLabel("Constitution"),priAttPanel);
		place(85,3,fieldCON,priAttPanel);
		place(5,4,new JLabel("Dexterity"),priAttPanel);
		place(85,4,fieldDEX,priAttPanel);
		place(5,5,new JLabel("Intelligence"),priAttPanel);
		place(85,5,fieldINT,priAttPanel);
		
		
		//Defensive Stats
		statsPanel = new JPanel();
		statsPanel.setLayout(new SpringLayout());
		statsPanel.setPreferredSize(new Dimension(400,250));
	
		place(5,1,new JLabel("Defensive stats"),statsPanel);
		place(5,2,new JLabel("Piercing"),statsPanel);
		place(65,2,fieldPiercingDef,statsPanel);
		place(5,3,new JLabel("Blunt"),statsPanel);
		place(65,3,fieldBluntDef,statsPanel);
		place(5,4,new JLabel("Slashing"),statsPanel);
		place(65,4,fieldSlashingDef,statsPanel);
		place(5,5,new JLabel("Fire"),statsPanel);
		place(65,5,fieldFireDef,statsPanel);
		place(5,6,new JLabel("Water"),statsPanel);
		place(65,6,fieldWaterDef,statsPanel);
		place(5,7,new JLabel("Earth"),statsPanel);
		place(65,7,fieldEarthDef,statsPanel);
		place(5,8,new JLabel("Air"),statsPanel);
		place(65,8,fieldAirDef,statsPanel);
		
		//Offensive stats
		place(205,1,new JLabel("Offensive stats"),statsPanel);
		place(205,2,new JLabel("Piercing"),statsPanel);
		place(265,2,fieldPiercingOff,statsPanel);
		place(205,3,new JLabel("Blunt"),statsPanel);
		place(265,3,fieldBluntOff,statsPanel);
		place(205,4,new JLabel("Slashing"),statsPanel);
		place(265,4,fieldSlashingOff,statsPanel);
		place(205,5,new JLabel("Fire"),statsPanel);
		place(265,5,fieldFireOff,statsPanel);
		place(205,6,new JLabel("Water"),statsPanel);
		place(265,6,fieldWaterOff,statsPanel);
		place(205,7,new JLabel("Earth"),statsPanel);
		place(265,7,fieldEarthOff,statsPanel);
		place(205,8,new JLabel("Air"),statsPanel);
		place(265,8,fieldAirOff,statsPanel);
		
		imageSelectionPanel = new JPanel();
		imageSelectionPanel.setLayout(new SpringLayout());
		imageSelectionPanel.setPreferredSize(new Dimension(400,250));
		
		place(5,1,new JLabel("ImageName"),imageSelectionPanel);
		place(100,1,ImagePath,imageSelectionPanel);
		place(285,1,browseButton,imageSelectionPanel);
		place (5,2,imageIcon,imageSelectionPanel);
		
		
		//Place panels
		place(305,1,mainFieldsPanel);
		place(305,5,enemyPanel);
		place(305,7,priAttPanel);
		place(305,13,statsPanel);
		place(600,3,imageSelectionPanel);
		
		
		clearDisplayOfFields();
	}
	@Override
	public void updatedEnemyNPCList(HashMap<String, EnemyNPC> npcList) {
		eNpcList = npcList;
		npcTree.setSelectionRow(0);
		DefaultMutableTreeNode enemyNode = (DefaultMutableTreeNode)((DefaultMutableTreeNode)npcTreeModel.getRoot()).getChildAt(0);
		//Clear node
		while(enemyNode.getChildCount()>0){
			npcTreeModel.removeNodeFromParent((DefaultMutableTreeNode)enemyNode.getChildAt(0));
		}
		for(String npcName: npcList.keySet()){
			npcTreeModel.insertNodeInto(new DefaultMutableTreeNode(npcName), enemyNode, 0);
		}
		
	}
	@Override
	public void updatedFriendlyNPCList(HashMap<String, FriendlyNPC> npcList) {
		fNpcList = npcList;
		npcTree.setSelectionRow(0);
		DefaultMutableTreeNode friendlyNode = (DefaultMutableTreeNode)((DefaultMutableTreeNode)npcTreeModel.getRoot()).getChildAt(1);
		//Clear node
		while(friendlyNode.getChildCount()>0){
			npcTreeModel.removeNodeFromParent((DefaultMutableTreeNode)friendlyNode.getChildAt(0));
		}
		for(String npcName: npcList.keySet()){
			npcTreeModel.insertNodeInto(new DefaultMutableTreeNode(npcName), friendlyNode, 0);
		}
		
	}
	@Override
	public void updatedStaticObjectList(HashMap<String, StaticObject> objList) {
		sObjList = objList;
		npcTree.setSelectionRow(0);
		DefaultMutableTreeNode sObjNode = (DefaultMutableTreeNode)((DefaultMutableTreeNode)npcTreeModel.getRoot()).getChildAt(2);
		//Clear node
		while(sObjNode.getChildCount()>0){
			npcTreeModel.removeNodeFromParent((DefaultMutableTreeNode)sObjNode.getChildAt(0));
		}
		for(String npcName: objList.keySet()){
			npcTreeModel.insertNodeInto(new DefaultMutableTreeNode(npcName), sObjNode, 0);
		}
	}
	/**
	 * Helper method for reseting the fields for a new NPC.
	 */
	private void clearFieldsForNewNPC(){
		//ATTRIBUTES
		fieldmaxHP.setText("0");
		fieldmaxMP.setText("0");
		fieldSTR.setText("0");
		fieldINT.setText("0");
		fieldDEX.setText("0");
		fieldCON.setText("0");
		fieldPiercingDef.setText("0");
		fieldBluntDef.setText("0");
		fieldSlashingDef.setText("0");
		fieldFireDef.setText("0");
		fieldWaterDef.setText("0");
		fieldEarthDef.setText("0");
		fieldAirDef.setText("0");
		fieldPiercingOff.setText("0");
		fieldBluntOff.setText("0");
		fieldSlashingOff.setText("0");
		fieldFireOff.setText("0");
		fieldWaterOff.setText("0");
		fieldEarthOff.setText("0");
		fieldAirOff.setText("0");
		//MainFields
		fieldName.setText("");
		radFemale.setSelected(true);
		fieldLVL.setValue(1);
		ImagePath.setText("");
		imageIcon.setIcon(null);
		fieldSleepTime.setText("30");
		fieldXP.setText("0");
		fieldAgressive.setText("0");
	}
/**
 * Helper method for setting the attributes of a NPC after level.
 * @param level
 */
	private void setAttributesByLevel(int level){
		fieldSTR.setText(level*2+"");
		fieldINT.setText(level+"");
		fieldDEX.setText(level+"");
		fieldCON.setText(level*2+"");
		fieldPiercingDef.setText(level+"");
		fieldBluntDef.setText(level+"");
		fieldSlashingDef.setText(level+"");
		fieldFireDef.setText(level+"");
		fieldWaterDef.setText(level+"");
		fieldEarthDef.setText(level+"");
		fieldAirDef.setText(level+"");
		fieldPiercingOff.setText(level+"");
		fieldBluntOff.setText(level+"");
		fieldSlashingOff.setText(level+"");
		fieldFireOff.setText(level+"");
		fieldWaterOff.setText(level+"");
		fieldEarthOff.setText(level+"");
		fieldAirOff.setText(level+"");
	}
	/**
	 * Initializes the components.
	 */
	private void initializeComponents(){
		saveNPCButton = new JButton("Save Changes");
		saveNPCButton.addActionListener(this);
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		deleteButton = new JButton("Delete");
		deleteButton.addActionListener(this);
		browseButton = new JButton("Browse...");
		browseButton.addActionListener(this);
		fieldName = new JTextField(15);
		radFemale = new JRadioButton("Female");
		radFemale.setSelected(true);
		radMale= new JRadioButton("Male");
		ButtonGroup sexGroup = new ButtonGroup();
		sexGroup.add(radFemale);
		sexGroup.add(radMale);
		//allMagicalAtt = new JList();
		//allPhysicalAtt = new JList();
		
		fieldmaxHP = new JTextField(4);
		fieldmaxMP = new JTextField(4);
		fieldSleepTime = new JTextField(4);
		fieldAgressive = new JTextField(4);
		fieldXP = new JTextField(4);
		fieldLVL = new JSpinner(new SpinnerNumberModel(1,1,20,1));
		fieldLVL.addChangeListener(this);
		//Imagepanel
		ImagePath  = new JTextField(15);
		ImagePath.addKeyListener(this);
		imageIcon = new JLabel();
		
		
		//Primary Attributes
		fieldSTR = new JTextField(4);
		fieldINT = new JTextField(4);
		fieldDEX = new JTextField(4);
		fieldCON = new JTextField(4);
		
		//Defense and Offensive attributes
		fieldPiercingDef = new JTextField(3);
		fieldBluntDef = new JTextField(3);
		fieldSlashingDef = new JTextField(3);
		fieldFireDef = new JTextField(3);
		fieldWaterDef = new JTextField(3);
		fieldEarthDef = new JTextField(3);
		fieldAirDef = new JTextField(3);
		fieldPiercingOff = new JTextField(3);
		fieldBluntOff = new JTextField(3);
		fieldSlashingOff = new JTextField(3);
		fieldFireOff = new JTextField(3);
		fieldWaterOff = new JTextField(3);
		fieldEarthOff = new JTextField(3);
		fieldAirOff = new JTextField(3);
	}
	
	/**
	 * HelperMethod for updating the fields according to NPC values.
	 * @param npc
	 */
	private void updateCharacterFields(shared.models.Character npc){
		//ATTRIBUTES
		Attributes attr = npc.getAttribute();
		fieldmaxHP.setText(attr.getMaxHP()+"");
		fieldmaxMP.setText(attr.getMaxMP()+"");
		fieldSTR.setText(attr.getSTR()+"");
		fieldINT.setText(attr.getINT()+"");
		fieldDEX.setText(attr.getDEX()+"");
		fieldCON.setText(attr.getCON()+"");
		fieldPiercingDef.setText(attr.getPiercingDef()+"");
		fieldBluntDef.setText(attr.getBluntDef()+"");
		fieldSlashingDef.setText(attr.getSlashingDef()+"");
		fieldFireDef.setText(attr.getFireDef()+"");
		fieldWaterDef.setText(attr.getWaterDef()+"");
		fieldEarthDef.setText(attr.getEarthDef()+"");
		fieldAirDef.setText(attr.getAirDef()+"");
		fieldPiercingOff.setText(attr.getPiercingOff()+"");
		fieldBluntOff.setText(attr.getBluntOff()+"");
		fieldSlashingOff.setText(attr.getSlashingOff()+"");
		fieldFireOff.setText(attr.getFireOff()+"");
		fieldWaterOff.setText(attr.getWaterOff()+"");
		fieldEarthOff.setText(attr.getEarthOff()+"");
		fieldAirOff.setText(attr.getAirOff()+"");
		//MainFields
		fieldName.setText(npc.getName());
		if(npc.getSex()==Gender.FEMALE){
			radFemale.setSelected(true);
		}else{
			radMale.setSelected(true);
		}
		fieldLVL.setValue(npc.getLVL());
		
	}
	
	private void setImageIcon(String imagePath){
		if(imagePath!=null){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)
				npcTree.getLastSelectedPathComponent();
			if (node == null)
				//Nothing is selected.	
				return;
			String nodeInfo = node.getUserObject().toString();
			
			if (node.isLeaf()) {
				if(node.getParent().toString().equals(ENEMY)){
					imageIcon.setIcon(new ImageIcon(ImageFactory.getInstance().getEnemyImageByImageName(imagePath)));
				}else if(node.getParent().toString().equals(FRIENDLY)){
					imageIcon.setIcon(new ImageIcon(ImageFactory.getInstance().getFriendlyImageByImageName(imagePath)));
				}else if(node.getParent().toString().equals(STATICOBJECT)){
					imageIcon.setIcon(new ImageIcon(ImageFactory.getInstance().getStaticObjectImageByImageName(imagePath)));
				
				}
				
			}else if(node.isRoot()){
				
			}else{
				if(nodeInfo.equals(ENEMY)){
					imageIcon.setIcon(new ImageIcon(ImageFactory.getInstance().getEnemyImageByImageName(imagePath)));
					
				}else if(nodeInfo.equals(FRIENDLY)){
					imageIcon.setIcon(new ImageIcon(ImageFactory.getInstance().getFriendlyImageByImageName(imagePath)));
				}else if(nodeInfo.equals(STATICOBJECT)){
					imageIcon.setIcon(new ImageIcon(ImageFactory.getInstance().getStaticObjectImageByImageName(imagePath)));
				}
			}
		}
	}
	/**
	 * Helper method for updatingFriendlyNPC fields.
	 * TODO: currently no friendly fields.
	 * @param npc
	 */
	private void updateFriendlyFields(FriendlyNPC npc){
		//Currently no friendly fields..
	}
	/**
	 * Helper method for updating EnemyNPC fields
	 * @param npc
	 */
	private void updateEnemyFields(EnemyNPC npc){
		fieldSleepTime.setText(npc.getSleepTime()+"");
		fieldXP.setText(npc.getXP()+"");
		fieldAgressive.setText(npc.getLevelOfAgressivenes()+"");
	}
	/**
	 * Shows fields used for EnemyNPC.
	 */
	private void showEnemyFields(){
		mainFieldsPanel.setVisible(true);
		showNPCMainFields();
		priAttPanel.setVisible(true);
		statsPanel.setVisible(true);
		imageSelectionPanel.setVisible(true);
		enemyPanel.setVisible(true);
		saveNPCButton.setVisible(true);
		cancelButton.setVisible(true);
	}
	/**
	 * Shows fields used for FriendlyNPC.
	 */
	private void showFriendlyFields(){
		mainFieldsPanel.setVisible(true);
		showNPCMainFields();
		priAttPanel.setVisible(true);
		statsPanel.setVisible(true);
		imageSelectionPanel.setVisible(true);
		enemyPanel.setVisible(false);
		saveNPCButton.setVisible(true);
		cancelButton.setVisible(true);
	}
	/**
	 * Shows Fields used for StaticObject.
	 * 
	 */
	private void showStaticObjectFields(){
		mainFieldsPanel.setVisible(true);
		hideNPCMainFields();
		priAttPanel.setVisible(false);
		statsPanel.setVisible(false);
		imageSelectionPanel.setVisible(true);
		enemyPanel.setVisible(false);
		saveNPCButton.setVisible(true);
		cancelButton.setVisible(true);
	}
	/**
	 * Hide NPCMainFields.
	 */
	private void hideNPCMainFields(){
		mainFieldsPanel.setPreferredSize(new Dimension(250,30));
	}
	/**
	 * ShowNPCMainFields.
	 */
	private void showNPCMainFields(){
		mainFieldsPanel.setPreferredSize(new Dimension(250,120));
	}
	/**
	 * Hides all inputs.
	 */
	private void clearDisplayOfFields(){
		mainFieldsPanel.setVisible(false);
		hideNPCMainFields();
		priAttPanel.setVisible(false);
		statsPanel.setVisible(false);
		imageSelectionPanel.setVisible(false);
		enemyPanel.setVisible(false);
		saveNPCButton.setVisible(false);
		cancelButton.setVisible(false);
		deleteButton.setVisible(false);
	}
	/**
	 * Sends new enemyNPC to StorageHandler for saving.
	 */
	private void sendNewEnemyNPC(){
		StorageHandler.getInstance().newENPC(getEnemyFromFields());
	}
	/**
	 * Sends new FriendlyNPC to StorageHandler for saving.
	 */
	private void sendNewFriendlyNPC(){
		StorageHandler.getInstance().newFNPC(getFriendlyFromFields());
	}
	/**
	 * Sends new StaticObject to StorageHandler for saving.
	 */
	private void sendNewStaticObject(){
		StorageHandler.getInstance().newStaticObject(getStaticObjectFromFields());
	}
	/**
	 * 
	 * Sends EnemyNPC to storageHandler for updating.
	 */
	private void updateEnemyNPCInDB(){
		EnemyNPC npc = getEnemyFromFields();
		npc.setEnemyNPCId(npcId);
		StorageHandler.getInstance().updateEnemyNPC(npc);
	}
	/**
	 * Sends FriendlyNPC to storageHandler for updating.
	 */
	private void updateFriendlyNPCInDB(){
		FriendlyNPC npc = getFriendlyFromFields();
		npc.setFriendlyNPCId(npcId);
		StorageHandler.getInstance().updateFriendlyNPC(npc);
	}
	/**
	 * Sends StaticObject to storageHandler for updating.
	 */
	private void updateStaticObjectInDB(){
		StaticObject obj = getStaticObjectFromFields();
		obj.setStaticObjectId(npcId);
		StorageHandler.getInstance().updateStaticObject(obj);
	}
	/**
	 * Sends id of selected EnemyNPC to StorageHandler for deletion.
	 */
	private void deleteEnemyNPC(){
		StorageHandler.getInstance().deleteEnemyNPC(npcId);
	}
	/**
	 * Sends id of selected FriendlyNPC to StorageHandler for deletion.
	 */
	private void deleteFriendlyNPC(){
		StorageHandler.getInstance().deleteFriendlyNPC(npcId);
	}
	/**
	 * Sends id of selected StaticObject to StorageHandler for deletion.
	 */
	private void deleteStaticObject(){
		StorageHandler.getInstance().deleteStaticObject(npcId);
	}
	/**
	 * Creates an EnemyNPC object from the input fields.
	 * @return
	 */
	private EnemyNPC getEnemyFromFields(){
		Gender gen;
		if(radMale.isSelected()){
			gen = Gender.MALE;
		}else{
			gen= Gender.FEMALE;
		}
		
		return new EnemyNPC(fieldName.getText(),gen,(Integer)fieldLVL.getValue(),attFromFields(),getIntFromField(fieldAgressive),getIntFromField(fieldSleepTime),getIntFromField(fieldXP),null,null,ImagePath.getText());
	}
	/**
	 * Creates a FriendlyNPC object from the input fields.
	 * @return
	 */
	private FriendlyNPC getFriendlyFromFields(){
		Gender gen;
		if(radMale.isSelected()){
			gen = Gender.MALE;
		}else{
			gen= Gender.FEMALE;
		}
		return new FriendlyNPC(fieldName.getText(),gen,(Integer)fieldLVL.getValue(),attFromFields(),null,null,null,null,null,null,ImagePath.getText());
	}
	/**
	 * Creates a StaticObject from the input fields.
	 * @return
	 */
	private StaticObject getStaticObjectFromFields(){
		return new StaticObject(fieldName.getText(),ImagePath.getText(),null);
	}
	/**
	 * Gets Attributes from input fields.
	 * @return
	 */
	private Attributes attFromFields(){
		return new Attributes(getIntFromField(fieldmaxHP),getIntFromField(fieldmaxMP),getIntFromField(fieldSTR),getIntFromField(fieldINT),getIntFromField(fieldDEX),getIntFromField(fieldCON),getIntFromField(fieldmaxHP),getIntFromField(fieldmaxMP),getIntFromField(fieldPiercingDef),getIntFromField(fieldBluntDef),getIntFromField(fieldSlashingDef),getIntFromField(fieldFireDef),getIntFromField(fieldWaterDef),getIntFromField(fieldEarthDef),getIntFromField(fieldAirDef),getIntFromField(fieldPiercingOff),getIntFromField(fieldBluntOff),getIntFromField(fieldSlashingOff),getIntFromField(fieldFireOff),getIntFromField(fieldWaterOff),getIntFromField(fieldEarthOff),getIntFromField(fieldAirOff));
	}
	/**
	 * Parses text in textfield to integer.
	 * @param field
	 * @return
	 */
	private int getIntFromField(JTextField field){
		return Integer.parseInt(field.getText());
	}
	/**
	 * Helper method for placing components in EditorPanel.
	 * @param x
	 * @param rowY
	 * @param comp
	 */
	private void place(int x,int rowY,Component comp){
		layout.putConstraint(SpringLayout.WEST,comp,
							x,
							SpringLayout.WEST,this);
		layout.putConstraint(SpringLayout.NORTH, comp,
							(rowY*30-25),
							SpringLayout.NORTH, this);
		this.add(comp);
	}
	/**
	 * Helper method for placing components in given JPanel.
	 * @param x
	 * @param rowY
	 * @param target
	 * @param inPanel
	 */
	private void place(int x, int rowY,Component target, JPanel inPanel){
		SpringLayout panelLayout = (SpringLayout)inPanel.getLayout();
		panelLayout.putConstraint(SpringLayout.WEST,target,
					x,
					SpringLayout.WEST,inPanel);
		panelLayout.putConstraint(SpringLayout.NORTH, target,
					(rowY*30-25),
					SpringLayout.NORTH, inPanel);
		inPanel.add(target);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==cancelButton){
			DefaultMutableTreeNode root = (DefaultMutableTreeNode)npcTreeModel.getRoot();
			while(root.getChildCount()>0){
				npcTreeModel.removeNodeFromParent((DefaultMutableTreeNode)root.getChildAt(0));
			}
		}else if(e.getSource()==saveNPCButton){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)
			npcTree.getLastSelectedPathComponent();
			if(saveNPCButton.getText().equals(saveChanges)){
				 String parentName = node.getParent().toString();
				if(parentName.equals(ENEMY)){
					updateEnemyNPCInDB();
				}else if(parentName.equals(FRIENDLY)){
					updateFriendlyNPCInDB();
				}else if(parentName.equals(STATICOBJECT)){
					updateStaticObjectInDB();
				}
				
			}else if(saveNPCButton.getText().equals(createNewNPC)){
				if(fieldName.getText().equals("")||fieldName.getText().isEmpty()){
					JOptionPane.showMessageDialog(this, "Please enter a name for the object.");

				}else{
					String nodeName = node.getUserObject().toString();
					if(nodeName.toString().equals(ENEMY)){
						sendNewEnemyNPC();
					}else if(nodeName.toString().equals(FRIENDLY)){
						sendNewFriendlyNPC();
					}else if(nodeName.toString().equals(STATICOBJECT)){
						sendNewStaticObject();
					}
				}
			}
			
		}else if(e.getSource()==deleteButton){
			 int response;
			 DefaultMutableTreeNode npcNode = (DefaultMutableTreeNode)npcTree.getLastSelectedPathComponent();
			 String npcName = npcNode.toString();
			 response = JOptionPane.showConfirmDialog(null, "Are you sure you want to delete "+npcName+"?");
			 System.out.println(response);
			 // 0 = yes , 1 = no, 2 = cancel
			 if(response == 0){
				 if(npcNode.getParent().toString().equals(ENEMY)){
					 deleteEnemyNPC();
				 }else if(npcNode.getParent().toString().equals(FRIENDLY)){
					 deleteFriendlyNPC(); 
				 }else if(npcNode.getParent().toString().equals(STATICOBJECT)){
					 deleteStaticObject();
				 }
			 }
		}else if(e.getSource()==browseButton){
			String imageName = EditorGUI.guiControls().browseForImage();
			if(imageName!=null){
				ImagePath.setText(imageName);
				setImageIcon(imageName);
			}
		}
	
		
	}
		
	@Override
	public void valueChanged(TreeSelectionEvent e) {
		 DefaultMutableTreeNode node = (DefaultMutableTreeNode)
			npcTree.getLastSelectedPathComponent();
		if (node == null)
			//Nothing is selected.	
			return;
		String nodeInfo = node.getUserObject().toString();
		if (node.isLeaf()) {
			if(node.getParent().toString().equals(ENEMY)){
				updateCharacterFields(eNpcList.get(nodeInfo));
				updateEnemyFields(eNpcList.get(nodeInfo));
				ImagePath.setText(eNpcList.get(nodeInfo).getImagePath());
				setImageIcon(eNpcList.get(nodeInfo).getImagePath());
				saveNPCButton.setText(saveChanges);
				this.npcId = eNpcList.get(nodeInfo).getEnemyNPCId();
				showEnemyFields();
				deleteButton.setVisible(true);
			}else if(node.getParent().toString().equals(FRIENDLY)){
				updateCharacterFields(fNpcList.get(nodeInfo));
				updateFriendlyFields(fNpcList.get(nodeInfo));
				ImagePath.setText(fNpcList.get(nodeInfo).getImagePath());
				setImageIcon(fNpcList.get(nodeInfo).getImagePath());
				saveNPCButton.setText(saveChanges);
				this.npcId = fNpcList.get(nodeInfo).getFriendlyNPCId();
				showFriendlyFields();
				deleteButton.setVisible(true);
			}else if(node.getParent().toString().equals(STATICOBJECT)){
				StaticObject obj = sObjList.get(nodeInfo);
				fieldName.setText(obj.getName());
				ImagePath.setText(obj.getTextureId());
				setImageIcon(obj.getTextureId());
				saveNPCButton.setText(saveChanges);
				this.npcId = obj.getStaticObjectId();
				showStaticObjectFields();
				deleteButton.setVisible(true);
			}else if(nodeInfo.equals(FRIENDLY)){
				makeReadyForNewFriendly();
			}else if(nodeInfo.equals(ENEMY)){
				makeReadyForNewEnemy();
			}else if(nodeInfo.equals(STATICOBJECT)){
				makeReadyForNewStaticObject();
			}
		}
		else if(node.isRoot()){
			clearDisplayOfFields();
		}
		else {
			if(nodeInfo.equals(ENEMY)){
				makeReadyForNewEnemy();
				
			}else if(nodeInfo.equals(FRIENDLY)){
				makeReadyForNewFriendly();
			}else if(nodeInfo.equals(STATICOBJECT)){
				makeReadyForNewStaticObject();
			}
		}
	}
	/**
	 * Makes input fields ready for new EnemyNPC.
	 */
	private void makeReadyForNewEnemy(){
		clearFieldsForNewNPC();
		saveNPCButton.setText(createNewNPC);
		this.npcId = 0;
		showEnemyFields();
		deleteButton.setVisible(false);
	}
	/**
	 * Makes input fields ready for new FriendlyNPC.
	 */
	private void makeReadyForNewFriendly(){
		//Can create new friendly
		clearFieldsForNewNPC();
		saveNPCButton.setText(createNewNPC);
		this.npcId = 0;
		showFriendlyFields();
		deleteButton.setVisible(false);
	}
	/**
	 * Makes input fields ready for new StaticObject.
	 */
	private void makeReadyForNewStaticObject(){
		clearFieldsForNewNPC();
		saveNPCButton.setText(createNewNPC);
		this.npcId = 0;
		showStaticObjectFields();
		deleteButton.setVisible(false);
	}
	@Override
	public void stateChanged(ChangeEvent e) {
		if(e.getSource() == fieldLVL){
			setAttributesByLevel((Integer)fieldLVL.getValue());
		}
	}
	@Override
	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void keyReleased(KeyEvent e) {
		if(e.getSource()==ImagePath){
				setImageIcon(ImagePath.getText());
			
		}
		
	}
	@Override
	public void keyTyped(KeyEvent e) {
	
		
	}

	
}
