package editor.graphics.editorPanels.mapEditorPanels;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SpinnerNumberModel;
import javax.swing.SpringLayout;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.StaticObject;
import shared.models.TravelTrigger;
import shared.models.WorldObject;
import shared.models.Zone;

import editor.EditorMain;
import editor.StorageHandler;
import editor.graphics.EditorGUI;
/**
 * Shows a Tree structure of all zones and objects in the current Kingdom in mapeditor.
 * 
 * @author thorgrun
 *
 */
public class MapObjects extends JPanel implements ActionListener,KeyListener, TreeSelectionListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//String constants
	private static String ENEMYNPC = "Enemy NPC";
	private static String FRIENDLYNPC = "Friendly NPC";
	private static String STATICOBJECT = "Static objects";
	private static String TRAVELTRIGGER = "Travel trigger";
	private static String EMPTYSELECTION = "No destination";
	private static String ADDZONE = "Create zone";
	private static String ADDTRAVEL = "Create travel";
	private static String LOADZONE = "Load zone";
	private static String SAVEZONE = "Save zone";

	private JButton addButton;
	private JButton removeButton;
	private DefaultTreeModel treeModel;
	private JTree worldTree;
	private SpringLayout layout;
	private ObjectSelectedListener listener;
	private String selectedZone;
	private JPanel propertiesPanel;
	
	//Properties panel components
	private JTextField nameField;
	private JLabel nameLbl;
	private JSpinner widthField;
	private JLabel widthLbl;
	private JSpinner heightField;
	private JLabel heightLbl;
	private JComboBox comboBox;//For toTrigger and themetopic
	private JLabel selectLbl;
	private JButton saveChanges;
	private String[] topics;
	private String[] zoneNames;
	private ArrayList<TravelTrigger> triggers;
	private SelectionType selectedType;
	private WorldObject selectedObject;
	/**
	 * Enum for which type of object is selected in the tree structure.
	 * @author thorgrun
	 *
	 */
	private enum SelectionType{
		ENEMY_NPC,
		FRIENDLY_NPC,
		STATIC_OBJECT,
		ZONE,
		TRAVEL_TRIGGER
	}
	/**
	 * Constructor.
	 * 
	 * @param listener ObjectSelectedListener
	 */
	public MapObjects(ObjectSelectedListener listener){
		layout = new SpringLayout();
		triggers = new ArrayList<TravelTrigger>();
		setLayout(layout);
		this.listener = listener;
		addButton = new JButton("Create new");
		addButton.addActionListener(this);
		
		removeButton = new JButton("Delete");
		removeButton.addActionListener(this);
		
		placeAtCoordinates(5,5,addButton);
		placeAtCoordinates(120,5,removeButton);
		treeModel = new DefaultTreeModel(new DefaultMutableTreeNode("Kingdom"));
		
		add(addButton);
		add(removeButton);
		setPreferredSize(new Dimension(200,700));

		worldTree = new JTree(treeModel);
		worldTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		worldTree.setSelectionRow(0);
		worldTree.addTreeSelectionListener(this);
		worldTree.addKeyListener(this);
		worldTree.setExpandsSelectedPaths(true);
		
		JScrollPane scroll = new JScrollPane(worldTree);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scroll.setPreferredSize(new Dimension(190,400));
		placeAtCoordinates(5,40,scroll);

		
		propertiesPanel = new JPanel();
		propertiesPanel.setLayout(new SpringLayout());
		propertiesPanel.setPreferredSize(new Dimension(190,200));
		nameField = new JTextField(10);
		widthField = new JSpinner(new SpinnerNumberModel(10,10,50,1));
		heightField = new JSpinner(new SpinnerNumberModel(10,10,50,1));;
		comboBox = new JComboBox();
		saveChanges = new JButton("Save changes");
		saveChanges.addActionListener(this);
		nameLbl = new JLabel("Name:");
		selectLbl = new JLabel("Select:");
		widthLbl = new JLabel("Width:");
		heightLbl = new JLabel("Height:");
		place(5,1,nameLbl,propertiesPanel);
		place(50,1,nameField,propertiesPanel);
		place(5,2,selectLbl,propertiesPanel);
		place(50,2,comboBox,propertiesPanel);
		place(5,3,widthLbl,propertiesPanel);
		place(50,3,widthField,propertiesPanel);
		place(5,4,heightLbl,propertiesPanel);
		place(50,4,heightField,propertiesPanel);
		place(15,5,saveChanges,propertiesPanel);
		placeAtCoordinates(0,445,propertiesPanel);
		add(propertiesPanel);
		propertiesPanel.setVisible(false);
		add(scroll);
	}
	/**
	 * HelperMethod for placing components in MapObjects panel.
	 * @param x
	 * @param y
	 * @param comp
	 */
	private void placeAtCoordinates(int x,int y,Component comp){
		layout.putConstraint(SpringLayout.WEST,comp,
							x,
							SpringLayout.WEST,this);
		layout.putConstraint(SpringLayout.NORTH, comp,
							y,
							SpringLayout.NORTH, this);
	}
	/**
	 * Helper method for placing components in given JPanel.
	 * @param x
	 * @param rowY
	 * @param target
	 * @param inPanel
	 */
	private void place(int x, int rowY,Component target, JPanel inPanel){
		SpringLayout panelLayout = (SpringLayout)inPanel.getLayout();
		panelLayout.putConstraint(SpringLayout.WEST,target,
					x,
					SpringLayout.WEST,inPanel);
		panelLayout.putConstraint(SpringLayout.NORTH, target,
					(rowY*30-25),
					SpringLayout.NORTH, inPanel);
		inPanel.add(target);
	}
	/**
	 * Updates the topiclist used by EnemyNPCs.
	 */
	public void updateTopicList(ArrayList<String> topicList){
		topics = new String[topicList.size()];
		for(int i = 0;i<topicList.size();i++){
			topics[i] = topicList.get(i);
		}
	}
	/**
	 * updates the names of the zone templates.
	 * @param zoneNameList
	 */
	public void updateZoneNameList(String[] zoneNameList){
		this.zoneNames = zoneNameList;
	}
	/**
	 * Displays the Properties of the selected EnemyNPC
	 * @param npc
	 */
	private void viewEnemyProperties(EnemyNPC npc){
		nameField.setText(npc.getName());
		nameField.setEnabled(false);
		widthField.setVisible(false);
		heightLbl.setVisible(false);
		heightField.setVisible(false);
		widthLbl.setVisible(false);
		selectLbl.setVisible(true);
		comboBox.setVisible(true);
		comboBox.setModel(new DefaultComboBoxModel(topics));
		comboBox.setSelectedItem(npc.getTopicTheme());
	}
	/**
	 * Displays the Properties of the selected FriendlyNPC
	 * @param npc
	 */
	private void viewFriendlyProperties(FriendlyNPC npc){
		nameField.setText(npc.getName());
		nameField.setEnabled(false);
		widthField.setVisible(false);
		heightLbl.setVisible(false);
		heightField.setVisible(false);
		widthLbl.setVisible(false);
		selectLbl.setVisible(false);
		comboBox.setVisible(false);
	}
	/**
	 * Displays the Properties of the selected StaticObject
	 * @param obj
	 */
	private void viewStaticObjectProperties(StaticObject obj){
		nameField.setText(obj.getName());
		nameField.setEnabled(false);
		widthField.setVisible(false);
		heightLbl.setVisible(false);
		heightField.setVisible(false);
		widthLbl.setVisible(false);
		selectLbl.setVisible(false);
		comboBox.setVisible(false);
	}
	/**
	 * Displays the Properties of the selected TravelTrigger.
	 * @param trigger
	 */
	private void viewTravelTriggerProperties(TravelTrigger trigger){
		nameField.setText(trigger.getName());
		nameField.setEnabled(true);
		widthLbl.setVisible(true);
		widthField.setVisible(true);
		widthField.setValue(trigger.getImageWidth());
		heightLbl.setVisible(true);
		heightField.setValue(trigger.getImageHeight());
		heightField.setVisible(true);
		selectLbl.setVisible(true);
		comboBox.setVisible(true);
		DefaultComboBoxModel model = new DefaultComboBoxModel(getTriggersAsArray());
		comboBox.setModel(model);
		if(trigger.getDestinationId()!=0){
			
			for(int i = 0;i<comboBox.getModel().getSize();i++){
				TravelTrigger trigg = (TravelTrigger)comboBox.getModel().getElementAt(i);
				if(trigg.getTravelTriggerId()==trigger.getDestinationId()){
					comboBox.setSelectedIndex(i);
					return;
				}
				
			}
			
		}
		
	}
	/**
	 * Called when root is selected.
	 * Updates panels and text on buttons.
	 */
	private void rootSelected(){
		selectedZone = "";
		addButton.setVisible(true);
		removeButton.setVisible(false);
		propertiesPanel.setVisible(false);
		listener.rootSelected();
	}
	/**
	 * Selects EnemyNPC in JTree
	 * @param zoneName name of zone where the EnemyNPC is
	 * @param id of the EnemyNPC
	 */
	public void selectedEnemyNPC(String zoneName, String id){
		DefaultMutableTreeNode enemyNode = getCategoryNode(zoneName, ENEMYNPC);
		for(Enumeration e=enemyNode.children(); e.hasMoreElements();){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
			EnemyNPC npc = (EnemyNPC)node.getUserObject();
			if(npc.getEnemyNPCId()==Integer.parseInt(id)){
				TreePath selectedPath = new TreePath(treeModel.getPathToRoot(node));
				worldTree.setSelectionPath(selectedPath);
				worldTree.scrollPathToVisible(selectedPath);
				return;
			}
		}
	}
	/**
	 * Helper method to get a category TreeNode
	 * @param zoneName Name of zone.
	 * @param categoryName Desired category.(EnemyNPC,FriendlyNPC,etc.)
	 * @return
	 */
	private DefaultMutableTreeNode getCategoryNode(String zoneName,String categoryName){
		DefaultMutableTreeNode root =(DefaultMutableTreeNode)treeModel.getRoot();
		DefaultMutableTreeNode zone = getChildWithName(zoneName, root);
		return getChildWithName(categoryName, zone);
	}
	/**
	 * Selects an FriendlyNPC in the JTree.
	 * @param zoneName Name of zone
	 * @param id Id of FriendlyNPC in zone.
	 */
	public void selectedFriendlyNPC(String zoneName, String id){
		DefaultMutableTreeNode friendlyNode = getCategoryNode(zoneName, FRIENDLYNPC);
		for(Enumeration e=friendlyNode.children(); e.hasMoreElements();){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
			FriendlyNPC npc = (FriendlyNPC)node.getUserObject();
			if(npc.getFriendlyNPCId()==Integer.parseInt(id)){
				TreePath selectedPath = new TreePath(treeModel.getPathToRoot(node));
				worldTree.setSelectionPath(selectedPath);
				worldTree.scrollPathToVisible(selectedPath);
				return;
			}
		}
	}
	/**
	 * Selects a StaticObject in the JTree.
	 * @param zoneName
	 * @param id
	 */
	public void selectedStaticObject(String zoneName, String id){
		DefaultMutableTreeNode staticObjectNode = getCategoryNode(zoneName, STATICOBJECT);
		for(Enumeration e=staticObjectNode.children(); e.hasMoreElements();){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
			StaticObject obj = (StaticObject)node.getUserObject();
			if(obj.getStaticObjectId()==Integer.parseInt(id)){
				TreePath selectedPath = new TreePath(treeModel.getPathToRoot(node));
				worldTree.setSelectionPath(selectedPath);
				worldTree.scrollPathToVisible(selectedPath);
				return;
			}
		}
	}
	/**
	 * Selects a TravelTrigger in the JTree.
	 * @param zoneName
	 * @param id
	 */
	public void selectedTravelTrigger(String zoneName, String id){
		DefaultMutableTreeNode triggerNode = getCategoryNode(zoneName, TRAVELTRIGGER);
		for(Enumeration e=triggerNode.children(); e.hasMoreElements();){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
			TravelTrigger trigger = (TravelTrigger)node.getUserObject();
			if(trigger.getTravelTriggerId()==Integer.parseInt(id)){
				TreePath selectedPath = new TreePath(treeModel.getPathToRoot(node));
				worldTree.setSelectionPath(selectedPath);
				worldTree.scrollPathToVisible(selectedPath);
				return;
			}
		}
	}
	/**
	 * Used to display list in combobox
	 * @return
	 */
	private TravelTrigger[] getTriggersAsArray(){
		TravelTrigger[] triggerArray = new TravelTrigger[triggers.size()+1];
		int i = 1;
		TravelTrigger noToTrigger = new TravelTrigger(EMPTYSELECTION,0,0,null);
		noToTrigger.setDestinationId(0);
		triggerArray[0] = noToTrigger;
		for(TravelTrigger trigger : triggers){
			triggerArray[i] = trigger;
			i++;
		}
		return triggerArray;
	}
	/**
	 * Collapses the zone nodes.
	 */
	public void collapseZone(){
		DefaultMutableTreeNode root =(DefaultMutableTreeNode)treeModel.getRoot();
		TreePath rootPath = new TreePath(root);
		for(Enumeration e=root.children(); e.hasMoreElements();){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
			worldTree.collapsePath(rootPath.pathByAddingChild(node));
		}
	}
	/**
	 * Collapses all other zones then selected.
	 * @param zoneName
	 */
	public void collapseOtherZones(String zoneName){
		DefaultMutableTreeNode root =(DefaultMutableTreeNode)treeModel.getRoot();
		TreePath rootPath = new TreePath(root);
		for(Enumeration e=root.children(); e.hasMoreElements();){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)e.nextElement();
			if(node.toString().equals(zoneName)){
				worldTree.expandPath(rootPath.pathByAddingChild(node));
			}else{
				worldTree.collapsePath(rootPath.pathByAddingChild(node));
			}
		}
	}
	/**
	 * Adds a zone to the JTree.
	 * @param zone
	 */
	public void addZoneToTree(Zone zone){
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)treeModel.getRoot();
		DefaultMutableTreeNode zoneNode = new DefaultMutableTreeNode(zone.getZoneName());
		treeModel.insertNodeInto(zoneNode, root, 0);
		addEnemyNodeWithChilds(zone.getEnemyList(),zoneNode);
		addFriendNodeWithChilds(zone.getFriendlyList(),zoneNode);
		addStaticNodeWithChilds(zone.getStaticObjectList(),zoneNode);
		addTravelTriggerNodeWithChilds(zone.getTravelTriggerList(), zoneNode);
	}
	
	/**
	 * Add enemyNPC to zone in JTree.
	 * @param zoneName
	 * @param npc
	 */
	public void addEnemyToZone(String zoneName,EnemyNPC npc){
		DefaultMutableTreeNode zoneNode = getZoneNode(zoneName);
		if(zoneNode!=null){
			DefaultMutableTreeNode enemyNode = getChildWithName(ENEMYNPC,zoneNode);
			if(enemyNode!=null){
				treeModel.insertNodeInto(new DefaultMutableTreeNode(npc),enemyNode, 0);
			}
		}
	}
	/**
	 * Add friendlyNPC to zone in JTree.
	 * @param zoneName
	 * @param npc
	 */
	public void addFriendlyToZone(String zoneName,FriendlyNPC npc){
		DefaultMutableTreeNode zoneNode = getZoneNode(zoneName);
		if(zoneNode!=null){
			DefaultMutableTreeNode friendlyNode = getChildWithName(FRIENDLYNPC,zoneNode);
			if(friendlyNode!=null){
				treeModel.insertNodeInto(new DefaultMutableTreeNode(npc),friendlyNode, 0);
			}
		}
	}
	/**
	 * Add staticObject to zone in JTree.
	 * @param zoneName
	 * @param obj
	 */
	public void addStaticObjectToZone(String zoneName,StaticObject obj){
		DefaultMutableTreeNode zoneNode = getZoneNode(zoneName);
		if(zoneNode!=null){
			DefaultMutableTreeNode sObjNode = getChildWithName(STATICOBJECT,zoneNode);
			if(sObjNode!=null){
				treeModel.insertNodeInto(new DefaultMutableTreeNode(obj),sObjNode, 0);
			}
		}
	}
	/**
	 * Add traveltrigger to zone in JTree.
	 * @param zoneName
	 * @param trigger
	 */
	public void addTravelTriggerToZone(String zoneName, TravelTrigger trigger){
		DefaultMutableTreeNode zoneNode = getZoneNode(zoneName);
		if(zoneNode!=null){
			DefaultMutableTreeNode triggerNode = getChildWithName(TRAVELTRIGGER,zoneNode);
			if(triggerNode!=null){
				triggers.add(trigger);
				treeModel.insertNodeInto(new DefaultMutableTreeNode(trigger),triggerNode, 0);
			}
		}
	}
	/**
	 * Removes zone from JTree.
	 * @param zoneName
	 */
	public void removeZone(String zoneName){
		DefaultMutableTreeNode zoneNode = getZoneNode(zoneName);
		if(zoneNode!=null){
			treeModel.removeNodeFromParent(zoneNode);
		}
	}
	/**
	 * Removes EnemyNPC from JTree.
	 * @param zoneName
	 * @param npcId
	 */
	public void removeEnemyNPC(String zoneName, int npcId){
		DefaultMutableTreeNode enemyNode = getChildWithName(ENEMYNPC,getZoneNode(zoneName));
		for(int i = 0; i<enemyNode.getChildCount();i++){
			EnemyNPC npc = (EnemyNPC)((DefaultMutableTreeNode)enemyNode.getChildAt(i)).getUserObject();
			if(npc.getEnemyNPCId()==npcId){
				treeModel.removeNodeFromParent((DefaultMutableTreeNode)enemyNode.getChildAt(i));
			}
		}
	}
	/**
	 * Removes FriendlyNPC from JTree.
	 * @param zoneName
	 * @param npcId
	 */
	public void removeFriendlyNPC(String zoneName, int npcId){
		DefaultMutableTreeNode friendlyNode = getChildWithName(FRIENDLYNPC,getZoneNode(zoneName));
		for(int i = 0; i<friendlyNode.getChildCount();i++){
			FriendlyNPC npc = (FriendlyNPC)((DefaultMutableTreeNode)friendlyNode.getChildAt(i)).getUserObject();
			if(npc.getFriendlyNPCId()==npcId){
				treeModel.removeNodeFromParent((DefaultMutableTreeNode)friendlyNode.getChildAt(i));
			}
		}
	}
	/**
	 * Removes StaticObject from JTree.
	 * @param zoneName
	 * @param objId
	 */
	public void removeStaticObject(String zoneName, int objId){
		DefaultMutableTreeNode sObjNode = getChildWithName(STATICOBJECT,getZoneNode(zoneName));
		for(int i = 0; i<sObjNode.getChildCount();i++){
			StaticObject obj = (StaticObject)((DefaultMutableTreeNode)sObjNode.getChildAt(i)).getUserObject();
			if(obj.getStaticObjectId()==objId){
				treeModel.removeNodeFromParent((DefaultMutableTreeNode)sObjNode.getChildAt(i));
			}
		}
	}
	/**
	 * Removes TravelTrigger from JTree.
	 * @param zoneName
	 * @param triggerId
	 */
	public void removeTravelTrigger(String zoneName, int triggerId){
		DefaultMutableTreeNode triggerNode = getChildWithName(TRAVELTRIGGER,getZoneNode(zoneName));
		for(int i = 0; i<triggerNode.getChildCount();i++){
			TravelTrigger trigger = (TravelTrigger)((DefaultMutableTreeNode)triggerNode.getChildAt(i)).getUserObject();
			if(trigger.getTravelTriggerId()==triggerId){
				triggers.remove(trigger);
				treeModel.removeNodeFromParent((DefaultMutableTreeNode)triggerNode.getChildAt(i));
			}
		}
	}
	/**
	 * Resets the JTree with new kingdom name.
	 * Removes all zone nodes.
	 * @param kingdomName
	 */
	public void clearWorld(String kingdomName){
		this.setVisible(true);
		triggers = new ArrayList<TravelTrigger>();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)treeModel.getRoot();
		root.setUserObject(kingdomName);
		rootSelected();
		while(root.getChildCount()>0){
			treeModel.removeNodeFromParent((DefaultMutableTreeNode)root.getChildAt(0));
		}
	}
	/**
	 * Returns zone node.
	 * @param zoneName
	 * @return
	 */
	private DefaultMutableTreeNode getZoneNode(String zoneName){
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)treeModel.getRoot();
		return getChildWithName(zoneName,root);
	}
	/**
	 * Returns childnode by name.
	 * @param name
	 * @param node
	 * @return
	 */
	private DefaultMutableTreeNode getChildWithName(String name, DefaultMutableTreeNode node){
		DefaultMutableTreeNode childNode = null;
		for(int i = 0; i<node.getChildCount();i++){
			if(((DefaultMutableTreeNode)node.getChildAt(i)).getUserObject().equals(name)){
				childNode = (DefaultMutableTreeNode)node.getChildAt(i);
			}
		}
		return childNode;
	}
	/**
	 * Adds a list of EnemyNPC to a zone.
	 * @param npcList
	 * @param zoneNode
	 */
	private void addEnemyNodeWithChilds(ArrayList<EnemyNPC> npcList, DefaultMutableTreeNode zoneNode){
		DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(ENEMYNPC);
		treeModel.insertNodeInto(categoryNode,zoneNode,0);
		for(EnemyNPC npc : npcList){
			treeModel.insertNodeInto(new DefaultMutableTreeNode(npc), categoryNode, 0);
		}
	}
	/**
	 * Adds a list of FriendlyNPC to a zone.
	 * @param npcList
	 * @param zoneNode
	 */
	private void addFriendNodeWithChilds(ArrayList<FriendlyNPC> npcList, DefaultMutableTreeNode zoneNode){
		DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(FRIENDLYNPC);
		treeModel.insertNodeInto(categoryNode,zoneNode,0);
		for(FriendlyNPC npc : npcList){
			treeModel.insertNodeInto(new DefaultMutableTreeNode(npc), categoryNode, 0);
		}
	}
	/**
	 * Adds a list of StaticObject to a zone.
	 * @param objList
	 * @param zoneNode
	 */
	private void addStaticNodeWithChilds(ArrayList<StaticObject> objList, DefaultMutableTreeNode zoneNode){
		DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(STATICOBJECT);
		treeModel.insertNodeInto(categoryNode,zoneNode,0);
		for(StaticObject obj : objList){
			treeModel.insertNodeInto(new DefaultMutableTreeNode(obj), categoryNode, 0);
		}
	}
	/**
	 * Adds a list of TravelTriggers to a zone.
	 * @param triggerList
	 * @param zoneNode
	 */
	private void addTravelTriggerNodeWithChilds(ArrayList<TravelTrigger> triggerList, DefaultMutableTreeNode zoneNode){
		DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(TRAVELTRIGGER);
		treeModel.insertNodeInto(categoryNode,zoneNode,0);
		for(TravelTrigger trigger : triggerList){
			triggers.add(trigger);
			treeModel.insertNodeInto(new DefaultMutableTreeNode(trigger), categoryNode, 0);
		}
	}

	
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==addButton){
			createNewOfSelected();
		
		}else if(e.getSource()==saveChanges){
			switch(selectedType){
			case ENEMY_NPC:
				EnemyNPC enpc  = (EnemyNPC)selectedObject;
				EditorMain.getControls().updateEnemyNPC(selectedZone,enpc.getEnemyNPCId(), comboBox.getSelectedItem().toString());
				break;
			case FRIENDLY_NPC:
				FriendlyNPC fnpc = (FriendlyNPC)selectedObject;
				//TODO: currently does not have any field that needs updating
				// Dialog, quests, inventory may be added in the future.
				break;
			case STATIC_OBJECT:
				StaticObject obj = (StaticObject)selectedObject;
				//TODO: currently does not have any fields that needs updating
				break;
			case TRAVEL_TRIGGER:
				TravelTrigger trigger = (TravelTrigger)selectedObject;
				EditorMain.getControls().updateTravelTrigger(trigger.getTravelTriggerId(), nameField.getText(), (Integer)widthField.getValue(), (Integer)heightField.getValue(), ((TravelTrigger)comboBox.getSelectedItem()).getTravelTriggerId());
				break;
			case ZONE:
				
				break;
				
			}
		}else if(e.getSource()==removeButton){
			deleteSelectedItem();
		}
		
	}
	public void createNewOfSelected(){
		DefaultMutableTreeNode selectedNode = 
			(DefaultMutableTreeNode)worldTree.getLastSelectedPathComponent();
		
		if(selectedNode == null){
			propertiesPanel.setVisible(false);
			return;
		}
		if(selectedNode.isRoot()){
			EditorGUI.guiControls().disableGui();
			new CreateZoneWindow(EditorGUI.guiControls().getLocationX(),EditorGUI.guiControls().getLocationY());
		}else if(isZone(selectedNode)){
			EditorGUI.guiControls().disableGui();
			new CreateZoneWindow(EditorGUI.guiControls().getLocationX(),EditorGUI.guiControls().getLocationY());
		}else if(selectedNode.isLeaf()){
			
			if(parentIsCategory(TRAVELTRIGGER,selectedNode)||
					(isCategoryNode(selectedNode)&&selectedNode.toString().equals(TRAVELTRIGGER))){
				EditorGUI.guiControls().disableGui();
				new CreateTriggerWindow(selectedZone,EditorGUI.guiControls().getLocationX(),EditorGUI.guiControls().getLocationY());
			}
		}else if(isCategoryNode(selectedNode)&&selectedNode.toString().equals(TRAVELTRIGGER)){
		
			EditorGUI.guiControls().disableGui();
			new CreateTriggerWindow(selectedZone,EditorGUI.guiControls().getLocationX(),EditorGUI.guiControls().getLocationY());
			
		}
	}
	
	@Override
	public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode selectedNode = 
			(DefaultMutableTreeNode)worldTree.getLastSelectedPathComponent();
		
		if(selectedNode == null){
			propertiesPanel.setVisible(false);
			return;
		}
		if(selectedNode.isRoot()){
			rootSelected();
		}else if(isZone(selectedNode)){
			selectedZone = selectedNode.toString();
			selectedType = SelectionType.ZONE;
			addButton.setVisible(true);
			removeButton.setVisible(true);
			propertiesPanel.setVisible(false);
			collapseOtherZones(selectedNode.toString());
			listener.zoneSelected(selectedNode.toString());
		}else if(isCategoryNode(selectedNode)){
			propertiesPanel.setVisible(false);
			removeButton.setVisible(false);
			if(selectedNode.toString().equals(TRAVELTRIGGER)){
				addButton.setVisible(true);
			}else{
				addButton.setVisible(false);
			}
			
		}else if(selectedNode.isLeaf()){
			propertiesPanel.setVisible(true);
			removeButton.setVisible(true);
			if(parentIsCategory(ENEMYNPC,selectedNode)){
				addButton.setVisible(false);
				EnemyNPC npc = (EnemyNPC)selectedNode.getUserObject();
				selectedObject = npc;
				selectedType = SelectionType.ENEMY_NPC;
				listener.enemyNPCSelected(selectedZone, npc.getEnemyNPCId()+"");
				viewEnemyProperties(npc);
			}else if(parentIsCategory(FRIENDLYNPC,selectedNode)){
				addButton.setVisible(false);
				FriendlyNPC npc = (FriendlyNPC)selectedNode.getUserObject();
				selectedObject = npc;
				selectedType = SelectionType.FRIENDLY_NPC;
				listener.friendlyNPCSelected(selectedZone, npc.getFriendlyNPCId()+"");
				viewFriendlyProperties(npc);
			}else if(parentIsCategory(STATICOBJECT,selectedNode)){
				addButton.setVisible(false);
				StaticObject obj = (StaticObject)selectedNode.getUserObject();
				selectedObject = obj;
				selectedType = SelectionType.STATIC_OBJECT;
				listener.staticObjectSelected(selectedZone,obj.getStaticObjectId()+"");
				viewStaticObjectProperties(obj);
			}else if(parentIsCategory(TRAVELTRIGGER,selectedNode)){
				addButton.setVisible(true);
				TravelTrigger trigger = (TravelTrigger)selectedNode.getUserObject();
				selectedObject = trigger;
				selectedType = SelectionType.TRAVEL_TRIGGER;
				listener.travelTriggerSelected(selectedZone, trigger.getTravelTriggerId()+"");
				viewTravelTriggerProperties(trigger);
			}else{
				propertiesPanel.setVisible(false);
				removeButton.setVisible(false);
			}
		}
	
	}
	/**
	 * Returns true if parent is a category.
	 * @param nameOfCategory
	 * @param node
	 * @return
	 */
	private boolean parentIsCategory(String nameOfCategory,DefaultMutableTreeNode node){
		if(!((DefaultMutableTreeNode)node.getParent().getParent()).isRoot()){
			if(isZone((DefaultMutableTreeNode)node.getParent().getParent())){
				if(node.getParent().toString().equals(nameOfCategory)){
					return true;
				}
			}
		}
		return false;
	}
	/**
	 * Returns true if current node is a category.
	 * @param node
	 * @return
	 */
	private boolean isCategoryNode(DefaultMutableTreeNode node){
		if(!((DefaultMutableTreeNode)node).isRoot()){
			if(isZone((DefaultMutableTreeNode)node.getParent())){
				return true;
			}
		}
		return false;
	}
	/**
	 * Returns true if current node is a zone.
	 * @param node
	 * @return
	 */
	private boolean isZone(DefaultMutableTreeNode node){
		if(((DefaultMutableTreeNode)node.getParent()).isRoot()){
			return true;
		}
		return false;
		
		
	}
	@Override
	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode()==KeyEvent.VK_DELETE){
			
			deleteSelectedItem();
		}
	}
	@Override
	public void keyReleased(KeyEvent e) {
		
	}
	@Override
	public void keyTyped(KeyEvent e) {
	
		
	}
	/**
	 * Deletes selected item in JTree.
	 */
	private void deleteSelectedItem(){
		DefaultMutableTreeNode selectedNode = 
			(DefaultMutableTreeNode)worldTree.getLastSelectedPathComponent();
		if(selectedNode == null){
			return;
		}
		if(selectedNode.isRoot()){
			JOptionPane.showMessageDialog(this, "If you want to create a new world. Select new world option under file menu.");

		}else if(isZone(selectedNode)){
			int yesno = JOptionPane.showConfirmDialog(
				    this,
				    "Are you sure you want to delete zone \""+selectedNode.toString()+"\"?",
				    "Delete zone confirmation",
				    JOptionPane.YES_NO_OPTION);
			if(yesno == JOptionPane.YES_OPTION){
				EditorMain.getControls().removeZoneFromWorld(selectedNode.toString());
			}
			
		}else if(selectedNode.isLeaf()){
			if(parentIsCategory(ENEMYNPC,selectedNode)){
				EnemyNPC npc = (EnemyNPC)selectedNode.getUserObject();
				int yesno = JOptionPane.showConfirmDialog(
					    this,
					    "Are you sure you want to delete NPC \""+npc.getName()+"\"?",
					    "Delete zone confirmation",
					    JOptionPane.YES_NO_OPTION);
				if(yesno == JOptionPane.YES_OPTION){
					EditorMain.getControls().removeEnemyNPCFromZone(selectedNode.getParent().getParent().toString(), npc.getEnemyNPCId());
				}
				
			}else if(parentIsCategory(FRIENDLYNPC,selectedNode)){
				FriendlyNPC npc = (FriendlyNPC)selectedNode.getUserObject();
				int yesno = JOptionPane.showConfirmDialog(
					    this,
					    "Are you sure you want to delete NPC \""+npc.getName()+"\"?",
					    "Delete NPC confirmation",
					    JOptionPane.YES_NO_OPTION);
				if(yesno == JOptionPane.YES_OPTION){
					
					EditorMain.getControls().removeFriendlyNPCFromZone(selectedNode.getParent().getParent().toString(), npc.getFriendlyNPCId());
				}
			}else if(parentIsCategory(STATICOBJECT,selectedNode)){
				StaticObject obj = (StaticObject)selectedNode.getUserObject();
				int yesno = JOptionPane.showConfirmDialog(
					    this,
					    "Are you sure you want to delete object \""+obj.getName()+"\"?",
					    "Delete Static object confirmation",
					    JOptionPane.YES_NO_OPTION);
				if(yesno == JOptionPane.YES_OPTION){
					
					EditorMain.getControls().removeStaticObjectFromZone(selectedNode.getParent().getParent().toString(), obj.getStaticObjectId());	
				}
			}else if(parentIsCategory(TRAVELTRIGGER,selectedNode)){
				TravelTrigger trigger = (TravelTrigger)selectedNode.getUserObject();
				int yesno= JOptionPane.showConfirmDialog(
						this,
						"Are you sure you want to delete trigger \""+trigger.getName()+"\"?",
						"Delete Traveltrigger confirmation",
						JOptionPane.YES_NO_OPTION);
				if(yesno == JOptionPane.YES_OPTION){
					EditorMain.getControls().removeTravelTriggerFromZone(selectedNode.getParent().getParent().toString(), trigger.getTravelTriggerId());
				}
			}
		}
	
	}
}
