package editor.graphics.editorPanels.mapEditorPanels;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import editor.graphics.ImageFactory;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.StaticObject;
/**
 * Panel that contains list of objects that can be placed in the Kingdom.
 * 
 * @author thorgrun
 *
 */
public class MapPlaceables extends JPanel implements ListSelectionListener, ActionListener, ChangeListener{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static String PLACE = "Place";
	private static String NOTOPIC = "No topic found.";
	private static String PAINT = "Paint background";
	private static String CANCEL_PAINT = "Cancel paint";
	private JTabbedPane pane;
	private DefaultListModel enemyNPCModel;
	private DefaultListModel friendlyNPCModel;
	private DefaultListModel staticObjModel;
	private DefaultListModel bTilesModel;
	private JList enemyNPCList;
	private JList friendlyNPCList;
	private JList staticObjList;
	private JList bTilesList;
	private JButton placeOnMap;
	private String topics[];
	private String currentZoneName;
	private JLabel icon;
	private PlaceableListener placeListener;
	private boolean isPaintingTiles;
	private enum SelectedType{
		ENEMY_NPC,
		FRIENDLY_NPC,
		STATIC_OBJECT,
		BACKGROUND_TILE
	}
	
	private SelectedType currentType;
	public MapPlaceables(PlaceableListener placeListener){
		isPaintingTiles = false;
		this.placeListener = placeListener;
		pane = new JTabbedPane();
		currentType = SelectedType.ENEMY_NPC;
		pane.setPreferredSize(new Dimension(140,500));
		setPreferredSize(new Dimension(150,700));
		enemyNPCList = new JList();
		friendlyNPCList = new JList();
		staticObjList = new JList();
		bTilesList = new JList();
		
		
		enemyNPCModel = createListPanel("Enemy NPC", enemyNPCList);
		friendlyNPCModel = createListPanel("Friendly NPC",friendlyNPCList);
		staticObjModel = createListPanel("Static Object",staticObjList);
		bTilesModel = createListPanel("Background tiles",bTilesList);
		JPanel placementPanel = new JPanel();
		icon = new JLabel();
		placeOnMap = new JButton(PLACE);
		placeOnMap.addActionListener(this);
		//placementPanel.add(icon);
		placementPanel.add(placeOnMap);
		placementPanel.setPreferredSize(new Dimension(150,300));
		updateBackgroundTileList(ImageFactory.getInstance().getBackgroundTiles());
		pane.addChangeListener(this);
		this.add(pane);
		this.add(placementPanel);
	}
	/**
	 * Sets the currently selected zoneName.
	 * @param zoneName
	 */
	public void setCurrentZoneName(String zoneName){
		currentZoneName= zoneName;
	}
	/**
	 * Helper method for creating list panels.
	 * @param name
	 * @param list
	 * @return
	 */
	private DefaultListModel createListPanel(String name, JList list){
		JPanel panel = new JPanel();
		

		DefaultListModel listModel = new DefaultListModel();
		list.setModel(listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		list.setLayoutOrientation(JList.HORIZONTAL_WRAP);
		list.addListSelectionListener(this);
		JScrollPane scroll = new JScrollPane(list);
		scroll.setPreferredSize(new Dimension(130,400));
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		panel.add(scroll);
		
		pane.addTab(name, panel);
		return listModel;
		
		
	}
	/**
	 * Updates the list of all EnemyNPCs
	 * @param npcList
	 */
	public void updateEnemyNPCList(HashMap<String,EnemyNPC> npcList){
		enemyNPCModel.clear();
		for(String str : npcList.keySet()){
			enemyNPCModel.addElement(str);
		}
	}
	/**
	 * Updates the list of all FriendlyNPCs
	 * @param npcList
	 */
	public void updateFriendlyNPCList(HashMap<String,FriendlyNPC> npcList){
		friendlyNPCModel.clear();
		for(String str : npcList.keySet()){
			friendlyNPCModel.addElement(str);
		}
	}
	/**
	 * Updates the list of all staticobjects.
	 * @param objList
	 */
	public void updateStaticObjectList(HashMap<String,StaticObject> objList){
		staticObjModel.clear();
		for(String str : objList.keySet()){
			staticObjModel.addElement(str);
		}
		
	}
	/**
	 * Updates the list of all Backgroundtiles.
	 * @param tileList
	 */
	private void updateBackgroundTileList(ArrayList<TileIcon> tileList){
		bTilesModel.clear();
		for(TileIcon img: tileList){
			bTilesModel.addElement(img);
		}
	}
	@Override
	public void valueChanged(ListSelectionEvent e) {
		JList list = (JList)e.getSource();
		if(list.getSelectedValue()!=null){
			if(e.getSource()==enemyNPCList){
				System.out.println();
				icon.setIcon(new ImageIcon(ImageFactory.getInstance().getEnemyImage(list.getSelectedValue().toString())));
			}else if(e.getSource() == friendlyNPCList){
				icon.setIcon(new ImageIcon(ImageFactory.getInstance().getFriendlyImage(list.getSelectedValue().toString())));
			}else if(e.getSource() == staticObjList){
				icon.setIcon(new ImageIcon(ImageFactory.getInstance().getStaticObjectImage(list.getSelectedValue().toString())));
			}else if(e.getSource() == bTilesList){
				if(isPaintingTiles){
					enablePainting(((TileIcon)list.getSelectedValue()).getTileName());
				}
				icon.setIcon(new ImageIcon(((TileIcon)list.getSelectedValue()).getImage()));
			}
		}
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==placeOnMap){
			switch(currentType){
			case ENEMY_NPC:
				if(enemyNPCList.getSelectedValue()==null)
					return;
				String selectedValue;
				if(topics.length>0){
					selectedValue = topics[0];
				}else{
					selectedValue =NOTOPIC;
				}
				
				String choice = (String)JOptionPane.showInputDialog(
	                    this,
	                    "Select a question topic for this NPC.",
	                    "Select topic",
	                    JOptionPane.PLAIN_MESSAGE,
	                    icon.getIcon(),
	                    topics,
	                    selectedValue);
				if(choice!=null&&!choice.equals(NOTOPIC)){
					placeListener.addEnemyToZone(currentZoneName,enemyNPCList.getSelectedValue()+"",choice);
					System.out.println(enemyNPCList.getSelectedValue());
					return;
				}
				break;
			case FRIENDLY_NPC:
				if(friendlyNPCList.getSelectedValue()==null)
					return;
				placeListener.addFriendlyToZone(currentZoneName, friendlyNPCList.getSelectedValue()+"");
				break;
			case STATIC_OBJECT:
				if(staticObjList.getSelectedValue()==null)
					return;
				placeListener.addStaticObjectToZone(currentZoneName, staticObjList.getSelectedValue()+"");
				break;
			}
		}
		
	}
	/**
	 * Updates the list of all theme topics.
	 * Used by EnemyNPC when they are placed in the Kingdom.
	 * @param topicList
	 */
	public void updateTopicList(ArrayList<String> topicList){
		topics = new String[topicList.size()];
		for(int i = 0;i<topicList.size();i++){
			topics[i] = topicList.get(i);
		}
	}
	/**
	 * Disables the painting of backgroundtiles.
	 */
	private void disablePainting(){
		isPaintingTiles = false;
		if(placeOnMap.getText().equals(CANCEL_PAINT)){
			placeOnMap.setText(PAINT);
		}
		placeListener.disablePainting();
	}
	/**
	 * Enables the painting of tiles.
	 * @param tileName
	 */
	private void enablePainting(String tileName){
		isPaintingTiles = true;
		if(placeOnMap.getText().equals(PAINT)){
			placeOnMap.setText(CANCEL_PAINT);
		}
		placeListener.setEnablePainting(tileName);
	}
	@Override
	public void stateChanged(ChangeEvent e) {
		if(e.getSource()==pane){
			switch(pane.getSelectedIndex()){
			case 0:
				currentType =SelectedType.ENEMY_NPC;
				if(isPaintingTiles)	
					disablePainting();
				placeOnMap.setText(PLACE);
				placeOnMap.setVisible(true);
				break;
			case 1:
				currentType = SelectedType.FRIENDLY_NPC;
				if(isPaintingTiles)
					disablePainting();
				placeOnMap.setText(PLACE);
				placeOnMap.setVisible(true);
				break;
			case 2:
				currentType = SelectedType.STATIC_OBJECT;
				if(isPaintingTiles)
					disablePainting();
				placeOnMap.setText(PLACE);
				placeOnMap.setVisible(true);
				break;
			case 3:
				currentType = SelectedType.BACKGROUND_TILE;
				isPaintingTiles = true;
				if(bTilesList.getSelectedValue()!=null)
					enablePainting(((TileIcon)bTilesList.getSelectedValue()).getTileName());
				
				placeOnMap.setVisible(false);
				break;
			}
		}
		
	}

}
