package editor.models;

import java.util.ArrayList;
import java.util.HashMap;

import editor.models.changeListeners.EnemyListChangedListener;
import editor.models.changeListeners.FriendlyListChangedListener;
import editor.models.changeListeners.QuestionListChangedListener;
import editor.models.changeListeners.StaticObjectListChangedListener;
import editor.models.changeListeners.TopicListChangedListener;
import editor.models.changeListeners.ZoneNameListChangedListener;

import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Question;
import shared.models.StaticObject;
/**
 * Contains the list of objects from Database that are used by several parts of the Editor.
 * 
 * @author thorgrun
 *
 */
public class WorldObjectLists {
	private HashMap<String,EnemyNPC> allEnemyNPCs;
	private HashMap<String,FriendlyNPC> allFriendlyNPCs;
	private HashMap<String,StaticObject> allStaticObjects;
	private ArrayList<String> topicList;
	private ArrayList<Question> questionList;
	private String[] zoneNames;//Names of zone templates in DB
	//LISTENERS
	private ArrayList<EnemyListChangedListener> enemyListener;
	private ArrayList<FriendlyListChangedListener> friendlyListener;
	private ArrayList<StaticObjectListChangedListener> staticObjListener;
	private ArrayList<QuestionListChangedListener> questionListener;
	private ArrayList<TopicListChangedListener> topicListener;
	private ArrayList<ZoneNameListChangedListener> zoneNameListener;
	/**
	 * Constructor of WorldObjectLists.
	 * Initializes the Listener lists.
	 * 
	 */
	public WorldObjectLists(){
		enemyListener = new ArrayList<EnemyListChangedListener>();
		friendlyListener = new ArrayList<FriendlyListChangedListener>();
		staticObjListener = new ArrayList<StaticObjectListChangedListener>();
		topicListener = new ArrayList<TopicListChangedListener>();
		questionListener = new ArrayList<QuestionListChangedListener>();
		zoneNameListener = new ArrayList<ZoneNameListChangedListener>();
	}
	/**
	 * Returns EnemyNPC by name.
	 * @param name
	 * @return
	 */
	public EnemyNPC getEnemyNPC(String name){
		return allEnemyNPCs.get(name);
	
	}
	/**
	 * Returns FriendlyNPC by name.
	 * @param name
	 * @return
	 */
	public FriendlyNPC getFriendlyNPC(String name){
		return allFriendlyNPCs.get(name);
	}
	/**
	 * Returns StaticObject by name.
	 * @param name
	 * @return
	 */
	public StaticObject getStaticObject(String name){
		return allStaticObjects.get(name);
	}
	/**
	 * Adds Listener to list of listeners.
	 * @param listener
	 */
	public void addEnemyListener(EnemyListChangedListener listener){
		enemyListener.add(listener);
	}
	/**
	 * Adds Listener to list of listeners.
	 * @param listener
	 */
	public void addFriendlyListener(FriendlyListChangedListener listener){
		friendlyListener.add(listener);
	}
	/**
	 * Adds Listener to list of listeners.
	 * @param listener
	 */
	public void addStaticObjListener(StaticObjectListChangedListener listener){
		staticObjListener.add(listener);
	}
	/**
	 * Adds Listener to list of listeners.
	 * @param listener
	 */
	public void addTopicListener(TopicListChangedListener listener){
		topicListener.add(listener);
	}
	/**
	 * Adds Listener to list of listeners.
	 * @param listener
	 */
	public void addQuestionListener(QuestionListChangedListener listener){
		questionListener.add(listener);
	}
	/**
	 * Adds Listener to list of listeners.
	 * @param listener
	 */
	public void addZoneNameListener(ZoneNameListChangedListener listener){
		zoneNameListener.add(listener);
	}
	/**
	 * Removes Listener from list of listeners.
	 * @param listener
	 */
	public void removeEnemyListener(EnemyListChangedListener listener){
		enemyListener.remove(listener);
	}
	/**
	 * Removes Listener from list of listeners.
	 * @param listener
	 */
	public void removeFriendlyListener(FriendlyListChangedListener listener){
		friendlyListener.remove(listener);
	}
	/**
	 * Removes Listener from list of listeners.
	 * @param listener
	 */
	public void removeStaticObjectListener(StaticObjectListChangedListener listener){
		staticObjListener.remove(listener);
	}
	/**
	 * Removes Listener from list of listeners.
	 * @param listener
	 */
	public void removeQuestionListener(QuestionListChangedListener listener){
		questionListener.remove(listener);
	}
	/**
	 * Removes Listener from list of listeners.
	 * @param listener
	 */
	public void removeTopicListener(TopicListChangedListener listener){
		topicListener.remove(listener);
	}
	/**
	 * Removes Listener from list of listeners.
	 * @param listener
	 */
	public void removeZoneNameListener(ZoneNameListChangedListener listener){
		zoneNameListener.remove(listener);
	}
	/**
	 * Updates EnemyList and notifies listeners of change.
	 * @param npcList
	 */
	public void setEnemyList(HashMap<String,EnemyNPC> npcList){
		this.allEnemyNPCs = npcList;
		for(EnemyListChangedListener list: enemyListener){
			list.updatedEnemyNPCList(npcList);
		}
	}
	/**
	 * Updates FriendlyList and notifies listeners of change.
	 * @param npcList
	 */
	public void setFriendlyList(HashMap<String,FriendlyNPC> npcList){
		this.allFriendlyNPCs = npcList;
		for(FriendlyListChangedListener list: friendlyListener){
			list.updatedFriendlyNPCList(npcList);
		}
	}
	/**
	 * Updates StaticObjectList and notifies listeners of change.
	 * @param objList
	 */
	public void setStaticObjectList(HashMap<String,StaticObject> objList){
		this.allStaticObjects = objList;
		for(StaticObjectListChangedListener list:staticObjListener){
			list.updatedStaticObjectList(objList);
		}
	}
	/**
	 * Updates TopicList and notifies listeners of change.
	 * @param topicList
	 */
	public void setTopicList(ArrayList<String> topicList){
		this.topicList = topicList;
		for(TopicListChangedListener list: topicListener){
			list.updateTopicList(this.topicList);
		}
	}
	/**
	 * Updates QuestionList and notifies listeners of change.
	 * @param questionList
	 */
	public void setQuestionList(ArrayList<Question> questionList){
		this.questionList = questionList;
		for(QuestionListChangedListener list: questionListener){
			list.updateQuestionList(this.questionList);
		}
	}
	/**
	 * Updates zoneNameList and notifies listeners of change.
	 * @param zoneNameList
	 */
	public void setZoneNameList(String[] zoneNameList){
		this.zoneNames = zoneNameList;
		for(ZoneNameListChangedListener list: zoneNameListener){
			list.updateZoneNameList(zoneNames);
		}
	}
	public String[] getZoneNames() {
		return zoneNames;
	}
	
}
