package lobbyServer.authenticate;


import shared.models.Player;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.PlayerInformationMessage;
import shared.network.messages.MessageType;
import shared.serverHelpers.*;
import lobbyServer.clientSessionHandler.LobbyControl;
/**
 * 
 * Authenticates a client using the username and password.
 */
public class ClientAuthenticator {

	private Player player;
	private boolean receivedResponse;
	private boolean dbResponsePositive;
	
	public ClientAuthenticator(){
	
		receivedResponse = false;
		dbResponsePositive = false;
		
	}
	
	public AuthenticationResult Authenticate(String username, String password, String sessionId){
	
		if(username != null && password != null){
	
			
			
			
			
			//check username and password with the db:
			LobbyControl.getDbConnection().sendMessage(new PlayerInformationMessage("LOBBYSERVER", "DBSERVER", "REQUEST CLIENT", username, password, sessionId, MessageType.GET_PLAYER ));
//			LobbyControl.getDbConnection().sendMessage(new PlayerInformationMessage("LOBBYSERVER", "DBSERVER", "REQUEST CLIENT", username, password, sessionId, MessageType.GET_BAN_LIST));
//			LobbyControl.getDbConnection().sendMessage(new PlayerInformationMessage("LOBBYSERVER", "DBSERVER", "REQUEST CLIENT", "newCoolMyUser", "myPassword", sessionId, "emailAddress", MessageType.CREATE_PLAYER));
//			LobbyControl.getDbConnection().sendMessage(new PlayerInformationMessage("LOBBYSERVER", "DBSERVER", "REQUEST CLIENT", "nyKulBruker2", "hemmelig2", sessionId, "2krllalphaDotCom", "oldPass", "oldName", MessageType.ALTER_PLAYER ));
			
				//wait for response:
				int timer = 0;
				while(timer < 10000 && !receivedResponse){
					timer += 500;
					try {
						Thread.sleep(500);
					} catch (InterruptedException e) {
						System.err.println("waiting for db response sleep interupted!");
						e.printStackTrace();
					}
				}
				if(receivedResponse){
					receivedResponse = false;
					if(dbResponsePositive){

						// db result is good, check if already connected.
						if(LobbyControl.getLobbyControl().clientAlreadyConnected(username)){
							return new AuthenticationResult(AuthenticationResultType.ALREADY_LOGGED_IN, "User already logged in!");
						}
						
						// db is good and not connected, check banlist:
						
						BanInfo bInfo = LobbyControl.getLobbyControl().checkBanList(player.getName()+"");
						
						if(bInfo != null){
							
							return new AuthenticationResult(AuthenticationResultType.CLIENT_BANNED, "Client banned. Reason : " + bInfo.getReason());
							
						}
						
						return new AuthenticationResult(AuthenticationResultType.OK, "Login Sucessfull", player);
					}
					else{
						return new AuthenticationResult(AuthenticationResultType.USERNAME_PASSWORD_WRONG, "Username or password wrong");
					}
				}
				else if(timer >= 10000){
					return new AuthenticationResult(AuthenticationResultType.TIMED_OUT, "Authentication timed out");
				}
			
		}
		else{
			
			return new AuthenticationResult(AuthenticationResultType.USERNAME_OR_PASSWORD_MISSING, "Username or password is missing");
		}
	
		
		return new AuthenticationResult(AuthenticationResultType.OTHER_ERROR, "Authentication: something unknown went wrong!");
	}
	
	public void dbResponseReceived(ObjectMessage omsg){
		receivedResponse = true;
		if(omsg.getObjectType() == ObjectType.PLAYER){
			dbResponsePositive = true;
			player = (Player)omsg.getObject();
		}
		else if(omsg.getObjectType() == ObjectType.IDENTIFICATION_ERROR){
			dbResponsePositive = false;
		}
		else{
			dbResponsePositive = false;
			System.err.println("authentication received somthing other than Player or Identification error!");
		}
	
	}
	
	
	
}
