package lobbyServer.clientSessionHandler;

import shared.network.ClientSocket;
import shared.network.MessageHandler;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.PlayerVerificationResult;
import shared.network.messages.PlayerVerifyMessage;
import shared.network.messages.RegisterResult;
import shared.serverHelpers.*;
/**
 * Handles network between World server and Lobby server.
 *
 */
public class WorldServerHandler implements MessageHandler{
	private shared.network.ClientSocket worldSocket;
	private WorldServer worldServer;
	private String worldSessionId;
	private WorldServerState state;
	
	
	public WorldServerHandler(ClientSocket worldSocket, String worldId){
		this.worldSocket = worldSocket;
		this.worldSocket.addMessageListener(this);
		this.worldSessionId = worldId;
		worldServer = new WorldServer("-",1, "-", 0);
		state = WorldServerState.CONNECTED;
		
	}
	/**
	 * Message received from the World Server.
	 */
	public void getMessage(Message message) {
		
		if(message.getMessageType() == MessageType.REGISTER_SERVER){
			
			ObjectMessage omsg = (ObjectMessage)message;
			if(omsg.getObjectType() == ObjectType.WORLDSERVER_INFO){
				handleRegister((WorldServer)omsg.getObject());
			}
			else{
				System.err.println("got register_server with wrong objecttype");
			}
		}
		
		else if(message.getMessageType() == MessageType.VERIFY_PLAYER){
			if(state == WorldServerState.REGISTERED){
				PlayerVerifyMessage pmsg = (PlayerVerifyMessage)message;
				if(LobbyControl.getLobbyControl().checkPlayer(pmsg.getPlayerSessionId(), pmsg.getPlayerName(), worldServer.getKingdomName(), pmsg.getCharacterName())){
					worldSocket.sendMessage(new PlayerVerificationResult("LOBBYSERVER", worldSessionId, "CLIENT APPROVED", true, pmsg.getPlayerSessionId()));
				}
				else{
					worldSocket.sendMessage(new PlayerVerificationResult("LOBBYSERVER", worldSessionId, "CLIENT DISSAPPROVED", false, pmsg.getPlayerSessionId()));
				}
			}
			else{
				worldSocket.sendMessage(new Message("LOBBYSERVER", worldSessionId, "PLEASE REGISTER BEFORE SENDING PLAYING VERIFICATIONS", MessageType.LOBBYSERVER_REQUEST_FAILURE));
			}
		}
		
	}
	
	/**
	 * Is called when the connection is interrupted.
	 */
	public void lostConnection() {
		
		System.out.println("lost connection with server " + worldSessionId);

		LobbyControl.getLobbyControl().removeServerHandler(worldSessionId);
		
		state = WorldServerState.DISCONNECTED;
	}
	
	private void handleRegister(WorldServer worldServerObject){
		if(LobbyControl.getLobbyControl().serverAlreadyRegistered(worldServerObject.getKingdomName(), worldServerObject.getIpAdresse(), worldServerObject.getPortNr())){
			worldSocket.sendMessage((new RegisterResult("LOBBYSERVER", worldSessionId, "Register unsuccessfull - name or ip/port combination already registered", false)));
			return;
		}
		worldServer = worldServerObject;
		
		worldSocket.sendMessage(new RegisterResult("LOBBYSERVER", worldSessionId, "Register successfull", true));
		state = WorldServerState.REGISTERED;
	}
	
	public void printOverview(){
		if(state == WorldServerState.REGISTERED){
			System.out.println(worldSessionId + ": " + worldServer.getKingdomName() + " " + worldServer.getIpAdresse() + ":" + worldServer.getPortNr());
		}
		else{
			System.out.println(worldSessionId + ": NOT REGISTERED");
		}
		
	}
	
	public boolean checkAgainstWorldServerConnection(String serverName, String ip, int port){
		
		if(serverName.equals(worldServer.getKingdomName()) || (ip.equals(worldServer.getIpAdresse()) && port == worldServer.getPortNr())){
			//the server has the same name OR ip/port combination as the input
			if(state == WorldServerState.CONNECTED){
				//the server is connected, so it's "similar" to the imput
				return true;
			}
		}
		return false;
	}
	
	public WorldServer getWorldServerIfServerRegistered(){
		if(state == WorldServerState.REGISTERED){
			return worldServer;
		}
		else{
			return null;
		}
	}

}
