package lobbyServer.network;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

import shared.network.messages.*;
import shared.models.*;

import lobbyServer.clientSessionHandler.LobbyControl;
/**
 * Creates the connection to the Database
 *
 */
public class DatabaseConnection implements Runnable{
	private Socket dbSocket;
	private ObjectOutputStream oos = null;
	private ObjectInputStream ois = null;
	private Thread listenThread;
	
	
	public DatabaseConnection(String serverIpAdress,int portNumber){

		try {
	        // open a socket connection
	    	System.out.println("Opening connection to DB");
	        dbSocket = new Socket(serverIpAdress, portNumber);
	        System.out.println("Connected to DB");
	        // open I/O streams for objects
	        oos = new ObjectOutputStream(dbSocket.getOutputStream());
	        ois = new ObjectInputStream(dbSocket.getInputStream());
	        // read an object from the db
	        listenThread = new Thread(this);
	        listenThread.start();
	      } catch(IOException e) {
	        System.out.println(e.getMessage());
	      }
	}

	/**
	 * Closes the connection
	 */
	public void closeConnection(){
		try{
			oos.close();
			ois.close();
			dbSocket.close();
		}catch(IOException e){
			System.out.println(e.getMessage());
		}
	}
	/**
	 * Sends a message to the Database.
	 * @param msg
	 */
	public void sendMessage(Message msg){
		try{
			msg.printMessage();
			oos.writeObject(msg);
			oos.flush();
			System.out.println("message sent");
		}catch(IOException ioEx){
			System.out.println(ioEx.getMessage());
		}
	}
	
	/**
	 * Listens for messages from the database, and forwards them to the LobbyControl.
	 */
	public void run() {
		boolean disconnect = false;
		try{
			while(!disconnect){
				Message msg = (Message)ois.readObject();
				
				LobbyControl.getLobbyControl().receiveDBMessage(msg);
				
			}
			
		}catch(ClassNotFoundException classEx) {
			System.out.println(classEx.getMessage());
		}catch(IOException ioEx){
			System.out.println(ioEx.getMessage());
		}
		
	}
	
	
	
	
	
}
