package shared;

import client.clientState.Settings;
import shared.models.Location;
/**
 * Commonfunctions used by the Client and the Worldserver.
 * @author thorgrun
 *
 */
public final class CommonFunctions {
	private static final double movePerMilliSec = 0.035;
	public static int drawXinPlayField(int playFieldPositionX,int screenFocusX,int imageWidth){
		return (playFieldPositionX-imageWidth/2) - (screenFocusX-Settings.playfieldWidth/2);
	}
	public static int drawYinPlayField(int playFieldPositionY,int screenFocusY,int imageHeight){
		return (playFieldPositionY-imageHeight) - (screenFocusY-Settings.playfieldHeight/2);
	}
	public static int drawXinPlayField(int playFieldPositionX,int screenFocusX){
		return playFieldPositionX - (screenFocusX-Settings.playfieldWidth/2);
	}
	public static int drawYinPlayField(int playFieldPositionY,int screenFocusY){
		return playFieldPositionY - (screenFocusY-Settings.playfieldHeight/2);
	}
	public static int getXinPlayField(int mouseX,int screenFocusX){
		return mouseX + (screenFocusX-Settings.playfieldWidth/2);
	}
	public static int getYinPlayField(int mouseY,int screenFocusY){
		return mouseY + (screenFocusY-Settings.playfieldHeight/2);
	}
	/**
	 * Updates the position of a object. Used by Character.
	 * @param elapsedTime
	 * @param currentLocation
	 * @param targetLocation
	 * @return
	 */
	public static Location updatePositions(long elapsedTime, Location currentLocation,Location targetLocation){
		Location endLocation = new Location(currentLocation.getXDouble()+0,currentLocation.getYDouble()+0);
		double arcTangens = Math.atan(Math.abs((targetLocation.getYDouble()-currentLocation.getYDouble()))/Math.abs((targetLocation.getXDouble()-currentLocation.getXDouble())));
		
		if(targetLocation.getY()<currentLocation.getY()){
			if(currentLocation.getY()-targetLocation.getY()<movePerMilliSec*elapsedTime){
				endLocation.setY(targetLocation.getY()+0);
			}else{
				endLocation.setY(currentLocation.getYDouble()-Math.sin(arcTangens)*movePerMilliSec*elapsedTime);
			}
		
		}else if(targetLocation.getY()>currentLocation.getY()){
			if(targetLocation.getY()-currentLocation.getY()<movePerMilliSec*elapsedTime){
				endLocation.setY(targetLocation.getY()+0);
			}else{
				endLocation.setY(currentLocation.getYDouble()+Math.sin(arcTangens)*movePerMilliSec*elapsedTime);
			}
		}
		
		
		if(targetLocation.getX()>currentLocation.getX()){
			if(targetLocation.getX()-currentLocation.getX()<movePerMilliSec*elapsedTime){
				endLocation.setX(targetLocation.getX()+0);
			}else{
				endLocation.setX(currentLocation.getXDouble()+Math.cos(arcTangens)*movePerMilliSec*elapsedTime);
			}
		}else if(targetLocation.getX()<currentLocation.getX()){
			if(currentLocation.getX()-targetLocation.getX()<movePerMilliSec*elapsedTime){
				endLocation.setX(targetLocation.getX()+0);
			}else{
				endLocation.setX(currentLocation.getXDouble()-Math.cos(arcTangens)*movePerMilliSec*elapsedTime);
			}
		}
		return endLocation;
	}
}
