package shared.logger;

import java.io.*;
import java.util.Calendar;

/**
 * Can be used to write messages to a txt file.
 *
 * @author thorgrun
 *
 */
public class LogSystem {
	private static LogSystem logSys;
	private boolean writeToLog;
	private boolean useTimeStamp;
	private boolean writeToSys;
	private LogSystem(){
		writeToLog = true;
		useTimeStamp = true;
		writeToSys = true;
		
	}
	public static LogSystem getInstance(){
		if(logSys==null){
			logSys = new LogSystem();
		}
		return logSys;
	}
	/**
	 * Enables writing to a txt file.
	 */
	public void enableLog(){
		writeToLog =true;
	}
	/**
	 * Disables writing to a txt file.
	 */
	public void disableLog(){
		writeToLog = false;
	}
	/**
	 * Enables time stamp on messages saved in txt.
	 */
	public void enableTimeStamp(){
		useTimeStamp =true;
	}
	/**
	 * Disables time stamp on messages saved in txt.
	 */
	public void disableTimeStamp(){
		useTimeStamp = false;
	}
	/**
	 * Enables sysout of messages.
	 */
	public void enableSysOutPrint(){
		writeToSys =true;
	}
	/**
	 * Disables sysout of messages.
	 */
	public void disableSysOutPrint(){
		writeToSys = false;
	}
	private String addTimeStamp(String msg){
		if(useTimeStamp){
			int hour = Calendar.getInstance().get(Calendar.HOUR_OF_DAY);
			int minute = Calendar.getInstance().get(Calendar.MINUTE);
			int second = Calendar.getInstance().get(Calendar.SECOND);
			msg =addZero(hour) +":"+addZero(minute)+":"+addZero(second)+" - "+msg;
		}
		return msg;
	}
	private static String addZero(int time){
		if(time<10){
			return "0"+time;
		}
		return time+"";
	}
	/**
	 * Writes to a DBLog.txt
	 * @param msg
	 */
	public void writeDBLog(String msg){
		if(writeToSys){
			System.out.println(msg);
		}
		if(writeToLog){
			writeToLogFile(msg,"DBLog.txt");
		}
	}
	/**
	 * Writes to ClientLog.txt
	 * @param msg
	 */
	public void writeClientLog(String msg){
		if(writeToSys){
			System.out.println(msg);
		}
		if(writeToLog){
			
			writeToLogFile(msg,"ClientLog.txt");
		}
	}
	/**
	 * Writes to LobbyLog.txt
	 * @param msg
	 */
	public void writeLobbyLog(String msg){
		if(writeToSys){
			System.out.println(msg);
		}
		if(writeToLog){
			writeToLogFile(msg,"LobbyLog.txt");
		}
	}
	/**
	 * Writes to WorldLog.txt
	 * @param msg
	 */
	public void writeWorldLog(String msg){
		if(writeToSys){
			System.out.println(msg);
		}
		if(writeToLog){
			writeToLogFile(msg,"WorldLog.txt");
		}
	}
	private void writeToLogFile(String msg, String fileName){
		try{
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(fileName),true));
		       writer.write(addTimeStamp(msg));
		       writer.newLine(); 
		       writer.close();
		}catch(IOException e){
			System.err.println("Write log error:"+e.getMessage());
		}

	}
	
	  
	  
	
}
