package shared.models;

import java.io.Serializable;
import java.util.ArrayList;
/**
 * Creates and holds an attack object 
 * @author Esben Fllesdal
 * @see Attributes
 * @see PlayableClasses
 * @see DamageType
 * @see Weapon
 * @see Effect
 * @see AttackType
 */
public class Attack implements Serializable{

	private static final long serialVersionUID = 1L;
	private int maxDamage;
	private int minDamage;
	private int questionLevel;
	private AttributeTypes attributeType;
	private int reloadTime;
	private int numberOfQuestions;
	private String name;
	private PlayableClasses classBeloning;
	private int levelActivation;
	private DamageType damageType;
	private WeaponType weaponType;
	private Effect effect;
	private AttackType attackType;
	private int range;
	private int answerPercentage;
	private boolean canAttackFriendly;
	private int mana;
	
	public Attack(int maxDamage, int minDamage, int questionLevel, AttributeTypes attributeType, 
			int reloadTime, int numberOfQuestions, String name, PlayableClasses classBeloning,
			int levelActivation, DamageType damageType, WeaponType weaponType,Effect effect,
			AttackType attackType, int range, int answerPercentage, boolean canAttackFriendly, 
			int mana){
		this.maxDamage = maxDamage;
		this.minDamage = minDamage;
		this.questionLevel = questionLevel;
		this.attributeType = attributeType;
		this.reloadTime = reloadTime;
		this.numberOfQuestions = numberOfQuestions;
		this.name = name;
		this.classBeloning = classBeloning;
		this.levelActivation = levelActivation;
		this.damageType = damageType;
		this.weaponType = weaponType;
		this.effect = effect;
		this.attackType = attackType;
		this.range = range;
		this.answerPercentage = answerPercentage;
		this.canAttackFriendly = canAttackFriendly;
		this.mana = mana;
	}
	
	public int getMana() {
		return mana;
	}

	public int getMaxDamage() {
		return maxDamage;
	}
	public void setMaxDamage(int maxDamage) {
		this.maxDamage = maxDamage;
	}
	public int getMinDamage() {
		return minDamage;
	}
	public void setMinDamage(int minDamage) {
		this.minDamage = minDamage;
	}
	public int getQuestionLevel() {
		return questionLevel;
	}
	public void setQuestionLevel(int questionLevel) {
		this.questionLevel = questionLevel;
	}
	public AttributeTypes getAttribute() {
		return attributeType;
	}
	public void setAttribute(AttributeTypes attribute) {
		this.attributeType = attribute;
	}
	public int getReloadTime() {
		return reloadTime;
	}
	public void setReloadTime(int reloadTime) {
		this.reloadTime = reloadTime;
	}
	public int getNumberOfQuestions() {
		return numberOfQuestions;
	}
	public void setNumberOfQuestions(int numberOfQuestions) {
		this.numberOfQuestions = numberOfQuestions;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public PlayableClasses getClassBeloning() {
		return classBeloning;
	}
	public void setClassBeloning(PlayableClasses classBeloning) {
		this.classBeloning = classBeloning;
	}
	public int getLevelActivation() {
		return levelActivation;
	}
	public void setLevelActivation(int levelActivation) {
		this.levelActivation = levelActivation;
	}
	public DamageType getDamageType() {
		return damageType;
	}
	public void setDamageType(DamageType damageType) {
		this.damageType = damageType;
	}
	public WeaponType getWeapon() {
		return weaponType;
	}
	public void setWeapon(WeaponType weapon) {
		this.weaponType = weapon;
	}
	public Effect getEffect() {
		return effect;
	}
	public void setEffect(Effect effect) {
		this.effect = effect;
	}
	public AttackType getAttackType() {
		return attackType;
	}
	public void setAttackType(AttackType attackType) {
		this.attackType = attackType;
	}

	public AttributeTypes getAttributeType() {
		return attributeType;
	}

	public void setAttributeType(AttributeTypes attributeType) {
		this.attributeType = attributeType;
	}

	public WeaponType getWeaponType() {
		return weaponType;
	}

	public void setWeaponType(WeaponType weaponType) {
		this.weaponType = weaponType;
	}

	public int getRange() {
		return range;
	}

	public void setRange(int range) {
		this.range = range;
	}

	public int getAnswerPercentage() {
		return answerPercentage;
	}

	public void setAnswerPercentage(int answerPercentage) {
		this.answerPercentage = answerPercentage;
	}

	public boolean isCanAttackFriendly() {
		return canAttackFriendly;
	}
}