package shared.models;

import java.io.Serializable;
import java.util.ArrayList;

import shared.CommonFunctions;


/**
 * 
 * @author Esben Fllesdal
 * @see Serializable
 */
public class Character extends WorldObject implements Serializable {

	private static final long serialVersionUID = 1L;
	private String name;
	private Gender sex;
	private int LVL;
	private Attributes attribute;
	private ArrayList<Attack> listOfPhysicalAttacks;
	private ArrayList<Attack> listOfMagicalAttacks;
	private Location currentLocation;//the current location of the character
	private Location targetLocation;//where the character is headed
	private int imageWidth;
	private int imageHeight;
	private String imagePath;

	public Character(String name, Gender sex, int LVL, 
			 Attributes attribute,
			ArrayList<Attack> listOfPhysicalAttacks,
			ArrayList<Attack> listOfMagicalAttacks,
			String imagePath){
		this.name = name;
		this.sex = sex;
		this.LVL = LVL;
		this.attribute = attribute;
		this.listOfMagicalAttacks = listOfMagicalAttacks;
		this.listOfPhysicalAttacks = listOfPhysicalAttacks;
		this.currentLocation = new Location(0,0);
		this.targetLocation = new Location(0,0);
		this.imageWidth = 0;
		this.imageHeight = 0;
		this.imagePath = imagePath;
	}

	public int getImageWidth() {
		return imageWidth;
	}

	public void setImageWidth(int imageWidth) {
		this.imageWidth = imageWidth;
	}

	public int getImageHeight() {
		return imageHeight;
	}

	public void setImageHeight(int imageHeight) {
		this.imageHeight = imageHeight;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Gender getSex() {
		return sex;
	}

	public void setSex(Gender sex) {
		this.sex = sex;
	}

	public int getLVL() {
		return LVL;
	}

	public void setLVL(int lvl) {
		LVL = lvl;
	}

	public ArrayList<Attack> getListOfPhysicalAttacks() {
		return listOfPhysicalAttacks;
	}

	public String getImagePath() {
		return imagePath;
	}
	
	public void setListOfPhysicalAttacks(ArrayList<Attack> listOfPhysicalAttacks) {
		this.listOfPhysicalAttacks = listOfPhysicalAttacks;
	}

	public ArrayList<Attack> getListOfMagicalAttacks() {
		return listOfMagicalAttacks;
	}

	public void setListOfMagicalAttacks(ArrayList<Attack> listOfMagicalAttacks) {
		this.listOfMagicalAttacks = listOfMagicalAttacks;
	}

	public Attributes getAttribute() {
		return attribute;
	}

	public void setAttribute(Attributes attribute) {
		this.attribute = attribute;
	}
	public Location getCurrentLocation() {
		return currentLocation;
	}

	public void setCurrentLocation(Location currentLocation) {
		this.currentLocation.setY(currentLocation.getYDouble());
		this.currentLocation.setX(currentLocation.getXDouble());
	}

	public Location getTargetLocation() {
		return targetLocation;
	}

	public void setTargetLocation(Location targetLocation) {
		this.targetLocation.setY(targetLocation.getY());
		this.targetLocation.setX(targetLocation.getX());
	}
	
	public void updatePosition(long elapsedTime){
		setCurrentLocation(CommonFunctions.updatePositions(elapsedTime, currentLocation, targetLocation));
	}
	
	/**
	 * @param questionLevel - the level of the question 
	 * @return the percentage to dodge an attack
	 */
	private int calculateDodge(int questionLevel){
		//TODO add feature effects
		float dodge = ((float)getAttribute().getDEX()/(float)questionLevel) + 2 * 10;
		if(dodge > 90){
			//maximum dodge is 90%
			return 90;
		}
		return (int)(dodge +0.5);
	}
	
	/**
	 * @param questionLevel -the level of the question
	 * @return if the character dodges or not
	 */
	public boolean dodgeSuccessful(int questionLevel){
		int randomNumber = (int) (Math.random()*101);
		if(calculateDodge(questionLevel) > randomNumber){
			return true;
		}
		return false;
	}
	
	
	public int calculateDefence(DamageType damageType){
		int defence  = getDefenceBasedOnDamageType(damageType);
		if( defence == -1){
			//the attacker uses the weapons damageType
		}
		else return defence; 
		return 0;
	}
	public boolean isAlive(){
		return attribute.getHP()>0;
	}
	public boolean updateHP(int damage){
		attribute.setHP(attribute.getHP()-damage);
		return attribute.getHP()<=0;
	}
	public void runEvent(EventType typeOfEvent){
		switch (typeOfEvent) {
		case RESPAWN:
			System.out.println("Reset values and respawn!");
			break;
		}
	}
	/**
	 * 
	 * @param damageType - the damagetype, given by the attack
	 * @return damage based on the damagetype
	 */
		protected int getDamageBasedOnDamageType(DamageType damageType){
			switch (damageType) {
			case SLASHING:
				return getAttribute().getSlashingOff(); 
			case BLUNT:
				return getAttribute().getBluntOff();
			case PIERCING:
				return getAttribute().getPiercingOff();
			case FIRE:
				return getAttribute().getFireOff();
			case WATER:
				return getAttribute().getWaterOff();
			case AIR:
				return getAttribute().getAirOff();
			case EARTH:
				return getAttribute().getEarthDef();
			case ALL:
				return -1;
				}
			return 0;
		}
		public String toString(){
			return this.getName();
		}
		/**
		 * 
		 * @param damageType - the damageType
		 * @return - the defense to a given character, based on the damage type
		 */
		protected int getDefenceBasedOnDamageType(DamageType damageType){
			switch (damageType) {
			case SLASHING:
				return getAttribute().getSlashingDef(); 
			case BLUNT:
				return getAttribute().getBluntDef();
			case PIERCING:
				return getAttribute().getPiercingDef();
			case FIRE:
				return getAttribute().getFireDef();
			case WATER:
				return getAttribute().getWaterDef();
			case AIR:
				return getAttribute().getAirDef();
			case EARTH:
				return getAttribute().getEarthDef();
			case ALL:
				return -1;
				}
			return 0;
		}
}