package shared.models;

import java.io.CharConversionException;
import java.io.Serializable;
import java.util.ArrayList;

import javax.smartcardio.ATR;

import shared.models.SimpleCharacter.CharacterType;

/**
 * 
 * @author Esben Fllesdal
 * @see Character
 * @see Serializable
 */
public class PlayableCharacter extends Character implements Serializable {

	private static final long serialVersionUID = 1L;
	private int playableCharacterId;
	
	private ArrayList<Item> inventory;
	private Item equipedItemHead;
	private Item equipedItemLeftRing;
	private Item equipedItemRightRing;
	private Item equipedItemLeftArm;
	private Item equipedItemRightArm;
	private Item equipedItemBody;
	private Item equipedItemLegs;
	private Item equipedItemAmulet;
	
	private int playerId;
	private PlayableClasses playerClass;
	private String respawnZone;
	private int money;
	private DamageType damageTypeOnLastAttack;
	private int XP;
	
	public PlayableCharacter(String name, Gender sex, PlayableClasses playerClass){
		super(name,sex,1,new Attributes(100,100,1,1,1,1,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0),null,null,"testDude");
		this.playerClass = playerClass;
		this.XP = 0;
		this.money = 100;
	}
	public PlayableCharacter(String name, Gender sex, int LVL, 
			Attributes attribute, 
			ArrayList<Item> inventory, int playerId, PlayableClasses playerClass,
			String respawnZone, int money, ArrayList<Attack> physicalAttackList,
			ArrayList<Attack> magicalAttackList,
			String imagePath,
			Item equipedItemHead, Item equipedItemLeftRing,	Item equipedItemRightRing,
			Item equipedItemLeftArm, Item equipedItemRightArm,	Item equipedItemBody,
			Item equipedItemLegs, Item equipedItemAmulet, int XP){
		super(name, sex, LVL,  attribute, physicalAttackList, magicalAttackList, imagePath);
		this.inventory = inventory;
		this.playerId = playerId;
		this.playerClass = playerClass;
		this.respawnZone = respawnZone;
		this.money = money;
		this.XP = XP;
		
		this.equipedItemHead = equipedItemHead;
		this.equipedItemLeftRing = equipedItemLeftRing;
		this.equipedItemRightRing = equipedItemRightRing;
		this.equipedItemLeftArm = equipedItemLeftArm;
		this.equipedItemRightArm = equipedItemRightArm;
		this.equipedItemBody = equipedItemBody;
		this.equipedItemLegs = equipedItemLegs;
		this.equipedItemAmulet = equipedItemAmulet;
		
	}
	
	@Deprecated
	/**
	 * removes an item from the inventroy 
	 * @param itemName - the name of the item to remove
	 */
	public void RemoveItemFromInventory(String itemName){
		//we want to remove an item from the inventory.
		//But if it is possible, we will remove the item
		//which is in the backpack.
		//item name is unique in the database
		Item deletableItem = null;
		for(Item item: inventory){
			if(item.getName().equals(itemName)){
				if(item.getEquipLocation().equals(EquipLocation.BACKPACK)){
					//this item is the correct item, and is in the backPack
					//we therefor deletes it from the inventory
					inventory.remove(item);
					return;
				}
				else {
					//this is the wanted item, but is is not in 
					//the backpack. We keep it, because there might
					//be another item which is not equipped on the 
					//character. If we find another in the 
					//backpack, we delete it. 
					deletableItem = item;
				}
			}
		}
		if(deletableItem != null){
			// we only found one item which, and this was equipped.
			// we remove this item, if it is found
			inventory.remove(deletableItem);
		}
	}
	
	public void removeItemFromInventory(Item item){
		inventory.remove(item);
	}
	
	/**
	 * Adds an item to the inventory
	 * @param item - the item which will be added to the inventory
	 * @param equipLocation - the location of the new item
	 */
	public void AddItemToInventory(Item item, EquipLocation equipLocation){
		//item.setEquipLocation(equipLocation);
		inventory.add(item);
	}
	
	/**
	 * Adds an item to the inventory. As default, it will be added to the backpack
	 * @param item - the item to add to the inventory
	 */
	public void AddItemToInventory(Item item){
//		item.setEquipLocation(EquipLocation.BACKPACK);
		inventory.add(item);
	}
	public SimpleCharacter getSimpleCharacter(){
		SimpleCharacter simpleCha =  new SimpleCharacter(getName(),getImagePath(),CharacterType.PLAYER_CHARACTER,getLVL(),getPlayerClass(),getSex(),getAttribute().getHP(),getAttribute().getMP(),getAttribute().getMaxHP(),getAttribute().getMaxMP());
		simpleCha.setCurrentLocationReference(getCurrentLocation());
		simpleCha.setTargetLocationReference(getTargetLocation());
		return simpleCha;
	}
	
	public int getPlayableCharacterId() {
		return playableCharacterId;
	}
	
	public void setPlayableCharacterId(int playableCharacterId) {
		this.playableCharacterId = playableCharacterId;
	}

	public ArrayList<Item> getInventory() {
		return inventory;
	}
	
	public int getAttackRange(){
		int shortestRange = 0;
		Weapon rightWeapon = getRightWeapon();
		if(rightWeapon != null){
			shortestRange = rightWeapon.getRange();
		}
		Weapon leftWeapon = getLeftWeapon();
		if(leftWeapon != null) {
			if(leftWeapon.getRange() < shortestRange || shortestRange == 0){
				shortestRange = leftWeapon.getRange();
			}
		}
		return shortestRange;
	}
	/**
	 * Returns the distance from the current position to position x,y
	 * @param x
	 * @param y
	 * @return
	 */
	public int getDistance(int x,int y){
		return (int)Math.sqrt(Math.pow((x-this.getCurrentLocation().getX()),2)+Math.pow((y-this.getCurrentLocation().getY()),2));
	}
	public void setInventory(ArrayList<Item> inventory) {
		this.inventory = inventory;
	}

	public int getPlayerId() {
		return playerId;
	}

	public void setPlayerId(int playerId) {
		this.playerId = playerId;
	}

	public PlayableClasses getPlayerClass() {
		return playerClass;
	}

	public void setPlayerClass(PlayableClasses playerClass) {
		this.playerClass = playerClass;
	}

	public String getRespawnZone() {
		return respawnZone;
	}

	public void setRespawnZone(String respawnZone) {
		this.respawnZone = respawnZone;
	}

	public int getMoney() {
		return money;
	}

	public void setMoney(int money) {
		this.money = money;
	}
	
	/**
	 * 
	 * @return the weapon equipped in the left hand
	 */
	private Weapon getLeftWeapon(){
		if(equipedItemLeftArm instanceof Weapon){
			return (Weapon)equipedItemLeftArm;
		}
		return null;
	}
	
	/**
	 * 
	 * @return the weapon equipped in the right arm
	 */
	private Weapon getRightWeapon(){
		if(equipedItemRightArm instanceof Weapon){
			return (Weapon)equipedItemRightArm;
		}
		return null;
	}
	
	/**
	 * will calculate the damage which an player deals
	 * @param attribute - which attribute to do add bonus from(STR, INT, DEX or CON)
	 * @param attackType - if this is an magical or physical attack
	 * @param effect - the effect on an attack
	 * @param damageType - the damage type
	 * @return total bonus damage( not included the damage given from the answer)
	 */
	public int calculateDamgeDealt(AttributeTypes attribute, 
			AttackType attackType, Effect effect, DamageType damageType){
		int damage = 0;
		Weapon rightWeapon = null;
		Weapon leftWeapon = null;
		//calculates the attribute associated with the attack
		damage += getAttributeValue(attribute);
		//calculates the damage based on damageType
		int damageTypeValue = getDamageBasedOnDamageType(damageType);
		if(damageTypeValue != -1){
			damage += damageTypeValue;
		}
		else {
			// there is not given any attack type, we use one of the equipped weapons
			rightWeapon = getRightWeapon();
			leftWeapon = getLeftWeapon();
			if(rightWeapon != null){
				damage += getBonusDamgeFromAttributes(rightWeapon.getWeaponType());
				damageTypeOnLastAttack = rightWeapon.getDamageType();
			}
			else{
				if(leftWeapon != null){
					damage += getBonusDamgeFromAttributes(leftWeapon.getWeaponType());
					damageTypeOnLastAttack = leftWeapon.getDamageType();
				}
			}
		}
		if(attackType == AttackType.PHYSICAL){
			if(leftWeapon == null){
				leftWeapon = getLeftWeapon();
			}
			if (rightWeapon == null){
				rightWeapon = getRightWeapon();
			}
			if(leftWeapon != null){
				damage += leftWeapon.getDamage();
			}
			if(rightWeapon != null){
				damage += rightWeapon.getDamage();
			}
			return damage;
		}
		else if(attackType == AttackType.MAGICAL){
			return damage;
		}
		//TODO: implement effect and feature bonus for the attack 
		return damage;
	}

	public DamageType getDamageTypeOnLastAttack() {
		return damageTypeOnLastAttack;
	}

	/**
	 * @param attribute - the given attribute
	 * @return the attribute value to a given attribute
	 */
	private int getAttributeValue(AttributeTypes attribute){
		switch (attribute) {
		case STR:
			return getAttribute().getSTR();
		case INT:
			return getAttribute().getINT();
		case DEX:
			getAttribute().getDEX();
		case CON:
			getAttribute().getCON();
		default:
			return 0;
		}
	}
	
	/**
	 * @param weaponType the weapontype
	 * @return bonusdamage fr om the weaponType
	 */
	private int getBonusDamgeFromAttributes(WeaponType weaponType){
		AttributeTypes attribute = null;
		switch (weaponType) {
		case BOW:
			attribute = AttributeTypes.PIERCING_OFF;
			break;
		case NATRUAL_PIERCING: 
			attribute = AttributeTypes.PIERCING_OFF;
			break;
		case SPEAR:
			attribute = AttributeTypes.PIERCING_OFF;
			break;
		case SWORD:
			attribute = AttributeTypes.SLASHING_OFF;
			break;
		case AXE:
			attribute = AttributeTypes.SLASHING_OFF;
			break;
		case DAGGER:
			attribute = AttributeTypes.SLASHING_OFF;
			break;
		case NATRUAL_SLASHING:
			attribute = AttributeTypes.SLASHING_OFF;
			break;
		case NATRUAL_BLUNT:
			attribute = AttributeTypes.BLUNT_OFF;
			break;
		case CLUB:
			attribute = AttributeTypes.BLUNT_OFF;
			break;
		case MACE:
			attribute = AttributeTypes.BLUNT_OFF;
			break;
		case STAFF:
			attribute = AttributeTypes.BLUNT_OFF;
			break;
		case WEAPON:
			attribute = AttributeTypes.BLUNT_OFF;
			break;
		}
		switch (attribute) {
		case PIERCING_OFF:
			return getAttribute().getPiercingOff();
		case BLUNT_OFF :
			return getAttribute().getBluntOff();
		case SLASHING_OFF:
			return getAttribute().getSlashingOff();
		case WATER_OFF:
			return getAttribute().getWaterOff();
		case EARTH_OFF:
			return getAttribute().getEarthOff();
		case AIR_OFF:
			return getAttribute().getAirOff();
		default:
			return 0;
		}
	}

	public Item getEquipedItemHead() {
		return equipedItemHead;
	}

	public Item getEquipedItemLeftRing() {
		return equipedItemLeftRing;
	}

	public Item getEquipedItemRightRing() {
		return equipedItemRightRing;
	}

	public Item getEquipedItemLeftArm() {
		return equipedItemLeftArm;
	}

	public Item getEquipedItemRightArm() {
		return equipedItemRightArm;
	}

	public Item getEquipedItemBody() {
		return equipedItemBody;
	}

	public Item getEquipedItemLegs() {
		return equipedItemLegs;
	}

	public Item getEquipedItemAmulet() {
		return equipedItemAmulet;
	}

	public void setEquipedItemHead(Item equipedItemHead) {
		this.equipedItemHead = equipedItemHead;
	}

	public void setEquipedItemLeftRing(Item equipedItemLeftRing) {
		this.equipedItemLeftRing = equipedItemLeftRing;
	}

	public void setEquipedItemRightRing(Item equipedItemRightRing) {
		this.equipedItemRightRing = equipedItemRightRing;
	}

	public void setEquipedItemLeftArm(Item equipedItemLeftArm) {
		this.equipedItemLeftArm = equipedItemLeftArm;
	}

	public void setEquipedItemRightArm(Item equipedItemRightArm) {
		this.equipedItemRightArm = equipedItemRightArm;
	}

	public void setEquipedItemBody(Item equipedItemBody) {
		this.equipedItemBody = equipedItemBody;
	}

	public void setEquipedItemLegs(Item equipedItemLegs) {
		this.equipedItemLegs = equipedItemLegs;
	}

	public void setEquipedItemAmulet(Item equipedItemAmulet) {
		this.equipedItemAmulet = equipedItemAmulet;
	}
	
	/**
	 * Gets the equiped item at the specified location
	 * 
	 * @param loc the location of the item belongs to
	 * @return the item at the location, null if none equipped, or
	 * the location is not one of the equippable locations
	 */
	public Item getEquipedItemFromEquipLocation(EquipLocation loc){
		if(loc == EquipLocation.HEAD){
			return getEquipedItemHead();
		}
		else if(loc == EquipLocation.LEFT_RING){
			return getEquipedItemLeftRing();
		}
		else if(loc == EquipLocation.RIGHT_RING){
			return getEquipedItemRightRing();
		}
		else if(loc == EquipLocation.LEFT_ARM){
			return getEquipedItemLeftArm();
		}
		else if(loc == EquipLocation.RIGHT_ARM){
			return getEquipedItemRightArm();
		}
		else if(loc == EquipLocation.BODY){
			return getEquipedItemBody();
		}
		else if(loc == EquipLocation.LEGS){
			return getEquipedItemLegs();
		}
		else if(loc == EquipLocation.AMULET){
			return getEquipedItemAmulet();
		}
		return null;
	}
	
	public void setEquipedItemFromEquipLocation(EquipLocation loc, Item item){
		if(loc == EquipLocation.HEAD){
			setEquipedItemHead(item);
		}
		else if(loc == EquipLocation.LEFT_RING){
			setEquipedItemLeftRing(item);
		}
		else if(loc == EquipLocation.RIGHT_RING){
			setEquipedItemRightRing(item);
		}
		else if(loc == EquipLocation.LEFT_ARM){
			setEquipedItemLeftArm(item);
		}
		else if(loc == EquipLocation.RIGHT_ARM){
			setEquipedItemRightArm(item);
		}
		else if(loc == EquipLocation.BODY){
			setEquipedItemBody(item);
		}
		else if(loc == EquipLocation.LEGS){
			setEquipedItemLegs(item);
		}
		else if(loc == EquipLocation.AMULET){
			setEquipedItemAmulet(item);
		}
	}
	
	public void equipItem(EquipLocation newLoc, EquipLocation oldLoc, Item item){

		//TODO: check attributes!
		
		
		
		
			if(oldLoc == EquipLocation.BACKPACK){
				
				if(inventory.contains(item)){
					
					Item alreadyEquipedItem = getEquipedItemFromEquipLocation(newLoc);
					if(alreadyEquipedItem != null){
						AddItemToInventory(alreadyEquipedItem);
					}
					setEquipedItemFromEquipLocation(newLoc, item);
					
					
					removeItemFromInventory(item);
				}
	
			}
			else{
				if(getEquipedItemFromEquipLocation(oldLoc) == item){
					Item alreadyEquipedItem = getEquipedItemFromEquipLocation(newLoc);
					if(alreadyEquipedItem != null){
						AddItemToInventory(alreadyEquipedItem);
					}
					setEquipedItemFromEquipLocation(newLoc, item);
					
					unEquip(oldLoc);	
				}
			}
		
		
	}
	
	public void moveItemToInventory(EquipLocation loc){
		Item item = getEquipedItemFromEquipLocation(loc);
		if(item != null){
			AddItemToInventory(item);			
		}
		unEquip(loc);
	}
	
	private void unEquip(EquipLocation loc){
		setEquipedItemFromEquipLocation(loc, null);
	}
	
	/**
	 * This method checks if the player have enough mana to cast a spell,
	 * and if he/she has, it subtract the mana cost
	 * @param magicCost - the amount of mana which is needed to perform a spell
	 * @return - true if the player has enough mana, false otherwise
	 */
	public boolean useMana(int magicCost){
		if(getAttribute().getMP() >= magicCost){
			getAttribute().setMP(getAttribute().getMP() - magicCost);
			return true;
		}
		return false;
	}
	
	/**
	 * This method checks if the attack can be performed with the equipped weapon
	 * @param weaponType
	 * @return true if the attack can be completed, false otherwise
	 */
	public boolean canPerformAttack(WeaponType weaponType){
		boolean returnValue = false;
		if(weaponType == WeaponType.WEAPON){
			return true;
		}
		if(getLeftWeapon() != null){
			if(getLeftWeapon().getWeaponType() == weaponType){
				return true;
			}
			else 
			{
				returnValue =  false;
			}
		}
		if(getRightWeapon() != null){
			if(getRightWeapon().getWeaponType() == weaponType){
				return true;
			}
			else 
			{
				returnValue = false;
			}
		}
		return returnValue;
	}

	public int getXP() {
		return XP;
	}

	public void setXP(int xp) {
		XP = xp;
	}
}