package shared.models;

import java.io.Serializable;
import java.util.ArrayList;


import shared.CommonFunctions;

/**
 * Holds limited information about a character object.
 * Used by the client to draw the relevant information.
 *
 */
public class SimpleCharacter extends WorldObject implements Serializable{
	public enum CharacterType{
		ENEMY_NPC,
		FRIENDLY_NPC,
		PLAYER_CHARACTER
	}
	
	private static final long serialVersionUID = 1L;
	private String name;
	private int LVL;
	private PlayableClasses characterClass;
	private Gender sex;
	private int HP;
	private int MP;
	private int MaxHP;
	private int MaxMP;
	private Location currentLocation;
	private Location targetLocation;
	private int imageHeight;
	private int imageWidth;
	private CharacterType charType;
	private String imagePath;
	private ArrayList<AttributeListener> attributeListeners;
	/**
	 * Constructor for simple Friendly and Enemy NPC
	 * @param name
	 * @param id
	 * @param charType
	 * @param LVL
	 * @param HP
	 * @param MP
	 * @param MaxHP
	 * @param MaxMP
	 */
	public SimpleCharacter(String name,String imagePath,CharacterType charType,int LVL,int HP,int MP,int MaxHP,int MaxMP){
		this.name = name;
		this.imagePath = imagePath;
		this.charType = charType;
		this.LVL = LVL;
		this.HP = HP;
		this.MP = MP;
		this.MaxHP = MaxHP;
		this.MaxMP = MaxMP;
		this.attributeListeners = new ArrayList<AttributeListener>();
		this.currentLocation = new Location(0,0);
		this.targetLocation = new Location(0,0);
		this.imageHeight = 0;
		this.imageWidth =0;
	}
	
	/**
	 * Constructor for simple player character
	 * @param name
	 * @param charType
	 * @param LVL
	 * @param characterClass
	 * @param sex
	 * @param HP
	 * @param MP
	 * @param MaxHP
	 * @param MaxMP
	 */
	public SimpleCharacter(String name,String imagePath,CharacterType charType, int LVL, PlayableClasses characterClass,Gender sex,int HP,int MP,int MaxHP,int MaxMP){
		this.name = name;
		this.imagePath = imagePath;
		this.attributeListeners = new ArrayList<AttributeListener>();
		this.charType = charType;
		this.LVL = LVL;
		this.characterClass = characterClass;
		this.sex = sex;
		this.HP = HP;
		this.MP = MP;
		this.MaxHP = MaxHP;
		this.MaxMP = MaxMP;
		this.currentLocation = new Location(0,0);
		this.targetLocation = new Location(0,0);
		this.imageHeight = 0;
		this.imageWidth =0;
	}
	
	public String getImagePath() {
		return imagePath;
	}

	public void setImagePath(String imagePath) {
		this.imagePath = imagePath;
	}

	public CharacterType getCharacterType() {
		return charType;
	}

	public void addAttributeListener(AttributeListener a){
		attributeListeners.add(a);
	}
	
	public void removeAttributeListener(AttributeListener a){
		attributeListeners.remove(a);
	}
	
	public void fireAttributeChanged(AttributeTypes attributeType, int oldValue, int newValue){
		for (AttributeListener a : attributeListeners) {
			a.AttributeChanged(attributeType, oldValue, newValue);
		}
	}
	public int getLVL() {
		return LVL;
	}
	public void setLVL(int lvl) {
		LVL = lvl;
	}
	public int getHP() {
		return HP;
	}
	public void setHP(int hp) {
		int oldval = HP;
		HP = hp;
		fireAttributeChanged(AttributeTypes.HP, oldval, hp);
	}
	public int getMP() {
		return MP;
	}
	public void setMP(int mp) {
		int oldval = MP;
		MP = mp;
		fireAttributeChanged(AttributeTypes.MP, oldval, mp);
	}
	public int getMaxHP() {
		return MaxHP;
	}
	public void setMaxHP(int maxHP) {
		MaxHP = maxHP;
	}
	public int getMaxMP() {
		return MaxMP;
	}
	public void setMaxMP(int maxMP) {
		MaxMP = maxMP;
	}
	public Location getCurrentLocation() {
		return currentLocation;
	}
	public void setCurrentLocation(Location currentLocation) {
		this.currentLocation.setX(currentLocation.getXDouble());
		this.currentLocation.setY(currentLocation.getYDouble());
	}
	public void setCurrentLocationReference(Location currentLocation){
		this.currentLocation = currentLocation;
	}
	public Location getTargetLocation() {
		return targetLocation;
	}
	public void setTargetLocation(Location targetLocation) {
		this.targetLocation.setX(targetLocation.getX());
		this.targetLocation.setY(targetLocation.getY());
	}
	public void setTargetLocationReference(Location targetLocation){
		this.targetLocation = targetLocation;
	}
	public String getName() {
		return name;
	}
	public PlayableClasses getCharacterClass() {
		return characterClass;
	}
	public Gender getSex() {
		return sex;
	}
	
	public void updatePosition(long elapsedTime){
		setCurrentLocation(CommonFunctions.updatePositions(elapsedTime, currentLocation, targetLocation));
	}
	public int getImageHeight() {
		return imageHeight;
	}
	public int getImageWidth() {
		return imageWidth;
	}
	public void setImageHeight(int height) {
		this.imageHeight = height;
		
	}
	public void setImageWidth(int width) {
		this.imageWidth = width;
		
	}
	
	
}
