package shared.models;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
/**
 * Holds limited information about a zone.
 * Used by client when drawing the zone.
 *
 */
public class SimpleZone implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private HashMap<String, StaticObject> staticObjectList;
	private Location respawn;
	private HashMap<String, SimpleCharacter> enemyNPCList;
	private HashMap<String, SimpleCharacter> friendNPCList;
	private HashMap<String, SimpleCharacter> characterList;
	private HashMap<String, TravelTrigger> triggerList;
	private String zoneId;
	private BackgroundTile[][] backgroundTileArray;// = MapSize*MapSize as array
	public SimpleZone(String zoneId,HashMap<String, StaticObject> staticObjectList,HashMap<String, EnemyNPC> enemyNPCList,
			HashMap<String, FriendlyNPC> friendNPCList,HashMap<String, SimpleCharacter> characterList,HashMap<String, TravelTrigger> triggerList,BackgroundTile[][] backgroundTileArray,Location respawn){
		this.staticObjectList = staticObjectList;
		this.characterList = characterList;
		this.zoneId = zoneId;
		this.backgroundTileArray = backgroundTileArray;
		this.respawn = respawn;
		this.enemyNPCList = new HashMap<String, SimpleCharacter>();
		this.friendNPCList = new HashMap<String, SimpleCharacter>();
		this.triggerList = triggerList;
		Iterator<String> itEnemy = enemyNPCList.keySet().iterator();
		while(itEnemy.hasNext()){
			String enemyId = itEnemy.next();
			this.enemyNPCList.put(enemyId, enemyNPCList.get(enemyId).getSimpleCharacter());
		}
		Iterator<String> itFriendly = friendNPCList.keySet().iterator();
		while(itFriendly.hasNext()){
			String friendlyId = itFriendly.next();
			this.friendNPCList.put(friendlyId, friendNPCList.get(friendlyId).getSimpleCharacter());
		}
		
		
	}
	public HashMap<String, SimpleCharacter> getCharacterList() {
		return characterList;
	}

	public Location getRespawn() {
		return respawn;
	}
	public void setCharacterList(HashMap<String, SimpleCharacter> characterList) {
		this.characterList = characterList;
	}
	public StaticObject getStaticObject(String id){
		return staticObjectList.get(id);
	}

	public HashMap<String, StaticObject> getStaticObjectList() {
		return staticObjectList;
	}

	public void setStaticObjectList(HashMap<String, StaticObject> staticObjectList) {
		this.staticObjectList = staticObjectList;
	}

	public HashMap<String, SimpleCharacter> getEnemyNPCList() {
		return enemyNPCList;
	}

	public void setEnemyNPCList(HashMap<String, SimpleCharacter> enemyNPCList) {
		this.enemyNPCList = enemyNPCList;
	}
	public HashMap<String, TravelTrigger> getTravelTriggerList(){
		return triggerList;
	}
	public HashMap<String, SimpleCharacter> getFriendNPCList() {
		return friendNPCList;
	}

	public void setFriendNPCList(HashMap<String, SimpleCharacter> friendNPCList) {
		this.friendNPCList = friendNPCList;
	}

	public String getZoneId() {
		return zoneId;
	}

	public void setZoneId(String zoneId) {
		this.zoneId = zoneId;
	}

	public BackgroundTile[][] getBackgroundTileArray() {
		return backgroundTileArray;
	}

	public void setBackgroundTileArray(BackgroundTile[][] backgroundTileArray) {
		this.backgroundTileArray = backgroundTileArray;
	}
	/**For testing purpose only
	 * 
	 * @param id
	 * @param obj
	 */
	public void addStaticObject(String id,StaticObject obj){
		staticObjectList.put(id, obj);
	}
	public void addEnemyNPC(String id,SimpleCharacter character){
		enemyNPCList.put(id, character);
	}
	public void addFriendlyNPC(String id,SimpleCharacter character){
		friendNPCList.put(id, character);
	}
	public void removeEnemyNPC(String id){
		enemyNPCList.remove(id);
	}
	public void removeFriendlyNPC(String id){
		friendNPCList.remove(id);
	}

	public void addPlayerCharacter(String name,SimpleCharacter character){
		characterList.put(name, character);
	}
	public void removePlayerCharacter(String name){
		characterList.remove(name);
	}
}
