package shared.models;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 * Zone object used by server. Holds the information about all the objects in a zone.
 *
 */
public class Zone implements Serializable{
	private SimpleZone simpleZone;
	private Location respawnPoint;
	private String zoneId;
	private String zoneName;
	private int horizontalSize=0;
	private int verticalSize=0;
	


	private HashMap<String, EnemyNPC> enemyNPCList;
	private HashMap<String, FriendlyNPC> friendNPCList;
	private HashMap<String, PlayableCharacter> clientList;
	private HashMap<String, StaticObject> staticObjectList;
	private HashMap<String, TravelTrigger> travelTriggerList;
	private BackgroundTile[][] backgroundTileArray;// =array of backgroundTiles
	public Zone(String zoneId,String zoneName,
			BackgroundTile[][] backgroundArray,Location respawnPoint,
			HashMap<String,EnemyNPC> eNPCList,
			HashMap<String,FriendlyNPC> fNPCList,
			HashMap<String,StaticObject> sObjList,
			HashMap<String, TravelTrigger> triggerList){
		this.zoneId = zoneId;
		this.zoneName = zoneName;
		this.respawnPoint = respawnPoint;
		clientList = new HashMap<String,PlayableCharacter>();
		enemyNPCList = eNPCList;
		friendNPCList = fNPCList;
		staticObjectList = sObjList;
		this.travelTriggerList = triggerList;
		if(triggerList==null){
			this.travelTriggerList = new HashMap<String, TravelTrigger>();
		}
		
		this.backgroundTileArray = backgroundArray;
		horizontalSize = backgroundArray.length;
		if(horizontalSize>0){
			verticalSize = backgroundArray[0].length;
		}
		simpleZone = new SimpleZone(zoneId,staticObjectList,enemyNPCList,friendNPCList,new HashMap<String, SimpleCharacter>(),travelTriggerList,backgroundTileArray,this.respawnPoint);
	}
	public Zone(String zoneId,String zoneName,BackgroundTile[][] backgroundArray,Location respawnPoint){
		this.zoneId = zoneId;
		this.zoneName = zoneName;
		this.respawnPoint = respawnPoint;
		clientList = new HashMap<String,PlayableCharacter>();
		enemyNPCList = new HashMap<String, EnemyNPC>();
		friendNPCList = new HashMap<String, FriendlyNPC>();
		staticObjectList = new HashMap<String, StaticObject>();
		travelTriggerList = new HashMap<String, TravelTrigger>();
		this.backgroundTileArray = backgroundArray;
		horizontalSize = backgroundArray.length;
		if(horizontalSize>0){
			verticalSize = backgroundArray[0].length;
		}
		simpleZone = new SimpleZone(zoneId,staticObjectList,enemyNPCList,friendNPCList,new HashMap<String, SimpleCharacter>(),travelTriggerList,backgroundTileArray,this.respawnPoint);
	}
	public Location getRespawnPoint(){
		return respawnPoint;
	}
	public String getZoneId() {
		return zoneId;
	}
	public void setZoneId(String zoneId){
		this.zoneId = zoneId;
	}
	public BackgroundTile[][] getBackgroundArray(){
		return backgroundTileArray;
	}
	public void setTileWithCascade(int positionX, int positionY,BackgroundTile newTile){
		String tileName = newTile.getTextureId();
		int arraySizeX = backgroundTileArray.length;
		int arraySizeY = 0;
		if(arraySizeX>0){
			arraySizeY = backgroundTileArray[0].length;
		}
		boolean atLeftEdge=false;
		boolean atTopEdge=false;
		boolean atBottomEdge=false;
		boolean atRightEdge=false;
		if(positionX==arraySizeX-1)
			atRightEdge = true;
		if(positionX == 0)
			atLeftEdge = true;
		if(positionY == arraySizeY-1)
			atBottomEdge = true;
		if(positionY == 0)
			atTopEdge = true;
		
		
		if(!atTopEdge&&!atLeftEdge)
			replaceTilePart(backgroundTileArray[positionX-1][positionY-1],4,tileName.charAt(0)+"");
		if(!atTopEdge){
			replaceTilePart(backgroundTileArray[positionX][positionY-1],3,tileName.charAt(0)+"");
			replaceTilePart(backgroundTileArray[positionX][positionY-1],4,tileName.charAt(1)+"");
		}
		if(!atTopEdge&&!atRightEdge)
			replaceTilePart(backgroundTileArray[positionX+1][positionY-1],3,tileName.charAt(1)+"");
		
		if(!atLeftEdge){
			replaceTilePart(backgroundTileArray[positionX-1][positionY],2,tileName.charAt(0)+"");
			replaceTilePart(backgroundTileArray[positionX-1][positionY],4,tileName.charAt(2)+"");
		}
		if(!atRightEdge){
			replaceTilePart(backgroundTileArray[positionX+1][positionY],1,tileName.charAt(1)+"");
			replaceTilePart(backgroundTileArray[positionX+1][positionY],3,tileName.charAt(3)+"");
		}
		if(!atLeftEdge&&!atBottomEdge)
			replaceTilePart(backgroundTileArray[positionX-1][positionY+1],2,tileName.charAt(2)+"");
		if(!atBottomEdge){
			replaceTilePart(backgroundTileArray[positionX][positionY+1],1,tileName.charAt(2)+"");
			replaceTilePart(backgroundTileArray[positionX][positionY+1],2,tileName.charAt(3)+"");
		}
		if(!atRightEdge&&!atBottomEdge)
			replaceTilePart(backgroundTileArray[positionX+1][positionY+1],1,tileName.charAt(3)+"");
		
		backgroundTileArray[positionX][positionY] = newTile;
	}
	private void replaceTilePart(BackgroundTile tile,int part, String replacement){
		String tileTexture = tile.getTextureId();
		String newTileTexture ="";
		switch (part){
		case 1:
			newTileTexture = replacement +tileTexture.substring(1,4);
			break;
		case 2:
			newTileTexture = tileTexture.substring(0, 1) + replacement + tileTexture.substring(2);
			break;
		case 3:
			newTileTexture = tileTexture.substring(0, 2) + replacement + tileTexture.substring(3);
			break;
		case 4:
			newTileTexture = tileTexture.substring(0, 3) + replacement;
			break;
		}
		tile.setTextureId(newTileTexture);
		
	}
	
	public void setTile(int positionX,int positionY,BackgroundTile newTile){
		backgroundTileArray[positionX][positionY] = newTile;
	}
	public SimpleZone getSimpleZone(){
		return simpleZone;
	}

	
	public void setClientCurrentAndTargetLocation(String clientId,Location currentLocation,Location targetLocation){
		clientList.get(clientId).setTargetLocation(targetLocation);
		clientList.get(clientId).setCurrentLocation(currentLocation);
	}
	/**
	 * Updates the position of the characters on the server side
	 * @param elapsedTime
	 */
	public void updatePositions(long elapsedTime){
		Iterator<FriendlyNPC> itFriendNPC = friendNPCList.values().iterator();
		while(itFriendNPC.hasNext()){
			itFriendNPC.next().updatePosition(elapsedTime);
		}
		Iterator<EnemyNPC> itEnemyNPC = enemyNPCList.values().iterator();
		while(itEnemyNPC.hasNext()){
			itEnemyNPC.next().updatePosition(elapsedTime);
		}
		Iterator<PlayableCharacter> itPC = clientList.values().iterator();
		while(itPC.hasNext()){
			itPC.next().updatePosition(elapsedTime);
		}
	}
//	public HashMap<String, EnemyNPC> getEnemyNPCList() {
//		return enemyNPCList;
//	}
//	public HashMap<String, FriendlyNPC> getFriendNPCList() {
//		return friendNPCList;
//	}
//	public HashMap<String, PlayableCharacter> getClientList() {
//		return clientList;
//	}
	
	public String getZoneName() {
		return zoneName;
	}
	public void setZoneName(String zoneName) {
		this.zoneName = zoneName;
	}

	public TravelTrigger getTravelTrigger(String id){
		return travelTriggerList.get(id);
	}

	public EnemyNPC getEnemyNPC(String enemyId){
		return enemyNPCList.get(enemyId);
	}
	public FriendlyNPC getFriendlyNPC(String friendlyId){
		return friendNPCList.get(friendlyId);
	}
	public StaticObject getStaticObject(String sObjId){
		return staticObjectList.get(sObjId);
	}
	public PlayableCharacter getClientCharacter(String clientId){
		return clientList.get(clientId);
	}
	public void removeCharacter(String clientId){
		clientList.remove(clientId);
	}
	public Iterator<String> getClientListIterator(){
		return clientList.keySet().iterator();
	}
	public ArrayList<EnemyNPC> getEnemyList(){
		ArrayList<EnemyNPC> enemyList = new ArrayList<EnemyNPC>();
		Iterator<String> it = enemyNPCList.keySet().iterator();
		while(it.hasNext()){
			enemyList.add(enemyNPCList.get(it.next()));
		}
		return enemyList;
	}
	public HashMap<String, EnemyNPC> getEnemyMap(){
		return enemyNPCList;
	}
	public HashMap<String, FriendlyNPC> getFriendlyMap(){
		return friendNPCList;
		
	}
	public HashMap<String, StaticObject> getStaticObjectMap(){
		return staticObjectList;
		
	}
	public HashMap<String, TravelTrigger> getTriggerMap(){
		return travelTriggerList;
	}
	public ArrayList<FriendlyNPC> getFriendlyList(){
		ArrayList<FriendlyNPC> friendList = new ArrayList<FriendlyNPC>();
		Iterator<String> it = friendNPCList.keySet().iterator();
		while(it.hasNext()){
			friendList.add(friendNPCList.get(it.next()));
		}
		return friendList;
	}
	public ArrayList<StaticObject> getStaticObjectList(){
		ArrayList<StaticObject> staticObjList = new ArrayList<StaticObject>();
		Iterator<String> it = staticObjectList.keySet().iterator();
		while(it.hasNext()){
			staticObjList.add(staticObjectList.get(it.next()));
		}
		return staticObjList;
	}
	public ArrayList<TravelTrigger> getTravelTriggerList(){
		ArrayList<TravelTrigger> triggList = new ArrayList<TravelTrigger>();
		Iterator<String> it = travelTriggerList.keySet().iterator();
		while(it.hasNext()){
			triggList.add(travelTriggerList.get(it.next()));
		}
		return triggList;
	}
	public void putClientInZone(String clientId,PlayableCharacter clientCharacter){
		clientList.put(clientId, clientCharacter);
		getSimpleZone().getCharacterList().put(clientCharacter.getName(),clientCharacter.getSimpleCharacter());
	}
	public void addStaticObject(String id,StaticObject object){
		staticObjectList.put(id,object );
	}
	
	public void addFriendlyNPC(String id,FriendlyNPC npc){
		friendNPCList.put(id, npc);
		simpleZone.addFriendlyNPC(id, npc.getSimpleCharacter());
	}
	
	public void addEnemyNPC(String id,EnemyNPC npc){
		enemyNPCList.put(id,npc);
		simpleZone.addEnemyNPC(id, npc.getSimpleCharacter());
	}
	public void addTravelTrigger(String id,TravelTrigger travel){
		travelTriggerList.put(id, travel);
	}
	public void removeStaticObject(String id){
		staticObjectList.remove(id);
	}
	public void removeFriendlyNPC(String id){
		friendNPCList.remove(id);
		simpleZone.removeFriendlyNPC(id);
	}
	public void removeEnemyNPC(String id){
		enemyNPCList.remove(id);
		simpleZone.removeEnemyNPC(id);
	}
	public void removeTravelTrigger(String id){
		travelTriggerList.remove(id);
	}
	public int getHorizontalSize() {
		return horizontalSize;
	}
	public int getVerticalSize() {
		return verticalSize;
	}
}
