package shared.network;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

import shared.network.MessageHandler;
import shared.network.messages.Message;


/**
 * Holds a socket connection.
 * Reads and writes messages to the network.
 * @author thorgrun
 *
 */
public class ClientSocket implements Runnable{
	private Thread receiveMessageThread;
	private Socket clientSocket;
	private ObjectInputStream in ;
	private ObjectOutputStream out;
	private boolean disconnect;
	private MessageHandler handler;
	public ClientSocket(Socket clientSocket){
		this.clientSocket = clientSocket;
		this.disconnect = false;
		receiveMessageThread = new Thread(this);
		receiveMessageThread.start();
	}
	/**
	 * Add a messageslistener. This listener will get the messages from network, 
	 * and lostconnection will be called when the connection is interrupted.
	 * @param handler
	 */
	public void addMessageListener(MessageHandler handler){
		this.handler= handler;
	}
	
	/**
	 * Reads objects from network, and passes it to the message handler.
	 */
	public void run() {
		try{
			in = new ObjectInputStream(clientSocket.getInputStream());
			out = new ObjectOutputStream(clientSocket.getOutputStream());
			Message netMsg;
			while(!disconnect){
				if(handler!=null){
					System.out.println("Waiting for message");
					netMsg = (Message) in.readObject();
					System.out.println("Received message");
					handler.getMessage(netMsg);
				}else{
					try {
						Thread.sleep(250);
					} catch (InterruptedException e) {
						System.out.println("InterruptedException: "+e.getMessage());
					}
				}
			}
		}catch(IOException ioEx){
			System.out.println("IoException:"+ioEx.getMessage());
		}catch(ClassNotFoundException classEx){
			System.out.println("ClassNotFoundException: "+classEx.getMessage());
		}finally{
			if(clientSocket!=null){
				try {
					clientSocket.close();
					System.out.println("connection closed!");
					handler.lostConnection();
				}catch (IOException er){
					System.out.println("close:"+er.getMessage());
				}
			}
		}
	}
	/**
	 * Disconnects and calls lostconnection at the messagehandler.
	 */
	public void disconnect(){
		this.disconnect = true;
		try {
			in.close();
			out.close();
			handler.lostConnection();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	/**
	 * Sends a message to the network.
	 * @param message
	 * @return
	 */
	public boolean sendMessage(Message message) {
		try{
			out.writeObject(message);
			out.flush();
			return true;
		}catch(IOException ioEx){
			System.out.println("Error on send message: " +ioEx.getMessage());
			return false;
		}
		
	}

	
	
}
