package shared.network;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;



import shared.network.MessageHandler;
import shared.network.messages.Message;
/**
 * Holds a connection.
 * @author thorgrun
 *
 */
public class Connection implements Runnable{
	private Socket clientSocket;
	private ObjectOutputStream oos = null;
	private ObjectInputStream ois = null;
	private Thread listenThread;
	private MessageHandler handler;
	private boolean connected;
	public Connection(String serverIpAdress,int portNumber,MessageHandler handler){
		this.handler = handler;
		try {
	        // open a socket connection
	    	System.out.println("Opening connection");
	        clientSocket = new Socket(serverIpAdress, portNumber);
	        connected = true;
	        System.out.println("Connected");
	        // open I/O streams for objects
	        oos = new ObjectOutputStream(clientSocket.getOutputStream());
	        ois = new ObjectInputStream(clientSocket.getInputStream());
	        // read an object from the server
	        listenThread = new Thread(this);
	        listenThread.start();
	      } catch(IOException e) {
	    	  connected = false;
	        System.out.println("Connection error:"+e.getMessage());
	      }
	}
	
	public boolean isConnected() {
		return connected;
	}

	public void setConnected(boolean connected) {
		this.connected = connected;
	}
	/**
	 * Disconnects and calls lostconnection at the messagehandler.
	 */
	public void closeConnection(){
		try{
			oos.close();
			ois.close();
			clientSocket.close();
			connected = false;
			handler.lostConnection();
		}catch(IOException e){
			System.out.println(e.getMessage());
			handler.lostConnection();
		}
	}
	/**
	 * Sends a message to the network.
	 * @param message
	 * @return
	 */
	public void sendMessage(Message msg){
		try{
			msg.printMessage();
			oos.writeObject(msg);
			oos.flush();
			System.out.println("message sent");
		}catch(IOException ioEx){
			System.out.println(ioEx.getMessage());
		}
	}
	
	/**
	 * Reads objects from network, and passes it to the message handler.
	 */
	public void run() {
		try{
			while(connected){
				Message msg = (Message)ois.readObject();
				System.out.println("Message received");
				handler.getMessage(msg);
			}
			
		}catch(ClassNotFoundException classEx) {
			System.out.println("ClassNotFoundException:"+classEx.getMessage());
		}catch(IOException ioEx){
			System.out.println("IOException:"+ioEx.getMessage());
			closeConnection();
		}
		connected = false;
	}
	
	
	
	
	
}
