package shared.network.messages;
/**
 * 
 * @author Esben Fllesdal
 * @see Message
 * @see MessageType
 * This class will contain the paramaeters needed to set/get or alter a player in the database
 * IMPORTANT: Have correct parameters according to messageType 
 */
public class PlayerInformationMessage extends Message {

	private static final long serialVersionUID = 1L;

	private String userName = null;
	private String password = null;
	private String sessionId = null;
	private String email = null;
	private String oldPassword = null;
	private String oldUserName = null; 
	
	/**
	 * Message used when altering a playerObject
	 * @param sourceId 
	 * @param targetId
	 * @param message
	 * @param username - the new username
	 * @param password - the new password
	 * @param sessionId
	 * @param email - the new email
	 * @param oldPassword - the old password
	 * @param oldUsername - the old username
	 * @param messageType 
	 * @see MessageType
	 * @see Message
	 */
	public PlayerInformationMessage(String sourceId, String targetId, String message, 
									String username, String password, String sessionId,
									String email, String oldPassword, String oldUsername,
									MessageType messageType) {
		super(sourceId, targetId, message, messageType);

		this.userName = username;
		this.password = password;
		this.sessionId = sessionId;
		this.email = email;
		this.oldPassword = oldPassword;
		this.oldUserName = oldUsername;
	}
/**
 * Message used when requesing a player
 * @param sourceId
 * @param targetId
 * @param message
 * @param username
 * @param password
 * @param sessionId
 * @param messageType
 * @see MessageType
 * @see Message
 */
	public PlayerInformationMessage(String sourceId, String targetId, String message,
			String username, String password, String sessionId, MessageType messageType){
		super(sourceId, targetId, message, messageType);
		this.userName = username;
		this.password = password;
		this.sessionId = sessionId;
	}

	/**
	 * Message used when creating a player
	 * @param sourceId
	 * @param targetId
	 * @param message
	 * @param username
	 * @param password
	 * @param sessionId
	 * @param email
	 * @param messageType
	 * @see MessageType
	 * @see Message
	 */
	public PlayerInformationMessage(String sourceId, String targetId, String message,
			String username, String password, String sessionId, String email, MessageType messageType){
		super(sourceId, targetId, message, messageType);
		this.userName = username;
		this.password = password;
		this.sessionId = sessionId;
		this.email = email;
	}
	
	public String getOldUserName() {
		return oldUserName;
	}

	public void setOldUserName(String oldUserName) {
		this.oldUserName = oldUserName;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getOldPassword() {
		return oldPassword;
	}

	public void setOldPassword(String newPassword) {
		this.oldPassword = newPassword;
	}

	public String getUserName() {
		return userName;
	}

	public String getPassword() {
		return password;
	}

	public String getSessionId() {
		return sessionId;
	}
}