package shared.network.messages.actions;

import shared.models.AttackType;
import shared.models.AttributeTypes;
import shared.models.DamageType;
import shared.models.Effect;

/**
 * This message will contain the information with damage and damagetype an
 * attack will inflict on a monster
 * @author Thor
 *
 */
public class AttackMessage extends ActionMessage{

	private static final long serialVersionUID = 1L;
	private int damage ;
	private DamageType damageType;
	private String attackerId;
	private String attackedId;
	private boolean successfullAttack = false;
	private AttributeTypes attributeType;
	private AttackType attackType;
	private Effect effect;
	private int questionLevel; 
	private boolean dodgedAttack = false; // is true, when an character successfully dodges an attack
	private int manaCost;
		
	public AttackMessage(String sourceId, String targetId, String message, String actionId,
			int baseDamage, DamageType damageType, String attackerId, String attackedId,
			AttributeTypes attributeType, AttackType attackType, Effect effect, int questionLevel, int mana) {
		super(sourceId, targetId, message, actionId,ActionType.ATTACK);
		this.damage = baseDamage;
		this.damageType = damageType;
		this.attackedId = attackedId;
		this.attackerId = attackerId; 
		this.attributeType = attributeType;
		this.attackType = attackType;
		this.effect = effect;
		this.questionLevel = questionLevel;
		this.manaCost  = mana;
		
	}
	public int getManaCost(){
		return this.manaCost;
	}
	public int getBaseDamage() {
		return damage;
	}

	public DamageType getDamageType() {
		return damageType;
	}

	public String getAttackerId() {
		return attackerId;
	}

	public String getAttackedId() {
		return attackedId;
	}
	
	public boolean isSuccessfullAttack() {
		return successfullAttack;
	}

	public void setSuccessfullAttack(boolean successfullAttack) {
		this.successfullAttack = successfullAttack;
	}
	
	/**
	 * 
	 * @return the total damage dealt if the attack succeeds, 0 other ways
	 */
	public int getDamage(){
		if(successfullAttack){
			return damage;
		}
		else{
			return 0;
		}
	}

	public void setBaseDamage(int baseDamage) {
		this.damage = baseDamage;
	}

	public AttributeTypes getAttributeType() {
		return attributeType;
	}
	
	/**
	 * updates the damage value
	 * @param damage - the damage value
	 */
	public void updateDamage(int damage){
		this.damage += damage;
	}

	public AttackType getAttackType() {
		return attackType;
	}

	public Effect getEffect() {
		return effect;
	}

	public int getQuestionLevel() {
		return questionLevel;
	}
	
	public boolean isDodgedAttack() {
		return dodgedAttack;
	}

	public void setDodgedAttack(boolean dodgedAttack) {
		this.dodgedAttack = dodgedAttack;
	}

	public void setDamageType(DamageType damageType) {
		this.damageType = damageType;
	}

}