package worldServer.messageHandler;

import java.util.ArrayList;
import java.util.HashMap;

import shared.network.MessageHandler;
import shared.network.messages.KingdomMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.SessionIdMessage;
import shared.models.PlayableCharacter;
import shared.models.Question;
import worldServer.serverControls.ServerControl;
import shared.network.Connection;
import shared.models.EnemyNPC;
/**
 * Holds a connection with the Database server. Handles messages received from the DB and sends messages to the DB.
 * @author thorgrun
 *
 */
public class DBHandler implements MessageHandler{
	
	private Connection conn;
	/**
	 * Connects to a DB with ip and port as params.
	 * @param serverIp
	 * @param portNr
	 */
	public DBHandler(String serverIp,int portNr){
		conn = new Connection(serverIp,portNr,this);
	}

	/**
	 * Return true if the connection is up.
	 * @return
	 */
	public boolean isConnected() {
		return conn.isConnected();
	}

	public void requestKingdom(String kingdomName){
		conn.sendMessage(new SessionIdMessage("WorldServer","Database","Retrieve kingdom: "+kingdomName,MessageType.GET_KINGDOM,"",kingdomName));
	}

	public void requestKingdomNames(){
		sendMessage(new SessionIdMessage("WorldServer","Database","Retrieve kingdomNames",MessageType.GET_ALL_KINGDOMNAMES,"",""));
	}
	/**
	 * Sends a message to the DB.
	 * @param msg
	 */
	public void sendMessage(Message msg){
		conn.sendMessage(msg);
	}
	
	/**
	 * Receives a message from the DB.
	 */
	public void getMessage(Message msg) {
		msg.printMessage();
		switch(msg.getMessageType()){
		case KINGDOM_OBJECT:
			KingdomMessage kingMsg = (KingdomMessage)msg;
			ServerControl.getServerControl().startUpKingdom(kingMsg.getKingdomName(),kingMsg.getSubjectName(), kingMsg.getKingdomId(), kingMsg.getZoneList());

			break;
		case OBJECT_MESSAGE:
			ObjectMessage objMsg = (ObjectMessage)msg;
			switch(objMsg.getObjectType()){
			case CHARACTER_OBJECT:
				ServerControl.getServerControl().getPlayer(objMsg.getSessionId()).setClientCharacter((PlayableCharacter)objMsg.getObject());
				break;
			case QUESTION_LIST:
				ServerControl.getServerControl().setQuestionList((ArrayList<Question>)objMsg.getObject());
				break;
			case KINGDOM_NAME_LIST:
				ServerControl.getServerControl().setKingdomNameList((ArrayList<String>)objMsg.getObject());
				break;
			}
			break;
		
		}
		
	}
	/**
	 * Sends a disconnect message to the DB.
	 */
	public void disconnect(){
		if(isConnected()){
			this.sendMessage(new Message("worldServer","DBserver","I want to disconnect!",MessageType.CLOSE_CONNECTION));
		}	
	}

	/**
	 * When the network connection is lost, this method is run.
	 */
	public void lostConnection() {
		// TODO Auto-generated method stub
		
	}
}
