package worldServer.messageHandler;

import shared.logger.LogSystem;
import shared.network.MessageHandler;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.ObjectMessage;
import shared.network.messages.ObjectType;
import shared.network.messages.PlayerVerificationResult;
import shared.network.messages.RegisterResult;
import shared.serverHelpers.WorldServer;
import shared.network.Connection;
import worldServer.serverControls.ServerControl;
/**
 * Holds a connection with the Lobby server. Handles messages received from the lobby and sends messages to the lobby.
 * @author thorgrun
 *
 */
public class LobbyHandler implements MessageHandler{
	private Connection conn;
	/**
	 * Creates a connection with the LobbyServer and sends a messsage for registration at the lobby.
	 * @param serverIp
	 * @param portNr
	 * @param serverInfo
	 */
	public LobbyHandler(String serverIp,int portNr,WorldServer serverInfo){
		conn = new Connection(serverIp,portNr,this);
		if(isConnected()){
			ObjectMessage registerMsg = new ObjectMessage("worldServer","Lobby","Register Server",MessageType.REGISTER_SERVER,serverInfo,ObjectType.WORLDSERVER_INFO,"Tha sessionId");
			sendMessage(registerMsg);
		}
	}
	/**
	 * Returns true if the connection is up
	 * @return
	 */
	public boolean isConnected() {
		return conn.isConnected();
	}

	/**
	 * Sends a message to the Lobby server.
	 * @param msg
	 */
	public void sendMessage(Message msg){
		conn.sendMessage(msg);
	}
	
	/**
	 * Receives a message from the Lobby Server.
	 */
	public void getMessage(Message msg) {
		writeLog("Received message from lobby to "+msg.getTargetId()+": "+msg.getMessage());
		switch(msg.getMessageType()){
		case REGISTER_RESULT:
			RegisterResult res = (RegisterResult)msg;
			if(res.isLoginSucessfull()){
				ServerControl.getServerControl().setSessionId(res.getTargetId());
			}
			break;
		case PLAYER_VERIFICATION_RESULT:
			PlayerVerificationResult verifyRes = (PlayerVerificationResult)msg;
			ServerControl.getServerControl().playerVerified(verifyRes.getSessionID(), verifyRes.isSuccessfull());
			
		}
		
		
	}
	/**
	 * Disconnects from the lobbyserver
	 * TODO: implement
	 */
	public void disconnect(){
		
	}
	/**
	 * Writes a message to a log file.
	 * @param logMessage
	 */
	private static void writeLog(String logMessage){
		LogSystem.getInstance().writeWorldLog(logMessage);
	}

	/**
	 * When the network connection is lost, this method is run.
	 * TODO: implement
	 */
	public void lostConnection() {
		// TODO Auto-generated method stub
		
	}
}
