package worldServer.serverControls;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import shared.network.messages.ChatMessage;
import shared.network.messages.Message;
import worldServer.messageHandler.MessageHandlerForWorld;
/**
 * The chatSystem for the game.
 * @author thorgrun
 *
 */
public class ChatSystem {
	HashMap<String, ArrayList<String>> chatChannels; //Key is name of channel, and value is a list of clientId that receives the messages
	private MessageHandlerForWorld sendMessage;//TODO: replace with proper listener
	/**
	 * Constructor for chatSystem. Creates an empty HashMap which will contain the channels, and a list of clients using the channel.
	 */
	public ChatSystem(MessageHandlerForWorld listener){
		this.sendMessage = listener;
		chatChannels = new HashMap<String, ArrayList<String>>();
	}
	/**
	 * Adds a client to a chatChannel. If the channel does not exists, it is created.
	 * @param clientId
	 * @param chatChannel
	 */
	public void addClientToChannel(String clientId,String chatChannel){
		//Creates channel if it does not exist
		if(chatChannels.get(chatChannel)==null){
			chatChannels.put(chatChannel, new ArrayList<String>());
		}
		chatChannels.get(chatChannel).add(clientId);
		
	}
	/**
	 * Removes a client from a chat channel.
	 * @param clientId
	 * @param chatChannel
	 */
	public void removeClientFromChannel(String clientId,String chatChannel){
		if(chatChannels.get(chatChannel)==null){
			System.err.println("Channel does not exist");
		}else{
			chatChannels.get(chatChannel).remove(clientId);
		}
		
	}
	public void removeClientFromAllChannels(String clientId){
		Iterator<ArrayList<String>> it = chatChannels.values().iterator();
		while(it.hasNext()){
			ArrayList<String> clientInChannel = it.next();
			if(clientInChannel.contains(clientId)){
				clientInChannel.remove(clientId);
			}
		}
	}
	/**
	 * Receives a message from a client
	 * @param msg
	 */
	public void receivedMessage(Message msg){
		ChatMessage chatMsg = (ChatMessage)msg;
		this.sendMessage(msg.getSourceId(), chatMsg.getToChannel(), chatMsg);
	}
	/**
	 * Sends a message from a client to all clients in a channel.
	 * @param fromClient
	 * @param toChannel
	 * @param msg
	 */
	private void sendMessage(String fromClient,String toChannel,Message msg){
		if(chatChannels.get(toChannel)==null){
			System.err.println("Channel does not exist");
		}else{
			for(String clientId: chatChannels.get(toChannel)){
				sendMessage.sendMessageToClient(clientId, msg);
			}
		}
	}
	/**
	 * For testing. Prints the id of the clients in the channel.
	 * @param chatChannel
	 */
	private void printChatMembers(String chatChannel){
		System.out.println("----"+chatChannel+"----");
		for(String client: chatChannels.get(chatChannel)){
			System.out.println(client);
		}
	}
	
}
