package worldServer.serverControls;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.Inet4Address;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import com.golden.gamedev.funbox.KeyCapture;

import shared.fileHandler.ReadBackgroundFromFile;
import shared.logger.LogSystem;
import shared.models.EnemyNPC;
import shared.models.FriendlyNPC;
import shared.models.Gender;
import shared.models.Location;
import shared.models.PlayableCharacter;
import shared.models.Question;
import shared.models.StaticObject;
import shared.models.Zone;
import shared.network.ConnectionListener;
import shared.network.NetworkServer;
import shared.network.messages.ChatMessage;
import shared.network.messages.LoginMessage;
import shared.network.messages.Message;
import shared.network.messages.MessageType;
import shared.network.messages.SessionIdMessage;
import shared.network.messages.actions.ActionMessage;
import shared.network.messages.actions.MovementMessage;
import shared.serverHelpers.WorldServer;
import worldServer.gameLoop.GameLoop;
import worldServer.gameLoop.GameWorld;
import worldServer.messageHandler.ClientHandler;
import worldServer.messageHandler.DBHandler;
import worldServer.messageHandler.LobbyHandler;
/**
 * Handles connections with network clients/db/lobby and startup of server.
 * @author thorgrun
 *
 */
public class ServerControl extends JFrame implements ConnectionListener, Runnable{
	private static String serverName = "TestKingdom";
	private static String NOKINGDOM = "No kingdoms found.";
	private HashMap<String, ClientHandler> clientList;
	private int lastClientNumber;
	private String sessionId;
	private String[] kingdomNames;
	private WorldServer worldInfo;
	private DBHandler dbHandler;
	private LobbyHandler lobbyHandler;
	private static ServerControl controlSingleton;
	private GameWorld game;
	private shared.fileHandler.Settings networkSettings;
	//Commandsline
	private boolean running;
	static BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
	static StringTokenizer st = new StringTokenizer("");
	/**
	 * Private constructor for servercontrol. Connects to the DB and Lobby and starts the GameWorld. And waits for clients to connect
	 * @param serverName
	 */
	private ServerControl(String serverName){
		super("World Server - "+ serverName);
		networkSettings = new shared.fileHandler.Settings("worldServer");
		controlSingleton = this;
		setVisible(true);
		setLocation(300, 300);
		setSize(200, 50);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JLabel lbl = new JLabel("Server is running..");
		add(lbl);
		running = true;
		dbHandler = new DBHandler(networkSettings.getDatabaseIp(),networkSettings.getDatabasePort());
		dbHandler.requestKingdomNames();
		new Thread(this).start();
	}
	/**
	 * Helper method for connecting to lobby after retrieving kingdom from DB.
	 */
	private void connectToLobby(String kingdomName,int kingdomId){
		try{
			this.worldInfo=new WorldServer(kingdomName,kingdomId,Inet4Address.getLocalHost().getHostAddress(),networkSettings.getClientListenPort());
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		writeLog("Connect to lobby");
		lobbyHandler = new LobbyHandler(networkSettings.getLobbyIp(),networkSettings.getLobbyPort(),worldInfo);
		writeLog("Listening to clients");
		new NetworkServer(networkSettings.getClientListenPort(),this);
		clientList = new HashMap<String, ClientHandler>();
		lastClientNumber = 0;
		writeLog("Server startup done");
	}
	/**
	 * Returns a singleton for the ServerControl
	 * @return
	 */
	public synchronized static ServerControl getServerControl(){
		if(controlSingleton == null){
			controlSingleton = new ServerControl(serverName);
		}
		
		return controlSingleton;
	}

	/**
	 * Creates the gameworld and zones.
	 * TODO: Implement, create the gameworld from db.
	 */
	public void startUpKingdom(String kingdomName,String subjectName, int kingdomId, HashMap<String,Zone> kingdom){
		//Connect to db, get kingdom id, zones and position of objects
		game = new GameWorld(kingdom);
		connectToLobby(kingdomName, kingdomId);
		requestQuestionList();
		
	}
	private void requestEnemyNpcList(){
		sendMsgToDB("Give me NPCs!", MessageType.GET_ALL_ENEMY_NPCS, "", "");
	}
	/**
	 * Returns the list of connected clients.
	 * @return
	 */
	public HashMap<String, ClientHandler> getClientList(){
		return clientList;
	}

	/**
	 * Sends a message to a client.
	 * @param clientId
	 * @param msg
	 */
	public void sendMessageToClient(String clientId,Message msg){
		clientList.get(clientId).sendMessage(msg);
	}
	/**
	 * When a client has been verified by the lobby server, it is sent to the GameWorld.
	 * @param clientId
	 */
	public void movePlayerToWorld(String clientId){
		game.addClientToWorld(clientId, clientList.get(clientId));
		clientList.remove(clientId);
	}
	/**
	 * Sends a message to the lobby server
	 * @param msg
	 */
	public void sendMsgToLobby(Message msg){
		lobbyHandler.sendMessage(msg);
	}
	
	/**
	 * Sends a message to the Database server.
	 * @param msg
	 */
	public void sendMsgToDB(String msg,MessageType type,String clientId,String characterName){
		
		dbHandler.sendMessage(new SessionIdMessage(sessionId,"DATABASE",msg,type,clientId,characterName));
	}

	/**
	 * Sets the list of questions that has been received from the Database handler.
	 * @param list
	 */
	public void setQuestionList(ArrayList<Question> list){
		game.setQuestionList(list);
	}
	
	
	public void setKingdomNameList(ArrayList<String> kingdoms){
		kingdomNames = new String[kingdoms.size()];
		int i = 0;
		for(String name : kingdoms){
			kingdomNames[i]= name;
			i++;
		}
		String selectedValue;
		if(kingdomNames.length>0){
			selectedValue = kingdomNames[0];
		}else{
			selectedValue = NOKINGDOM;
		}
		String choice = (String)JOptionPane.showInputDialog(
                this,
                "Select the kingdom you want to load from database.",
                "Select kingdom",
                JOptionPane.PLAIN_MESSAGE,
                null,
                kingdomNames,
                selectedValue);
		if(choice!=null&&!choice.equals(NOKINGDOM)){
			dbHandler.requestKingdom(choice);
			return;
		}
	}
	/**
	 * Sets the sessionId of the WorldServer that is received from the Lobby
	 * @param sessionId
	 */
	public void setSessionId(String sessionId){
		this.sessionId= sessionId;
		
		
	}
	/**
	 * returns the sessionId
	 * @return
	 */
	public String getSessionId() {
		return sessionId;
	}
	
	/**
	 * Is run when a new client connects to the WorldServer.
	 * Creates a temporary id, and creates the ClientHandler.
	 */
	public void newConnectionReceived(Socket newsocket) {
		lastClientNumber++;
		String clientID = "tempClientId"+lastClientNumber;
		clientList.put(clientID, new ClientHandler(new shared.network.ClientSocket(newsocket),clientID));
		System.out.println("New client added");
	}
	
	/**
	 * Changes the clientId of a client. (From temporary id to the assigned id from Lobby)
	 */
	public void changeClientSessionId(String oldId,String newId){
		ClientHandler temp = clientList.get(oldId);
		temp.setClientId(newId);
		clientList.remove(oldId);
		clientList.put(newId, temp);
	}
	/**
	 * Return the clientHandler.
	 * @param clientId
	 * @return
	 */
	public ClientHandler getPlayer(String clientId){
		return clientList.get(clientId);
	}
	/**
	 * Kicks a player with a given reason.
	 * @param clientId
	 * @param reason
	 */
	public void kickPlayer(String clientId,String reason){
		clientList.get(clientId).kickPlayer(reason);
		clientList.remove(clientId);
		writeLog("Removed client :"+clientId);
	}
	/**
	 * Removes a player from the client list.
	 * @param clientId
	 */
	public void removePlayer(String clientId){
		clientList.remove(clientId);
		
	}
	/**
	 * Request the List of questions from the DB.
	 */
	private void requestQuestionList(){
		sendMsgToDB("ME wants sie questions.",MessageType.GET_ALL_QUESTIONS,"","");
	}
	/**
	 * Is run when the verification message result comes from the Lobby. If the client has been verified, it retrieves his character from the DB. 
	 * If the client hasn't been verified, the client is kicked.
	 * @param clientId
	 * @param verified
	 */
	public void playerVerified(String clientId,boolean verified){
		if(verified){
			clientList.get(clientId).isVerified();
			sendMsgToDB("Get character",MessageType.GET_PLAYABLE_CHARACTER,clientId,clientList.get(clientId).getCharName());
		}else{
			kickPlayer(clientId,"Player is not authorized to connect.");
		}
	}
	/**
	 * Disconnects from DB and Lobby and closes the server
	 * TODO: Implement, and also include clients that are connected.
	 */
	private void close(){
		dbHandler.disconnect();
		lobbyHandler.disconnect();
		System.exit(1);
	}
	
	/**
	 * Private method used to get the network status of the server.
	 */
	private void status(){
		writeLog("----------STATUS-----------");
		writeLog("Connected to lobbyServer: "+lobbyHandler.isConnected());
		writeLog("Connected to databaseServer: "+dbHandler.isConnected());
		Iterator<ClientHandler> it = clientList.values().iterator();
		writeLog("Connected and waiting for verify:");
		while(it.hasNext()){
			writeLog(it.next().getClientId());
		}
		Iterator<ClientHandler> its = game.getPlayerList().values().iterator();
		writeLog("Connected and in the game world:");
		while(its.hasNext()){
			ClientHandler client = its.next();
			writeLog(client.getClientId()+" in zone: "+client.getInZone());
		}
	}
	/** 
	 * Read commands from commandline
	 */
	private void readCommands(){
		while(running){
		String command;
			try {
				command = readString();
				if(command.equals("close")){
					close();
				}else if(command.equals("status")){
					status();
				}else if(command.equals("print")){
					String type = readString();
					if(type.equals("attacks")){
						
					}
				}
				
			
			} catch (Exception e) {
				System.out.println("Command Read Error!");
			}
		}
	}
	
	
	// Read next input-token as string.
	private static String readString() throws Exception {
		while (!st.hasMoreTokens()) {
			st = new StringTokenizer(stdin.readLine());
		}
		return st.nextToken();
	}
	@Override
	public void run() {
		readCommands();
	}
	private static void writeLog(String logMessage){
		LogSystem.getInstance().writeWorldLog(logMessage);
	}
	
	/**
	 * Starts the WorldServer
	 * @param args
	 */
	public static void main(String args[]){
		new ServerControl("No kingdom loaded.");
	}
}
